program comp_clim_3d
!! #############################################################################
!!
!! Copyright 2018 IRD
!!
!! This file is part of NCSTAT.
!!
!! NCSTAT is free software: you can redistribute it and/or modify
!! it under the terms of the GNU Lesser General Public License as 
!! published by the Free Software Foundation, either version 3 of 
!! the License, or (at your option) any later version.
!!
!! NCSTAT is distributed in the hope that it will be useful,
!! but WITHOUT ANY WARRANTY; without even the implied warranty of
!! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!! GNU Lesser General Public License for more details.
!!
!! You can find a copy of the GNU Lesser General Public License
!! in the NCSTAT/doc directory.
!!
!! #############################################################################
!
!+
!
! .. _comp_clim_3d:
!
! ============
! comp_clim_3d
! ============
!
! Authors
! =======
!
! Pascal Terray (LOCEAN/IPSL)
!
! Latest revision
! ===============
!
! 12/09/2018
!
! Purpose
! =======
!
! Compute a climatology (e.g. means and standard-deviations) from a
! tridimensional variable extracted from a NetCDF dataset and, 
! optionally, the mask and scale factors of the 2-D grid-mesh
! associated with the input NetCDF variable.
!
! Mean and standard-deviation are computed for each point in the time series
! of the 2-D grid-mesh associated with the input NetCDF variable.
! These means and standard-deviations may be computed by taking into account
! the periodicity of the data. These statistics are stored in an output NetCDF dataset.
!
! The mean is a simple, but informative, measure, of the central tendency of a variable [vonStorch_Zwiers]_.
! The standard-deviation is a conventional measure of variation of a variable [vonStorch_Zwiers]_.
! If *X*\ (:) is a vector of *ntime* observations for one grid-point in the time series of the 2-D
! grid-mesh, the mean and standard-deviation statistics for this grid-point are estimated,
! respectively, by
!
! * **MEAN** = sum( *X*\ (:) ) / *ntime*
!
! * **STD** = sqrt( sum( [*X*\ (:)-\ **MEAN**\ ]**2 ) / *ntime* )
!
! Note that the divisor used in calculating standard-deviation is the number of observations,
! this is in contrast with the formulae used in :ref:`comp_stat_3d`, which uses the number of degrees
! of freedom.
!
! If your data contains missing values, use  :ref:`comp_clim_miss_3d` instead of comp_clim_3d
! to estimate means and standard deviations from your gappy dataset.
!
! If you need more univariate statistics on your input variable such as skewness, kurtosis, etc.,
! refer to :ref:`comp_stat_3d`. Finally, if the NetCDF variable is fourdimensional use :ref:`comp_clim_4d` 
! instead of comp_clim_3d.
!
! This procedure is parallelized if OpenMP is used and the NCSTAT software has been built
! with the _PARALLEL_READ CPP macro. Moreover, this procedure computes the means and
! standard-deviations with only one pass through the data and an out-of-core strategy
! which is highly efficient on huge datasets.
!
! Optionally, a mesh-mask NetCDF dataset may also be created. This dataset will contain
! a presence-absence mask and scale factor variables, which may be used to compute the surface
! of each cell in the 2-D grid-mesh associated with the input NetCDF variable. This
! mesh-mask NetCDF dataset will be used by other NCSTAT procedures such as :ref:`comp_serie_3d`,
! :ref:`comp_eof_3d`, etc.
!
! Further Details
! ===============
!
! Usage
! -----
!
! ::
!
!       $ comp_clim_3d \
!         -f=input_netcdf_file \
!         -v=netcdf_variable \
!         -p=periodicity                    (optional) \
!         -x=lon1,lon2                      (optional) \
!         -y=lat1,lat2                      (optional) \
!         -t=time1,time2                    (optional) \
!         -c=output_climatology_netcdf_file (optional) \
!         -m=output_mesh_mask_netcdf_file   (optional) \
!         -yl=latl1,latl2                   (optional) \
!         -mi=missing_value                 (optional) \
!         -fmsk=input_mesh_mask_netcdf_file (optional) \
!         -vmsk=mesh_mask_netcdf_variable   (optional) \
!         -val=mask_value                   (optional) \
!         -rel=mask_relation                (optional : eq, gt, ge, lt, le) \
!         -ntr=number_of_time_records       (optional) \
!         -double                           (optional) \
!         -bigfile                          (optional) \
!         -hdf5                             (optional) \
!         -tlimited                         (optional)
!
! By default
! ----------
!
!        *-p=*
!         the *periodicity* is equal to ``1``
!        *-x=*    
!         the whole longitude domain associated with the *netcdf_variable*
!        *-y=*    
!         the whole latitude domain associated with the *netcdf_variable*
!        *-t=*        
!         the whole time period associated with the *netcdf_variable*
!        *-c=*
!         the *output_climatology_netcdf_file* is named ``clim_``\ *netcdf_variable*\ ``.nc``
!        *-m=*        
!         the *output_mesh_mask_netcdf_file* is not created
!        *-yl=*       
!         it is assumed that the domain is the whole globe when computing the scale factors
!        *-mi=*       
!         the *missing_value* for the STD variable in the output NetCDF file is set to ``1.e+20``
!        *-fmsk=*     
!         an *input_mesh_mask_netcdf_file* is not used when computing the presence-absence mask
!        *-vmsk=*     
!         a *mesh_mask_netcdf_variable* is not used when computing the presence-absence mask
!        *-val=*      
!         this argument is set to ``1`` when computing the presence-absence mask
!        *-rel=*      
!         this argument is set to ``eq`` when computing the presence-absence mask
!        *-ntr=*      
!         the *number_of_time_records* read in each iteration of the procedure is set
!         to the *periodicity*
!        *-double*
!         the standard-deviations are stored as single-precision floating
!         point numbers in the output NetCDF file. If **-double** is activated, the
!         standard-deviations are stored as double-precision floating point numbers 
!        *-bigfile*
!         a NetCDF classical format file is created. If **-bigfile** is activated,
!         the output NetCDF file is a 64-bit offset format file
!        *-hdf5*
!         a NetCDF classical format file is created. If **-hdf5** is activated,
!         the output NetCDF file is a NetCDF-4/HDF5 format file
!        *-tlimited*
!         the time dimension is defined as unlimited in the output NetCDF file.
!         However, if **-tlimited** is activated, the time dimension is defined
!         as limited in the output NetCDF file
!
! Remarks
! -------
!
!   #) The **-v=**\ *netcdf_variable* argument specifies the NetCDF variable for 
!      which statistics must be computed and the **-f=**\ *input_netcdf_file* 
!      argument specifies that this NetCDF variable must be extracted from
!      the NetCDF file *input_netcdf_file*.
!
!   #) The **-p=**\ *periodicity* argument gives the periodicity of the input data.
!      For example, with monthly data **-p=**\ ``12`` should be specified,
!      with yearly data **-p=**\ ``1`` may be used, etc.
!      By default, the *periodicity* is set to ``1``. Note that the
!      output NetCDF file will have *periodicity* time observations.    
!
!   #) If the **-x=**\ *lon1,lon2* and **-y=**\ *lat1,lat2* arguments are missing, the whole
!      geographical domain associated with the *netcdf_variable* is used to
!      construct the climatology.
!      
!      The longitude or latitude range must be a vector of two integers
!      specifying the first and last selected indices along each dimension.
!      The indices are relative to ``1``.
!      Negative values for *lon1* are not allowed.
!
!      Refer to :ref:`comp_mask_3d` for transforming geographical coordinates as indices before
!      using comp_clim_3d.
!
!   #) If the **-t=**\ *time1,time2* argument is missing, the whole time period
!      associated with the *netcdf_variable* is used to construct the climatology
!      and to compute the statistics.
!
!      The selected time period is a vector of two integers specifying the
!      first and last time observations.
!      The indices are relative to ``1``.
!
!   #) It is assumed that the input data has no missing values (excepted missing
!      values associated with a constant land-sea mask as indicated by a 
!      *missing_value* or *_FillValue* attribute).
!
!      If it is the case, use :ref:`comp_clim_miss_3d` instead of comp_clim_3d.
!
!   #) If the **-m=**\ *output_mesh_mask_netcdf_file* argument is present and the
!      **-yl=** argument is missing, it is assumed that the whole geographical 
!      domain associated with the NetCDF variable is the earth and that the 2-D grid-mesh
!      is regular or Gaussian when computing the scale factors.
!
!      If the domain is not the whole globe, the **-yl=** argument must be
!      specified, otherwise the first and last columns (elements) of the first two scale
!      factors are wrong.
!
!      The **-yl=** argument specifies the latitude limits of the domain in degrees
!      (*latl1* and *latl2* must be real numbers).
!
!   #) If the **-m=**\ *output_mesh_mask_netcdf_file* argument is present, and if the 
!      **-fmsk=** and  **-vmsk=** arguments are also specified, 
!      the presence-absence mask in the *output_mesh_mask_netcdf_file* is computed
!      from the input *mesh_mask_netcdf_variable* (as specified by the **-vmsk=** and **-fmsk=**
!      arguments) as follows :
!
!      * **output_mask**\ (i,j) = ``1`` if **input_mask**\ (i,j) .\ *mask_relation*\ . *mask_value* is ``true``
!
!      * **output_mask**\ (i,j) = ``0`` otherwise
!
!      where *mask_relation* is determined from the **-rel=** argument and *mask_value*
!      from the **-val=** argument (*mask_value* is a real number). 
!
!      By default, *mask_relation* is ``eq`` and *mask_value* is ``1.``\  . Both the **-fmsk=** and
!      **-vmsk=** arguments must be present, otherwise the procedure will stop with an error
!      message.
!
!   #) If the **-m=**\ *output_mesh_mask_netcdf_file* argument is present and some
!      scale factors can not be computed, these scale factors are set to ``1``.    
!
!   #) The **-mi=**\ *missing_value* argument specifies the missing value indicator for
!      the standard-deviation (STD) variable in the *output_climatology_netcdf_file*. 
!      If the **-mi=** argument is not specified and the NetCDF variable has a
!      *missing_value* or *_FillValue* attribute, the *missing_value* is set
!      to ``1.e+20``.
!      This argument is not used if the NetCDF variable specified in
!      the **-v=** argument has no *missing_value* or *_FillValue* attribute.
!
!   #) The **-ntr=**\ *number_of_time_records* argument specifies how many time records are read
!      in each iteration of the loop for reading the input NetCDF variable.
!      By default, the *number_of_time_records* read in each iteration of the procedure is set
!      to the *periodicity* (as specified by the **-p=** argument). On very large dataset, it may be useful
!      to reduce the *number_of_time_records* in order to decrease the memory used by the procedure.
!
!   #) The **-double** argument specifies that the standard-deviation variable
!      must be stored as double-precision floating point numbers instead of
!      single-precision floating point numbers in the *output_climatology_netcdf_file*.
!
!   #) The **-bigfile** argument is allowed only if the NCSTAT software has been compiled with the
!      _USE_NETCDF36 or _USE_NETCDF4 CPP macros (e.g. ``-D_USE_NETCDF36`` or ``-D_USE_NETCDF4``) and linked
!      to the NetCDF 3.6 library or higher.
!      If this argument is specified, the *output_netcdf_file* will be a 64-bit offset format file
!      instead of a NetCDF classic format file. However, this argument is recognized in the procedure
!      only if the NCSTAT software has been built with the _USE_NETCDF36 or _USE_NETCDF4 CPP macros.
!
!   #) The **-hdf5** argument is allowed only if the NCSTAT software has been
!      compiled with the _USE_NETCDF4 CPP macro (e.g. ``-D_USE_NETCDF4``) and linked
!      to the NetCDF 4 library or higher. 
!      If this argument is specified, the *output_netcdf_file* will be a NetCDF-4/HDF5
!      format file instead of a NetCDF classic format file. However, this
!      argument is recognized in the procedure only if the NCSTAT software has
!      been built with the _USE_NETCDF4 CPP macro.
!
!   #) Duplicate parameters are allowed, but this is always the last occurrence
!      of a parameter which will be used for the computations. Moreover, the
!      number of specified parameters must not be greater than the total number
!      of allowed parameters.
!
!   #) For more details on the use of univariate statistics in the climate literature, see
!
!      * "Statistical Analysis in Climate Research",
!        by von Storch, H., and Zwiers, F.W., Cambridge University press,
!        Cambridge, UK, Chapter 2, 484 pp., 2002. ISBN: `9780521012300`_
!
! .. _9780521012300: http://www.cambridge.org/gb/academic/subjects/earth-and-environmental-science/climatology-and-climate-change/statistical-analysis-climate-research?format=PB
!
! Outputs
! -------
!
!   comp_clim_3d creates an output NetCDF file that contains the means, standard-deviations and number
!   of observations of the input NetCDF variable, taking into account eventually the periodicity of the
!   data as determined by the **-p=**\ *periodicity* argument. The output NetCDF dataset contains the
!   following NetCDF variables (in the description below, nlat and nlon are the length of the dimensions
!   of the input NetCDF variable) :
!
!   #) *netcdf_variable*\ _mean\ ``(periodicity,nlat,nlon)`` : the means for each point in the time series 
!      of the 2-D grid-mesh associated with the input NetCDF variable.
!
!   #) *netcdf_variable*\ _std\ ``(periodicity,nlat,nlon)`` : the standard-deviations for each point 
!      in the time series of the 2-D grid-mesh associated with the input NetCDF variable.
!
!   #) *netcdf_variable*\ _nobs\ ``(periodicity)`` : the number of observations used to compute
!      the statistics.
!
!   The means and standard-deviations are packed in tridimensional variables whose first and second
!   dimensions are exactly the same as those associated with the input NetCDF variable *netcdf_variable*
!   even if you restrict the geographical domain with the **-x=** and **-y=** arguments. However,
!   outside the selected domain, these output NetCDF variables are filled with missing values.
!
!   Optionally, comp_clim_3d can also create an output mesh-mask NetCDF file that contains
!   the following NetCDF variables :
!
!   #) *netcdf_variable*\ _nmask\ ``(nlat,nlon)`` : a presence-absence or land-sea 2-D mask associated
!      with the input NetCDF variable.
!
!   #) *netcdf_variable*\ _e1n\ ``(nlat,nlon)`` : the first scale factor associated with the
!      2-D grid-mesh of the input NetCDF variable.
!
!   #) *netcdf_variable*\ _e2n\ ``(nlat,nlon)`` : the second scale factor associated with the
!      2-D grid-mesh of the input NetCDF variable.
!
!   Multiplying the two scale factors together gives the surface of each cell in the 2-D grid-mesh
!   associated with the input NetCDF variable.
!
! Examples
! --------
!
!   #)   For computing monthly means and standard-deviations from the NetCDF
!        file ``ST7_1m_00101_20012_grid_T_sosstsst.nc``, which includes a NetCDF
!        variable ``sosstsst`` and store the results in the NetCDF file
!        ``clim_ST7_1m_00101_20012_grid_T_sosstsst.nc``, use the following
!        command :
!
!        ::
!
!          $ comp_clim_3d \
!            -f=ST7_1m_00101_20012_grid_T_sosstsst.nc \
!            -v=sosstsst \
!            -p=12 \
!            -c=clim_ST7_1m_00101_20012_grid_T_sosstsst.nc
!
!   #)   For computing monthly means and standard-deviations from the NetCDF
!        file ``sst.mnmean.nc``, which includes a NetCDF variable ``sst`` and
!        store the results in a NetCDF file named ``clim_sst.nc`` and, in addition,
!        generate an associated *mesh_mask_netcdf_file* named ``mesh_mask_sst.nc``,
!        use the following command :
!
!        ::
!
!          $ comp_clim_3d \
!            -f=sst.mnmean.nc \
!            -v=sst \
!            -p=12 \
!            -m=mesh_mask_sst.nc
!
!-
! ================================================================================================
!
#ifndef _OPENMP
#undef _PARALLEL_READ
#endif
!
#ifdef _USE_NETCDF4
#define _USE_NETCDF36
#endif
!
! USED MODULES 
! ============
!
#ifdef _USE_NAGWARE
    use f90_unix_env,     only : iargc, getarg
#endif
!
#ifdef _PARALLEL_READ
    use netcdf,    only : nf90_share
    use omp_lib,   only : omp_get_num_procs, omp_get_max_threads, omp_get_thread_num
#endif
!
#ifdef _USE_NETCDF36
    use netcdf,    only : nf90_64bit_offset
#endif
!
#ifdef _USE_NETCDF4
    use netcdf,    only : nf90_hdf5
#endif
!
    use netcdf,   only : nf90_max_var_dims, nf90_max_name, nf90_clobber, nf90_real4, nf90_real8,       &
                         nf90_int, nf90_short, nf90_byte,                                              & 
                         nf90_sizehint_default, nf90_open, nf90_noerr,                                 &
                         nf90_nowrite, nf90_strerror, nf90_inq_varid, nf90_inquire_variable,           &
                         nf90_inquire_dimension, nf90_get_var, nf90_close, nf90_get_att,               &
                         nf90_create, nf90_unlimited, nf90_def_dim, nf90_def_var, nf90_inq_attname,    &
                         nf90_copy_att, nf90_put_att, nf90_inquire_attribute, nf90_rename_att,         &
                         nf90_global, nf90_enddef, nf90_put_var, nf90_set_fill, nf90_close, nf90_nofill
!
    use Statpack, only : i1b, i4b, stnd, extd, lgl, zero, one, two, c90, c180, c360, twopi, pio2,   &
                         true, false, allocate_error, merror, comp_mvs_grp, arth,                   &
                         kfix, kexp, kint, tolower, case_change, string_to_val
!
    use NCSTAT_Utilities, only : prtunit, nchar, sp, dp, radius, def_missing_scal,       &
                                 def_missing, get_listarg, get_variable_dims
!
!
! STRONG TYPING IMPOSED 
! =====================
!
    implicit none
!
!
! PARAMETERS 
! ==========
!
!
    character(len=*), parameter :: name_proc='comp_clim_3d : ',              &
!
                                   crelation(5)=(/"eq","gt","ge","lt","le"/)         
!
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
!
#ifndef _USE_NAGWARE
    integer(i4b)  :: iargc
#endif
    integer(i4b)  :: narg, status, ncid, ncid_clim, ncid_mask, ncid_fmsk,                   & 
                     varid, meanid, stdid, periodid, nobsid, maskid, vmskid, e1id, e2id,    &
                     nlon, nlat, nt, nperiod, natts, xtype, xtype_att,                      &
                     i, j, j2, k, i1, i2, ioffset,                                          &
                     lon1, lon2, lon3, lat1, lat2, lat3, time1, time2, time3,               &
                     output_type, output_mode, mskrel, ntr
    integer(i4b),              dimension(3)                 :: coor_natts, coor_type,       &
                                                               coor_id, coor_id2, startlec
    integer(i4b),              dimension(nf90_max_var_dims) :: dimids, dimlens
    integer(i4b), allocatable, dimension(:)                 :: period
!
    integer(i1b)  :: kcode
!
    real(stnd), allocatable, dimension(:,:,:) :: data, mean, std
    real(stnd), allocatable, dimension(:,:)   :: mask
    real(stnd), allocatable, dimension(:)     :: n_period, temp1, temp2
    real(stnd)                                :: missing, missing2, mskval, latl1, latl2
!
    real(extd), allocatable, dimension(:)     :: record
!
    logical(lgl)                              :: first, last, use_yl,  use_tlimited,       &
                                                 use_missing, use_fillvalue, create_hdf5
    logical(lgl),              dimension(3)   :: use_coor_variable
    logical(lgl), allocatable, dimension(:,:) :: maskl
!
    character(len=14)                          :: fmt
    character(len=nchar)                       :: name, history
    character(len=nf90_max_name)               :: fic_name, var_name, clim_name, mask_name, att_name, &
                                                  fmsk_name, vmsk_name
    character(len=nf90_max_name), dimension(3) :: dim_names
!
#ifdef _PARALLEL_READ
!
    real(stnd)                                :: tmp
    real(stnd), allocatable, dimension(:,:,:) :: mean2, std2
!
    integer(i4b)  :: ncid2, varid2
!
    logical  :: test_par
!
#endif
!
!
! EXECUTABLE STATEMENTS
! =====================
!
!   GET THE NUMBER OF COMMAND LINE ARGUMENTS.
!
    narg = iargc()
!
!   CHECK THE NUMBER OF COMMAND LINE ARGUMENTS.
!
    if ( narg==0 ) then
!
        write (prtunit,*)
        write (prtunit,*) ' Purpose : '
        write (prtunit,*)
        write (prtunit,*) ' Compute a climatology from a tridimensional variable '
        write (prtunit,*) ' extracted from a NetCDF dataset.'
        write (prtunit,*)
        write (prtunit,*) ' Usage : '
        write (prtunit,*)
        write (prtunit,*) ' comp_clim_3d -f=input_netcdf_file'
        write (prtunit,*) '              -v=netcdf_variable'
        write (prtunit,*) '              -p=periodicity                    (optional)'
        write (prtunit,*) '              -x=lon1,lon2                      (optional)'
        write (prtunit,*) '              -y=lat1,lat2                      (optional)'
        write (prtunit,*) '              -t=time1,time2                    (optional)'
        write (prtunit,*) '              -c=output_climatology_netcdf_file (optional)'
        write (prtunit,*) '              -m=output_mesh_mask_netcdf_file   (optional)'
        write (prtunit,*) '              -yl=latl1,latl2                   (optional)'
        write (prtunit,*) '              -mi=missing_value                 (optional)'
        write (prtunit,*) '              -fmsk=input_mesh_mask_netcdf_file (optional)'
        write (prtunit,*) '              -vmsk=mesh_mask_netcdf_variable   (optional)'
        write (prtunit,*) '              -val=mask_value                   (optional)'
        write (prtunit,*) '              -rel=mask_relation                (optional : eq, gt, ge, lt, le)'
        write (prtunit,*) '              -ntr=number_of_time_records       (optional)'
        write (prtunit,*) '              -double                           (optional)'
#ifdef _USE_NETCDF36
        write (prtunit,*) '              -bigfile                          (optional)'
#endif
#ifdef _USE_NETCDF4
        write (prtunit,*) '              -hdf5                             (optional)'
#endif
        write (prtunit,*) '              -tlimited                         (optional)'
        write (prtunit,*)
        write (prtunit,*) ' By default : '
        write (prtunit,*)
        write (prtunit,*) '-p= the periodicity is set to 1'
        write (prtunit,*) '-x= the whole longitude domain associated with the netcdf_variable'
        write (prtunit,*) '-y= the whole latitude domain associated with the netcdf_variable'
        write (prtunit,*) '-t= the whole time period associated with the netcdf_variable'
        write (prtunit,*) '-c=clim_netcdf_variable.nc'
        write (prtunit,*) '-m= the mesh-mask NetCDF file is not created'
        write (prtunit,*) '-yl= it is assumed that the domain is the whole globe'
        write (prtunit,*) '-mi= the missing_value for the STD variable is equal to 1.e+20'
        write (prtunit,*) '-fmsk= an input_mesh_mask_netcdf_file is not used'
        write (prtunit,*) '-vmsk= an input mesh_mask_netcdf_variable is not used'
        write (prtunit,*) '-val=1.'
        write (prtunit,*) '-rel=eq'
        write (prtunit,*) '-ntr= the number_of_time_records is equal to the periodicity'
        write (prtunit,*) '-double   : by default, the STD variable is stored as single floating numbers'
#ifdef _USE_NETCDF36
        write (prtunit,*) '-bigfile  : by default, a NetCDF classical format file is created'
#endif
#ifdef _USE_NETCDF4
        write (prtunit,*) '-hdf5     : by default, a NetCDF classical format file is created'
#endif
        write (prtunit,*) '-tlimited : by default, the time dimension is defined as unlimited'
        write (prtunit,*)
#ifdef _PARALLEL_READ
        write (prtunit,*) 'This procedure is parallelized with OpenMP'
#endif
        write (prtunit,*)
!
        stop
!
    else if ( narg<2 .or. narg>19 ) then
!
        call merror( name_proc//"Wrong number of line arguments" )
!
    end if
!
    fic_name  = " "
    var_name  = " "
    clim_name = " "
    mask_name = " "
    fmsk_name = " "
    vmsk_name = " "
!
    history = "comp_clim_3d"
!
!   DEFAULT VALUES FOR COMMAND LINE ARGUMENTS.
!
    nperiod     = 1
    ntr         =-1
    output_mode = nf90_clobber
    output_type = nf90_real4
    mskrel      = 1
!
    missing2    = def_missing
    mskval      = one
!
    lon1      = 1
    lon2      = huge( lon2 )
    lat1      = 1
    lat2      = huge( lat2 )
    time1     = 1
    time2     = huge( time2 )
!
    latl1     = zero
    latl2     = zero
!
    use_yl       = false
    create_hdf5  = false
    use_tlimited = false
!
!   CHECK THE COMMAND LINE ARGUMENTS.
!
    do i = 1, narg
!
        call getarg( i, name )
!
        history = trim(history)//" "//trim(name)
!
        call case_change( name(:3), tolower )
!
        select case ( name(1:3 ) )
!
            case ('-f=')
!
!               GET THE NAME OF THE NETCDF DATASET.
!
                fic_name = name(4:)
!
            case ('-v=')
!
!               GET THE NAME OF THE NETCDF VARIABLE IN THE DATASET.
!
                var_name = name(4:)
!
            case ('-p=')
!
!               GET THE PERIODICITY OF THE DATA.
!
                call string_to_val( name(4:), kcode, fmt )
                if ( kcode==kint ) then
                    read( name(4:), fmt ) nperiod
                end if
!
                if ( kcode/=kint .or. nperiod<=0 )      &
                call merror( name_proc//"The periodicity is an integer > 0 " )
!
            case ('-x=')
!
!               GET THE DOMAIN DEFINITION (Longitude).
!
                call get_listarg( name_proc, name(4:),                    &
                                  "lon1 and lon2 are integer values",     &
                                  "Wrong domain definition (longitude)",  &
                                  lon1, lon2 )
!
            case ('-y=')
!
!               GET THE DOMAIN DEFINITION (latitude).
!
                call get_listarg( name_proc, name(4:),                    &
                                  "lat1 and lat2 are integer values",     &
                                  "Wrong domain definition (latitude)",   &
                                  lat1, lat2 )
!
            case ('-t=')
!
!               GET THE PERIOD DEFINITION.
!
                call get_listarg( name_proc, name(4:),                    &
                                  "time1 and time2 are integer values",   &
                                  "Wrong period definition",              &
                                  time1, time2 )
!
            case ('-c=')
!
!               GET THE NAME OF THE NEW NETCDF DATASET FOR THE CLIMATOLOGY.
!
                clim_name = name(4:)
!
            case ('-m=')
!
!               GET THE NAME OF THE OPTIONAL MASK NETCDF DATASET.
!
                mask_name = name(4:)
!
            case default
!
                select case ( name(1:4 ) )
!
                    case ('-yl=')
!
!                       GET THE DOMAIN LIMITS.
!
                        call get_listarg( name_proc, name(5:),                              &
                                          "latl1 and latl2 must be real numbers",           &
                                          "Wrong domain definition limits (latl1 latl2)",   &
                                          latl1, latl2 )
!
                        if ( abs(latl1)>c90 .or. abs(latl2)>c90 )        &
                 call merror( name_proc//" abs(latl1)>90. or abs(latl2)>90. in -yl= argument" )
!
                        use_yl = true
!
                    case ('-mi=')
!
!                       GET THE MISSING VALUE INDICATOR FOR THE STD VARIABLE.
!
                        call string_to_val( name(5:), kcode, fmt )
                        if ( kcode/=kfix .and. kcode/=kexp )                &
                    call merror( name_proc//"The -mi= argument must be a real number " )
                        read( name(5:), fmt ) missing2
!
                    case default
!
                        call case_change( name(4:5), tolower )
!
                        select case ( name(1:5 ) )
!
                            case ('-val=')
!
!                               GET THE MASK VALUE FOR THE INPUT MESH-MASK FILE.
!
                                call string_to_val( name(6:), kcode, fmt )
                                if ( kcode/=kfix .and. kcode/=kexp )                &
                                call merror( name_proc//"The -val= argument must be a real number " )
                                read( name(6:), fmt ) mskval
!
                            case ('-rel=')
!
!                               DETERMINE THE MASK RELATION FOR THE INPUT MESH-MASK FILE.
!
                                call case_change( name(6:), tolower )
!
                                select case ( name(6:) )
                                    case ( crelation(1) )
                                        mskrel = 1
                                    case ( crelation(2) )
                                        mskrel = 2
                                    case ( crelation(3) )
                                        mskrel = 3
                                    case ( crelation(4) )
                                        mskrel = 4
                                    case ( crelation(5) )
                                        mskrel = 5
                                    case default
                                        call merror( name_proc//"Unknown mask_relation option" )
                                end select
!
                            case ('-ntr=')
!
!                               GET THE NUMBER OF TIME OBSERVATIONS TO READ.
!
                                call string_to_val( name(6:), kcode, fmt )
                                if ( kcode==kint ) then
                                    read( name(6:), fmt ) ntr
                                end if
!
                                if ( kcode/=kint .or. ntr<=0 )      &
                                call merror( name_proc//"The -ntr= argument is an integer > 0 " )
!
                            case default
!
                                select case ( name(1:6 ) )
!
                                    case ('-fmsk=')
!
!                                       GET THE NAME OF THE INPUT MESH-MASK FILE.
!
                                        fmsk_name = name(7:)
!
                                    case ('-vmsk=')
!
!                                       GET THE NAME OF THE INPUT MESH-MASK VARIABLE.
!
                                        vmsk_name = name(7:)
!
                                    case default
!
                                         call case_change( name(6:), tolower )
!
                                         select case ( name(1:) )
!
                                             case ('-tlimited')
!
!                                                DEFINE THE TIME DIMENSION AS LIMITED.
!
                                                 use_tlimited = true
!
                                             case ('-double')
!
!                                                STORE STATISTICS AS DOUBLE PRECISION NUMBERS.
!
                                                 output_type = nf90_real8
!
#ifdef _USE_NETCDF36
                                             case ('-bigfile')
!
!                                                REDEFINE THE CREATION MODE FLAG FOR THE OUTPUT FILE.
!
                                                 output_mode = nf90_64bit_offset
#endif
!
#ifdef _USE_NETCDF4
                                             case ('-hdf5')
!
!                                                REDEFINE THE CREATION MODE FLAG FOR THE OUTPUT FILE.
!
                                                 output_mode = nf90_hdf5
                                                 create_hdf5 = true
#endif
!
                                             case default
!
!                                                WRONG ARGUMENT.
!
                                                 call merror( name_proc//"Wrong argument" )
!
                                        end select
!
                                end select
!
                        end select
!
                end select
!
        end select
!
    end do
!
!   CHECK THE COMMAND LINE ARGUMENTS.
!
    if ( fic_name==" " ) call merror( name_proc//"Input NETCDF file is missing" )
!
    if ( var_name==" " ) call merror( name_proc//"Input NETCDF variable is missing" )
!
    if ( fmsk_name/=" " ) then
        if ( vmsk_name==" " ) call merror( name_proc//"Input NETCDF mesh_mask_variable is missing" )
    end if
!
    if ( vmsk_name/=" " ) then
        if ( fmsk_name==" " ) call merror( name_proc//"Input NETCDF mesh_mask_file is missing" )
    end if
!
    if ( ntr<=-1 )  ntr = nperiod
!
    if ( clim_name==" " ) clim_name="clim_"//trim(var_name)//".nc"
!
!   OPEN THE NETCDF DATASET.
!
    k = nf90_sizehint_default
    status = nf90_open( trim(fic_name), nf90_nowrite, ncid, chunksize=k )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!   GET THE VARIABLE ID FROM ITS NAME.
!
    status = nf90_inq_varid( ncid, trim(var_name), varid )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!   FIND THE EXTERNAL TYPE, THE DIMENSION LENGTHS AND NAMES, 
!   THE NUMBER OF ATTRIBUTS FOR THE NETCDF VARIABLE.
!
    status = nf90_inquire_variable( ncid, varid, xtype=xtype, ndims=i, dimids=dimids, natts=natts )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    if ( i/=3 ) call merror( name_proc//"Input NETCDF variable is not tridimensional" )
!
    do i = 1, 3
!
        status = nf90_inquire_dimension( ncid, dimids(i), name=dim_names(i), len=dimlens(i) )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    end do
!
    nlon = dimlens(1)
    nlat = dimlens(2)
    nt   = dimlens(3)
!
    do i = 1, 3
!
!       GET THE COORDINATE VARIABLES IDS FROM THEIR NAME, IF ANY.
!
        status = nf90_inq_varid( ncid, trim(dim_names(i)), coor_id(i) )
        if ( status==nf90_noerr ) then
!
!           FIND THE EXTERNAL TYPE, THE DIMENSION ID AND THE NUMBER OF ATTRIBUTES 
!           FOR THE COORDINATE VARIABLES.
!
            status = nf90_inquire_variable( ncid, coor_id(i), xtype=coor_type(i), ndims=k,    &
                                            dimids=dimlens, natts=coor_natts(i) )
            if ( status/=nf90_noerr )                                                         &
            call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            use_coor_variable(i) = k==1 .and. dimlens(1)==dimids(i)
        else
            use_coor_variable(i) = false
        end if
!
        if ( .not.use_coor_variable(i) ) coor_type(i) = nf90_int
!
    end do
!
!   CHECK THE BOUNDARIES OF THE GEOGRAPHICAL DOMAIN.
!
    lat1 = max( lat1, 1 )
    lat2 = min( lat2, nlat )
    lat3 = lat2 - lat1 + 1
    if ( lat3<=0  )  call merror( name_proc//"Wrong domain definition (latitude)" )
!
    if ( lon1<=0  )  call merror( name_proc//"Wrong domain definition (longitude)" )
    lon2 = min( lon2, nlon )
    lon3 = lon2 - lon1 + 1
    if ( lon3<=0  )  call merror( name_proc//"Wrong domain definition (longitude)" )
!
!   CHECK THE TIME DEFINITION.
!
    time1 = max( time1, 1 )
    time2 = min( time2, nt )
    time3 = time2 - time1 + 1
    if ( time3<=0 ) call merror( name_proc//"Wrong time definition" )
!
!   CHECK THE PERIOD DEFINITION.
!
    if ( nperiod>time3 )  call merror( name_proc//"-t=, -p= options mismatch" )
!
!   CHECK THE NUMBER OF TIME RECORDS DEFINITION.
!
    ntr = min( ntr, time3 )
!
!   ALLOCATE WORK ARRAYS.
!
    allocate( mean(lon3,lat3,nperiod),           &
              std(lon3,lat3,nperiod),            &
              period(time3),                     &
              n_period(nperiod),                 &
              stat=status )
    if ( status/=0 ) call merror( name_proc//allocate_error )
!
!   CONSTRUCT SEASONAL AXIS.
!
    j = 1
!
    do i = 1, time3
!
        period(i) = j
!
        if ( j==nperiod ) then
            j = 1
        else
            j = j + 1
        end if
!
    end do
!
    ioffset = 1 - time1
!
#ifdef _PARALLEL_READ
!
    i = omp_get_num_procs()
    j = omp_get_max_threads()
    test_par = i>1 .and. j>1 .and. time3/ntr>=j
!
    if ( test_par ) then
!
!$OMP PARALLEL PRIVATE(i,i1,i2,k,status,startlec,ncid2,varid2,data,mean2,std2,temp2,tmp,first)     &
!$OMP         ,SHARED(period,mean,std,n_period,nperiod)
!
!       ALLOCATE WORK ARRAYS.
!
        allocate( data(lon3,lat3,ntr),                &
                  mean2(lon3,lat3,nperiod),           &
                  std2(lon3,lat3,nperiod),            &
                  temp2(nperiod),                     &
                  stat=status )
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
!$OMP CRITICAL (opennc)
!
!       OPEN THE NETCDF DATASET.
!
        status = nf90_open( trim(fic_name), nf90_share, ncid2 )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!       GET THE VARIABLE ID FROM ITS NAME.
!
        status = nf90_inq_varid( ncid2, trim(var_name), varid2 )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!$OMP END CRITICAL (opennc)
!
        startlec(1)  = lon1
        startlec(2)  = lat1
        first        = true
!
!$OMP DO SCHEDULE(STATIC)
        do i = time1, time2, ntr
!
            k = min( time2-i+1, ntr )
            startlec(3) = i
!
!           FILL THE WORK ARRAY WITH THE NETCDF VARIABLE.
!
            status = nf90_get_var( ncid2, varid2, data(:lon3,:lat3,:k), start=startlec )     
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           COMPUTE UNIVARIATE STATISTICS FOR THE NETCDF VARIABLE.
!
            i1 = i + ioffset
            i2 = i1 + k - 1
!
            call comp_mvs_grp( data(:lon3,:lat3,:k), first, false, nperiod, period(i1:i2),       &
                               mean2, std2, temp2 )
            first = false
!
        end do
!$OMP END DO
!
!$OMP MASTER
        mean(:lon3,:lat3,:nperiod) = mean2(:lon3,:lat3,:nperiod)
        std(:lon3,:lat3,:nperiod)  = std2(:lon3,:lat3,:nperiod)
        n_period(:nperiod)         = temp2(:nperiod)
!$OMP END MASTER
!
!$OMP BARRIER
!
!$OMP CRITICAL (sumstat)
        if ( omp_get_thread_num()/=0 ) then
!
            do i = 1, nperiod
!
                tmp = n_period(i) + temp2(i)
!
                data(:lon3,:lat3,1)  = std(:lon3,:lat3,i) + std2(:lon3,:lat3,i)    +   &
                         ((n_period(i)*temp2(i))/tmp)*( mean(:lon3,:lat3,i) - mean2(:lon3,:lat3,i) )**2
                std(:lon3,:lat3,i)  = data(:lon3,:lat3,1)
!
                data(:lon3,:lat3,1) = (n_period(i)/tmp)*mean(:lon3,:lat3,i)    +   &
                                      (temp2(i)/tmp)*mean2(:lon3,:lat3,i)
                mean(:lon3,:lat3,i) = data(:lon3,:lat3,1)
!
                n_period(i) = tmp
!
            end do
!
        end if
!$OMP END CRITICAL (sumstat)
!
!       DEALLOCATE WORK ARRAYS.
!
        deallocate( data, temp2, mean2, std2 )
!
!$OMP CRITICAL (closenc)
!
!       CLOSE THE NETCDF DATASET.
!
        status = nf90_close( ncid2 )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!$OMP END CRITICAL (closenc)
!
!$OMP END PARALLEL
!
!       REALLOCATE WORK ARRAY.
!
        allocate( data(lon3,lat3,1), stat=status )
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
    else
!
#endif
!
!       ALLOCATE WORK ARRAY.
!
        allocate( data(lon3,lat3,ntr), stat=status )
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
        startlec(1)  = lon1
        startlec(2)  = lat1
        first        = true
!
        do i = time1, time2, ntr
!
            k = min( time2-i+1, ntr )
            startlec(3) = i
!
!           FILL THE WORK ARRAY WITH THE NETCDF VARIABLE.
!
            status = nf90_get_var( ncid, varid, data(:lon3,:lat3,:k), start=startlec )     
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           COMPUTE UNIVARIATE STATISTICS FOR THE NETCDF VARIABLE.
!
            i1 = i + ioffset
            i2 = i1 + k - 1
            call comp_mvs_grp( data(:lon3,:lat3,:k), first, false, nperiod, period(i1:i2),       &
                               mean, std, n_period )
            first = false
!
        end do
!
#ifdef _PARALLEL_READ
!
    end if
!
#endif
!
!   FINISH COMPUTATIONS OF UNIVARIATE STATISTICS FOR THE NETCDF VARIABLE.
!
    call comp_mvs_grp( data(:lon3,:lat3,1:0), false, true, nperiod, period(1:0),      &
                       mean, std, n_period )
!
!   FIND THE MISSING ATTRIBUTE, IF ANY.
!
    status = nf90_get_att( ncid, varid, "missing_value", missing )
!
    use_missing = status==nf90_noerr
    if (.not.use_missing ) then
!
!       TRY THE _FillValue ATTRIBUTE IF THE MISSING ATTRIBUTE IS ABSENT.
!
        status = nf90_get_att( ncid, varid, "_FillValue", missing )
        use_fillvalue = status==nf90_noerr
!
    else
!
        use_fillvalue = false
!
    end if
!
    if ( use_missing .or. use_fillvalue ) then
!
!       ALLOCATE LOGICAL MASK ARRAY.
!
        allocate( maskl(lon3,lat3), stat=status )
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
!       APPLY THE LAND-SEA MASK TO THE MEAN AND STD ARRAYS.
!
#ifdef _PARALLEL_READ
!
        if ( test_par ) then
!
            startlec(1:3)  = (/ lon1, lat1, time1 /)
            status = nf90_get_var( ncid, varid, data(:lon3,:lat3,:1), start=startlec )     
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end if
!
#endif
!
        maskl(:lon3,:lat3) = data(:lon3,:lat3,1)==missing
!
        if ( any(maskl(:lon3,:lat3)) ) then
!
            do i = 1, nperiod
!
                where( maskl(:lon3,:lat3) )
                    mean(:lon3,:lat3,i) = missing
                    std(:lon3,:lat3,i)  = missing2
                end where
!
            end do
!
!           DEALLOCATE LOGICAL MASK ARRAY, IF NOT NEEDED LATER.
!
            if ( mask_name==" " ) deallocate( maskl )
!
        else
!
!           DEALLOCATE LOGICAL MASK ARRAY.
!
            deallocate( maskl )
!
        end if
!
    end if
!
!   CREATE THE NEW NETCDF DATASET FOR THE CLIMATOLOGY.
!
    if ( create_hdf5 ) then
        status = nf90_create( trim(clim_name), cmode=output_mode, ncid=ncid_clim )
    else
        k = nf90_sizehint_default
        status = nf90_create( trim(clim_name), cmode=output_mode, ncid=ncid_clim, chunksize=k )
    end if
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!   DEFINE THE DIMENSIONS FOR THE CLIMATOLOGY NETCDF DATASET.
!
    dimlens(1)   = nlon
    dimlens(2)   = nlat
    if ( use_tlimited ) then
        dimlens(3)   = nperiod
    else
        dimlens(3)   = nf90_unlimited
    end if
!
    do i = 1, 3
!
        status = nf90_def_dim( ncid_clim, name=trim(dim_names(i)), len=dimlens(i), dimid=dimids(i) )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    end do
!
!   DEFINE THE VARIABLES FOR THE CLIMATOLOGY NETCDF DATASET.
!
    do i = 1, 3
!
        status = nf90_def_var( ncid_clim, trim(dim_names(i)), coor_type(i),                &
                               dimids=dimids(i), varid=coor_id2(i) )
        if ( status/=nf90_noerr )                                                          &
        call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end do
!
    status = nf90_def_var( ncid_clim, trim(var_name)//"_mean", xtype, dimids=dimids(:3), varid=meanid )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_def_var( ncid_clim, trim(var_name)//"_std", output_type, dimids=dimids(:3), varid=stdid )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_def_var( ncid_clim, trim(var_name)//"_nobs", nf90_int, dimids=dimids(3), varid=nobsid )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_def_var( ncid_clim, "period", nf90_int, dimids=dimids(3), varid=periodid )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!   ASSIGN ATTRIBUTES FOR THE CLIMATOLOGY NETCDF DATASET.
!
    do j = 1, 3
!
        if ( .not.use_coor_variable(j) ) cycle
!
        do i = 1, coor_natts(j)
!
            status = nf90_inq_attname( ncid, coor_id(j), i, att_name )
            if ( status/=nf90_noerr )                                                     &
            call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           COPY ATTRIBUTES FOR THE COORDINATE VARIABLES.
!
            status = nf90_copy_att(  ncid, coor_id(j), att_name, ncid_clim, coor_id2(j) )
            if ( status/=nf90_noerr )                                                     &
            call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end do
!
    end do
!
    status = nf90_put_att( ncid_clim, periodid, "long_name", "Period axis" )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_clim, periodid, "short_name", "Period" )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    do i = 1, natts
!
        status = nf90_inq_attname( ncid, varid, i, att_name )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!       COPY AND MODIFY USEFUL ATTRIBUTES.
!
        select case ( att_name )
!
            case ( 'units', 'scale_factor' )
!
!               COPY USEFUL ATTRIBUTES.
!
                status = nf90_copy_att(  ncid, varid, att_name, ncid_clim, meanid )
                if ( status/=nf90_noerr )                                             &
                call merror( name_proc//trim( nf90_strerror( status ) ) )
!
                status = nf90_copy_att(  ncid, varid, att_name, ncid_clim, stdid )
                if ( status/=nf90_noerr )                                             &
                call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            case ( 'long_name', 'short_name' )
!
#ifdef _USE_NETCDF36
!
! Due to a bug in the NETCDF 3.5 library, the following section of code can't be used.
! However, if you are using NETCDF 3.6.1 or higher, you can use this section safely.
!
                status = nf90_inquire_attribute( ncid, varid, att_name, len=j )
                if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!               CHECK LENGTH OF STRING ATTRIBUTE.
!
                if ( j <= nchar ) then
                    status = nf90_get_att( ncid, varid, att_name, name )
                    if ( status/=nf90_noerr )                                             &
                    call merror( name_proc//trim( nf90_strerror( status ) ) )
                    name(j+1:nchar) = " "
                else
                    name = var_name
                end if
#else
!
! This section of code must be used if you are using NETCDF 3.5 .
!
                name(1:nchar) = " "
                status = nf90_get_att( ncid, varid, att_name, name )
                if ( status/=nf90_noerr )                                             &
                call merror( name_proc//trim( nf90_strerror( status ) ) )
#endif
!
!               MODIFY USEFUL ATTRIBUTES.
!
                status = nf90_put_att( ncid_clim, meanid, att_name, "Means of "//trim(name)  )
                if ( status/=nf90_noerr )                                             &
                call merror( name_proc//trim( nf90_strerror( status ) ) )
!
                status = nf90_put_att(  ncid_clim, stdid, att_name, "STDs of "//trim(name) )
                if ( status/=nf90_noerr )                                             &
                call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            case ( 'add_offset' )
!
!               COPY USEFUL ATTRIBUTES.
!
                status = nf90_copy_att(  ncid, varid, att_name, ncid_clim, meanid )
                if ( status/=nf90_noerr )                                             &
                call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            case ( 'missing_value' )
!
!               COPY missing_value ATTRIBUTE, IF REQUIRED.
!
                status = nf90_copy_att(  ncid, varid, att_name, ncid_clim, meanid )
                if ( status/=nf90_noerr )                                             &
                call merror( name_proc//trim( nf90_strerror( status ) ) )
!
                status = nf90_inquire_attribute(  ncid, varid, att_name, xtype=xtype_att )
                if ( status/=nf90_noerr )                                             &
                call merror( name_proc//trim( nf90_strerror( status ) ) )
!
                if ( xtype==xtype_att ) then
!
                    status = nf90_rename_att( ncid_clim, meanid, att_name, "_FillValue" )
                    if ( status/=nf90_noerr )                                 &
                    call merror( name_proc//trim( nf90_strerror( status ) ) )
!
                    status = nf90_copy_att(  ncid, varid, att_name, ncid_clim, meanid )
                    if ( status/=nf90_noerr )                                 &
                    call merror( name_proc//trim( nf90_strerror( status ) ) )
!
                end if
!
            case ( '_FillValue' )
!
!               COPY fill_value ATTRIBUTE, IF REQUIRED.
!
                if ( use_fillvalue ) then
!
                    status = nf90_copy_att(  ncid, varid, att_name, ncid_clim, meanid )
                    if ( status/=nf90_noerr )                                 &
                    call merror( name_proc//trim( nf90_strerror( status ) ) )
!
                    status = nf90_rename_att( ncid_clim, meanid, att_name, "missing_value" )
                    if ( status/=nf90_noerr )                                 &
                    call merror( name_proc//trim( nf90_strerror( status ) ) )
!
                    status = nf90_copy_att(  ncid, varid, att_name, ncid_clim, meanid )
                    if ( status/=nf90_noerr )                                 &
                    call merror( name_proc//trim( nf90_strerror( status ) ) )
!
                end if
!
        end select
!
    end do
!
    if ( use_missing .or. use_fillvalue ) then
!
        if ( output_type==nf90_real4 ) then
!
            status = nf90_put_att( ncid_clim, stdid, "missing_value", real( missing2, sp )  )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_clim, stdid, "_FillValue", real( missing2, sp )  )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        else
!
            status = nf90_put_att( ncid_clim, stdid, "missing_value", real( missing2, dp )  )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_clim, stdid, "_FillValue", real( missing2, dp )  )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end if
!
    end if
!
    status = nf90_put_att( ncid_clim, nobsid, "long_name", "Number of observations" )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_clim, nobsid, "short_name", "Nobs" )
    if ( status/=nf90_noerr ) call merror( name_proc// trim( nf90_strerror( status ) ) )
!
!   SET GLOBAL ATTRIBUTES.
!
    status = nf90_put_att( ncid_clim, nf90_global, "title", trim(var_name)//" Climatology" )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_clim, nf90_global, "lon1_"//trim(var_name), lon1 )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_clim, nf90_global, "lon2_"//trim(var_name), lon2 )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_clim, nf90_global, "lat1_"//trim(var_name), lat1 )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_clim, nf90_global, "lat2_"//trim(var_name), lat2 )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_clim, nf90_global, "history", trim(history) )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!   LEAVE DEFINE MODE FOR THE CLIMATOLOGY NETCDF DATASET.
!
    status = nf90_enddef( ncid_clim )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!   WRITE VARIABLES FOR THE CLIMATOLOGY NETCDF DATASET.
!
    dimlens(3) = nperiod
!
!   ALLOCATE WORK ARRAY.
!
    i = max( nlon, nlat, nperiod )
    allocate( record(i), stat=status )
    if ( status/=0 ) call merror( name_proc//allocate_error )
!
    do j = 1, 3   
!
        j2 = dimlens(j)
!
        if ( use_coor_variable(j) ) then
            status = nf90_get_var( ncid, coor_id(j), record(:j2), count=(/ j2 /) )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_var( ncid_clim, coor_id2(j), record(:j2), count=(/ j2 /) )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
        else
            status = nf90_put_var( ncid_clim, coor_id2(j), arth(1_i4b,1_i4b,j2) )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
        end if
!
    end do
!
!   DEALLOCATE WORK ARRAY.
!
    deallocate( record )
!
    startlec(1:3) = (/ lon1, lat1, 1_i4b /)
!
    status = nf90_put_var( ncid_clim, periodid, period(:nperiod) )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!   WRITE CLIMATOLOGY VARIABLES.
!
    if ( xtype == nf90_int .or. xtype == nf90_short .or. xtype == nf90_byte ) then 
       status = nf90_put_var( ncid_clim, meanid, nint(mean), start=startlec )
    else
       status = nf90_put_var( ncid_clim, meanid, mean, start=startlec )
    end if
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_var( ncid_clim, stdid, std, start=startlec )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_var( ncid_clim, nobsid, n_period )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!   CONSTRUCT THE MESH-MASK DATASET IF REQUIRED.
!
    if ( mask_name/=" " ) then
!
!       DEALLOCATE WORK ARRAYS.
!
        deallocate( data, mean, std, period, n_period  )
!
!       ALLOCATE WORK ARRAYS.
!
        allocate( mask(nlon,nlat), temp1(nlat), temp2(nlat+1), stat=status )
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
!       CONSTRUCT THE MASK.
!
        if ( fmsk_name==" " ) then
!
            mask(:nlon,:nlat) = zero
!
            if ( allocated(maskl) ) then
!
                where( .not.maskl(:lon3,:lat3) )  mask(lon1:lon2,lat1:lat2) = one
!
!               DEALLOCATE LOGICAL MASK ARRAY.
!
                deallocate( maskl )
!
            else
                mask(lon1:lon2,lat1:lat2) = one
            end if
!
        else
!
!           OPEN THE INPUT NETCDF MESH-MASK DATASET.
!
            status = nf90_open( trim(fmsk_name), nf90_nowrite, ncid_fmsk )
            if ( status/=nf90_noerr )                                            &
            call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           GET THE MASK ID FROM ITS NAME.
!           VERIFY THE DIMENSION LENGTHS FOR THE NETCDF MASK VARIABLE.
!
            call get_variable_dims( name_proc, trim(vmsk_name), ncid_fmsk,    &
                                    vmskid, dimids, dimlens )
!
            if ( dimlens(1)/=nlon .or. dimlens(2)/=nlat )                      &
            call merror( name_proc//"Shape disagreement with the input mask" )
!
!           GET THE MASK.
!
            status = nf90_get_var( ncid_fmsk, vmskid, mask(:nlon,:nlat)  )     
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           DETERMINE THE GRID POINTS USED FROM THE MASK, MASK RELATION AND MASK VALUE.
!
            select case ( mskrel )
!
                case ( 1 )
!
                    where( mask(:nlon,:nlat)==mskval )
                        mask(:nlon,:nlat) = one
                    elsewhere
                        mask(:nlon,:nlat) = zero
                    end where
!
                case ( 2 )
!
                    where( mask(:nlon,:nlat)>mskval )
                        mask(:nlon,:nlat) = one
                    elsewhere
                        mask(:nlon,:nlat) = zero
                    end where
!
                case ( 3 )
!
                    where( mask(:nlon,:nlat)>=mskval )
                        mask(:nlon,:nlat) = one
                    elsewhere
                        mask(:nlon,:nlat) = zero
                    end where
!
                case ( 4 )
!
                    where( mask(:nlon,:nlat)<mskval )
                        mask(:nlon,:nlat) = one
                    elsewhere
                        mask(:nlon,:nlat) = zero
                    end where
!
                case ( 5 )
!
                    where( mask(:nlon,:nlat)<=mskval )
                        mask(:nlon,:nlat) = one
                    elsewhere
                        mask(:nlon,:nlat) = zero
                    end where
!
            end select
!
!           COMPUTE INDICE BOUNDS.
!
            do lon1=1,nlon
                if ( any( mask(lon1,:nlat)==one ) ) exit
            end do
!
            do lon2=nlon,1,-1
                if ( any( mask(lon2,:nlat)==one ) ) exit
            end do
!
            do lat1=1,nlat
                if ( any( mask(:nlon,lat1)==one ) ) exit
            end do
!
            do lat2=nlat,1,-1
                if ( any( mask(:nlon,lat2)==one ) ) exit
            end do
!
!            if ( lat1>lat2 .or. lon1>lon2 )    &
!            call merror( name_proc//"Selected domain(mask) is empty" )
!
        end if
!
!       CREATE THE NEW NETCDF DATASET FOR THE MASK.
!
        status = nf90_create( trim(mask_name), cmode=nf90_clobber, ncid=ncid_mask )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_set_fill( ncid_mask, nf90_nofill, i )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!       DEFINE THE DIMENSIONS FOR THE MESH-MASK NETCDF DATASET.
!
        do i = 1, 2
!
            status = nf90_def_dim( ncid_mask, name=trim(dim_names(i)), len=dimlens(i), dimid=dimids(i) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end do
!
!       DEFINE THE VARIABLES FOR THE MESH-MASK NETCDF DATASET.
!
        status = nf90_def_var( ncid_mask, "nmask", nf90_real4, dimids=dimids(:2), varid=maskid )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_def_var( ncid_mask, "e1n", nf90_real8, dimids=dimids(:2), varid=e1id )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_def_var( ncid_mask, "e2n", nf90_real8, dimids=dimids(:2), varid=e2id )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!       ASSIGN ATTRIBUTES FOR THE MESH-MASK NETCDF DATASET.
!
        status = nf90_put_att( ncid_mask, maskid, "valid_min", zero )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_mask, maskid, "valid_max", one )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!       ADD LONGITUDE / LATITUDE BOUNDS ATTRIBUTES.
!
        status = nf90_put_att( ncid_mask, maskid, "lon1_Eastern_limit", lon1 )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_mask, maskid, "lon2_Western_limit", lon2 )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_mask, maskid, "lat1_Southern_limit", lat1 )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_mask, maskid, "lat2_Northern_limit", lat2 )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_mask, e1id, "units", "meters" )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_mask, e1id, "missing_value", def_missing_scal )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_mask, e2id, "units", "meters" )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_mask, e2id, "missing_value", def_missing_scal )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!       SET GLOBAL ATTRIBUTES.
!
        status = nf90_put_att( ncid_mask, nf90_global, "title", trim(var_name)//" Mesh-mask" )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_mask, nf90_global, "history", trim(history) )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!       LEAVE DEFINE MODE FOR THE MESH-MASK NETCDF DATASET.
!
        status = nf90_enddef( ncid_mask )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!       WRITE VARIABLES FOR THE MESH-MASK NETCDF DATASET.
!
        status = nf90_put_var( ncid_mask, maskid, mask )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!       CONSTRUCT THE SCALE FACTORS.
!
        if ( use_coor_variable(2) ) then
!
            status = nf90_get_var( ncid, coor_id(2), temp1(:nlat), count=(/ nlat /) )     
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        else
!
!           TRY nav_lat VARIABLE, IF COORDINATE VARIABLE IS MISSING.
!
            status = nf90_inq_varid( ncid, "nav_lat", coor_id(2) )
!
            if ( status==nf90_noerr ) then
!
                use_coor_variable(2) = true
!
!               FIND THE DIMENSION LENGTHS FOR THE nav_lat VARIABLE.
!
                status = nf90_inquire_variable( ncid, coor_id(2), ndims=i,  dimids=dimids )
                if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
                if ( i/=2 ) call merror( name_proc//"Coordinate variable nav_lat is not twodimensional" )
!
                do i = 1, 2
                    status = nf90_inquire_dimension( ncid, dimids(i), len=dimlens(i) )
                    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
                end do
!
                if ( dimlens(1)/=nlon .or. dimlens(2)/=nlat )    &
                call merror( name_proc//"Shape disagreement with the coordinate variable nav_lat" )
!
                status = nf90_get_var( ncid, coor_id(2), temp1(:nlat), count=(/ 1, nlat /) )     
                if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            else
!
                use_coor_variable(2) = false
!
            end if
!
        end if
!
        if ( use_coor_variable(2) ) then
!
!           VERIFY THAT THE COORDINATE VARIABLE IS STRICTLY MONOTONIC.
!
            last                 = all(temp1(:nlat-1)<temp1(2:nlat))
            use_coor_variable(2) = all(temp1(:nlat-1)>temp1(2:nlat)) .or. last
!
        end if
!
        if ( use_coor_variable(2) ) then
!
            if ( use_yl ) then
!
!               CHECK THE SPECIFIED LATITUDE LIMITS AGAINST THE COORDINATE VARIABLE.
!
                first = merge( latl1>temp1(1) .or. latl2<temp1(nlat),    &
                               latl1<temp1(1) .or. latl2>temp1(nlat),    &
                               last )
                if ( first ) call merror( name_proc//"Wrong specification for the -yl= argument" )
!
                temp2(1)      = two*latl1
                temp2(nlat+1) = two*latl2
!
            else
!
!               IT IS ASSUMED HERE THAT THE NETCDF VARIABLE COVERS THE WHOLE GLOBE.
!
                temp2(1)      = merge( -c180, c180, temp1(1)<zero )
                temp2(nlat+1) = -temp2(1)
!
            end if
!
            temp2(2:nlat)  = ( temp1(:nlat-1) + temp1(2:nlat) )
            temp2(:nlat+1) = temp2(:nlat+1)*( pio2/c360 )
!
!           CONSTRUCT THE FIRST SCALE FACTOR.
!
            temp1(:nlat)  = ( twopi*radius/real(nlon,stnd) )*abs( cos( temp2(2:nlat+1)+temp2(1:nlat) ) )
            mask(:nlon,:nlat) = spread( temp1(:nlat), dim=1, ncopies=nlon )
!
            status = nf90_put_var( ncid_mask, e1id, mask )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           CONSTRUCT THE SECOND SCALE FACTOR.
!
            temp1(:nlat) = ( two*radius )*abs( temp2(2:nlat+1)-temp2(1:nlat) )
            mask(:nlon,:nlat) = spread( temp1(:nlat), dim=1, ncopies=nlon )
!
        else
!
!           NO COORDINATE VARIABLES CONSTRUCT DUMMY SCALE FACTORS.
!
            mask(:nlon,:nlat) = def_missing_scal
!
            status = nf90_put_var( ncid_mask, e1id, mask )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end if
!
        status = nf90_put_var( ncid_mask, e2id, mask )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!       DEALLOCATE WORK ARRAYS.
!
        deallocate( mask, temp1, temp2  )
!
!       CLOSE THE NETCDF DATASET.
!
        status = nf90_close( ncid_mask )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    else
!
!       DEALLOCATE WORK ARRAYS.
!
        deallocate( data, mean, std, period, n_period )
!
    end if
!
!   CLOSE THE NETCDF DATASETS.
!
    status = nf90_close( ncid )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_close( ncid_clim )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    if ( fmsk_name/=" " ) then
        status = nf90_close( ncid_fmsk )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
    end if
!
!
! END OF PROGRAM comp_clim_3d
! ===========================
!
end program comp_clim_3d
