program comp_composite_4d
!! #############################################################################
!!
!! Copyright 2018 IRD
!!
!! This file is part of NCSTAT.
!!
!! NCSTAT is free software: you can redistribute it and/or modify
!! it under the terms of the GNU Lesser General Public License as 
!! published by the Free Software Foundation, either version 3 of 
!! the License, or (at your option) any later version.
!!
!! NCSTAT is distributed in the hope that it will be useful,
!! but WITHOUT ANY WARRANTY; without even the implied warranty of
!! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!! GNU Lesser General Public License for more details.
!!
!! You can find a copy of the GNU Lesser General Public License
!! in the NCSTAT/doc directory.
!!
!! #############################################################################
!
!+
!
! .. _comp_composite_4d:
!
! =================
! comp_composite_4d
! =================
!
! Authors
! =======
!
! Pascal Terray (LOCEAN/IPSL)
!
! Latest revision
! ===============
!
! 12/09/2018
!
! Purpose
! =======
!
! Compute a composite analysis from a fourdimensional variable extracted from
! a NetCDF dataset and perform statistical tests on the differences between the
! composite mean and the overall mean (e.g. the mean of the parent finite population)
! for each cell of the 3-D grid-mesh associated with the tridimensional NetCDF variable.
! The composite statistics and tests may be computed by taking into account the periodicity
! of the data. These statistics are stored in an output NetCDF dataset.
!
! The use of Student t values and test in composite analyses, as is traditionally used in the
! climate literature is not valid [Brown_Hall]_. Thus, to obtain more meaningful results, comp_composite_4d
! uses a resampling scheme or probabilities based on the finite population of the observed time observations
! and a normal approximation to estimate the statistical significance of the composite means [Terray_etal]_ .
!
! For more details on the statistical test used in comp_composite_4d to assess the significance
! of the composite means, consult the references cited below [Terray_etal]_ [Noreen]_.
!
! If the NetCDF variable is tridimensional, use :ref:`comp_composite_3d` 
! instead of comp_composite_4d.
!
! This procedure is parallelized if OpenMP is used and the NCSTAT software has been built
! with the _PARALLEL_READ CPP macro.
!
! Further Details
! ===============
!
! Usage
! -----
!
! ::
!
!       $ comp_composite_4d \
!         -f=input_netcdf_file \
!         -v=netcdf_variable \
!         -c=input_climatology_netcdf_file \
!         -a=year1,year2,year3,..., yearn \
!         -m=input_mesh_mask_netcdf_file    (optional) \
!         -g=grid_type                      (optional : n, t, u, v, w, f) \
!         -x=lon1,lon2                      (optional) \
!         -y=lat1,lat2                      (optional) \
!         -z=level1,level2                  (optional) \
!         -t=time1,time2                    (optional) \
!         -s=type_of_statistics             (optional : comp, diff) \
!         -alg=algorithm                    (optional : normal, simul) \
!         -o=output_composite_netcdf_file   (optional) \
!         -nb=number_of_shuffles            (optional) \
!         -mi=missing_value                 (optional) \
!         -nostd                            (optional) \
!         -double                           (optional) \
!         -bigfile                          (optional) \
!         -hdf5                             (optional) \
!         -tlimited                         (optional)
!
! By default
! ----------
!
!        *-m=*        
!         an *output_mesh_mask_netcdf_file* is not used
!        *-g=*
!         the *grid_type* is set to ``n`` which means that the 3-D grid-mesh associated with
!         the input *netcdf_variable* is assumed to be regular or Gaussian
!        *-x=*    
!         the whole longitude domain associated with the *netcdf_variable*
!        *-y=*    
!         the whole latitude domain associated with the *netcdf_variable*
!        *-z=*
!         the whole vertical resolution associated with the *netcdf_variable*
!        *-t=*        
!         the whole time period associated with the *netcdf_variable*
!        *-s=*       
!         the *type_of_statistics* is set to ``comp``
!        *-alg=*
!         the *algorithm* is set to ``normal``
!        *-nb=*
!         *number_of_shuffles* is set to ``99``
!        *-o=*
!         the *output_composite_netcdf_file* is named ``composite_``\ *netcdf_variable*\ ``.nc``
!        *-mi=*
!         the *missing_value* attribute in the output NetCDF file is set to ``1.e+20``
!        *-nostd*
!         the composite fields of the input NetCDF variable are standardized.
!         If **-nostd** is activated, the composite fields are not standardized
!        *-double*
!         the composite statistics are stored as single-precision floating
!         point numbers in the output NetCDF file. If **-double** is activated, the
!         results are stored as double-precision floating point numbers 
!        *-bigfile*
!         a NetCDF classical format file is created. If **-bigfile** is activated,
!         the output NetCDF file is a 64-bit offset format file
!        *-hdf5*
!         a NetCDF classical format file is created. If **-hdf5** is activated,
!         the output NetCDF file is a NetCDF-4/HDF5 format file
!        *-tlimited*
!         the time dimension is defined as unlimited in the output NetCDF file.
!         However, if **-tlimited** is activated, the time dimension is defined
!         as limited in the output NetCDF file
!
! Remarks
! -------
!
!   #) The **-v=**\ *netcdf_variable* argument specifies the NetCDF variable for which a
!      composite analysis must be computed and the **-f=**\ *input_netcdf_file* argument
!      specifies that this NetCDF variable must be extracted from the NetCDF file
!      *input_netcdf_file*.
!
!   #) The *input_climatology_netcdf_file* specified with the **-c=** argument must contain
!      the means and standard deviations for the parent population. The periodicity for the
!      composite analysis is also deduced from the number of observations in the 
!      *input_climatology_netcdf_file*. This *input_climatology_netcdf_file* must have been created
!      by :ref:`comp_clim_4d` applied to the same *netcdf_variable* with an identical **-t=**\ *time1,time2* argument
!      as used in comp_composite_4d in order to obtain correct statistics in the output NetCDF file.
!
!      The geographical shapes of the *netcdf_variable* (in the *input_netcdf_file*) and the climatology 
!      (in the *input_climatology_netcdf_file*) must agree.
!
!   #) The **-a=** argument lists the indices of the years (or seasons, months or days depending on the
!      sampling of the observations in the *input_netcdf_file*), which must be included in
!      the composite analysis (e.g. for computing the composite means). The indices of the years are counted from
!      the start of the (selected) time period (e.g. *time1* in the **-t=**\ *time1,time2* argument or ``1`` if this
!      argument is missing).
!
!      The list may be specified in different formats:
!
!      * **-a=**\ *n1,n2,...nn* allows to select for years *n1*, *n2*, ... and *nn* in the *input_netcdf_file*
!
!      * **-a=**\ *n1:n2* allows to select for years *n1* to *n2* in the *input_netcdf_file*
!
!      The two forms of the **-a=** argument may be combined and repeated any number of times, but
!      duplicate years are not allowed.
!
!      Remember also that the length of a year in the *input_netcdf_file* is determined by the periodicity
!      of the observations as deduced from the number of time observations in the *input_climatology_netcdf_file*,
!      when specifying the indices of the years in the **-a=** argument.
!      This periodicity will also determine how many composite means and statistics will be computed and
!      stored in the *output_composite_netcdf_file* by comp_composite_4d.
!
!   #) The optional argument **-m=**\ *input_mesh_mask_netcdf_file* specifies the land-sea mask
!      to apply to *netcdf_variable* for transforming this fourdimensional NetCDF variable as a
!      rectangular matrix of observed variables before computing the composite analysis. By default,
!      it is assumed that each cell in the 3-D grid-mesh associated with the input fourdimensional
!      NetCDF variable is a valid time series (e.g. missing values are not present).
!
!      The geographical shapes of the *netcdf_variable* (in the *input_netcdf_file*), the climatology 
!      (in the *input_climatology_netcdf_file*) and the mask (if an *input_mesh_mask_netcdf_file* is used) 
!      must agree.
!
!      Refer to :ref:`comp_clim_4d` or :ref:`comp_mask_4d` for creating a valid
!      *input_mesh_mask_netcdf_file* NetCDF file for regular or gaussian grids before
!      using comp_composite_4d.
!
!   #) If **-g=** is set to ``t``, ``u``, ``v``, ``w`` or ``f``, it is assumed that
!      the input NetCDF variable is from an experiment with the ORCA model (R2, R4 or R05 resolutions).
!      This argument is also used to determined the name of the mesh_mask variable if
!      an *input_mesh_mask_netcdf_file* is used.
!
!   #) If the **-x=**\ *lon1,lon2*, **-y=**\ *lat1,lat2* and **-z=**\ *level1,level2* arguments are missing,
!      the whole geographical domain and vertical resolution associated with
!      the *netcdf_variable* are used in the composite analysis.
!
!      The longitude, latitude or level range must be a vector of two integers specifying
!      the first and last selected indices along each dimension. The indices are relative to ``1``\ .
!      Negative values are allowed for *lon1*. In this case the longitude domain
!      is from ``nlon``\ +\ *lon1*\ +\ ``1`` to *lon2* where ``nlon`` is the number of longitude
!      points in the grid associated with the NetCDF variable and it is assumed that the grid
!      is periodic.
!
!      Refer to :ref:`comp_mask_4d` for transforming geographical coordinates as indices or
!      generating an appropriate mesh-mask before using comp_composite_4d.
!
!   #) If the **-t=**\ *time1,time2* argument is missing, data in the whole time period
!      associated with the *netcdf_variable* is taken into account.
!      The selected time period is a vector of two integers specifying the first
!      and last time observations. The indices are relative to ``1``\ .
!
!      The selected time period (e.g.  *time2* - *time1* + ``1``) must be a whole multiple of
!      the *periodicity* as deduced from the number of time observations in the *input_climatology_netcdf_file*
!      if **-alg=**\ ``simul`` (see remark below).
!
!      It is also assumed that the selected time period matches exactly the time period used to
!      compute the climatology in the *input_climatology_netcdf_file*. Moreover, the first selected time
!      observation ( *time1* if the **-t=** argument is present) must correspond to the 
!      first day, month, season of the climatology specified with the **-c=** argument.
!
!   #) The **-s=**\ *type_of_statistics* argument specified how the composite variable *netcdf_variable*\ _composite 
!      stored in the *output_composite_netcdf_file* is computed:
!
!      * **-s=**\ ``comp`` means that the composites are computed as the standardized differences
!        between the mean of the composite years and the overall mean (e.g. the mean of the parent
!        finite population) for each grid-point.
!
!      * **-s=**\ ``diff`` means that the composites are computed as the standardized differences 
!        between the mean of the composite years and the mean of the other years in the parent
!        finite population for each grid-point.
!
!      The standard deviations from the parent population are used for the standardization in both cases
!      and are read from the *input_climatology_netcdf_file*.
!      Finally, note that this argument does not affect the other variables stored in the *output_composite_netcdf_file*.
!
!   #) The **-nostd** argument specifies that the composite variable *netcdf_variable*\ _composite in
!      the *output_composite_netcdf_file* must not be standardized.
!
!      By default, the composites are standardized in the *output_composite_netcdf_file*.
!
!   #) The **-alg=** argument selects the method for computing critical probabilities associated with
!      the composite means and U statistics (see the second reference cited below for the definition of this statistic):
!
!      * If **-alg=**\ ``normal``, a normal approximation is used to compute the critical probabilities
!        associated with the composite means and U statistics.
!
!      * If **-alg=**\ ``simul``, the critical probabilities are estimated by a resampling method. If **-alg=**\ ``simul`` 
!        the selected time period (e.g.  *time2* - *time1* + ``1``) must be a whole multiple of the periodicity.
!
!   #) The **-nb=**\ *number_of_shuffles* argument specifies the number of shuffles for the resampling
!      procedure if **-alg=**\ ``simul``.
!
!   #) The **-mi=**\ *missing_value* argument specifies the missing value indicator associated with
!      the NetCDF variables in the *output_netcdf_file*.
!      If the **-mi=** argument is not specified the *missing_value* attribute is set to ``1.e+20``\ for
!      the NetCDF variables in the *output_netcdf_file*.
!
!   #) The **-double** argument specifies that the results are stored as double-precision floating
!      point numbers in the output NetCDF file.
!      By default, the results are stored as single-precision floating point numbers
!      in the output NetCDF file.
!
!   #) The **-bigfile** argument is allowed only if the NCSTAT software has been compiled with the
!      _USE_NETCDF36 or _USE_NETCDF4 CPP macros (e.g. ``-D_USE_NETCDF36`` or ``-D_USE_NETCDF4``) and linked
!      to the NetCDF 3.6 library or higher.
!      If this argument is specified, the *output_netcdf_file* will be a 64-bit offset format file
!      instead of a NetCDF classic format file. However, this argument is recognized in the procedure
!      only if the NCSTAT software has been built with the _USE_NETCDF36 or _USE_NETCDF4 macros.
!
!   #) The **-hdf5** argument is allowed only if the NCSTAT software has been
!      compiled with the _USE_NETCDF4 CPP macro (e.g. ``-D_USE_NETCDF4``) and linked
!      to the NetCDF 4 library or higher. 
!      If this argument is specified, the *output_netcdf_file* will be a NetCDF-4/HDF5
!      format file instead of a NetCDF classic format file. However, this
!      argument is recognized in the procedure only if the NCSTAT software has
!      been built with the _USE_NETCDF4 CPP macro.
!
!   #) Duplicate parameters are allowed, but this is always the last occurrence of a parameter
!      which will be used for the computations. Moreover, the number of specified parameters
!      must not be greater than the total number of allowed parameters.
!
!   #) It is assumed that the data has no missing values.
!
!   #) For more details on composite analysis and statistical testing in composite analysis, see
!
!      * "The Use of t values in Composite Analyses"
!        by Brown, T.J., and Hall, B.L.,
!        Journal of climate, vol. 12. 2941-2944, 1999. doi: `10.1175/1520-0442(1999)012<2941:TUOTVI>2.0.CO;2`_
!
!      * "Sea Surface Temperature associations with the Late Indian Summer Monsoon",
!        by Terray, P., Delecluse P., Labattu S., Terray L.,
!        Climate Dynamics, vol. 21, 593-618, 2003. doi: `10.1007/s00382-003-0354-0`_
!
!      * "Computer-intensive methods for testing hypotheses: an introduction",
!        by Noreen, E.W.,
!        Wiley and Sons, New York, USA, 1989. ISBN: `978-0-471-61136-3`_
!
! .. _`10.1175/1520-0442(1999)012<2941:TUOTVI>2.0.CO;2` : https://doi.org/10.1175/1520-0442(1999)012<2941:TUOTVI>2.0.CO;2
! .. _`10.1007/s00382-003-0354-0` : http://dx.doi.org/10.1007/s00382-003-0354-0
! .. _`978-0-471-61136-3` : https://www.wiley.com/en-us/Computer+Intensive+Methods+for+Testing+Hypotheses%3A+An+Introduction-p-9780471611363
!
! Outputs
! -------
!
!   comp_composite_4d creates an output NetCDF file that contains the composite statistics and
!   critical probabilities associated with the composite means, taking into account eventually the periodicity
!   of the data as determined by the **-c=**\ *input_climatology_netcdf_file* argument. The output NetCDF dataset
!   contains the following NetCDF variables (in the description below, nlev, nlat and nlon are the length of the vertical
!   and spatial dimensions of the input NetCDF variable) and *periodicity* time observations (*periodicity* is the number
!   of time observations in the *input_climatology_netcdf_file* ) :
!
!   #) *netcdf_variable*\ _compmean\ ``(periodicity,nlev,nlat,nlon)`` : the mean fields of the time observations selected with
!      the help of the **-a=** argument, taking into account the periodicity of the data as determined by the number of
!      observations in the *input_climatology_netcdf_file* (specified with the help of the **-c=** argument).
!
!   #) *netcdf_variable*\ _compstd\ ``(periodicity,nlev,nlat,nlon)`` : the standard-deviation fields of the time
!      observations selected with the help of the **-a=** argument, taking into account the periodicity of 
!      the data as determined by the  **-c=**\ *input_climatology_netcdf_file* argument.
!
!   #) *netcdf_variable*\ _qstd\ ``(periodicity,nlev,nlat,nlon)`` : the ratio between the standard deviations fields
!      of the selected time observations to the standard deviations fields of all the time observations, taking
!      into account the periodicity of the data as determined by the  **-c=**\ *input_climatology_netcdf_file* argument.
!      The standard deviations fields for all the observations are extracted from the *input_climatology_netcdf_file*
!      specified in the **-c=** argument.
!
!   #) *netcdf_variable*\ _composite\ ``(periodicity,nlev,nlat,nlon)`` : the composite fields of the time observations
!      selected with the help of the **-a=** argument, taking into account the periodicity of the data as
!      determined by the number of observations in the *input_climatology_netcdf_file* (specified with the help
!      of the **-c=** argument).
!
!      How these composite fields are calculated is determined by the **-s=** and **-nostd** arguments.
!
!   #) *netcdf_variable*\ _u\ ``(periodicity,nlev,nlat,nlon)`` : the U statistics for the time observations selected with
!      the help of the **-a=** argument, taking into account the periodicity of the data as determined by the 
!      number of observations in the *input_climatology_netcdf_file* (specified with the help of the **-c=** argument).
!
!      See the second publication cited above for more details on the definition of the U statistic.
!
!   #) *netcdf_variable*\ _prob\ ``(periodicity,nlev,nlat,nlon)`` : the critical probabilities associated with
!      the U statistics.
!
!      These critical probabilities are computed under the null hypothesis that the selected time observations
!      (with the help of the **-a=** argument) come from the same population as the other observations in
!      the *input_netcdf_file*. Small probabilities indicate a large departure from the null hypothesis.
!
!      The **-alg=**\ *algorithm* argument determines how these critical probabilities are
!      computed.
!
!   #) *netcdf_variable*\ _compnobs\ ``(periodicity)`` : the number of observations used to compute
!      the composite means.
!
!   All these statistics are packed in fourdimensional variables whose first, second and third dimensions
!   are exactly the same as those associated with the input NetCDF variable *netcdf_variable*
!   even if you restrict the geographical domain with the **-x=**, **-y=** and **-z=** arguments. However,
!   outside the selected domain, the output NetCDF variables are filled with missing values.
!
! Example
! -------
!
!   #)   For computing a composite analysis from a fourdimensional NetCDF variable
!        ``votemper`` in the NetCDF file ``ST7_1m_00101_20012_votemper_grid_T.nc``,
!        assessing the significance of the results with a monte carlo simulation with
!        999 shuffles and, finally, storing the results in a NetCDF file named
!        ``composite_ST7_1m_votemper_grid_T.nc``, use the following
!        command (note that the critical probabilities associated with the U statistics
!        are estimated with the help of a resampling method using ``999`` surrogate
!        time series since **-alg**\ =\ ``simul`` and **-nb**\ =\ ``999`` are specified) :
!
!        ::
!
!          $ comp_composite_4d \
!            -f=ST7_1m_00101_20012_T_votemper_grid_T.nc \
!            -v=votemper \
!            -c=clim_votemper_grid_T.nc \
!            -a=10,11,20,55,100 \
!            -g=t \
!            -m=mesh_mask_ST7_votemper_grid_T.nc \
!            -alg=simul \
!            -nb=999 \
!            -o=composite_ST7_1m_votemper_grid_T.nc
!
!-
! ================================================================================================
!
#ifndef _OPENMP
#undef _PARALLEL_READ
#endif
!
#ifdef _USE_NETCDF4
#define _USE_NETCDF36
#endif
!
! USED MODULES 
! ============
!
#ifdef _USE_NAGWARE
    use f90_unix_env,     only : iargc, getarg
#endif
!
#ifdef _OPENMP
    use omp_lib,     only : omp_get_num_procs, omp_get_max_threads
#ifdef _PARALLEL_READ
    use netcdf,      only : nf90_share
    use Statpack,    only : omp_limit
#endif
#endif
!
#ifdef _USE_NETCDF36
    use netcdf,   only : nf90_inquire_attribute, nf90_64bit_offset
#endif
!
#ifdef _USE_NETCDF4
    use netcdf,   only : nf90_hdf5
#endif
!
    use netcdf,   only : nf90_max_name, nf90_max_var_dims, nf90_max_name, nf90_clobber,          &
                         nf90_real4, nf90_real8, nf90_int,                                       &
                         nf90_sizehint_default, nf90_noerr, nf90_open, nf90_nowrite,             &
                         nf90_strerror, nf90_get_att, nf90_get_var, nf90_close, nf90_create,     &
                         nf90_set_fill, nf90_nofill, nf90_def_dim, nf90_def_var, nf90_unlimited, &
                         nf90_inq_attname, nf90_copy_att, nf90_put_att,                          &
                         nf90_global, nf90_enddef, nf90_put_var, nf90_close 
!
    use Statpack, only : i1b, i4b, stnd, extd, lgl, true, false, zero, one, two,    & 
                         kfix, kexp, kint, tolower, case_change, string_to_val,     &
                         allocate_error, merror, arth,                              &
                         comp_mvs_grp, probn, drawsample, random_seed_
!
    use NCSTAT_Utilities, only : prtunit, nchar, sp, dp, def_missing,                &
                                 get_listarg, get_int_list, get_variable_dims,       &
                                 get_variable_dimcoords
!
!
! STRONG TYPING IMPOSED 
! =====================
!
    implicit none
!
!
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='comp_composite_4d : ',       &
!
                                   cmethod(2)=(/"comp","diff"/),           &  
!
                                   calgorithm(2)=(/"normal","simul "/)         
!
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
!
#ifndef _USE_NAGWARE
    integer(i4b)  :: iargc
#endif
    integer(i4b)  :: narg, status, ncid, ncid_clim, ncid_mask, ncid_comp,          &
                     varid, maskid, meanid, stdid, compid, uid, probid, nobsid,    &
                     qstdid, natts,                                                &
                     i, j, j1, j2, k, l, nlon, nlat, nt, nlevel, nm, ndat,         &
                     lon1, lon2, lon3, lat1, lat2, lat3, level1, level2, level3,   &
                     time1, time2, time3, nyears, method, alg, offset_rec, nrep,   &
                     output_type, output_mode
    integer(i4b), dimension(4)                 :: startlec, countlec
    integer(i4b), dimension(5)                 :: coor_id2
    integer(i4b), dimension(6)                 :: id_var
    integer(i4b), dimension(nf90_max_var_dims) :: dimids, dimlens, coor_natts, coor_id, coor_type
    integer(i4b), dimension(:),    allocatable :: period
    integer(i4b), dimension(:),    pointer     :: years
    integer(i4b), dimension(:,:),  allocatable :: nge
    integer(i1b)                               :: kcode
!
    real(stnd), dimension(:,:,:), allocatable  :: temp
    real(stnd), dimension(:,:),   allocatable  :: dat, compo, comp_mean, comp_std, u, mean, std
    real(stnd), dimension(:),     allocatable  :: n_period, df
    real(stnd)                                 :: missing, tmp
    real(extd), dimension(:),     allocatable  :: record
!
    logical(lgl)                                :: first, last, use_tlimited, use_x, use_y, use_z,  &
                                                   use_mask, use_nostd, create_hdf5
    logical(lgl), dimension(nf90_max_var_dims)  :: use_coor_variable
    logical(lgl), dimension(:,:,:), allocatable :: mask
!
    character(len=nchar)                       :: name, name2, history
    character(len=nf90_max_name)               :: fic_name, var_name, clim_name, mask_name,    &
                                                  comp_name, att_name 
    character(len=14)                          :: fmt
    character                                  :: grid_type, chr
    character(len=nf90_max_name), dimension(nf90_max_var_dims) :: dim_names
!
#ifdef _PARALLEL_READ
!
    integer(i4b)  :: ncid2
!
#endif
!
#ifdef _OPENMP
!
    integer(i4b), dimension(:,:),  allocatable :: nge2
    logical                                    :: test_par
!
#endif
!
!
! EXECUTABLE STATEMENTS
! =====================
!
!   GET THE NUMBER OF COMMAND LINE ARGUMENTS.
!
    narg = iargc()
!
!   CHECK THE NUMBER OF COMMAND LINE ARGUMENTS.
!
    if ( narg==0 ) then
!
        write (prtunit,*)
        write (prtunit,*) ' Purpose : '
        write (prtunit,*)
        write (prtunit,*) ' Compute a composite analysis from a fourdimensional variable '
        write (prtunit,*) ' extracted from a NetCDF dataset.'
        write (prtunit,*)
        write (prtunit,*) ' Usage : '
        write (prtunit,*)
        write (prtunit,*) ' comp_composite_4d  '
        write (prtunit,*) '              -f=input_netcdf_file'
        write (prtunit,*) '              -v=netcdf_variable'
        write (prtunit,*) '              -c=input_climatology_netcdf_file'
        write (prtunit,*) '              -a=year1,year2,year3,...,yearn'
        write (prtunit,*) '              -m=input_mesh_mask_netcdf_file   (optional)'
        write (prtunit,*) '              -g=grid_type                     (optional : n, t, u, v, w, f)'
        write (prtunit,*) '              -x=lon1,lon2                     (optional)'
        write (prtunit,*) '              -y=lat1,lat2                     (optional)'
        write (prtunit,*) '              -z=level1,level2                 (optional)'
        write (prtunit,*) '              -t=time1,time2                   (optional)'
        write (prtunit,*) '              -s=type_of_statistics            (optional : comp, diff)'
        write (prtunit,*) '              -alg=algorithm                   (optional : normal, simul)'
        write (prtunit,*) '              -nb=number_of_shuffles           (optional)'
        write (prtunit,*) '              -o=output_composite_netcdf_file  (optional)'
        write (prtunit,*) '              -mi=missing_value                (optional)'
        write (prtunit,*) '              -nostd                           (optional)'
        write (prtunit,*) '              -double                          (optional)'
#ifdef _USE_NETCDF36
        write (prtunit,*) '              -bigfile                         (optional)'
#endif
#ifdef _USE_NETCDF4
        write (prtunit,*) '              -hdf5                            (optional)'
#endif
        write (prtunit,*) '              -tlimited                        (optional)'
        write (prtunit,*)
        write (prtunit,*) ' By default : '
        write (prtunit,*)
        write (prtunit,*) '-m= an input_mesh_mask_netcdf_file is not used'
        write (prtunit,*) '-g= the grid type is set to n'
        write (prtunit,*) '-x= the whole longitude domain associated with the netcdf_variable'
        write (prtunit,*) '-y= the whole latitude domain associated with the netcdf_variable'
        write (prtunit,*) '-z= the whole vertical resolution associated with the netcdf_variable'
        write (prtunit,*) '-t= the whole time period  associated with the netcdf_variable'
        write (prtunit,*) '-s=comp'
        write (prtunit,*) '-alg=normal'
        write (prtunit,*) '-nb=99'
        write (prtunit,*) '-o=composite_netcdf_variable.nc'
        write (prtunit,*) '-mi= by default, the missing_value is equal to 1.e+20'
        write (prtunit,*) '-nostd    : by default, the composites are standardized'
        write (prtunit,*) '-double   : by default, the results are stored as single floating numbers'
#ifdef _USE_NETCDF36
        write (prtunit,*) '-bigfile  : by default, a netCDF classical format file is created'
#endif
#ifdef _USE_NETCDF4
        write (prtunit,*) '-hdf5     : by default, a NetCDF classical format file is created'
#endif
        write (prtunit,*) '-tlimited : by default, the time dimension is defined as unlimited'
        write (prtunit,*)
#ifdef _OPENMP
        write (prtunit,*) 'This procedure is parallelized with OpenMP'
#endif
        write (prtunit,*)
!
        stop
!
    else if ( narg<4 .or. narg>20 ) then
!
        call merror( name_proc//"Wrong number of line arguments" )
!
    end if
!
    fic_name   = " "
    var_name   = " "
    mask_name  = " "
    clim_name  = " "
    comp_name  = " "
!
    history = "comp_composite_4d"
!
!   DEFAULT VALUES FOR COMMAND LINE ARGUMENTS.
!
    output_mode = nf90_clobber
    output_type = nf90_real4
    nyears = 0
    method = 1
    alg    = 1
    nrep   = 99
!
    missing = def_missing
!
    grid_type = "n"
!
    lon1      = 1
    lon2      = huge( lon2 )
    lat1      = 1
    lat2      = huge( lat2 )
    level1    = 1
    level2    = huge( level2 )
    time1     = 1
    time2     = huge( time2 )
!
    use_mask     = false
    use_nostd    = false
    use_x        = false
    use_y        = false
    use_z        = false
    create_hdf5  = false
    use_tlimited = false
!
!   GET THE COMMAND LINE ARGUMENTS.
!
    do i = 1, narg
!
        call getarg( i, name )
!
        history = trim(history)//" "//trim(name)
!
        call case_change( name(:3), tolower )
!
        select case ( name(1:3 ) )
!
            case ('-f=')
!
!               GET THE NAME OF THE NETCDF DATASET.
!
                fic_name = name(4:)
!
            case ('-v=')
!
!               GET THE NAME OF THE NETCDF VARIABLE IN THE DATASET.
!
                var_name = name(4:)
!
            case ('-m=')
!
!               GET THE NAME OF THE NETCDF MESH-MASK DATASET.
!
                mask_name = name(4:)
!
                use_mask = true
!
            case ('-c=')
!
!               GET THE NAME OF THE NETCDF CLIMATOLOGY DATASET.
!
                clim_name = name(4:)
!
            case ('-n=')
!
!               GET THE NUMBER OF YEARS (OBSOLETE ARGUMENT).
!
!
            case ('-a=')
!
!               GET THE INDICES OF YEARS FOR THE COMPOSITE.
!
                if ( nyears/=0 ) deallocate( years )
!
                call get_int_list(  name_proc, name, 4, nyears, years )
!
            case ('-s=')
!
!               DETERMINE THE METHOD OF PRESENTATION OF THE COMPOSITES.
!
                call case_change( name(4:), tolower )
!
                select case ( name(4:) )
                    case ( cmethod(1) )
                        method = 1
                    case ( cmethod(2) )
                        method = 2
                    case default
                        call merror( name_proc//"Unknown type of statistics option" )
                end select
!
            case ('-g=')
!
!               GET THE GRID-TYPE.
!
                call case_change( name(4:), tolower )
!
                select case ( name(4:) )
                    case ( 'n', 'u', 'v', 'w', 't', 'f' )
                        grid_type = name(4:4)
                    case default
                        call merror( name_proc//"Unknown grid type" )
                end select
!
            case ('-x=')
!
!               GET THE DOMAIN DEFINITION (Longitude).
!
                call get_listarg( name_proc, name(4:),                    &
                                  "lon1 and lon2 are integer values",     &
                                  "Wrong domain definition (longitude)",  &
                                  lon1, lon2 )
!
                use_x = true
!
            case ('-y=')
!
!               GET THE DOMAIN DEFINITION (latitude).
!
                call get_listarg( name_proc, name(4:),                    &
                                  "lat1 and lat2 are integer values",     &
                                  "Wrong domain definition (latitude)",   &
                                  lat1, lat2 )
!
                use_y = true
!
            case ('-z=')
!
!               GET THE LEVEL DEFINITION.
!
                call get_listarg( name_proc, name(4:),                    &
                                  "level1 and level2 are integer values", &
                                  "Wrong level definition",               &
                                  level1, level2 )
!
                use_z = true
!
            case ('-t=')
!
!               GET THE PERIOD DEFINITION.
!
                call get_listarg( name_proc, name(4:),                    &
                                  "time1 and time2 are integer values",   &
                                  "Wrong period definition",              &
                                  time1, time2 )
!
!
            case ('-o=')
!
!               GET THE NAME OF THE NEW NETCDF DATASET FOR STORING THE COMPOSITE ANALYSIS.
!
                comp_name = name(4:)
!
            case default
!
                select case ( name(1:4 ) )
!
                    case ('-nb=')
!
!                       GET THE NUMBER OF SHUFFLES FOR THE SIMULATION.
!
                        call string_to_val( name(5:), kcode, fmt )
                        if ( kcode==kint ) then
                            read( name(5:), fmt ) nrep
                        end if
                        if ( kcode/=kint .or. nrep<=0  )                                      &
                        call merror( name_proc//"The number of shuffles is a positive integer value" )
!
                    case ('-mi=')
!
!                       GET THE MISSING VALUE INDICATOR.
!
                        call string_to_val( name(5:), kcode, fmt )
                        if ( kcode/=kfix .and. kcode/=kexp )                &
                        call merror( name_proc//"The -mi= argument must be a real number " )
                        read( name(5:), fmt ) missing
!
                    case default
!
                        call case_change( name(4:), tolower )
!
                        select case ( name(1:5 ) )
!
                            case ('-alg=')
!
!                               DETERMINE THE ALGORITHM FOR THE COMPUTATION.
!
                                select case ( name(6:) )
                                    case ( calgorithm(1) )
                                        alg = 1
                                    case ( calgorithm(2) )
                                        alg = 2
                                    case default
                                        call merror( name_proc//"Unknown algorithm option" )
                                end select
!
                            case default
!
                                select case ( name(1:) )
!
                                    case ('-nostd')
!
!                                       COMPOSITES ARE NOT STANDARDIZED.
!
                                        use_nostd = true
!
                                    case ('-double')
!
!                                       STORE RESULTS AS DOUBLE PRECISION NUMBERS.
!
                                        output_type = nf90_real8
!
#ifdef _USE_NETCDF36
                                    case ('-bigfile')
!
!                                       REDEFINE THE CREATION MODE FLAG FOR THE OUTPUT FILE.
!
                                        output_mode = nf90_64bit_offset
#endif
!
#ifdef _USE_NETCDF4
                                    case ('-hdf5')
!
!                                       REDEFINE THE CREATION MODE FLAG FOR THE OUTPUT FILE.
!
                                        output_mode = nf90_hdf5
                                        create_hdf5 = true
#endif
!
                                    case ('-tlimited')
!
!                                       DEFINE THE TIME DIMENSION AS LIMITED.
!
                                        use_tlimited = true
!
                                    case default
!
!                                       WRONG ARGUMENT.
!
                                        call merror( name_proc//"Wrong argument" )
!
                                end select
!
                        end select
!
                end select
!
        end select
!
    end do
!
!   CHECK THE COMMAND LINE ARGUMENTS.
!
    if ( fic_name==" " )                                                 &
    call merror( name_proc//"Input NETCDF file is missing" )
!
    if ( var_name==" " )                                                 &
    call merror( name_proc//"Input NETCDF variable is missing" )
!
    if ( clim_name==" " )                                                &
    call merror( name_proc//"Climatology NETCDF file is missing" )
!
    if ( nyears==0 )                                                     &
    call merror( name_proc//"The -a= parameter is missing" )
!
    if ( comp_name==" " ) comp_name="composite_"//trim(var_name)//".nc"
!
!   OPEN THE NETCDF DATASETS.
!
    k = nf90_sizehint_default
    status = nf90_open( trim(fic_name), nf90_nowrite, ncid, chunksize=k )
    if ( status/=nf90_noerr )                                            &
    call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    if ( use_mask ) then
        status = nf90_open( trim(mask_name), nf90_nowrite, ncid_mask )
        if ( status/=nf90_noerr )                                                &
        call merror( name_proc//trim( nf90_strerror( status ) ) )
    end if
!
    status = nf90_open( trim(clim_name), nf90_nowrite, ncid_clim )
    if ( status/=nf90_noerr )                                        &
    call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!   GET THE VARIABLE ID FROM ITS NAME.
!   FIND THE DIMENSION LENGTHS AND NAMES FOR THE NETCDF VARIABLE.
!   FIND THE NUMBER OF ATTRIBUTES FOR THE NETCDF VARIABLE.
!   GET THE COORDINATE VARIABLES IDS FROM THEIR NAMES, IF ANY.
!   FIND THE EXTERNAL TYPE, THE DIMENSION ID AND THE NUMBER OF ATTRIBUTES
!   FOR THE COORDINATE VARIABLES.
!
    call get_variable_dimcoords(  name_proc, trim(var_name), ncid,                      &
                                  varid,  dimids_out=dimids, dim_names_out=dim_names,   &
                                  dimlens_out=dimlens, coor_id_out=coor_id,             &
                                  coor_natts_out=coor_natts, coor_type_out=coor_type,   &
                                  use_coor_variable_out=use_coor_variable, natts=natts  )
!
    if ( count(dimlens/=0)/=4 )           &
    call merror( name_proc//"Input NETCDF variable is not fourdimensional" )
!
    nlon   = dimlens(1)
    nlat   = dimlens(2)
    nlevel = dimlens(3)
    nt     = dimlens(4)
!
    if ( use_mask ) then
!
        chr = merge( 't', grid_type, grid_type=='w' )
!
!       GET THE MASK ID FROM ITS NAME.
!       VERIFY THE DIMENSION LENGTHS FOR THE NETCDF MASK VARIABLE.
!
        call get_variable_dims( name_proc, chr//"mask", ncid_mask, maskid, dimids, dimlens )
!
        if ( dimlens(1)/=nlon .or. dimlens(2)/=nlat .or. dimlens(3)/=nlevel )    &
        call merror( name_proc//"Shape disagreement with the mask" )
!
        if ( .not.use_x ) then
!
            status = nf90_get_att( ncid_mask, maskid, "lon1_Eastern_limit", i )
            if ( status==nf90_noerr ) lon1 = i
!
            status = nf90_get_att( ncid_mask, maskid, "lon2_Western_limit", i )
            if ( status==nf90_noerr ) lon2 = i
!
        end if
!
        if ( .not.use_y ) then
!
            status = nf90_get_att( ncid_mask, maskid, "lat1_Southern_limit", i )
            if ( status==nf90_noerr ) lat1 = i
!
            status = nf90_get_att( ncid_mask, maskid, "lat2_Northern_limit", i )
            if ( status==nf90_noerr ) lat2 = i
!
        end if
!
        if ( .not.use_z ) then
!
            status = nf90_get_att( ncid_mask, maskid, "level1_First_level", i )
            if ( status==nf90_noerr ) level1 = i
!
            status = nf90_get_att( ncid_mask, maskid, "level2_Last_level", i )
            if ( status==nf90_noerr ) level2 = i
!
        end if
!
    end if
!
!   CHECK THE BOUNDARIES OF THE GEOGRAPHICAL DOMAIN.
!
    if ( lon1<0 ) then
        lon2 = max( lon2, 1 )
        lon3 = nlon + lon1 - lon2 + 1
    else
        lon1 = max( lon1, 1 )
        lon2 = min( lon2, nlon )
        lon3 = lon2 - lon1 + 1
    end if
!
    if ( lon3<=0  )  call merror( name_proc//"Wrong domain definition (longitude)" )
!
    if ( lon1<0 ) then
       lon3 = nlon
       j1   = 1
       j2   = nlon
    else
       j1 = lon1
       j2 = lon2
    end if
!
    lat1 = max( lat1, 1 )
    lat2 = min( lat2, nlat )
    lat3 = lat2 - lat1 + 1
    if ( lat3<=0  )  call merror( name_proc//"Wrong domain definition (latitude)" )
!
!   CHECK THE LEVEL DEFINITION.
!
    level1 = max( level1, 1 )
    level2 = min( level2, nlevel )
    level3 = level2 - level1 + 1
    if ( level3<=0  ) call merror( name_proc//"Wrong vertical definition" )
!
!   CHECK THE PERIOD DEFINITION.
!
    time1 = max( time1, 1 )
    time2 = min( time2, nt )
    time3 = time2 - time1 + 1
    if ( time3<=0  ) call merror( name_proc//"Wrong time definition" )
!
!   GET THE MEAN FIELD ID FROM ITS NAME.
!   FIND THE DIMENSION LENGTHS FOR THE NETCDF MEAN VARIABLE.
!
    call get_variable_dims( name_proc, trim(var_name)//"_mean", ncid_clim,    &
                            meanid, dimids, dimlens )
!
    if ( count(dimlens/=0)/=4 )     &
    call merror( name_proc//"Input climatology is not fourdimensional" )
!
    if ( dimlens(1)/=nlon .or. dimlens(2)/=nlat .or. dimlens(3)/=nlevel )          &
    call merror( name_proc//"Shape disagreement with the climatology (mean field)" )
!
    first = false
!
    status = nf90_get_att( ncid_clim, nf90_global, "lon1_"//trim(var_name), i )
    if ( status==nf90_noerr ) first = j1<i
!
    status = nf90_get_att( ncid_clim, nf90_global, "lon2_"//trim(var_name), i )
    if ( status==nf90_noerr .and. .not.first ) first = j2>i
!
    status = nf90_get_att( ncid_clim, nf90_global, "lat1_"//trim(var_name), i )
    if ( status==nf90_noerr .and. .not.first ) first = lat1<i
!
    status = nf90_get_att( ncid_clim, nf90_global, "lat2_"//trim(var_name), i )
    if ( status==nf90_noerr .and. .not.first ) first = lat2>i
!
    status = nf90_get_att( ncid_clim, nf90_global, "level1_"//trim(var_name), i )
    if ( status==nf90_noerr .and. .not.first ) first = level1<i
!
    status = nf90_get_att( ncid_clim, nf90_global, "level2_"//trim(var_name), i )
    if ( status==nf90_noerr .and. .not.first ) first = level2>i
!
    if ( first ) call merror( name_proc//"Missing values in the climatology " )
!
!   CHECK PERIOD, PERIODICITY AND LIST OF YEARS COMPATIBILITY.
!
    nm = dimlens(4)
    tmp = time3/nm
    k = ceiling( tmp )
    if ( any( years>k ) )     &
    call merror( name_proc//"-t= and -a= options mismatch" )
!
    if ( alg==2 ) then
        if ( (time3/nm)*nm/=time3 )    &
        call merror( name_proc//"The selected time period must be a multiple of the periodicity if -alg=simul " )
    end if
!
!   GET THE STD ID FROM ITS NAME.
!   FIND THE DIMENSION LENGTHS FOR THE NETCDF STD VARIABLE.
!
    call get_variable_dims( name_proc, trim(var_name)//"_std", ncid_clim,    &
                            stdid, dimids, dimlens )
!
    if ( count(dimlens/=0)/=4 )          &
    call merror( name_proc//"Input climatology is not fourdimensional" )
!
    if ( dimlens(1)/=nlon .or. dimlens(2)/=nlat .or. dimlens(3)/=nlevel )          &
    call merror( name_proc//"Shape disagreement with the climatology (std field)" )
!
    if ( dimlens(4)/=nm )                                                          &
    call merror( name_proc//"Shape disagreement in the climatology" )
!
!   ALLOCATE WORK ARRAYS.
!
    allocate( temp(nlon,nlat,nlevel), mask(nlon,nlat,nlevel), stat=status )
    if ( status/=0 ) call merror( name_proc//allocate_error )
!
    if ( use_mask ) then
!
!       GET THE MASK.
!
        status = nf90_get_var( ncid_mask, maskid, temp )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!       DETERMINE THE GRID POINTS USED IN THE COMPOSITE ANALYSIS FROM THE MASK.
!
        mask(:nlon,:nlat,:nlevel) = temp(:nlon,:nlat,:nlevel)==one
!
    else
!
!       CONSTRUCT A DUMMY MASK.
!
        mask(:nlon,:nlat,:nlevel) = true
!
    end if
!
!   SELECT THE DOMAIN.
!
    if ( lon1>0 ) then
        mask(1:lon1-1,:nlat,:nlevel)    = false
        mask(lon2+1:nlon,:nlat,:nlevel) = false
    else
        mask(lon2+1:nlon+lon1,:nlat,:nlevel) = false
    end if
!
    mask(:nlon,1:lat1-1,:nlevel)    = false
    mask(:nlon,lat2+1:nlat,:nlevel) = false
!
!   SELECT THE VERTICAL RESOLUTION.
!
    mask(:nlon,:nlat,1:level1-1)      = false
    mask(:nlon,:nlat,level2+1:nlevel) = false
!
    ndat = count( mask )
    if ( ndat<=0  )  call merror( name_proc//"Wrong domain definition : domain is empty" )
!
!   ALLOCATE WORK ARRAYS .
!
    i = merge( nm, time3, alg==1 )
!
    allocate( dat(ndat,i), compo(ndat,nm), comp_mean(ndat,nm), comp_std(ndat,nm), u(ndat,nm), &
              period(nm), n_period(nm), df(nm),  mean(ndat,nm), std(ndat,nm), stat=status     )
    if ( status/=0 ) call merror( name_proc//allocate_error )
!
    startlec(1:3) = (/ j1, lat1, level1 /)
    countlec(1:4) = (/ lon3, lat3, level3, 1_i4b /)
!
!   GET THE MEANS.
!
    do j = 1, nm
!
        startlec(4) = j
!
        status = nf90_get_var( ncid_clim, meanid, temp(:lon3,:lat3,:level3),        &
                               start=startlec(:4), count=countlec(:4) )
        if ( status/=nf90_noerr )                                                   &
        call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!       PACK THE MEAN FIELD.
!
        mean(:ndat,j) = pack( temp(:lon3,:lat3,:level3), mask(j1:j2,lat1:lat2,level1:level2) )
!
    end do
!
!   GET THE STDS.
!
    do j = 1, nm
!
        startlec(4) = j
!
        status = nf90_get_var( ncid_clim, stdid, temp(:lon3,:lat3,:level3),        &
                               start=startlec(:4), count=countlec(:4) )
        if ( status/=nf90_noerr )                                                  &
        call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!       PACK THE STD FIELD.
!
        std(:ndat,j) = pack( temp(:lon3,:lat3,:level3), mask(j1:j2,lat1:lat2,level1:level2) )
!
    end do
!
!   COMPUTE THE OFFSET FOR THE RECORD NUMBER.
!
    offset_rec  = time1 - 1
    period(:nm) = (/ (i,i=1,nm) /)
    years(:nyears) = years(:nyears) - 1
!
!   COMPUTE THE STATISTICS FOR THE COMPOSITE YEARS.
!
    if ( alg==1 ) then
!
!       GET THE DATA FOR THE COMPOSITE YEARS.
!
        do i = 1, nyears
!
            l = years(i)*nm + offset_rec
!
            do j = 1, nm
!
                k = l + j
!
                if ( k>time2 ) exit
!
                startlec(4) = k
!
!               FILL THE WORK ARRAY WITH A RECORD FROM THE NETCDF VARIABLE.
!
                status = nf90_get_var( ncid, varid, temp(:lon3,:lat3,:level3),              &
                                       start=startlec(:4), count=countlec(:4) )
                if ( status/=nf90_noerr )                                                   &
                call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!               PACK THE DATA.
!
                dat(:ndat,j) =  pack( temp(:lon3,:lat3,:level3), mask(j1:j2,lat1:lat2,level1:level2) )
!
            end do
!
!           COMPUTE UNIVARIATE STATISTICS FOR THE COMPOSITE YEARS.
!
            first = i==1
            last  = i==nyears
            l     = j - 1
!
            call comp_mvs_grp( dat(:ndat,:l), first, last, nm, period(:l),           &
                               comp_mean, comp_std, n_period )
!
        end do
!
    else
!
!       GET THE DATA FOR ALL THE YEARS FIRST.
!
!
#ifdef _PARALLEL_READ
!
        i = omp_get_num_procs()
        j = omp_get_max_threads()
        test_par = i>1 .and. j>1 .and. time3*ndat>omp_limit .and. time3>=j
!
        if ( test_par ) then
!
!           DEALLOCATE WORK ARRAY.
!
            deallocate( temp )
!
!$OMP PARALLEL PRIVATE(k,l,status,startlec,ncid2,temp)           &
!$OMP         ,SHARED(dat,mask,countlec,varid,j1,j2,lon3,lat1,lat2,lat3,level1,level2,level3,time1,time2)
!
!           ALLOCATE WORK ARRAYS.
!
            allocate( temp(lon3,lat3,level3), stat=status )
            if ( status/=0 ) call merror( name_proc//allocate_error )
!
            k = nf90_sizehint_default
!
!$OMP CRITICAL (opennc)
!
!           OPEN THE NETCDF DATASET.
!
            status = nf90_open( trim(fic_name), nf90_share, ncid2, chunksize=k  )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!$OMP END CRITICAL (opennc)
!
            startlec(1)  = j1
            startlec(2)  = lat1
            startlec(3)  = level1
!
!$OMP DO SCHEDULE(STATIC)
              do k = time1, time2
!
                  startlec(4) = k
!
!                 FILL THE WORK ARRAY WITH A RECORD FROM THE NETCDF VARIABLE.
!
                  status = nf90_get_var( ncid2, varid, temp(:lon3,:lat3,:level3),             &
                                         start=startlec(:4), count=countlec(:4) )
                  if ( status/=nf90_noerr )                                                   &
                  call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!                 PACK THE DATA.
!
                  l = k - offset_rec
                  dat(:ndat,l) = pack( temp(:lon3,:lat3,:level3), mask(j1:j2,lat1:lat2,level1:level2) )
!
              end do
!$OMP END DO NOWAIT
!
!           DEALLOCATE WORK ARRAY.
!
            deallocate( temp )
!
            status = nf90_close( ncid2 )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!$OMP END PARALLEL

!           REALLOCATE WORK ARRAY.
!
            allocate( temp(nlon,nlat,nlevel), stat=status )
            if ( status/=0 ) call merror( name_proc//allocate_error )
!
        else
!
#endif
            do k = time1, time2
!
                startlec(4) = k
!
!               FILL THE WORK ARRAY WITH A RECORD FROM THE NETCDF VARIABLE.
!
                status = nf90_get_var( ncid, varid, temp(:lon3,:lat3,:level3),              &
                                       start=startlec(:4), count=countlec(:4) )
                if ( status/=nf90_noerr )                                                   &
                call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!               PACK THE DATA.
!
                l = k - offset_rec
                dat(:ndat,l) = pack( temp(:lon3,:lat3,:level3), mask(j1:j2,lat1:lat2,level1:level2) )
!
            end do
!
#ifdef _PARALLEL_READ
!
        end if
!
#endif
!
!       COMPUTE UNIVARIATE STATISTICS FOR THE COMPOSITE YEARS.
!
        do i = 1, nyears
!
            l = years(i)*nm
!
!           COMPUTE UNIVARIATE STATISTICS FOR THE COMPOSITE YEARS.
!
            first = i==1
            last  = i==nyears
!
            call comp_mvs_grp( dat(:ndat,l+1:l+nm), first, last, nm, period,           &
                               comp_mean, comp_std, n_period )
!
        end do
!
    end if
!
    years(:nyears) = years(:nyears) + 1
!
!   CREATE THE NEW NETCDF DATASET FOR THE COMPOSITE ANALYSIS.
!
    if ( create_hdf5 ) then
        status = nf90_create( comp_name, cmode=output_mode, ncid=ncid_comp )
    else
        k = nf90_sizehint_default
        status = nf90_create( comp_name, cmode=output_mode, ncid=ncid_comp, chunksize=k )
    end if
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_set_fill( ncid_comp, nf90_nofill, i )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!   DEFINE THE DIMENSIONS FOR THE NEW NETCDF DATASET.
!
    dimlens(1)   = nlon
    dimlens(2)   = nlat
    dimlens(3)   = nlevel
    if ( use_tlimited ) then
        dimlens(4)   = nm
    else
        dimlens(4)   = nf90_unlimited
    end if
    dimlens(5)   = nyears
!
    dim_names(4) = "period"
    dim_names(5) = "composite_years"
!
    coor_type(4:5) = nf90_int
!
    use_coor_variable(4) = false
!
    do i = 1, 5
!
        status = nf90_def_dim( ncid_comp, name=trim(dim_names(i)),                         &
                               len=dimlens(i), dimid=dimids(i) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim(nf90_strerror(status)) )
!
    end do
!
!   DEFINE THE VARIABLES FOR THE NEW NETCDF DATASET.
!
    do i = 1, 5
!
        status = nf90_def_var( ncid_comp, trim(dim_names(i)), coor_type(i),                &
                               dimids=dimids(i), varid=coor_id2(i) )
        if ( status/=nf90_noerr )                                                          &
        call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end do
!
    status = nf90_def_var( ncid_comp, trim(var_name)//"_compmean", output_type, dimids=dimids(:4), varid=meanid )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_def_var( ncid_comp, trim(var_name)//"_compstd", output_type, dimids=dimids(:4), varid=stdid )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_def_var( ncid_comp, trim(var_name)//"_qstd", output_type, dimids=dimids(:4), varid=qstdid )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_def_var( ncid_comp, trim(var_name)//"_composite", output_type, dimids=dimids(:4), varid=compid )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_def_var( ncid_comp, trim(var_name)//"_u", output_type, dimids=dimids(:4), varid=uid )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_def_var( ncid_comp, trim(var_name)//"_prob", output_type, dimids=dimids(:4), varid=probid )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_def_var( ncid_comp, trim(var_name)//"_compnobs", nf90_int, dimids=dimids(4), varid=nobsid )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!   ASSIGN ATTRIBUTES FOR THE NEW NETCDF DATASET.
!
    do j = 1, 3
!
        if ( .not.use_coor_variable(j) ) cycle
!
        do i = 1, coor_natts(j)
!
            status = nf90_inq_attname( ncid, coor_id(j), i, att_name )
            if ( status/=nf90_noerr )                                                     &
            call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           COPY ATTRIBUTES FOR THE COORDINATE VARIABLES.
!
            status = nf90_copy_att(  ncid, coor_id(j), att_name, ncid_comp, coor_id2(j) )
            if ( status/=nf90_noerr )                                                     &
            call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end do
!
    end do
!
    status = nf90_put_att( ncid_comp, coor_id2(4), "long_name", "Period axis" )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_comp, coor_id2(4), "short_name", "Period" )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_comp, coor_id2(5), "long_name", "Composite years (indices)" )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_comp, coor_id2(5), "short_name", "Years" )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    name = " "
    if ( method==1 ) then
        name = "anomalies"
    else
        name = "differences"
    end if
!
    name2 = " "
    if ( use_nostd ) then
        name2 = "Composite "//trim(name)//" of"
    else
        name2 = "Composite standardized "//trim(name)//" of"
    end if
!
    do i = 1, natts
!
        status = nf90_inq_attname( ncid, varid, i, att_name )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!       COPY AND MODIFY USEFUL ATTRIBUTES.
!
        select case ( att_name )
!
            case ( 'long_name', 'short_name' )
!
#ifdef _USE_NETCDF36
!
! Due to a bug in the NETCDF 3.5 library, the following section of code can't be used.
! However, if you are using NETCDF 3.5.1 or higher, you can use this section safely.
!
                status = nf90_inquire_attribute( ncid, varid, att_name, len=j )
                if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!               CHECK LENGTH OF STRING ATTRIBUTE.
!
                if ( j <= nchar ) then
                    status = nf90_get_att( ncid, varid, att_name, name )
                    if ( status/=nf90_noerr )                                             &
                    call merror( name_proc//trim( nf90_strerror( status ) ) )
                    name(j+1:nchar) = " "
                else
                    name = var_name
                end if
#else
!
! This section of code must be used if you are using NETCDF 3.5.
!
                name(1:nchar) = " "
                status = nf90_get_att( ncid, varid, att_name, name )
                if ( status/=nf90_noerr )                                             &
                call merror( name_proc//trim( nf90_strerror( status ) ) )
#endif
!
!               MODIFY USEFUL ATTRIBUTES.
!
                status = nf90_put_att( ncid_comp, meanid, att_name,                   &
                                       "Composite Means of "//trim(name)  )
                if ( status/=nf90_noerr )                                             &
                call merror( name_proc//trim( nf90_strerror( status ) ) )
!
                status = nf90_put_att( ncid_comp, stdid, att_name,                    &
                                       "Composite STDs of "//trim(name)  )
                if ( status/=nf90_noerr )                                             &
                call merror( name_proc//trim( nf90_strerror( status ) ) )
!
                status = nf90_put_att( ncid_comp, qstdid, att_name,                   &
                                       "Composite STDs/STDs of "//trim(name)  )
                if ( status/=nf90_noerr )                                             &
                call merror( name_proc//trim( nf90_strerror( status ) ) )
!
                status = nf90_put_att( ncid_comp, compid, att_name,                   &
                                       trim(name2)//" "//trim(name)  )
                if ( status/=nf90_noerr )                                             &
                call merror( name_proc//trim( nf90_strerror( status ) ) )
!
                status = nf90_put_att( ncid_comp, uid, att_name,                      &
                                       "U Statistics of "//trim(name)  )
                if ( status/=nf90_noerr )                                             &
                call merror( name_proc//trim( nf90_strerror( status ) ) )
!
                status = nf90_put_att( ncid_comp, probid, att_name,                   &
                                       "Critical Probabilities of "//trim(name)  )
                if ( status/=nf90_noerr )                                             &
                call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            case ( 'units', 'scale_factor' )
!
!               COPY USEFUL ATTRIBUTES.
!
                status = nf90_copy_att(  ncid, varid, att_name, ncid_comp, meanid )
                if ( status/=nf90_noerr )                                               &
                call merror( name_proc//trim( nf90_strerror( status ) ) )
!
                status = nf90_copy_att(  ncid, varid, att_name, ncid_comp, stdid )
                if ( status/=nf90_noerr )                                               &
                call merror( name_proc//trim( nf90_strerror( status ) ) )
!
                if ( use_nostd ) then
                    status = nf90_copy_att(  ncid, varid, att_name, ncid_comp, compid )
                    if ( status/=nf90_noerr )                                           &
                    call merror( name_proc//trim( nf90_strerror( status ) ) )
                end if
!
            case ( 'add_offset' )
!
!               COPY add_offset ATTRIBUTE, IF REQUIRED.
!
                status = nf90_copy_att(  ncid, varid, att_name, ncid_comp, meanid )
                if ( status/=nf90_noerr )                                             &
                call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end select
!
    end do
!
    if ( .not.use_nostd ) then
        status = nf90_put_att( ncid_comp, compid, "units", "standard-deviation" )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
    end if
!
    status = nf90_put_att( ncid_comp, nobsid, "long_name", "Number of observations" )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_comp, nobsid, "short_name", "Nobs" )
    if ( status/=nf90_noerr ) call merror( name_proc// trim( nf90_strerror( status ) ) )
!
    id_var(1:6) = (/ meanid, stdid, qstdid, compid, uid, probid /)
!
    if (  output_type==nf90_real8 ) then
!
        do i=1, 6
!
            status = nf90_put_att( ncid_comp, id_var(i), "missing_value", real( missing, dp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_comp, id_var(i), "_FillValue", real( missing, dp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end do
!
    else
!
        do i=1, 6
!
            status = nf90_put_att( ncid_comp, id_var(i), "missing_value", real( missing, sp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_comp, id_var(i), "_FillValue", real( missing, sp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end do
!
    end if
!
!   CREATE GLOBAL ATTRIBUTES.
!
    status = nf90_put_att( ncid_comp, nf90_global, "title", "Composite analysis of "//trim(var_name) )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_comp, nf90_global, "grid_type", grid_type )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_comp, nf90_global, "algorithm", calgorithm(alg) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    if ( alg>1 ) then
        status = nf90_put_att( ncid_comp, nf90_global, "number_of_shuffles", nrep )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
    end if
!
    status = nf90_put_att( ncid_comp, nf90_global, "statistic", cmethod(method) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    if ( use_nostd ) then
        i = 0
    else
        i = 1
    end if
!
    status = nf90_put_att( ncid_comp, nf90_global, "standardization", i )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_comp, nf90_global, "history", trim(history) )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!   LEAVE DEFINE MODE FOR THE NEW NETCDF DATASET.
!
    status = nf90_enddef( ncid_comp )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!   WRITE VARIABLES FOR THE NEW  NETCDF DATASET.
!
!
!   ALLOCATE WORK ARRAY.
!
    i = max( nlon, nlat, nlevel )
    allocate( record(i), stat=status )
    if ( status/=0 ) call merror( name_proc//allocate_error )
!
    dimlens(4) = nm
!
    do j = 1, 4
!
        j2 = dimlens(j)
!
        if ( use_coor_variable(j) ) then
            status = nf90_get_var( ncid, coor_id(j), record(:j2), count=(/ j2 /) )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_var( ncid_comp, coor_id2(j), record(:j2), count=(/ j2 /) )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
        else
            status = nf90_put_var( ncid_comp, coor_id2(j), arth(1_i4b,1_i4b,j2 ) )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
        end if
!
    end do
!
!   DEALLOCATE WORK ARRAY.
!
    deallocate( record )
!
    status = nf90_put_var( ncid_comp, coor_id2(5), years )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    startlec(1:3) = 1
!
!   WRITE THE MEANS OF THE COMPOSITE YEARS.
!
    do i = 1, nm
!
        startlec(4) = i
!
        temp(:nlon,:nlat,:nlevel) = unpack( comp_mean(:ndat,i), mask(:nlon,:nlat,:nlevel), missing )
!
        status = nf90_put_var( ncid_comp, meanid, temp, start=startlec(:4) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end do
!
!   WRITE THE STDS OF THE COMPOSITE YEARS.
!
    do i = 1, nm
!
        startlec(4) = i
!
        temp(:nlon,:nlat,:nlevel) = unpack( comp_std(:ndat,i), mask(:nlon,:nlat,:nlevel), missing )
!
        status = nf90_put_var( ncid_comp, stdid, temp, start=startlec(:4) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end do
!
!   WRITE THE NUMBER OF OBSERVATIONS FOR THE COMPOSITES.
!
    status = nf90_put_var( ncid_comp, nobsid, n_period )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!   COMPUTE QUOTIENT OF STDS AND THE COMPOSITES.
!
    where( std(:ndat,:nm)/=zero )
        compo(:ndat,:nm)     = comp_mean(:ndat,:nm) - mean(:ndat,:nm)
        u(:ndat,:nm)         = compo(:ndat,:nm)/std(:ndat,:nm)
        comp_std(:ndat,:nm)  = comp_std(:ndat,:nm)/std(:ndat,:nm)
    elsewhere
        compo(:ndat,:nm)     = zero
        u(:ndat,:nm)         = zero
        comp_std(:ndat,:nm)  = one
    end where
!
!   WRITE QUOTIENT OF STDS.
!
    do i = 1, nm
!
        startlec(4) = i
!
        temp(:nlon,:nlat,:nlevel) = unpack( comp_std(:ndat,i), mask(:nlon,:nlat,:nlevel), missing )
!
        status = nf90_put_var( ncid_comp, qstdid, temp, start=startlec(:4) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end do
!
!   COMPUTE THE TOTAL NUMBERS OF OBSERVATIONS.
!
    k = time3/nm
    if ( time3==k*nm ) then
        df(:nm) = real( k, stnd )
    else
        l          = time3 - k*nm
        df(:l)     = real( k, stnd ) + one
        df(l+1:nm) = real( k, stnd )
    end if
!
!   WRITE THE COMPOSITES.
!
    if ( use_nostd ) then
        comp_std(:ndat,:nm)  = compo(:ndat,:nm)
    else
        comp_std(:ndat,:nm)  = u(:ndat,:nm)
    end if
!
    if ( method==2 ) then
!
        do i = 1, nm
            comp_std(:ndat,i) = comp_std(:ndat,i)*( df(i)/(df(i)-n_period(i)) )
        end do
!
    end if
!
    do i = 1, nm
!
        startlec(4) = i
!
        temp(:nlon,:nlat,:nlevel) = unpack( comp_std(:ndat,i), mask(:nlon,:nlat,:nlevel), missing )
!
        status = nf90_put_var( ncid_comp, compid, temp, start=startlec(:4) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end do
!
!   COMPUTE THE U STATISTICS.
!
    where ( n_period(:nm)<df(:nm) )
        df(:nm) = sqrt( ( n_period(:nm)*( df(:nm) - one ) )/( df(:nm)  - n_period(:nm) ) )
    elsewhere
        df(:nm) = zero
    end where
!
    u(:ndat,:nm) = u(:ndat,:nm)*spread( df(:nm), 1, ndat )
!
!   WRITE THE U STATISTICS.
!
    do i = 1, nm
!
        startlec(4) = i
!
        temp(:nlon,:nlat,:nlevel) = unpack( u(:ndat,i), mask(:nlon,:nlat,:nlevel), missing )
!
        status = nf90_put_var( ncid_comp, uid, temp, start=startlec(:4) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end do
!
!   COMPUTE THE PROBABILITIES.
!
    if ( alg==1 ) then
!
!       USE A NORMAL APPROXIMATION.
!
        comp_std(:ndat,:nm)  = abs( u(:ndat,:nm) )
!
!        compo(:ndat,:nm)     = two*probn( comp_std(:ndat,:nm), upper=true )
!
        do i = 1, nm
            compo(:ndat,i) = two*probn( comp_std(:ndat,i), upper=true )
        end do
!
!       DEALLOCATE WORK ARRAYS.
!
        deallocate( dat, mean, std, comp_mean, comp_std, u, years, n_period, period, df )
!
    else
!
!       USE SIMULATION.
!
        compo(:ndat,:nm) = abs( compo(:ndat,:nm) )
!
!       DEALLOCATE WORK ARRAYS.
!
        deallocate( std, comp_std, u, years, n_period, period, df )
!
!       INITIALIZE THE RANDOM GENERATOR.
!
        call random_seed_()
!
!       ALLOCATE WORK ARRAY.
!
        allocate( nge(ndat,nm), stat=status )
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
        nge(:ndat,:nm) = 1
        tmp            = one/real( nyears, stnd )
!
#ifdef _OPENMP
!
        i = omp_get_num_procs()
        j = omp_get_max_threads()
        test_par = i>1 .and. j>1
!
        if ( test_par ) then
!
!           DEALLOCATE WORK ARRAY.
!
            deallocate( comp_mean )
!
!$OMP PARALLEL PRIVATE(comp_mean, nge2, years, i, j, j1, j2, status)
!
!           ALLOCATE WORK ARRAYS.
!
            i = time3/nm
            allocate( years(i),comp_mean(ndat,nm),nge2(ndat,nm), stat=status )
            if ( status/=0 ) call merror( name_proc//allocate_error )
!
            nge2(:ndat,:nm) = 0
!
!$OMP DO SCHEDULE(STATIC)
!
            do i=1_i4b, nrep
!
!$OMP CRITICAL
!
!               DRAW A SAMPLE, WITHOUT REPLACEMENT OF SIZE nyears .
!
                call drawsample( nyears, years )
!
!$OMP END CRITICAL
!
!               COMPUTE COMPOSITE MEANS FROM THE SAMPLE YEARS.
!
                comp_mean(:ndat,:nm) = zero
!
                do j = 1, nyears
                    j2 = years(j)*nm
                    j1 = j2 - nm + 1
                    comp_mean(:ndat,:nm) = comp_mean(:ndat,:nm) + dat(:ndat,j1:j2)
                end do
!
                comp_mean(:ndat,:nm) = abs( comp_mean(:ndat,:nm)*tmp - mean(:ndat,:nm) )
!
                where ( comp_mean(:ndat,:nm)>=compo(:ndat,:nm) )   &
                nge2(:ndat,:nm) = nge2(:ndat,:nm) + 1
!
            end do
!
!$OMP END DO NOWAIT
!
!$OMP CRITICAL
!
            nge(:ndat,:nm) = nge(:ndat,:nm) + nge2(:ndat,:nm)
!
!$OMP END CRITICAL
!
!           DEALLOCATE WORK ARRAYS.
!
            deallocate( comp_mean, nge2, years )
!
!$OMP END PARALLEL
!
        else
!
#endif
!
!           ALLOCATE WORK ARRAY.
!
            i = time3/nm
            allocate( years(i), stat=status )
            if ( status/=0 ) call merror( name_proc//allocate_error )
!
            do i=1_i4b, nrep
!
!               DRAW A SAMPLE, WITHOUT REPLACEMENT OF SIZE nyears.
!
                call drawsample( nyears, years )
!
!               COMPUTE COMPOSITE MEANS FROM THE SAMPLE YEARS.
!
                comp_mean(:ndat,:nm) = zero
!
                do j = 1, nyears
                    j2 = years(j)*nm
                    j1 = j2 - nm + 1
                    comp_mean(:ndat,:nm) = comp_mean(:ndat,:nm) + dat(:ndat,j1:j2)
                end do
!
                comp_mean(:ndat,:nm) = abs( comp_mean(:ndat,:nm)*tmp - mean(:ndat,:nm) )
!
                where ( comp_mean(:ndat,:nm)>=compo(:ndat,:nm) )   &
                nge(:ndat,:nm) = nge(:ndat,:nm) + 1
!
            end do
!
!           DEALLOCATE WORK ARRAYS.
!
            deallocate( comp_mean, years )
!
#ifdef _OPENMP
!
        end if
!
#endif
!
!       COMPUTE THE SIGNIFICANCE LEVEL.
!
        tmp = one/real( nrep+1, stnd )
        compo(:ndat,:nm) = real( nge(:ndat,:nm), stnd )*tmp
!
!       DEALLOCATE WORK ARRAYS.
!
        deallocate( dat, nge, mean )
!
    end if
!
!
!   WRITE THE PROBABILITIES.
!
    do i = 1, nm
!
        startlec(4) = i
!
        temp(:nlon,:nlat,:nlevel) = unpack( compo(:ndat,i), mask(:nlon,:nlat,:nlevel), missing )
!
        status = nf90_put_var( ncid_comp, probid, temp, start=startlec(:4) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end do
!
!   DEALLOCATE WORK ARRAYS.
!
    deallocate( temp, mask, compo  )
!
!   CLOSE THE NETCDF DATASETS.
!
    status = nf90_close( ncid )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    if ( use_mask ) then
        status = nf90_close( ncid_mask )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
    end if
!
    status = nf90_close( ncid_clim )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_close( ncid_comp )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!
! END OF PROGRAM comp_composite_4d
! ================================
!
end program comp_composite_4d
