program comp_eof_4d
!! #############################################################################
!!
!! Copyright 2021 IRD
!!
!! This file is part of NCSTAT.
!!
!! NCSTAT is free software: you can redistribute it and/or modify
!! it under the terms of the GNU Lesser General Public License as 
!! published by the Free Software Foundation, either version 3 of 
!! the License, or (at your option) any later version.
!!
!! NCSTAT is distributed in the hope that it will be useful,
!! but WITHOUT ANY WARRANTY; without even the implied warranty of
!! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!! GNU Lesser General Public License for more details.
!!
!! You can find a copy of the GNU Lesser General Public License
!! in the NCSTAT/doc directory.
!!
!! #############################################################################
!
!+
!
! .. _comp_eof_4d:
!
! ===========
! comp_eof_4d
! ===========
!
! Authors
! =======
!
! Pascal Terray (LOCEAN/IPSL)
!
! Latest revision
! ===============
!
! 15/03/2021
!
! Purpose
! =======
!
! Compute an Empirical Orthogonal Function (EOF) analysis , also
! known as Principal Component Analysis (PCA) from
! a fourdimensional variable extracted from a NetCDF dataset.
! The procedure first transforms the input fourdimensional NetCDF variable
! as a *ntime* by *nv* rectangular matrix, **X**\ , of observed variables (e.g. the selected
! cells of the 3-D grid-mesh associated with the fourdimensional NetCDF variable)
! and then performs an EOF analysis of this rectangular matrix.
!
! The eigenvalues, eigenvectors and principal components time series
! of the EOF analysis are computed by a full or partial Singular Value 
! Decomposition (SVD) of the rectangular matrix of the observed variables [Bjornsson_Venegas]_  [Hannachi]_
! [vonStorch_Zwiers]_. Both algorithms find square roots of eigenvalues (e.g. singular values of the
! data matrix **X**\ ) and associated eigenvectors of the sums of squares
! and cross-products, covariance or correlation matrix between the observed variables
! without actually computing this symmetric matrix.
!
! An output NetCDF dataset containing singular values, eigenvectors
! and standardized principal component time series is created.
! The eigenvectors are repacked as a fourdimensional variable in
! the output NetCDF dataset.
!
! You should use EOF analysis if you are interested in summarizing
! data and/or detecting linear relationships between the observed
! variables. EOF analysis can also be used to reduce the number of
! variables or the noise in a dataset before a regression, cluster or
! Maximum Covariance Analysis (MCA). More specifically, the first *k*
! principal component time series and eigenvectors give a least-squares
! solution to the model
!
!           **X** = **A**\ **B** + **E**
!
! where
!
!  * **X** is the *ntime* by *nv* matrix of observed variables
!  * **A** is the *ntime* by *k* matrix of the first *k* principal component time series
!  * **B** is the *k* by *nv* matrix of the first *k* eigenvectors (stored rowwise)
!  * **E** is an *ntime* by *nv* matrix of residuals
!
! and you want to minimize the squared Frobenius norm of **E** (e.g. the sum of all the
! squared elements of **E**).
!
! Refer to :ref:`comp_invert_eof_4d`, if you want to compute such approximation of your
! dataset.
!
! Refer to :ref:`comp_svd_3d` , :ref:`comp_reg_3d` and :ref:`comp_reg_4d` for more
! details on MCA and regression procedures available in NCSTAT, respectively.
!
! If the NetCDF variable is tridimensional use :ref:`comp_eof_3d` instead of comp_eof_4d.
!
! This procedure is parallelized if OpenMP is used. Moreover, this procedure may use partial SVD
! algorithms which are highly efficient on huge datasets if you are interested only in the few
! leading terms of the SVD of the data matrix **X**\ .
!
! Further Details
! ===============
!
! Usage
! -----
!
! ::
!
!       $ comp_eof_4d \
!         -f=input_netcdf_file \
!         -v=netcdf_variable \
!         -m=input_mesh_mask_netcdf_file \
!         -g=grid_type                         (optional : n, t, u, v, w, f) \
!         -r=resolution                        (optional : r2, r4) \
!         -b=nlon_orca, nlat_orca, nlevel_orca (optional) \
!         -x=lon1,lon2                         (optional) \
!         -y=lat1,lat2                         (optional) \
!         -z=level1,level2                     (optional) \
!         -t=time1,time2                       (optional) \
!         -a=type_of_analysis                  (optional : scp, cov, cor) \
!         -c=input_climatology_netcdf_file     (optional) \
!         -d=type_of_distance                  (optional : dist2, dist3, ident) \
!         -alg=algorithm                       (optional : svd, inviter, deflate) \
!         -n=number_of_eofs                    (optional) \
!         -o=output_eof_netcdf_file            (optional) \
!         -mi=missing_value                    (optional) \
!         -explvar                             (optional) \
!         -double                              (optional) \
!         -bigfile                             (optional) \
!         -hdf5                                (optional) \
!         -tlimited                            (optional)
!
! By default
! ----------
!
!        *-g=*
!         the *grid_type* is set to ``n`` which means that the 3-D grid-mesh associated with
!         the input NetCDF variable is assumed to be regular or Gaussian
!        *-r=*
!         if the input netcdf_variable is from the NEMO or ORCA model (e.g. if **-n=** argument
!         is not set to ``n``) the resolution is assumed to be ``r2``
!        *-b=*
!         if **-n=** is not set to ``n``\ , the dimensions of the 3-D grid-mesh, *nlon_orca*, 
!         *nlat_orca* and *nlevel_orca* are determined from the **-r=** argument. However, you
!         may override this choice by default with the **-b=** argument
!        *-x=*
!         the whole longitude domain associated with the *netcdf_variable*
!        *-y=*
!         the whole latitude domain associated with the *netcdf_variable*
!        *-z=*
!         the whole vertical resolution associated with the *netcdf_variable*
!        *-t=*
!         the whole time period associated with the *netcdf_variable*
!        *-a=*
!         the *type_of_analysis* is set to ``scp``. This means that the eigenvectors
!         and eigenvalues are computed from the sums of squares and cross-products matrix
!         between the observed variables
!        *-c=*
!         an *input_climatology_netcdf_file* is not needed if
!         the *type_of_analysis* is set to ``scp``
!        *-d=*
!         the *type_of_distance* is set to ``dist3``. This means that distances and scalar products
!         in the EOF analysis are computed with the diagonal metric associated with the 3-D grid-mesh
!         associated with the input NetCDF variable
!        *-alg=*
!         the *algorithm* option is set to ``inviter``\ . This means that the EOF model is
!         computed by a partial SVD analysis of the matrix of the observed variables using
!         an inverse iteration algorithm
!        *-n=*
!         *number_of_eofs* is set to ``10`` and a 10-component EOF model is stored in
!         the output NetCDF file *output_eof_netcdf_file*
!        *-o=*
!         the *output_eof_netcdf_file* is named ``eof_``\ *netcdf_variable*\ ``.nc``
!        *-mi=*
!         the *missing_value* attribute in the output NetCDF file is set to ``1.e+20``
!        *-explvar*
!         the **-n=** option specifies the number of eofs to be computed and
!         stored in the output NetCDF file. If **-explvar** is activated, the **-n=** option
!         specifies the minimum value of explained variance by the EOF model for selecting
!         the order (e.g. the number of components) of the EOF model
!        *-double*
!         the results of the EOF analysis are stored as single-precision floating
!         point numbers in the output NetCDF file. If **-double** is activated, the
!         results are stored as double-precision floating point numbers 
!        *-bigfile*
!         a NetCDF classical format file is created. If **-bigfile** is activated,
!         the output NetCDF file is a 64-bit offset format file
!        *-hdf5*
!         a NetCDF classical format file is created. If **-hdf5** is activated,
!         the output NetCDF file is a NetCDF-4/HDF5 format file
!        *-tlimited*
!         the time dimension is defined as unlimited in the output NetCDF file.
!         However, if **-tlimited** is activated, the time dimension is defined
!         as limited in the output NetCDF file
!
! Remarks
! -------
!
!   #) The **-v=**\ *netcdf_variable* argument specifies the NetCDF variable for which an
!      EOF analysis must be computed and the **-f=**\ *input_netcdf_file* argument
!      specifies that this NetCDF variable must be extracted from the NetCDF file,
!      *input_netcdf_file*.
!
!   #) The argument **-m=**\ *input_mesh_mask_netcdf_file* specifies the land-sea mask
!      to apply to the *netcdf_variable* for transforming this fourdimensional NetCDF variable as a
!      rectangular matrix before computing the EOF analysis. The scale factors
!      associated with the 3-D grid-mesh of this NetCDF variable (needed if **-d=**\ ``dist2``\  or
!      ``dist3``\  are specified when calling the procedure) are also read from the
!      *input_mesh_mask_netcdf_file*.
!
!   #) If the **-x=**\ *lon1,lon2*, **-y=**\ *lat1,lat2* and **-z=**\ *level1,level2* arguments
!      are missing, the geographical
!      domain used in the EOF analysis is determined from the attributes of the input mesh mask
!      NetCDF variable named *grid_type*\ mask (e.g. lon1_Eastern_limit, lon2_Western_limit, lat1_Southern_limit
!      , lat2_Northern_limit, level1_First_level and level2_Last_level ) which is read from the input NetCDF
!      file *input_mesh_mask_netcdf_file*\ .
!      If these attributes are missing, the whole geographical domain and vertical resolution associated
!      with the *netcdf_variable* is used in the EOF analysis.
!
!      The longitude, latitude or level range must be a vector of two integers specifying
!      the first and last selected indices along each dimension. The indices are relative to ``1``\ .
!      Negative values are allowed for *lon1*. In this case the longitude domain
!      is from ``nlon``\ +\ *lon1*\ +\ ``1`` to *lon2* where ``nlon`` is the number of longitude
!      points in the grid associated with the NetCDF variable and it is assumed that the grid
!      is periodic.
!
!      Refer to :ref:`comp_mask_4d` for transforming geographical coordinates as indices or generating an
!      appropriate mesh-mask before using comp_eof_4d.
!
!   #) If the **-t=**\ *time1,time2* argument is missing the whole time period
!      associated with the *netcdf_variable* is used to estimate eigenvectors and
!      principal component time series.
!
!      The selected time period is a vector of two integers specifying the first
!      and last time observations. The indices are relative to ``1``\ . Note that the
!      output NetCDF file will have ``ntime`` = *time2* - *time1* + ``1`` time observations.
!
!   #) It is assumed that the data has no missing values excepted those associated with a
!      constant land-sea mask.
!
!   #) If **-g=** is set to ``t``, ``u``, ``v``, ``w`` or ``f`` it is assumed that the
!      NetCDF variable is from an experiment with the NEMO or ORCA model.
!      In this case, the duplicate points from the ORCA grid are removed 
!      before the EOF analysis, as far as possible, and, in particular,
!      if the 3-D grid-mesh of the input NetCDF variable covers the whole globe. On output,
!      the duplicate points are restored when writing the EOFs (e.g. the eigenvectors), if the geographical
!      domain of the input NetCDF variable is the whole globe.
!
!      If **-g=** is set to ``n``, it is assumed that the 3-D grid-mesh is regular or Gaussian
!      and as such has no duplicate points.
!
!      The **-g=** argument is also used to determine the name of the NetCDF variables which
!      contain the mesh-mask and the scale factors in the *input_mesh_mask_netcdf_file* (e.g.
!      these variables are named *grid_type*\ ``mask``, ``e1``\ *grid_type*\ , ``e2``\ *grid_type* and 
!      ``e3``\ *grid_type*\ , respectively). This *input_mesh_mask_netcdf_file* may
!      be created by :ref:`comp_clim_4d` if the 3-D grid-mesh is regular or gaussian.
!
!   #) If **-g=** is set to ``t``, ``u``, ``v``, ``w`` or ``f`` (e.g. if the NetCDF variable
!      is from an experiment with the NEMO or ORCA model), the **-r=** argument gives the 
!      resolution used. If:
!
!      * **-r=**\ ``r2``, the NetCDF variable is from an experiment with the ORCA R2 model
!
!      * **-r=**\ ``r4``, the NetCDF variable is from an experiment with the ORCA R4 model.
!
!   #) If the NetCDF variable is from an experiment with the NEMO or ORCA model,
!      but the resolution is not ``r2`` or ``r4``, the dimensions of the ORCA grid
!      must be specified explicitly with the **-b=** argument.
!
!   #) The **-a=** argument specifies if the observed variables are centered or standardized
!      with an input climatology (specified with the **-c=** argument) before the EOF analysis. If:
!
!      * **-a=**\ ``scp``, the EOF analysis is done on the raw data
!
!      * **-a=**\ ``cov``, the EOF analysis is done on the anomalies
!
!      * **-a=**\ ``cor``, the EOF analysis is done on the standardized anomalies.
!
!   #) The *input_climatology_netcdf_file* specified with the **-c=** argument is needed
!      only if **-a=**\ ``cov`` or **-a=**\ ``cor``.
!
!   #) If **-a=**\ ``cov`` or **-a=**\ ``cor``, the selected time period must agree with the climatology.
!      This means that the first selected time observation (*time1* if the **-t=** argument
!      is present) must correspond to the first day, month, season of the climatology
!      specified with the **-c=** argument.
!
!   #) The geographical shapes of the *netcdf_variable* (in the *input_netcdf_file*), the mask
!      (*input_mesh_mask_netcdf_file*), the scale factors (in the *input_mesh_mask_netcdf_file*),
!      and the climatology (in the *input_climatology_netcdf_file*) must agree.
!
!   #) The **-n=**\ *number_of_eofs* argument specifies the number of components of the EOF model which
!      must be stored (and also computed if **-alg=**\ ``inviter`` or **-alg=**\ ``deflate``\  is
!      specified) in the output NetCDF file specified by the **-o=** argument.
!      See also the **-explvar** argument.
!
!   #) If the **-explvar** argument is activated, the number specified in **-n=** option is not the
!      number of required EOFs, but the minimum of explained variance that the EOFs must describe.
!      Number of EOFs is then calculated with reference with the minimum of explained variance required
!      by the **-n=** argument. Express the explained variance in percentage (0-100).
!      If **-explvar** is specified, the **-n=** argument must be less or equal to ``100``.
!
!   #) The **-d=** argument specifies the metric and scalar product used in the EOF analysis. If:
!
!      * **-d=**\ ``dist2``, the EOF analysis is done with the diagonal distance
!        associated with the horizontal 2-D grid-mesh (e.g. each grid point is weighted accordingly to
!        the surface associated with it)
!
!      * **-d=**\ ``dist3``, the analysis is done with the diagonal distance associated
!        with the whole 3D grid-mesh (e.g. each grid point is weighted accordingly to the volume
!        or weight associated with it)
!
!      * **-d=**\ ``ident``, the EOF analysis is done with the identity metric: the
!        Euclidean distance and the usual scalar product are used in the EOF analysis.
!
!      By default, the **-d=** argument is set to ``dist3``\ .
!
!   #) The **-alg=** argument determines how eigenvectors and principal components are
!      computed. If:
!
!      * **-alg=**\ ``svd``, a full SVD of the data matrix is computed, even if you ask only for
!        the leading eigenvectors
!
!      * **-alg=**\ ``inviter``, a partial SVD of the data matrix is computed by inverse iteration
!
!      * **-alg=**\ ``deflate``, a partial SVD of the data matrix is computed by a deflation technique.
!
!      All algorithms are parallelized if OpenMP is used. The default is **-alg=**\ ``inviter``
!      since computing a partial SVD is generally much faster than computing a full SVD. But,
!      **-alg=**\ ``deflate`` is generally as fast as **-alg=**\ ``inviter``.
!
!   #) The **-bigfile** argument is allowed only if the NCSTAT software has been compiled with the
!      _USE_NETCDF36 or _USE_NETCDF4 CPP macros (e.g. ``-D_USE_NETCDF36`` or ``-D_USE_NETCDF4``) and linked
!      to the NetCDF 3.6 library or higher.
!      If this argument is specified, the *output_netcdf_file* will be a 64-bit offset format file
!      instead of a NetCDF classic format file. However, this argument is recognized in the procedure
!      only if the NCSTAT software has been built with the _USE_NETCDF36 or _USE_NETCDF4 CPP macros.
!
!   #) The **-hdf5** argument is allowed only if the NCSTAT software has been
!      compiled with the _USE_NETCDF4 CPP macro (e.g. ``-D_USE_NETCDF4``) and linked
!      to the NetCDF 4 library or higher. 
!      If this argument is specified, the *output_netcdf_file* will be a NetCDF-4/HDF5
!      format file instead of a NetCDF classic format file. However, this
!      argument is recognized in the procedure only if the NCSTAT software has
!      been built with the _USE_NETCDF4 CPP macro.
!
!   #) The **-mi=**\ *missing_value* argument specifies the missing value indicator associated with
!      the netcdf_variables in the *output_netcdf_file*.
!      If the **-mi=** argument is not specified *missing_value* is set to ``1.e+20``.
!
!   #) The **-double** argument specifies that the results are stored as double-precision floating
!      point numbers in the output NetCDF file.
!
!      By default, the results are stored as single-precision floating point numbers
!      in the output NetCDF file.
!
!   #) Duplicate parameters are allowed, but this is always the last occurrence of a parameter
!      which will be used for the computations. Moreover, the number of specified parameters
!      must not be greater than the total number of allowed parameters.
!
!   #) For more details on EOF analysis in the climate literature, see
!
!      * "A manual for EOF and SVD analyses of climate data",
!        by Bjornsson, H., and Venegas, S.A., McGill University, CCGCR Report No. 97-1,
!        Montral, Qubec, 52pp, 1997. https://www.jsg.utexas.edu/fu/files/EOFSVD.pdf
!
!      * "A primer for EOF analysis of climate data",
!        by Hannachi, A., Reading University,
!        Reading, UK, 33pp, 2004. http://www.met.reading.ac.uk/~han/Monitor/eofprimer.pdf
!
!      * "Statistical Analysis in Climate Research",
!        by von Storch, H., and Zwiers, F.W., Cambridge University press,
!        Cambridge, UK, Chapter 13, 484 pp., 2002. ISBN: `9780521012300`_
!
! .. _9780521012300: http://www.cambridge.org/gb/academic/subjects/earth-and-environmental-science/climatology-and-climate-change/statistical-analysis-climate-research?format=PB
!
! Outputs
! -------
!
!   comp_eof_4d creates an output NetCDF file that contains the principal component time
!   series, the eigenvectors and the eigenvalues of the EOF analysis. The number of principal
!   components, eigenvectors and eigenvalues stored in the output NetCDF dataset is determined by
!   the **-n=**\ *number_of_eofs* argument. The output NetCDF dataset contains the following NetCDF
!   variables (in the description below, nlev, nlat and nlon are the length of the dimensions of
!   the input NetCDF variable) :
!
!   #) *netcdf_variable*\ _eof\ ``(number_of_eofs,nlev,nlat,nlon)`` : the selected eigenvectors of the sums of squares
!      and cross-products (**-a=**\ ``scp``), covariance (**-a=**\ ``cov``) or correlation (**-a=**\ ``cor``)
!      matrix between the observed variables. The eigenvectors are sorted by descending order of the
!      associated eigenvalues. The eigenvectors are scaled such that they give the scalar products 
!      (**-a=**\ ``scp``), covariances (**-a=**\ ``cov``) or correlations (**-a=**\ ``cor``) 
!      between the original observed variables and the associated principal component time series.
!
!      The eigenvectors are packed in a fourdimensional variable whose first, second and third dimensions
!      are exactly the same as those associated with the input NetCDF variable *netcdf_variable*
!      even if you restrict the geographical domain with the **-x=**\ , **-y=** and **-z=** arguments.
!      However, outside the selected domain, the output NetCDF variable is filled with missing values.
!      If this is a problem, you can use :ref:`comp_norm_4d` for restricting the geographical
!      domain in the input dataset before using comp_eof_4d.
!
!   #) *netcdf_variable*\ _pc\ ``(ntime,number_of_eofs)`` : the principal component time series
!      sorted by descending order of the eigenvalues (e.g. the square of the singular values of the
!      input matrix of observed variables).
!      The principal component time series are always standardized to unit variance.
!
!   #) *netcdf_variable*\ _sing\ ``(number_of_eofs)`` : the singular values of the input data matrix
!      in decreasing order. Up to a constant scaling factor (equal to the square root of ``1/ntime``),
!      these singular values are the square roots of the eigenvalues of the sums of squares
!      and cross-products (**-a=**\ ``scp``) or covariance (**-a=**\ ``cov``) or 
!      correlation (**-a=**\ ``cor``) matrix between the observed variables.
!
!      These eigenvalues are equal to the variance described by the principal component time series.
!
!   #) *netcdf_variable*\ _var\ ``(number_of_eofs)`` : the proportion of variance explained by each
!      principal component time series.
!
! Examples
! --------
!
!   #)   For computing an EOF analysis from the NetCDF file
!        ``z.1970_2002.apm.nc``, which includes a NetCDF 
!        variable ``z``, and store the results in a NetCDF file named
!        ``eof_era40_1m_z850_1979_2001.nc``, use the following
!        command (the analysis is done on the centered data
!        and only the level 21, which corresponds to 850 hPa,
!        is considered in the analysis) :
!
!        ::
!
!          $ comp_eof_4d \
!            -f=z.1970_2002.apm.nc  \
!            -v=z \
!            -z=21,21 \
!            -m=mask_era40_z.nc  \
!            -g=n \
!            -c=clim_era40_1m_z_1979_2001.nc  \
!            -d=dist2 \
!            -a=cov \
!            -n=10 \
!            -o=eof_era40_1m_z850_1979_2001.nc
!
!
!   #)   For computing an EOF analysis from the NetCDF file
!        ``ST7_1m_0101_20012_grid_T_votemper.nc``, which includes a NetCDF 
!        variable ``votemper`` from a numerical simulation with the ORCA R2 model,
!        and store the results in a NetCDF file named ``eof_votemper.nc``,
!        use the following command (the analysis is done on the standardized data
!        and all the depths/levels are included in the analysis) :
!        ::
!
!          $ comp_eof_4d \
!            -f=ST7_1m_0101_20012_grid_T_votemper.nc \
!            -v=votemper \
!            -c=clim_ST7_1m_0101_20012_grid_T_votemper.nc \
!            -a=cor \
!            -d=dist3 \
!            -m=meshmask.orca2.nc \
!            -g=t
!
!-
! ================================================================================================
!
#ifndef _OPENMP
#undef _PARALLEL_READ
#endif
!
#ifdef _USE_NETCDF4
#define _USE_NETCDF36
#endif
!
! USED MODULES 
! ============
!
#ifdef _USE_NAGWARE
    use f90_unix_env,     only : iargc, getarg
#endif
!
#ifdef _PARALLEL_READ
    use netcdf,           only : nf90_share
    use Statpack,         only : omp_limit
    use omp_lib,          only : omp_get_num_procs, omp_get_max_threads
#endif
!
#ifdef _TRANSPOSE
    use Utilities,        only : transpose=>transpose2
#endif
!
#ifdef _USE_NETCDF36
    use netcdf,           only : nf90_64bit_offset
#endif
!
#ifdef _USE_NETCDF4
    use netcdf,           only : nf90_hdf5
#endif
!
    use netcdf,           only : nf90_max_var_dims, nf90_max_name, nf90_clobber, nf90_real4, nf90_real8,     &
                                 nf90_sizehint_default, nf90_open, nf90_nowrite, nf90_close,                 &
                                 nf90_noerr, nf90_strerror, nf90_get_att, nf90_get_var, nf90_create,         &
                                 nf90_set_fill, nf90_nofill, nf90_unlimited, nf90_def_dim, nf90_def_var,     &
                                 nf90_inq_attname, nf90_copy_att, nf90_put_att, nf90_enddef, nf90_inq_varid, &
                                 nf90_global, nf90_put_var
!
    use Statpack,         only : i1b, i4b, stnd, extd, lgl, true, false, zero, one, c100,               &
                                 kfix, kexp, kint, tolower, case_change, string_to_val, allocate_error, &
                                 merror, lamch, arth, norm, svd_cmp, select_singval_cmp2, bd_inviter2,  &
                                 bd_deflate2
!
    use NCSTAT_Utilities, only : prtunit, nchar, sp, dp, nlon_def2, nlat_def2, ndepth_def2,        &
                                 nlon_def4, nlat_def4, ndepth_def4, def_missing,                   &
                                 get_listarg, get_variable_dims, get_variable_dimcoords,           &
                                 check_lon_domain, select_grid_points_3d, duplicate_orca_points_3d
!
!
! STRONG TYPING IMPOSED 
! =====================
!
    implicit none
!
!
! PARAMETERS 
! ==========
!
    integer(i4b), parameter ::                                      &
!
!  maxiter IS THE NUMBER OF INVERSE ITERATIONS PERFORMED WHEN COMPUTING A
!  PARTIAL SVD OF THE DATA MATRIX WITH THE INVERSE ITERATION METHOD.
!
                                   maxiter=2
    logical(lgl), parameter ::                                      &
!
!  ortho DETERMINES IF THE ORTHOGONALITY OF THE SINGULAR VECTORS IS AUTOMATICALLY PRESERVED
!  WHEN COMPUTING A PARTIAL SVD OF THE DATA MATRIX WITH THE DEFLATION METHOD.
!
                                   ortho=false
!
    character(len=*), parameter :: name_proc='comp_eof_4d : ',                  &
!
                                   cmethod(3)=(/"scp","cov","cor"/),            &          
!
                                   cdistance(3)=(/"dist2","dist3","ident"/),    &
!
                                   cresolution(2)=(/"r2","r4"/),                &        
!
                                   calgorithm(3)=(/"inviter","deflate","svd    "/)
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
!
#ifndef _USE_NAGWARE
    integer(i4b)  :: iargc
#endif
    integer(i4b)   :: narg, status, ncid, ncid_clim, ncid_mask, ncid_eof,                     &
                      varid, maskid, meanid, stdid, e1id, e2id, e3id,                         &
                      eofid, pcid, singid, expvarid,                                          &
                      i, i1, i2, j, j1, j2, k, nlon, nlat, nlevel, nt, nm, nvar, neof, nsing, &
                      lon1, lon2, lon3, lat1, lat2, lat3,                                     &
                      time1, time2, time3, level1, level2, level3,                            &
                      method, distance, rec, offset_rec, nlon_def, nlat_def, nlevel_def, alg, &
                      output_type, output_mode
    integer(i4b), dimension(4)                   :: coor_id2, startlec, countlec
    integer(i4b), dimension(nf90_max_var_dims)   :: dimids, dimlens, coor_natts, coor_type, coor_id
    integer(i1b)                                 :: kcode
!
    real(stnd), dimension(:,:,:),   allocatable  :: temp
    real(stnd), dimension(:,:),     allocatable  :: mat, mean, std, right_vec, left_vec
    real(stnd), dimension(:),       allocatable  :: sing_val, scal, d, e, tauq, taup
    real(stnd)                                   :: missing, safmin, abstol, tmp, tmp2, ssqmat, min_explvar
    real(extd), dimension(:),       allocatable  :: record
!
    logical(lgl)                                  :: failure, use_tlimited, use_x, use_y, use_z,  &
                                                     use_explvar, create_hdf5
    logical(lgl), dimension(nf90_max_var_dims)    :: use_coor_variable
    logical(lgl), dimension(:,:,:), allocatable   :: mask
!
    character(len=nchar)                       :: name, history
    character(len=nf90_max_name)               :: fic_name, var_name, clim_name, mask_name,     &
                                                  eof_name, att_name
    character(len=nf90_max_name), dimension(nf90_max_var_dims) :: dim_names
    character(len=14)                                          :: fmt
    character                                                  :: grid_type, chr
!
#ifdef _PARALLEL_READ
    integer(i4b)  :: ncid2, varid2
!
    logical  :: test_par
#endif
!
!
! EXECUTABLE STATEMENTS
! =====================
!
!   GET THE NUMBER OF COMMAND LINE ARGUMENTS.
!
    narg = iargc()
!
!   CHECK THE NUMBER OF COMMAND LINE ARGUMENTS.
!
    if ( narg==0 ) then
!
        write (prtunit,*)
        write (prtunit,*) ' Purpose : '
        write (prtunit,*)
        write (prtunit,*) ' Compute an EOF analysis from a fourdimensional variable '
        write (prtunit,*) ' extracted from a NetCDF dataset.'
        write (prtunit,*)
        write (prtunit,*) ' Usage : '
        write (prtunit,*)
        write (prtunit,*) ' comp_eof_4d  -f=input_netcdf_file'
        write (prtunit,*) '              -v=netcdf_variable'
        write (prtunit,*) '              -m=input_mesh_mask_netcdf_file'
        write (prtunit,*) '              -g=grid_type                         (optional : n, t, u, v, w, f)'
        write (prtunit,*) '              -r=resolution                        (optional : r2, r4)'
        write (prtunit,*) '              -b=nlon_orca, nlat_orca, nlevel_orca (optional)'
        write (prtunit,*) '              -x=lon1,lon2                         (optional)'
        write (prtunit,*) '              -y=lat1,lat2                         (optional)'
        write (prtunit,*) '              -z=level1,level2                     (optional)'
        write (prtunit,*) '              -t=time1,time2                       (optional)'
        write (prtunit,*) '              -a=type_of_analysis                  (optional : scp, cov, cor)'
        write (prtunit,*) '              -c=input_climatology_netcdf_file     (optional)'
        write (prtunit,*) '              -d=type_of_distance                  (optional : dist2, dist3, ident)'
        write (prtunit,*) '              -alg=algorithm                       (optional : inviter, svd, deflate)'
        write (prtunit,*) '              -n=number_of_eofs                    (optional)'
        write (prtunit,*) '              -o=output_eof_netcdf_file            (optional)'
        write (prtunit,*) '              -mi=missing_value                    (optional)'
        write (prtunit,*) '              -explvar                             (optional)'
        write (prtunit,*) '              -double                              (optional)'
#ifdef _USE_NETCDF36
        write (prtunit,*) '              -bigfile                             (optional)'
#endif
#ifdef _USE_NETCDF4
        write (prtunit,*) '              -hdf5                                (optional)'
#endif
        write (prtunit,*) '              -tlimited                            (optional)'
        write (prtunit,*)
        write (prtunit,*) ' By default : '
        write (prtunit,*)
        write (prtunit,*) '-g=n'
        write (prtunit,*) '-r=r2'
        write (prtunit,*) '-b= by default, nlon_orca, nlat_orca and nlevel_orca are determined'
        write (prtunit,*) '    from the -r= argument'
        write (prtunit,*) '-x= the whole longitude domain associated with the netcdf_variable'
        write (prtunit,*) '-y= the whole latitude domain associated with the netcdf_variable'
        write (prtunit,*) '-z= the whole vertical resolution associated with the netcdf_variable'
        write (prtunit,*) '-t= the whole time period associated with the netcdf_variable'
        write (prtunit,*) '-a=scp'
        write (prtunit,*) '-c= is not needed if -a=scp'
        write (prtunit,*) '-d=dist3'
        write (prtunit,*) '-alg=inviter'
        write (prtunit,*) '-n=10, a 10-component EOF model is stored'
        write (prtunit,*) '-o=eof_netcdf_variable.nc'
        write (prtunit,*) '-mi= the missing_value is equal to 1.e+20'
        write (prtunit,*) '-explvar  : by default, -n= option specifies number of EOFs'
        write (prtunit,*) '-double   : by default, the results are stored as single floating numbers'
#ifdef _USE_NETCDF36
        write (prtunit,*) '-bigfile  : by default, a NetCDF classical format file is created'
#endif
#ifdef _USE_NETCDF4
        write (prtunit,*) '-hdf5     : by default, a NetCDF classical format file is created'
#endif
        write (prtunit,*) '-tlimited : by default, the time dimension is defined as unlimited'
        write (prtunit,*)
#ifdef _OPENMP
        write (prtunit,*) 'This procedure is parallelized with OpenMP'
#endif
        write (prtunit,*)
!
        stop
!
    else if ( narg<3 .or. narg>22 ) then
!
        call merror( name_proc//"Wrong number of line arguments" )
!
    end if
!
    fic_name  = " "
    var_name  = " "
    mask_name = " "
    clim_name = " "
    eof_name  = " "
!
    history = "comp_eof_4d"
!
!   DEFAULT VALUES FOR COMMAND LINE ARGUMENTS.
!
    output_mode = nf90_clobber
    output_type = nf90_real4
    nlat_def    = nlat_def2
    nlon_def    = nlon_def2
    nlevel_def  = ndepth_def2
    method      = 1
    distance    = 2
    alg         = 1
    neof        = 10
!
    missing     = def_missing
!
    grid_type   = "n"
!
    lon1      = 1
    lon2      = huge( lon2 )
    lat1      = 1
    lat2      = huge( lat2 )
    level1    = 1
    level2    = huge( level2 )
    time1     = 1
    time2     = huge( time2 )
!
    use_x        = false
    use_y        = false
    use_z        = false
    use_explvar  = false
    create_hdf5  = false
    use_tlimited = false
!
!   GET THE COMMAND LINE ARGUMENTS.
!
    do i = 1, narg
!
        call getarg( i, name )
!
        history = trim(history)//" "//trim(name)
!
        call case_change( name(:3), tolower )
!
        select case ( name(1:3 ) )
!
            case ('-f=')
!
!               GET THE NAME OF THE NETCDF DATASET.
!
                fic_name = name(4:)
!
            case ('-v=')
!
!               GET THE NAME OF THE NETCDF VARIABLE IN THE DATASET.
!
                var_name = name(4:)
!
            case ('-g=')
!
!               GET THE GRID-TYPE.
!
                call case_change( name(4:), tolower )
!
                select case ( name(4:) )
                    case ( 'n', 'u', 'v', 'w', 't', 'f' )
                        grid_type = name(4:4)
                    case default
                        call merror( name_proc//"Unknown grid type" )
                end select
!
            case ('-r=')
!
!               DETERMINE THE RESOLUTION FOR THE ORCA GRID.
!
                call case_change( name(4:), tolower )
!
                select case ( name(4:) )
                    case ( cresolution(1) )
                        nlat_def   = nlat_def2
                        nlon_def   = nlon_def2
                        nlevel_def = ndepth_def2
                    case ( cresolution(2) )
                        nlat_def   = nlat_def4
                        nlon_def   = nlon_def4
                        nlevel_def = ndepth_def4
                    case default
                        call merror( name_proc//"Unknown resolution option" )
                end select
!
            case ('-b=')
!
!               GET THE DIMENSIONS OF THE ORCA GRID.
!
                call get_listarg( name_proc, name(4:),                                        &
                                  "nlon_orca, nlat_orca and nlevel_orca are integer values",  &
                                  "Wrong ORCA grid definition ",                              &
                                  nlon_def, nlat_def, nlevel_def )
!
            case ('-x=')
!
!               GET THE DOMAIN DEFINITION (Longitude).
!
                call get_listarg( name_proc, name(4:),                    &
                                  "lon1 and lon2 are integer values",     &
                                  "Wrong domain definition (longitude)",  &
                                  lon1, lon2 )
!
                use_x = true
!
            case ('-y=')
!
!               GET THE DOMAIN DEFINITION (latitude).
!
                call get_listarg( name_proc, name(4:),                    &
                                  "lat1 and lat2 are integer values",     &
                                  "Wrong domain definition (latitude)",   &
                                  lat1, lat2 )
!
                use_y = true
!
            case ('-z=')
!
!               GET THE LEVEL DEFINITION.
!
                call get_listarg( name_proc, name(4:),                    &
                                  "level1 and level2 are integer values", &
                                  "Wrong level definition",               &
                                  level1, level2 )
!
                use_z = true
!
            case ('-t=')
!
!               GET THE PERIOD DEFINITION.
!
                call get_listarg( name_proc, name(4:),                    &
                                  "time1 and time2 are integer values",   &
                                  "Wrong period definition",              &
                                  time1, time2 )
!
            case ('-m=')
!
!               GET THE NAME OF THE NETCDF MESH-MASK DATASET.
!
                mask_name = name(4:)
!
            case ('-c=')
!
!               GET THE NAME OF THE NETCDF CLIMATOLOGY DATASET.
!
                clim_name = name(4:)
!
            case ('-a=')
!
!               DETERMINE THE METHOD OF ANALYSIS.
!
                call case_change( name(4:), tolower )
!
                select case ( name(4:) )
                    case ( cmethod(1) )
                        method = 1
                    case ( cmethod(2) )
                        method = 2
                    case ( cmethod(3) )
                        method = 3
                    case default
                        call merror( name_proc//"Unknown analysis option" )
                end select
!
            case ('-d=')
!
!               DETERMINE THE DISTANCE FOR THE ANALYSIS.
!
                call case_change( name(4:), tolower )
!
                select case ( name(4:) )
                    case ( cdistance(1) )
                        distance = 1
                    case ( cdistance(2) )
                        distance = 2
                    case ( cdistance(3) )
                        distance = 3
                    case default
                        call merror( name_proc//"Unknown distance option" )
                end select
!
            case ('-o=')
!
!               GET THE NAME OF THE NEW NETCDF DATASET FOR STORING THE EOF ANALYSIS.
!
                eof_name = name(4:)
!
            case ('-n=')
!
!               GET THE NUMBER OF EOFs.
!
                call string_to_val( name(4:), kcode, fmt )
                if ( kcode==kint ) then
                    read( name(4:), fmt ) neof
                end if
                if ( kcode/=kint .or. neof<=0  )                                      &
          call merror( name_proc//"The number of EOFs or minimum variance is a positive integer value" )
!
            case default
!
                select case ( name(1:4 ) )
!
                    case ('-mi=')
!
!                       GET THE MISSING VALUE INDICATOR.
!
                        call string_to_val( name(5:), kcode, fmt )
                        if ( kcode/=kfix .and. kcode/=kexp )                &
                    call merror( name_proc//"The -mi= argument must be a real number " )
                        read( name(5:), fmt ) missing
!
                    case default
!
                        call case_change( name(4:), tolower )
!
                        select case ( name(1:5 ) )
!
                            case ('-alg=')
!
!                               DETERMINE THE ALGORITHM FOR THE COMPUTATION.
!
                                select case ( name(6:) )
                                    case ( calgorithm(1) )
                                        alg = 1
                                    case ( calgorithm(2) )
                                        alg = 2
                                    case ( calgorithm(3) )
                                        alg = 3
                                    case default
                                        call merror( name_proc//"Unknown algorithm option" )
                                end select
!
                            case default
!
                                select case ( name(1:) )
!
                                    case ('-explvar')
!
!                                       DEFINE EOFS NUMBER WITH REFERENCE TO 
!                                       MINIMUM OF EXPLAINED VARIANCE .
!
                                        use_explvar = true
!
                                    case ('-double')
!
!                                       STORE RESULTS AS DOUBLE PRECISION NUMBERS.
!
                                        output_type = nf90_real8
!
#ifdef _USE_NETCDF36
                                    case ('-bigfile')
!
!                                       REDEFINE THE CREATION MODE FLAG FOR THE OUTPUT FILE.
!
                                        output_mode = nf90_64bit_offset
#endif
!
#ifdef _USE_NETCDF4
                                    case ('-hdf5')
!
!                                       REDEFINE THE CREATION MODE FLAG FOR THE OUTPUT FILE.
!
                                        output_mode = nf90_hdf5
                                        create_hdf5 = true
#endif
!
                                    case ('-tlimited')
!
!                                       DEFINE THE TIME DIMENSION AS LIMITED.
!
                                        use_tlimited = true
!
                                    case default
!
!                                       WRONG ARGUMENT.
!
                                        call merror( name_proc//"Wrong argument" )
!
                                end select
!
                        end select
!
                end select
!
        end select
!
    end do
!
!   CHECK THE COMMAND LINE ARGUMENTS.
!
    if ( fic_name==" " )                                                 &
    call merror( name_proc//"Input NETCDF file is missing" )
!
    if ( var_name==" " )                                                 &
    call merror( name_proc//"Input NETCDF variable is missing" )
!
    if ( mask_name==" " )                                                &
    call merror( name_proc//"Input mesh-mask NETCDF file is missing" )
!
    if ( clim_name==" " .and. method>=2 )                                &
    call merror( name_proc//"Climatology NETCDF file is missing" )
!
    if ( use_explvar ) then
        if ( neof > 100 )                                  &
        call merror( name_proc//"with -explvar, the -n= argument must be less or equal to 100" )
        min_explvar = real( neof, stnd )
    endif
!
    if ( eof_name==" " ) eof_name="eof_"//trim(var_name)//".nc"
!
!   OPEN THE NETCDF DATASETS.
!
    k = nf90_sizehint_default
    status = nf90_open( trim(fic_name), nf90_nowrite, ncid, chunksize=k )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_open( trim(mask_name), nf90_nowrite, ncid_mask )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    if ( method>=2 ) then
        status = nf90_open( trim(clim_name), nf90_nowrite, ncid_clim )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
    end if
!
!   GET THE VARIABLE ID FROM ITS NAME.
!   FIND THE DIMENSION LENGTHS AND NAMES FOR THE NETCDF VARIABLE.
!   GET THE COORDINATE VARIABLES IDS FROM THEIR NAMES, IF ANY.
!   FIND THE EXTERNAL TYPE, THE DIMENSION ID AND THE NUMBER OF ATTRIBUTES 
!   FOR THE COORDINATE VARIABLES.
!
    call get_variable_dimcoords(  name_proc, trim(var_name), ncid,                    &
                                  varid,  dimids_out=dimids, dim_names_out=dim_names, &
                                  dimlens_out=dimlens, coor_id_out=coor_id,           &
                                  coor_natts_out=coor_natts, coor_type_out=coor_type, &
                                  use_coor_variable_out=use_coor_variable             )
!
    if ( count(dimlens/=0)/=4 )           &
    call merror( name_proc//"Input NETCDF variable is not fourdimensional" )
!
    nlon   = dimlens(1)
    nlat   = dimlens(2)
    nlevel = dimlens(3)
    nt     = dimlens(4)
!
    chr = merge( 't', grid_type, grid_type=='w' )
!
!   GET THE MASK ID FROM ITS NAME.
!   VERIFY THE DIMENSION LENGTHS FOR THE NETCDF MASK VARIABLE.
!
    call get_variable_dims( name_proc, chr//"mask", ncid_mask, maskid, dimids, dimlens )
!
    if ( dimlens(1)/=nlon .or. dimlens(2)/=nlat .or. dimlens(3)/=nlevel )    &
    call merror( name_proc//"Shape disagreement with the mask" )
!
!   GET THE BOUNDARIES OF THE GEOGRAPHICAL DOMAIN FROM THE MESH-MASK DATASET.
!
    if ( .not.use_x ) then
!
        status = nf90_get_att( ncid_mask, maskid, "lon1_Eastern_limit", i )
        if ( status==nf90_noerr ) lon1 = i
!
        status = nf90_get_att( ncid_mask, maskid, "lon2_Western_limit", i )
        if ( status==nf90_noerr ) lon2 = i
!
    end if
!
    if ( .not.use_y ) then
!
        status = nf90_get_att( ncid_mask, maskid, "lat1_Southern_limit", i )
        if ( status==nf90_noerr ) lat1 = i
!
        status = nf90_get_att( ncid_mask, maskid, "lat2_Northern_limit", i )
        if ( status==nf90_noerr ) lat2 = i
!
    end if
!
    if ( .not.use_z ) then
!
        status = nf90_get_att( ncid_mask, maskid, "level1_First_level", i )
        if ( status==nf90_noerr ) level1 = i
!
        status = nf90_get_att( ncid_mask, maskid, "level2_Last_level", i )
        if ( status==nf90_noerr ) level2 = i
!
    end if
!
!   CHECK THE BOUNDARIES OF THE GEOGRAPHICAL DOMAIN.
!   CHECK THE GEOGRAPHICAL DOMAIN.
!   REMOVE DUPLICATE POINTS FROM THE ORCA GRID.
!
    if ( grid_type/='n' ) then
!
        if ( nlon>nlon_def )                                                     &
        call merror( name_proc//"Input NETCDF variable is not on the ORCA GRID (longitude)" )
!
        if ( nlat>nlat_def )                                                     &
        call merror( name_proc//"Input NETCDF variable is not on the ORCA GRID (latitude)" )
!
        if ( nlevel>nlevel_def )    &
        call merror( name_proc//"Input NETCDF variable is not on the ORCA GRID (level)" )
!
!       A TRICK TO DISABLE THE ELIMINATION AND DUPLICATION IN THE HIGH LATITUDES OF THE ORCA GRID
!       IF THE RESOLUTION IS NOT R2 OR R4.
!
        if ( ( nlat_def/=nlat_def2 .or. nlon_def/=nlon_def2 ) .and. ( nlat_def/=nlat_def4 .or. nlon_def/=nlon_def4 ) )    &
        nlat_def = nlat_def + 1
!
    end if
!
    call check_lon_domain( grid_type, nlon, nlon_def, lon1, lon2, lon3 )
!
    if ( lon3<=0  )  call merror( name_proc//"Wrong domain definition (longitude)" )
!
    lat1 = max( lat1, 1 )
    lat2 = min( lat2, nlat )
    lat3 = lat2 - lat1 + 1
    if ( lat3<=0  )  call merror( name_proc//"Wrong domain definition (latitude)" )
!
!   CHECK THE LEVEL DEFINITION.
!
    level1 = max( level1, 1 )
    level2 = min( level2, nlevel )
    level3 = level2 - level1 + 1
    if ( level3<=0  ) call merror( name_proc//"Wrong vertical definition" )
!
!   CHECK THE PERIOD DEFINITION.
!
    time1 = max( time1, 1 )
    time2 = min( time2, nt )
    time3 = time2 - time1 + 1
    if ( time3<=0  ) call merror( name_proc//"Wrong time definition" )
!
!   ALLOCATE WORK ARRAYS.
!
    allocate( temp(nlon,nlat,nlevel), mask(nlon,nlat,nlevel), stat=status )
    if ( status/=0 ) call merror( name_proc//allocate_error )
!
!   GET THE MASK.
!
    status = nf90_get_var( ncid_mask, maskid, temp )     
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!   DETERMINE THE GRID POINTS USED IN THE ANALYSIS FROM THE MASK.
!
    mask(:nlon,:nlat,:nlevel) = temp(:nlon,:nlat,:nlevel)==one
!
!   SELECT THE DOMAIN.
!
    call select_grid_points_3d( grid_type, lon1, lon2, lat1, lat2, level1, level2, &
                                nlon_def, nlat_def, mask, j1, j2  )
!
    if ( lon1<0 ) lon3 = nlon
!
    nvar = count( mask )
    if ( nvar<=0  )  call merror( name_proc//"Wrong domain definition : domain is empty" )
!
!   ALLOCATE WORK ARRAY.
!
    allocate( mat(nvar,time3), stat=status )
    if ( status/=0 ) call merror( name_proc//allocate_error )
!
!   COMPUTE THE OFFSET FOR THE RECORD NUMBER.
!
    offset_rec = time1 - 1
!
    countlec(1:4)  = (/ lon3, lat3, level3, 1_i4b /)
!
!   COMPUTE MACHINE CONSTANTS.
!
    safmin = lamch('s')
    abstol = sqrt( safmin )
!
!   GET THE DATA.
!
#ifdef _PARALLEL_READ
!
    i = omp_get_num_procs()
    j = omp_get_max_threads()
    test_par = i>1 .and. j>1 .and. time3*nvar>omp_limit .and. time3>=j
!
    if ( test_par ) then
!
!       DEALLOCATE WORK ARRAY.
!
        deallocate( temp )
!
!$OMP PARALLEL PRIVATE(k,status,rec,startlec,ncid2,varid2,temp)                          &
!$OMP         ,SHARED(mat,mask,countlec,j1,j2,lon3,lat1,lat2,lat3,level1,level2,level3,  &
!$OMP                 nvar,offset_rec,time1,time2)
!
!       ALLOCATE WORK ARRAYS.
!
        allocate( temp(lon3,lat3,level3), stat=status )
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
!$OMP CRITICAL (opennc)
!
!       OPEN THE NETCDF DATASET.
!
        status = nf90_open( trim(fic_name), nf90_share, ncid2  )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!       GET THE VARIABLE ID FROM ITS NAME.
!
        status = nf90_inq_varid( ncid2, trim(var_name), varid2 )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!$OMP END CRITICAL (opennc)
!
        startlec(1)  = j1
        startlec(2)  = lat1
        startlec(3)  = level1
!
!$OMP DO SCHEDULE(STATIC)
!
        do k = time1, time2
!
            startlec(4) = k
!
!           FILL THE WORK ARRAY WITH A RECORD FROM THE NETCDF VARIABLE.
!
            status = nf90_get_var( ncid2, varid2, temp(:lon3,:lat3,:level3),          &
                                   start=startlec, count=countlec )     
            if ( status/=nf90_noerr )                                                &
            call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           PACK THE DATA.
!
            rec            = k - offset_rec
            mat(:nvar,rec) =                                                         &
            pack( temp(:lon3,:lat3,:level3), mask(j1:j2,lat1:lat2,level1:level2) )
!
        end do
!
!$OMP END DO
!
!       DEALLOCATE WORK ARRAY.
!
        deallocate( temp )
!
!$OMP CRITICAL (closenc)
!
        status = nf90_close( ncid2 )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!$OMP END CRITICAL (closenc)
!
!$OMP END PARALLEL
!
!       REALLOCATE WORK ARRAY.
!
        allocate( temp(nlon,nlat,nlevel), stat=status )
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
    else
!
#endif
!
        startlec(1:3)  = (/ j1, lat1, level1 /)
!
        do k = time1, time2
!
            startlec(4) = k
!
!           FILL THE WORK ARRAY WITH A RECORD FROM THE NETCDF VARIABLE.
!
            status = nf90_get_var( ncid, varid, temp(:lon3,:lat3,:level3),              &
                                   start=startlec, count=countlec )     
            if ( status/=nf90_noerr )                                                   &
            call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           PACK THE DATA.
!
            rec            = k - offset_rec
            mat(:nvar,rec) =                                                           &
            pack( temp(:lon3,:lat3,:level3), mask(j1:j2,lat1:lat2,level1:level2) )
!
        end do
!
#ifdef _PARALLEL_READ
!
    end if
!
#endif
!
!   SELECT THE METHOD OF ANALYSIS.
!
    select case ( method )
!
        case ( 1 )
!
!           EOF ANALYSIS IS DONE ON THE RAW DATA.
!
!
!           MULTIPLY THE NETCDF VARIABLE BY THE SCALE FACTOR IF REQUIRED.
!
            status = nf90_get_att( ncid, varid, "scale_factor", tmp )
            if ( status==nf90_noerr ) mat(:nvar,:time3) = mat(:nvar,:time3)*tmp
!
!           ADD OFFSET TO THE NETCDF VARIABLE IF REQUIRED.
!
            status = nf90_get_att( ncid, varid, "add_offset", tmp )
            if ( status==nf90_noerr ) mat(:nvar,:time3) = mat(:nvar,:time3) + tmp
!
        case ( 2, 3 )
!
!           EOF ANALYSIS IS DONE ON THE (STANDARDIZED) ANOMALIES.
!
!
!           GET THE MEAN FIELD ID FROM ITS NAME.
!           FIND THE DIMENSION LENGTHS FOR THE NETCDF MEAN VARIABLE.
!
            call get_variable_dims( name_proc, trim(var_name)//"_mean", ncid_clim,    &
                                    meanid, dimids, dimlens )
!
            if ( count(dimlens/=0)/=4 )     &
            call merror( name_proc//"Input climatology is not fourdimensional" )
!
            if ( dimlens(1)/=nlon .or. dimlens(2)/=nlat .or. dimlens(3)/=nlevel )    &
            call merror( name_proc//"Shape disagreement with the climatology (mean field)" )
!
            nm = dimlens(4)
!
            failure = false
!
            status = nf90_get_att( ncid_clim, nf90_global, "lon1_"//trim(var_name), i )
            if ( status==nf90_noerr ) failure = j1<i
!
            status = nf90_get_att( ncid_clim, nf90_global, "lon2_"//trim(var_name), i )
            if ( status==nf90_noerr .and. .not.failure ) failure = j2>i
!
            status = nf90_get_att( ncid_clim, nf90_global, "lat1_"//trim(var_name), i )
            if ( status==nf90_noerr .and. .not.failure ) failure = lat1<i
!
            status = nf90_get_att( ncid_clim, nf90_global, "lat2_"//trim(var_name), i )
            if ( status==nf90_noerr .and. .not.failure ) failure = lat2>i
!
            status = nf90_get_att( ncid_clim, nf90_global, "level1_"//trim(var_name), i )
            if ( status==nf90_noerr .and. .not.failure ) failure = level1<i
!
            status = nf90_get_att( ncid_clim, nf90_global, "level2_"//trim(var_name), i )
            if ( status==nf90_noerr .and. .not.failure ) failure = level2>i
!
            if ( failure ) call merror( name_proc//"Missing values in the climatology " )
!
!           ALLOCATE WORK ARRAY FOR THE MEANS.
!
            allocate( mean(nvar,nm), stat=status )
            if ( status/=0 ) call merror( name_proc//allocate_error )
!
            startlec(1:3)  = (/ j1, lat1, level1 /)
!
!           GET THE MEANS.
!
            do j = 1, nm
!
                startlec(4)  = j
!
                status = nf90_get_var( ncid_clim, meanid, temp(:lon3,:lat3,:level3),        &   
                                       start=startlec, count=countlec)     
                if ( status/=nf90_noerr )                                                   &
                call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!               PACK THE MEAN FIELD.
!
                mean(:nvar,j) =                                                           &
                pack( temp(:lon3,:lat3,:level3), mask(j1:j2,lat1:lat2,level1:level2) )
            end do
!
            if ( method==2 ) then
!
!               EOF ANALYSIS IS DONE ON THE ANOMALIES.
!
                k = ( time3/nm )*nm
!
!               COMPUTE ANOMALIES.
!
                do i = 0, k-1, nm
!
                    i1 = i + 1
                    i2 = i + nm
!
                    mat(:nvar,i1:i2) = mat(:nvar,i1:i2) - mean(:nvar,:nm)
!
                end do
!
                if ( k/=time3 )                                    &
                mat(:nvar,k+1:time3) = mat(:nvar,k+1:time3) - mean(:nvar,1:time3-k)
!
!               DEALLOCATE WORK ARRAY.
!
                deallocate( mean )
!
!               MULTIPLY THE NETCDF VARIABLE BY THE SCALE FACTOR IF REQUIRED.
!
                status = nf90_get_att( ncid, varid, "scale_factor", tmp )
                if ( status==nf90_noerr ) mat(:nvar,:time3) = mat(:nvar,:time3)*tmp
!
            else
!
!               EOF ANALYSIS IS DONE ON THE STANDARDIZED ANOMALIES.
!
!               GET THE STD ID FROM ITS NAME.
!               FIND THE DIMENSION LENGTHS FOR THE NETCDF STD VARIABLE.
!
                call get_variable_dims( name_proc, trim(var_name)//"_std", ncid_clim,    &
                                        stdid, dimids, dimlens )
!
                if ( count(dimlens/=0)/=4 )          &
                call merror( name_proc//"Input climatology is not fourdimensional" )
!
                if ( dimlens(1)/=nlon .or. dimlens(2)/=nlat .or. dimlens(3)/=nlevel )          &
                call merror( name_proc//"Shape disagreement with the climatology (std field)" )
!
                if ( dimlens(4)/=nm )                                                          &
                call merror( name_proc//"Shape disagreement in the climatology" )
!
!               ALLOCATE WORK ARRAYS.
!
                allocate( std(nvar,nm), stat=status )
                if ( status/=0 ) call merror( name_proc//allocate_error )
!
!               GET THE STDS.
!
                do j = 1, nm
!
                    startlec(4)  = j
!
                    status = nf90_get_var( ncid_clim, stdid, temp(:lon3,:lat3,:level3),            &
                                           start=startlec, count=countlec)     
                    if ( status/=nf90_noerr )                                                      &
                    call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!                   PACK THE STD FIELD.
!
                    std(:nvar,j) =                                                                &
                    pack( temp(:lon3,:lat3,:level3), mask(j1:j2,lat1:lat2,level1:level2) )
!
                end do
!
                where ( std(:nvar,:nm)>safmin )
                    std(:nvar,:nm) = one/std(:nvar,:nm)
                elsewhere
                    std(:nvar,:nm) = zero
                end where
!
                k = ( time3/nm )*nm
!
!               COMPUTE STANDARDIZED ANOMALIES.
!
                do i = 0, k-1, nm
!
                    i1 = i + 1
                    i2 = i + nm
!
                    mat(:nvar,i1:i2) = ( mat(:nvar,i1:i2) - mean(:nvar,:nm) )*std(:nvar,:nm)
!
                end do
!
                if ( k/=time3 ) mat(:nvar,k+1:time3) =                                  &
                ( mat(:nvar,k+1:time3) - mean(:nvar,:time3-k) )*std(:nvar,:time3-k)
!
!               DEALLOCATE WORK ARRAYS.
!
                deallocate( mean, std )
!
            end if
!
    end select
!
!   COMPUTE THE DISTANCE.
!
    if ( distance<=2 ) then
!
!       THE DISTANCE IS ASSOCIATED WITH THE (HORIZONTAL) GRID.
!
!
!       ALLOCATE WORK ARRAYS.
!
        allocate( scal(nvar), right_vec(lon3,lat3), stat=status )
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
!       GET THE FIRST AND SECOND SCALE FACTORS IDs FROM THEIR NAMES.
!       VERIFY THE DIMENSION LENGTHS FOR THE SCALE FACTOR VARIABLES.
!
        call get_variable_dims( name_proc, "e1"//chr, ncid_mask,    &
                                e1id, dimids, dimlens )
!
        if ( dimlens(1)/=nlon .or. dimlens(2)/=nlat )               &
        call merror( name_proc//"Shape disagreement with the first scale factor" )
!
        call get_variable_dims( name_proc, "e2"//chr, ncid_mask,    &
                                e2id, dimids, dimlens )
!
        if ( dimlens(1)/=nlon .or. dimlens(2)/=nlat )               &
        call merror( name_proc//"Shape disagreement with the second scale factor" )
!
!       GET THE FIRST AND SECOND SCALE FACTORS.
!
        status = nf90_get_var( ncid_mask, e1id, right_vec, start=(/j1,lat1/),  &     
                               count=(/ lon3, lat3 /)  )     
        if ( status/=nf90_noerr )                                              &
        call merror( name_proc//trim( nf90_strerror( status ) ) )
        temp(:lon3,:lat3,1) = right_vec(:lon3,:lat3)
!
        status = nf90_get_var( ncid_mask, e2id, right_vec, start=(/j1,lat1/),  &     
                               count=(/ lon3, lat3 /)  )     
        if ( status/=nf90_noerr )                                              &
        call merror( name_proc//trim( nf90_strerror( status ) ) )
        right_vec(:lon3,:lat3) = right_vec(:lon3,:lat3)*temp(:lon3,:lat3,1)

        temp(:lon3,:lat3,:level3) = spread( right_vec(:lon3,:lat3), dim=3, ncopies=level3 )
        scal(:nvar) = pack( temp(:lon3,:lat3,:level3), mask(j1:j2,lat1:lat2,level1:level2) )
!
        if ( distance==2 ) then
!
!           ALLOCATE WORK ARRAY.
!
            allocate( d(level3), stat=status )
            if ( status/=0 ) call merror( name_proc//allocate_error )
!
!           THE DISTANCE IS ASSOCIATED WITH THE WHOLE 3D GRID.
!
            chr = merge( grid_type, 't', grid_type=='w' .or. grid_type=='n' )
!
!           GET THE THIRD SCALE FACTOR ID FROM ITS NAME.
!           VERIFY THE DIMENSION LENGTHS FOR THE THIRD SCALE FACTOR VARIABLE.
!
            call get_variable_dims( name_proc, "e3"//chr, ncid_mask,    &
                                    e3id, dimids, dimlens )
!
            if ( dimlens(1)/=1 .or. dimlens(2)/=1 .or. dimlens(3)/=nlevel )             &
            call merror( name_proc//"Shape disagreement with the third scale factor" )
!
!           GET THE THIRD SCALE FACTOR VARIABLE.
!
            status = nf90_get_var( ncid_mask, e3id, d,                             &
                                   start=(/1_i4b,1_i4b,level1/), count=(/1_i4b,1_i4b,level3/) )    
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           CHECK THE THIRD SCALE FACTOR.
!
            if ( d(1)<zero )                        &
    call merror( name_proc//" First level with -d=dist3 are not allowed for mesh-mask "//trim(mask_name) )
!
            if ( d(level3)<zero )                   &
    call merror( name_proc//" Last level with -d=dist3 are not allowed for mesh-mask "//trim(mask_name) )
!
            temp(:lon3,:lat3,:level3) =                                                   &
            spread( spread( d(:level3), dim=1, ncopies=lat3), dim=1, ncopies=lon3 )
            scal(:nvar) =                                                               &
            scal(:nvar)*pack( temp(:lon3,:lat3,:level3), mask(j1:j2,lat1:lat2,level1:level2) )
!
!           DEALLOCATE WORK ARRAY.
!
            deallocate( d )
!
        end if
!
!       COMPUTE THE DIAGONAL METRIC ASSOCIATED WITH THE (HORIZONTAL) GRID.
!
        scal(:nvar) = sqrt( scal(:nvar)*(one/maxval(scal(:nvar))) )
!        scal(:nvar) = sqrt( scal(:nvar) )
!
!       DEALLOCATE WORK ARRAYS.
!
        deallocate( right_vec, temp )
!
!       SCALE THE NETCDF VARIABLE.
!
        mat(:nvar,:time3) = mat(:nvar,:time3)*spread(scal(:nvar),dim=2,ncopies=time3 ) 
!
    else
!
!       DEALLOCATE WORK ARRAY.
!
        deallocate( temp )
!
    end if
!
    k = min( nvar, time3 )
!
    if ( alg==1 .or. alg==2 ) then
!
!       COMPUTE A PARTIAL SVD OF mat USING THE GOLUB-REINSCH
!       BIDIAGONAL REDUCTION ALGORITHM, A BISECTION ALGORITHM FOR
!       SINGULAR VALUES AND THE INVERSE ITERATION TECHNIQUE OR A
!       DEFLATION TECHNIQUE FOR SINGULAR VECTORS.
!
!
!       ALLOCATE WORK ARRAYS FOR THE PARTIAL SVD ANALYSIS.
!
        allocate(  sing_val(k), d(k), e(k), tauq(k), taup(k), stat=status )
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
        if ( use_explvar ) then
!
!           COMPUTE ALL SINGULAR VALUES OF mat AND STORE INTERMEDIATE RESULTS
!           WITH SUBROUTINE svd_cmp :
!               THE INTERMEDIATE BIDIAGONAL FORM OF mat IS STORED IN d AND e.
!               THE ORTHOGONAL MATRICES USED TO REDUCE mat TO BIDIAGONAL FORM d_e
!               ARE STORED IN mat, tauq, taup.
!               THE SINGULAR VALUES ARE STORED IN sing_val IN DECREASING ORDER (sort='d').
!
            call svd_cmp( mat, sing_val, failure, sort='d', d=d, e=e, tauq=tauq, taup=taup  )
!
            if ( .not.failure ) then
!
!               COMPUTE MEAN SQUARE VARIANCE FROM THE SINGULAR VALUES.
!
                ssqmat = sum( sing_val(:k)**2 )
!
!               DETERMINE THE NUMBER OF EOFS IF REQUIRED.
!
                tmp  = zero
                tmp2 = c100/ssqmat
!
                do neof= 1, k
!
                    tmp = tmp + tmp2*sing_val(neof)**2
                    if ( tmp>=min_explvar ) exit
!
                end do
!
!               CHECK THE neof VALUE.
!
                neof = min( neof, k )
!
            end if
!
        else
!
!           COMPUTE FIRST THE MEAN SQUARE VARIANCE.
!
            ssqmat = norm( mat(:nvar,:time3) )**2
!
!           COMPUTE SINGULAR VALUES OF mat AND STORE INTERMEDIATE RESULTS
!           WITH SUBROUTINE select_singval_cmp2 :
!           THE INTERMEDIATE BIDIAGONAL FORM OF mat IS STORED IN d AND e.
!           THE ORTHOGONAL MATRICES USED TO REDUCE mat TO BIDIAGONAL FORM d_e
!           ARE STORED IN mat, tauq, taup.
!           THE SINGULAR VALUES ARE STORED IN sing_val IN DECREASING ORDER (sort='d').
!
            call select_singval_cmp2( mat, nsing, sing_val, failure=failure, sort='d', d=d, e=e, &
                                      ls=neof, abstol=abstol, tauq=tauq, taup=taup )
!
!           RESET THE neof VALUE IN CASE OF MULTIPLE SINGULAR VALUES.
!
            neof = nsing
!
        end if
!
        if ( failure ) then
!
            call merror(  name_proc//"Error in the partial SVD analysis (bidiagonal algorithm and singular values)" )
!
        end if
!
!       ALLOCATE WORK ARRAYS FOR THE PARTIAL SVD ANALYSIS.
!
        allocate( left_vec(nvar,neof), right_vec(time3,neof), stat=status )
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
        if ( alg==1 ) then
!
!           COMPUTE THE FIRST neof LEFT AND RIGHT SINGULAR VECTORS OF mat BY maxiter
!           INVERSE ITERATIONS ON THE INTERMEDIATE BIDIAGONAL MATRIX
!           d_e AND BACK-TRANSFORMATION WITH SUBROUTINE bd_inviter2 .
!
            call bd_inviter2( mat, tauq, taup, d, e, sing_val(:neof), left_vec,    &
                              right_vec, failure, maxiter=maxiter  )
!
            if ( failure ) call merror( name_proc//&
            &"Error in the partial SVD analysis (inviter algorithm)" )
!
        else
!
!           COMPUTE THE FIRST neof LEFT AND RIGHT SINGULAR VECTORS OF mat BY A
!           DEFLATION TECHNIQUE ON THE INTERMEDIATE BIDIAGONAL MATRIX
!           d_e AND BACK-TRANSFORMATION WITH SUBROUTINE bd_deflate2.
!
            call bd_deflate2( mat, tauq, taup, d, e, sing_val(:neof),            &
                              left_vec, right_vec, failure=failure, ortho=ortho  )
!
            if ( failure ) call merror( name_proc//&
            &"Error in the partial SVD analysis (deflate algorithm)" )
!
        end if
!
!       SAVE THE LEFT SINGULAR VECTORS IN mat.
!
        mat(:nvar,:neof) = left_vec(:nvar,:neof)
!
!       DEALLOCATE UNUSED WORK ARRAYS.
!
        deallocate( d, e, tauq, taup, left_vec )
!
    else
!
!       COMPUTE THE FULL SVD OF mat.
!
!
!       ALLOCATE WORK ARRAYS FOR THE SVD ANALYSIS.
!
        allocate(  sing_val(k), right_vec(time3,k), stat=status )
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
!       COMPUTE THE EOF ANALYSIS BY THE SVD OF THE DATA MATRIX.
!
        call  svd_cmp( mat, sing_val, failure, right_vec, sort='d'  )
        if ( failure ) call merror(  name_proc//"Error in the SVD analysis" )
!
!       COMPUTE MEAN SQUARE VARIANCE FROM THE SINGULAR VALUES.
!
        ssqmat = sum( sing_val(:k)**2 )
!
!       DETERMINE THE NUMBER OF EOFS TO STORE IF REQUIRED.
!
        if ( use_explvar ) then
!
            tmp  = zero
            tmp2 = c100/ssqmat
!
            do neof= 1, k
!
                tmp = tmp + tmp2*sing_val(neof)**2
                if ( tmp>=min_explvar ) exit
!
            end do
!
        end if
!
!       CHECK THE neof VALUE.
!
        neof = min( neof, k )
!
    end if
!
!   CREATE THE NEW NETCDF DATASET FOR THE EOF ANALYSIS.
!
    if ( create_hdf5 ) then
        status = nf90_create( eof_name, cmode=output_mode, ncid=ncid_eof )
    else
        i = nf90_sizehint_default
        status = nf90_create( eof_name, cmode=output_mode, ncid=ncid_eof, chunksize=i )
    end if
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_set_fill( ncid_eof, nf90_nofill, i )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!   DEFINE THE DIMENSIONS FOR THE NEW NETCDF DATASET.
!
    dimlens(1) = nlon
    dimlens(2) = nlat
    dimlens(3) = nlevel
    dimlens(4) = neof
    if ( use_tlimited ) then
        dimlens(5)   = time3
    else
        dimlens(5)   = nf90_unlimited
    end if
    dim_names(5) = dim_names(4)
    dim_names(4) = "pc"
!
    do i = 1, 5
!
        status = nf90_def_dim( ncid_eof, name=trim(dim_names(i)),                     &
                               len=dimlens(i), dimid=dimids(i) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end do
!
!   DEFINE THE VARIABLES FOR THE NEW NETCDF DATASET.
!
    do i = 1, 3
!
        status = nf90_def_var( ncid_eof, trim(dim_names(i)), coor_type(i),                &
                               dimids=dimids(i), varid=coor_id2(i) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end do
!
    status = nf90_def_var( ncid_eof, trim(dim_names(5)), coor_type(4),                    &
                           dimids=dimids(5), varid=coor_id2(4) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_def_var( ncid_eof, trim(var_name)//"_eof", output_type,                 &
                           dimids=dimids(1:4), varid=eofid )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_def_var( ncid_eof, trim(var_name)//"_pc", output_type,                   &
                           dimids=dimids(4:5), varid=pcid )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_def_var( ncid_eof, trim(var_name)//"_sing", nf90_real8,                  &
                           dimids=dimids(4), varid=singid )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_def_var( ncid_eof, trim(var_name)//"_var", output_type,                  &
                           dimids=dimids(4), varid=expvarid )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!   ASSIGN ATTRIBUTES FOR THE NEW NETCDF DATASET.
!
    do j = 1, 4
!
        if ( .not.use_coor_variable(j) ) cycle
!
        do i = 1, coor_natts(j)
!
            status = nf90_inq_attname( ncid, coor_id(j), i, att_name )
            if ( status/=nf90_noerr )                                                     &
            call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           COPY ATTRIBUTES FOR THE COORDINATE VARIABLES.
!
            status = nf90_copy_att(  ncid, coor_id(j), att_name, ncid_eof, coor_id2(j) )
            if ( status/=nf90_noerr )                                                     &
            call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end do
!
    end do
!
    if ( .not.use_coor_variable(4) ) then
!
        status = nf90_put_att( ncid_eof, coor_id2(4), "long_name", "Time axis" )
        if ( status/=nf90_noerr )                                                  &
        call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_eof, coor_id2(4), "short_name", "Time" )
        if ( status/=nf90_noerr )                                                  &
        call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end if
!
    status = nf90_put_att( ncid_eof, eofid, "long_name", "Empirical Orthogonal Functions" )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_eof, eofid, "short_name", "EOFs" )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    if (  output_type==nf90_real8 ) then
!
        status = nf90_put_att( ncid_eof, eofid, "missing_value", real( missing, dp ) )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_eof, eofid, "_FillValue", real( missing, dp ) )
!
    else
!
        status = nf90_put_att( ncid_eof, eofid, "missing_value", real( missing, sp ) )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_eof, eofid, "_FillValue", real( missing, sp ) )
!
    end if
!
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_eof, pcid, "long_name", "Principal Components" )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_eof, pcid, "short_name", "PCs" )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_eof, singid, "long_name", "Singular Values" )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_eof, singid, "short_name", "SingVals" )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_eof, expvarid, "long_name", "Explained Variances" )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_eof, expvarid, "short_name", "ExpVars" )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!   SET GLOBAL ATTRIBUTES.
!
    status = nf90_put_att( ncid_eof, nf90_global, "title", trim(var_name)//" EOF analysis" )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_eof, nf90_global, "distance", cdistance(distance) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_eof, nf90_global, "method", cmethod(method) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_eof, nf90_global, "algorithm", calgorithm(alg) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_eof, nf90_global, "grid_type", grid_type )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_eof, nf90_global, "history", trim(history) )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!   LEAVE DEFINE MODE FOR THE NEW NETCDF DATASET.
!
    status = nf90_enddef( ncid_eof )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!   WRITE VARIABLES FOR THE NEW  NETCDF DATASET.
!
    do j = 1, 4        
!
        select case ( j )
            case ( 1 )
                j1 = 1
                j2 = nlon
            case ( 2 )
                j1 = 1
                j2 = nlat
            case ( 3 )
                j1 = 1
                j2 = nlevel
            case ( 4 )
                j1 = time1
                j2 = time3
        end select
!
        if ( use_coor_variable(j) ) then
            allocate( record(j2), stat=status )
            if ( status/=0 ) call merror( name_proc//allocate_error )
!
            status = nf90_get_var( ncid, coor_id(j), record, start=(/ j1 /), count=(/ j2 /) )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_var( ncid_eof, coor_id2(j), record )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
            deallocate( record )
        else
            status = nf90_put_var( ncid_eof, coor_id2(j), arth(j1,1_i4b,j2 ) )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
        end if
!
    end do
!
    if ( distance<=2 ) then
!
!       THE DISTANCE IS ASSOCIATED WITH THE (HORIZONTAL) GRID.
!
        scal(:nvar) = one/scal(:nvar)
!
!       UNSCALE THE EOFs.
!
        mat(:nvar,:neof) = mat(:nvar,:neof)*spread(scal(:nvar),dim=2,ncopies=neof ) 
!
!       DEALLOCATE WORK ARRAY.
!
        deallocate( scal )
!
    end if
!
    tmp = sqrt( real(time3, stnd) )
    sing_val(:k) = sing_val(:k)/tmp
!
    startlec(1:3) = 1
!
!   ALLOCATE WORK ARRAY.
!
    allocate( temp(nlon,nlat,nlevel), stat=status )
    if ( status/=0 ) call merror( name_proc//allocate_error )
!
!   WRITE EOFS.
!
    do i = 1, neof
!
        mat(:nvar,i) = mat(:nvar,i)*sing_val(i)
        temp(:nlon,:nlat,:nlevel) = unpack( mat(:nvar,i), mask, missing )
!
        if ( grid_type/='n' .and. nlon==nlon_def ) then
!
            if ( lat2>=nlat_def-1 .and. nlat==nlat_def ) then
!
!               ADD DUPLICATE POINTS TO THE ORCA GRID IF THE WHOLE GLOBE IS USED.
!
                call duplicate_orca_points_3d( grid_type, mask, temp )
!
            else
!
!               ADD DUPLICATE LONGITUDES LINES TO THE ORCA GRID IF THE WHOLE LONGITUDE
!               DOMAIN IS USED.
!
                temp(1,:nlat,:nlevel) = temp(nlon-1,:nlat,:nlevel)
                temp(nlon,:nlat,:nlevel) = temp(2,:nlat,:nlevel)
!
            end if
!
        end if
!
        startlec(4) = i
!
        status = nf90_put_var( ncid_eof, eofid, temp, start=startlec )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end do
!
!   WRITE EOF AMPLITUDE TIME SERIES.
!
    right_vec(:time3,:neof) = right_vec(:time3,:neof)*tmp
!
    status = nf90_put_var( ncid_eof, pcid, transpose(right_vec(:time3,:neof)) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!   WRITE SINGULAR VALUES.
!
    status = nf90_put_var( ncid_eof, singid, sing_val(:neof) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!   WRITE EXPLAINED VARIANCES.
!
    sing_val(:neof) = ( (sing_val(:neof)**2)*real(time3,stnd) )/ssqmat
!
    status = nf90_put_var( ncid_eof, expvarid, sing_val(:neof) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!   DEALLOCATE WORK ARRAYS.
!
    deallocate( temp, mask, mat, right_vec, sing_val )
!
!   CLOSE THE NETCDF DATASETS.
!
    status = nf90_close( ncid )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_close( ncid_mask )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    if ( method>=2 ) then
        status = nf90_close( ncid_clim )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
    end if
!
    status = nf90_close( ncid_eof )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!
! END OF PROGRAM comp_eof_4d
! ==========================
!
end program comp_eof_4d
