program comp_project_eof_4d
!! #############################################################################
!!
!! Copyright 2018 IRD
!!
!! This file is part of NCSTAT.
!!
!! NCSTAT is free software: you can redistribute it and/or modify
!! it under the terms of the GNU Lesser General Public License as 
!! published by the Free Software Foundation, either version 3 of 
!! the License, or (at your option) any later version.
!!
!! NCSTAT is distributed in the hope that it will be useful,
!! but WITHOUT ANY WARRANTY; without even the implied warranty of
!! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!! GNU Lesser General Public License for more details.
!!
!! You can find a copy of the GNU Lesser General Public License
!! in the NCSTAT/doc directory.
!!
!! #############################################################################
!
!+
!
! .. _comp_project_eof_4d:
!
! ===================
! comp_project_eof_4d
! ===================
!
! Authors
! =======
!
! Pascal Terray (LOCEAN/IPSL) and Eric Maisonnave (CERFACS)
!
! Latest revision
! ===============
!
! 13/09/2018
!
! Purpose
! =======
!
! Project a fourdimensional NetCDF variable (or parts of it) extracted from a NetCDF
! dataset onto eigenvectors or singular vectors computed from a previous Empirical
! Orthogonal Function (EOF) or Singular Value Decomposition (SVD) analysis.
!
! Using as input, an EOF (or SVD) NetCDF file produced by :ref:`comp_eof_4d`
! or :ref:`comp_svd_3d`, this procedure computes the projection of a given fourdimensional
! variable extracted from another NetCDF dataset onto the orthonormal basis formed by the
! eigenvectors or singular vectors of the EOF (or SVD) analysis.
!
! The procedure first transforms the selected time steps of the input fourdimensional NetCDF variable
! as a *ntime* by *nv* rectangular matrix, **X**\ , of observed variables (e.g. the
! selected cells of the 3-D grid-mesh associated with the fourdimensional NetCDF variable),
! does the same repacking operation for the selected eigenvectors or singular vectors (which
! must have been computed on the same selected cells of the 3-D grid-mesh associated with the input
! fourdimensional NetCDF variable) and then computes the projections of the selected time steps
! onto the selected eigenvectors (or singular vectors) by performing the following matrix product
!
!           **A** = **X**\ .transpose(**B**\ )
!
! where
!
!  * **A** is the *ntime* by *k* matrix of *k* selected principal components (or singular variables)
!    time series to be computed
!  * **B** is the *k* by *nv* matrix of the *k* eigenvectors or singular vectors (stored rowwise) 
!    readed from an input NetCDF file produced by :ref:`comp_eof_4d` or :ref:`comp_svd_3d`.
!
! If the NetCDF variable is tridimensional use :ref:`comp_project_eof_3d` instead of comp_project_eof_4d.
!
! An output NetCDF dataset containing the expansion coefficients for the selected time steps of
! the principal components (or singular variables) time series is created.
!
! This procedure is parallelized if OpenMP is used.
!
! Further Details
! ===============
!
! Usage
! -----
!
! ::
!
!       $ comp_project_eof_4d \
!         -f=input_netcdf_file \
!         -v=netcdf_variable \
!         -fe=eof_netcdf_file \
!         -ve=eof_netcdf_variable \
!         -m=input_mesh_mask_netcdf_file \
!         -se=selected_eof                  (optional) \
!         -g=grid_type                      (optional : n, t, u, v, w, f) \
!         -r=resolution                     (optional : r2, r4) \
!         -b=nlon_orca, nlat_orca           (optional) \
!         -x=lon1,lon2                      (optional) \
!         -y=lat1,lat2                      (optional) \
!         -z=level1,level2                  (optional) \
!         -t=time1,time2                    (optional) \
!         -l=selected_time_period           (optional) \
!         -a=type_of_analysis               (optional : scp, cov, cor) \
!         -c=input_climatology_netcdf_file  (optional) \
!         -d=type_of_distance               (optional : dist2, dist3, ident) \
!         -o=output_pc_netcdf_file          (optional) \
!         -normpc                           (optional) \
!         -svd                              (optional) \
!         -double                           (optional) \
!         -bigfile                          (optional) \
!         -hdf5                             (optional) \
!         -tlimited                         (optional)
!
! By default
! ----------
!
!        *-se=*
!         all the eigenvectors or singular vectors stored in the *eof_netcdf_file*
!        *-g=*
!         the *grid_type* is set to ``n`` which means that the 3-D grid-mesh associated with
!         the input NetCDF variable is assumed to be regular or Gaussian
!        *-r=*
!         if the input netcdf_variable is from the NEMO or ORCA model (e.g. if **-g=** argument
!         is not set to ``n``) the resolution is assumed to be ``r2``
!        *-b=*
!         if **-n=** is not set to ``n``\ , the dimensions of the 3-D grid-mesh, *nlon_orca*, 
!         *nlat_orca* and *nlevel_orca* are determined from the **-r=** argument. However, you
!         may override this choice by default with the **-b=** argument
!        *-x=*
!         the whole longitude domain associated with the *netcdf_variable*
!        *-y=*
!         the whole latitude domain associated with the *netcdf_variable*
!        *-z=*
!         the whole vertical resolution associated with the *netcdf_variable*
!        *-t=*
!         the whole time period associated with the *netcdf_variable*
!        *-l=*
!         all the time steps from *time1* to *time2* as specified in the **-t=** argument
!        *-a=*
!         the *type_of_analysis* is set to ``scp``. This means that the projection onto
!         the eigenvectors is done on the raw data without centering or standardizing the
!         input time series
!        *-c=*
!         an *input_climatology_netcdf_file* is not needed if
!         the *type_of_analysis* is set to ``scp``
!        *-d=*
!         the *type_of_distance* is set to ``dist3``. This means that the scalar products
!         for computing the projections are computed with the diagonal metric associated with
!         the 3-D grid-mesh associated with the input NetCDF variable
!        *-o=*
!         the *output_pc_netcdf_file* is named ``proj_``\ *netcdf_variable*\ ``.nc``
!        *-normpc*
!         the computed PC (or SV) time series are not normalized in the output NetCDF file.
!         If **-normpc** is activated, the computed PC (or SV) time series are normalized 
!         in the output NetCDF file
!        *-svd*
!         the *eof_netcdf_file* is assumed to be produced by :ref:`comp_eof_4d` or
!         :ref:`comp_eof_miss_3d`\ . If **-svd** is activated, a file produced by
!         :ref:`comp_svd_3d` is assumed, this means that the projection is done onto
!         singular vectors of a previous SVD analysis.
!        *-double*
!         the results of the projection analysis are stored as single-precision floating
!         point numbers in the output NetCDF file. If **-double** is activated, the
!         results are stored as double-precision floating point numbers 
!        *-bigfile*
!         a NetCDF classical format file is created. If **-bigfile** is activated,
!         the output NetCDF file is a 64-bit offset format file
!        *-hdf5*
!         a NetCDF classical format file is created. If **-hdf5** is activated,
!         the output NetCDF file is a NetCDF-4/HDF5 format file
!        *-tlimited*
!         the time dimension is defined as unlimited in the output NetCDF file.
!         However, if **-tlimited** is activated, the time dimension is defined
!         as limited in the output NetCDF file
!
! Remarks
! -------
!
!   #) The **-v=**\ *netcdf_variable* argument specifies the NetCDF variable for which a
!      projection analysis must be computed and the **-f=**\ *input_netcdf_file* argument
!      specifies that this NetCDF variable must be extracted from the NetCDF file,
!      *input_netcdf_file*.
!
!   #) The **-ve=**\ *eof_netcdf_variable* argument specifies the NetCDF variable for which an
!      EOF (or SVD) analysis was originally computed by :ref:`comp_eof_4d` (or :ref:`comp_svd_3d`\ )
!      and the **-fe=**\ *input_eof_netcdf_file* argument specifies that the resulting EOF (or SVD)
!      patterns must be extracted from the NetCDF file, *eof_netcdf_file*\ . 
!      This NetCDF file must have exactly the same format as the files produced
!      by :ref:`comp_eof_4d` or :ref:`comp_svd_3d`\ .
!      These EOF (or SVD) patterns will be used to compute the projections of the *netcdf_variable*
!      specified by the **-v=** argument.
!
!   #) The argument **-m=**\ *input_mesh_mask_netcdf_file* specifies the land-sea mask
!      to apply to the *netcdf_variable* for transforming this fourdimensional NetCDF variable as a
!      rectangular matrix before computing the EOF (or SVD) projection. The same land-sea mask is
!      assumed and apply to the *eof_netcdf_variable* which contains the eigenvectors of the EOF
!      (or SVD) analysis.
!
!      The scale factors associated with the 2-D grid-mesh of these NetCDF variables (needed if
!      **-d=**\ ``dist2``\  is specified when calling the procedure) are also read from the
!      *input_mesh_mask_netcdf_file*\ .
!
!   #) The **-se=** argument allows the user to select the eigenvectors (or singular vectors)
!      which must be included in the projection analysis. The list of selected vectors may
!      be given in two formats:
!
!      * **-se=**\ *1,3,...,nn* allows to include *eof1,eof3,...* and *eofnn* in the EOF (or SVD) projection
!
!      * **-se=**\ ``1:4`` allows to include from *eof1* to *eof4* in the EOF (or SVD) projection.
!
!      The two forms of the **-se=** argument may be combined and repeated any number of times.
!      Duplicate EOF or SVD numbers are not allowed.
!      If the **-se=** argument is not specified, all the eigenvectors (or singular vectors)
!      stored in the *input_eof_netcdf_file* are used in the projection analysis.
!
!   #) If **-g=** is set to ``t``, ``u``, ``v``, ``w`` or ``f`` it is assumed that the
!      NetCDF variables are from an experiment with the NEMO or ORCA model.
!      In this case, the duplicate points from the ORCA grid are removed 
!      before the EOF (or SVD) projection, as far as possible, and, in particular,
!      if the 3-D grid-mesh of the input NetCDF variables covers the whole globe. 
!
!      If **-g=** is set to ``n``, it is assumed that the 3-D grid-mesh is regular or Gaussian
!      and as such has no duplicate points.
!
!      The **-g=** argument is also used to determine the name of the NetCDF variables which
!      contain the 2-D mesh-mask and the scale factors in the *input_mesh_mask_netcdf_file* (e.g.
!      these variables are named *grid_type*\ mask, e1\ *grid_type* and e2\ *grid_type*\ ,
!      respectively). This *input_mesh_mask_netcdf_file* may be created by  :ref:`comp_clim_4d`
!      if the 3-D grid-mesh is regular or gaussian.
!
!   #) If **-g=** is set to ``t``, ``u``, ``v``, ``w`` or ``f`` (e.g. if the input NetCDF variables
!      are from an experiment with the NEMO or ORCA model), the **-r=** argument gives the 
!      resolution used. If:
!
!      * **-r=**\ ``r2`` the NetCDF variables are from an experiment with the ORCA R2 model.
!
!      * **-r=**\ ``r4`` the NetCDF variables are from an experiment with the ORCA R4 model.
!
!   #) If the NetCDF variables are from an experiment with the NEMO or ORCA model,
!      but the resolution is not R2 or R4, the dimensions of the ORCA grid
!      must be specified explicitly with the **-b=** argument.
!
!   #) If the **-x=**\ *lon1,lon2*, **-y=**\ *lat1,lat2* and **-z=**\ *level1,level2* arguments
!      are missing, the geographical
!      domain used in the EOF projection is determined from the attributes of the input mesh mask
!      NetCDF variable named *grid_type*\ mask (e.g. lon1_Eastern_limit, lon2_Western_limit, lat1_Southern_limit
!      , lat2_Northern_limit, level1_First_level and level2_Last_level ) which is read from the input NetCDF
!      file *input_mesh_mask_netcdf_file*\ .
!      If these attributes are missing, the whole geographical domain associated with the *netcdf_variable*
!      is used in the EOF projection.
!
!      The longitude, latitude or level range must be a vector of two integers specifying
!      the first and last selected indices along each dimension. The indices are relative to ``1``\ .
!      Negative values are allowed for *lon1*. In this case the longitude domain
!      is from ``nlon``\ +\ *lon1*\ +\ ``1`` to *lon2* where ``nlon`` is the number of longitude
!      points in the grid associated with the NetCDF variable and it is assumed that the grid
!      is periodic.
!
!      Refer to :ref:`comp_mask_4d` for transforming geographical coordinates as indices or generating an
!      appropriate mesh-mask before using comp_project_eof_4d.
!
!   #) If the **-t=**\ *time1,time2* argument is missing, the whole time period
!      associated with the *netcdf_variable* is used in the projection analysis.
!
!      The selected time period is a vector of two integers specifying the first
!      and last time observations. The indices are relative to ``1``\ . Note that the
!      output NetCDF file will have ``ntime`` = *time2* - *time1* + ``1`` observations
!      if the **-l=** argument is missing.
!
!   #) The **-l=** argument lists the indices of the time steps which must be included in
!      the output file. The indices of the time steps are counted from the start of the (selected)
!      time period (e.g. *time1* in the **-t=**\ *time1,time2* argument or ``1`` if this argument
!      is not used). The list may be specified in two formats:
!
!      * **-l=**\ *n1,n2,...nn* allows to select for *n1*, *n2*, ... and *nn* time steps
!
!      * **-l=**\ *n1:n2* allows to select time steps from *n1* to *n2*\ .
!
!      The two forms of the **-l=** argument may be combined and repeated any number of times.
!      Duplicate time steps are not allowed.
!      Be careful with time period limits when specifying the **-l=** argument.
!
!   #) The **-a=** argument specifies if the observed variables have to be centered or standardized
!      with an input climatology (specified with the **-c=** argument) before the projection analysis:
!
!      * **-a=**\ ``scp`` means that the projection analysis must be done on the raw data.
!
!      * **-a=**\ ``cov`` means that the projection analysis must be done on the anomalies.
!
!      * **-a=**\ ``cor`` means that the projection analysis must be done on the standardized anomalies.
!
!   #) The *input_climatology_netcdf_file* is needed only if **-a=**\ ``cov`` or **-a=**\ ``cor``.
!
!   #) If **-a=**\ ``cov`` or **-a=**\ ``cor``, the selected time period must agree with the climatology.
!      This means that the first selected time observation (*time1* if the **-t=** argument
!      is present) must correspond to the first day, month, season of the climatology 
!      specified with the **-c=** argument.
!
!   #) The geographical shapes of the *netcdf_variable* (in the *input_netcdf_file*), the *eof_netcdf_file*
!      (in the *eof_netcdf_file*), the climatology (in the *input_climatology_netcdf_file*) and the scale factors
!      (in the *input_mesh_mask_netcdf_file*) must agree.
!
!   #) The **-d=** argument specifies the metric and scalar product used in the EOF or (SVD) projection:
!
!      * **-d=**\ ``dist2`` means that the projection is done with the diagonal distance
!        associated with the horizontal 2-D grid-mesh (e.g. each grid point is weighted accordingly to
!        the surface associated with it)
!
!      * **-d=**\ ``dist3`` means that the projection is done with the diagonal distance associated
!        with the whole 3D grid-mesh (e.g. each grid point is weighted accordingly to the volume
!        or weight associated with it)
!
!      * **-d=**\ ``ident`` means that the projection is done with the identity metric : the usual
!        Euclidean distance and scalar product are used in the EOF (or SVD) projection.
!
!      By default, the **-d=** argument is set to ``dist3``\ .
!
!   #) The **-normpc** argument specifies that the computed PC (or SV) time series must be normalized
!      with the reciprocal of the singular value of the associated EOF pattern stored in the
!      *eof_netcdf_file* (or with the reciprocal of the previously computed standard-deviations of the
!      SV time series stored in the *eof_netcdf_file* if the **-svd** argument is used).
!
!   #) The **-svd** argument specifies that the *eof_netcdf_file* is produced by :ref:`comp_svd_3d`
!      instead of :ref:`comp_eof_4d`\ . This means that the projection will be done onto the
!      singular vectors of a previous SVD analysis stored in :ref:`comp_svd_3d`\ .
!
!   #) The **-double** argument specifies that the results are stored as double-precision floating
!      point numbers in the output NetCDF file.
!
!      By default, the results are stored as single-precision floating point numbers
!      in the output NetCDF file.
!
!   #) The **-bigfile** argument is allowed only if the NCSTAT software has been compiled with the
!      _USE_NETCDF36 or _USE_NETCDF4 CPP macros (e.g. ``-D_USE_NETCDF36`` or ``-D_USE_NETCDF4``) and linked
!      to the NetCDF 3.6 library or higher.
!      If this argument is specified, the *output_netcdf_file* will be a 64-bit offset format file
!      instead of a NetCDF classic format file. However, this argument is recognized in the procedure
!      only if the NCSTAT software has been built with the _USE_NETCDF36 or _USE_NETCDF4 CPP macros.
!
!   #) The **-hdf5** argument is allowed only if the NCSTAT software has been
!      compiled with the _USE_NETCDF4 CPP macro (e.g. ``-D_USE_NETCDF4``) and linked
!      to the NetCDF 4 library or higher. 
!      If this argument is specified, the *output_netcdf_file* will be a NetCDF-4/HDF5
!      format file instead of a NetCDF classic format file. However, this
!      argument is recognized in the procedure only if the NCSTAT software has
!      been built with the _USE_NETCDF4 CPP macro.
!
!   #) It is assumed that the data has no missing values excepted those associated with
!      a constant land-sea mask.
!
!   #) Duplicate parameters are allowed, but this is always the last occurrence of a parameter
!      which will be used for the computations. Moreover, the number of specified parameters
!      must not be greater than the total number of allowed parameters.
!
!   #) For more details on EOF or SVD analysis in the climate literature, see
!
!      * "A manual for EOF and SVD analyses of climate data",
!        by Bjornsson, H., and Venegas, S.A., McGill University, CCGCR Report No. 97-1,
!        Montral, Qubec, 52pp, 1997. https://www.jsg.utexas.edu/fu/files/EOFSVD.pdf
!
!      * "Statistical Analysis in Climate Research",
!        by von Storch, H., and Zwiers, F.W., Cambridge University press,
!        Cambridge, UK, Chapter 13, 484 pp., 2002. ISBN: `9780521012300`_
!
! .. _9780521012300: http://www.cambridge.org/gb/academic/subjects/earth-and-environmental-science/climatology-and-climate-change/statistical-analysis-climate-research?format=PB
!
! Outputs
! -------
!
!   comp_project_eof_4d creates an output NetCDF file that contains the new principal component
!   (or singular variables) time series computed from the eigenvectors (or singular vectors) of
!   a previous EOF (or SVD) analysis.
!   The number of principal components stored in the output NetCDF dataset is determined by
!   the **-se=**\ *selected_eofs* argument. The number of observations in the output NetCDF dataset
!   is determined from the **-t=**\ *time1,time2* and **-l=**\ *selected_time_period* arguments.
!   The output NetCDF dataset contains the following NetCDF variable :
!
!   #) *netcdf_variable*\ _pc\ ``(ntime,number_of_eofs)`` : the new principal component time series
!      corresponding to the projection onto the selected eigenvectors or singular vectors.
!
!      The new principal component time series are standardized with the standard-deviations estimated
!      from the previous EOF (or SVD) analysis if the **-normpc** argument is specified.
!
! Examples
! --------
!
!   #)   For computing a ``5``\ -EOF projection of a NetCDF variable named ``votemper``
!        in the NetCDF file ``ST7_1m_20101_30012_grid_T_votemper.nc`` on the ``5`` first
!        EOF patterns of a previous EOF analysis stored in the file named
!        ``eof_ST7_1m_0101_20012_grid_T_votemper.nc`` and store the results in
!        a NetCDF file named ``ST7_1m_20101_30012_grid_T_votemper_10pc.nc``, use the
!        following command :
!
!        ::
!
!          $ comp_project_eof_4d \
!            -f=ST7_1m_20101_30012_grid_T_votemper.nc  \
!            -v=votemper  \
!            -fe=eof_ST7_1m_0101_20012_grid_T_votemper.nc  \
!            -ve=votemper \
!            -se=1:5 \
!            -m=ST7_grid_T_votemper_mask.nc \
!            -o=ST7_1m_20101_20012_grid_T_votemper_10pc.nc
!
!-
! ================================================================================================
!
#ifndef _OPENMP
#undef _PARALLEL_READ
#endif
!
#ifdef _USE_NETCDF4
#define _USE_NETCDF36
#endif
!
! USED MODULES 
! ============
!
#ifdef _USE_NAGWARE
    use f90_unix_env,     only : iargc, getarg
#endif
!
#ifdef _MATMUL
    use Statpack,         only : matmul=>matmul2
#endif
!
#ifdef _PARALLEL_READ
    use netcdf,           only : nf90_share
    use Statpack,         only : omp_limit
    use omp_lib,          only : omp_get_num_procs, omp_get_max_threads
#endif
!
#ifdef _USE_NETCDF36
    use netcdf,    only : nf90_64bit_offset
#endif
!
#ifdef _USE_NETCDF4
    use netcdf,    only : nf90_hdf5
#endif
!
    use netcdf,    only : nf90_clobber, nf90_nowrite, nf90_share, nf90_int, nf90_real4, nf90_real8,    &
                          nf90_sizehint_default, nf90_max_name, nf90_max_var_dims, nf90_global,        &
                          nf90_noerr, nf90_unlimited, nf90_nofill,                                     &
                          nf90_open, nf90_strerror, nf90_get_att, nf90_get_var, nf90_inq_varid,        &
                          nf90_close, nf90_create, nf90_set_fill, nf90_def_dim, nf90_def_var,          &
                          nf90_inq_attname, nf90_copy_att, nf90_put_att, nf90_enddef, nf90_put_var,    &
                          nf90_close
!
    use Statpack,         only : i1b, i4b, stnd, extd, lgl, true, false, zero, one,         & 
                                 kint, tolower, case_change, string_to_val,                 &
                                 allocate_error, merror, arth, lamch
!
    use NCSTAT_Utilities, only : prtunit, nchar, nlon_def2, nlat_def2, ndepth_def2,                     &
                                 nlon_def4, nlat_def4, ndepth_def4,                                     &
                                 get_listarg, get_int_list, get_variable_dims, get_variable_dimcoords,  &
                                 check_lon_domain, select_grid_points_3d
!
!
! STRONG TYPING IMPOSED 
! =====================
!
    implicit none
!
!
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='comp_project_eof_4d : ',          &
!
                                   cmethod(3)=(/"scp","cov","cor"/),            &          
!
                                   cdistance(3)=(/"dist2","dist3","ident"/),    &   
!
                                   cresolution(2)=(/"r2","r4"/)
!
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
!
#ifndef _USE_NAGWARE
    integer(i4b)  :: iargc
#endif
    integer(i4b)  :: narg, status, ncid, ncid_clim, ncid_mask, ncid_eof, ncid_proj,    &
                     varid, meanid, stdid, maskid, e1id, e2id, e3id, eofid, projid,    &
                     i, j, j1, j2, k, l, m, nlon, nlat, nlevel, nt, nvar,              &
                     lon1, lon2, lon3, lat1, lat2, lat3, period,                       &
                     time1, time2, time3, level1, level2, level3,                      &
                     ioffset, method, distance, nlon_def, nlat_def, nlevel_def,        &
                     ntime_list, neof_list, neof, output_type, output_mode

    integer(i4b), dimension(3)                 :: coor_id2
    integer(i4b), dimension(4)                 :: startlec, countlec
    integer(i4b), dimension(nf90_max_var_dims) :: dimids, dimlens, coor_natts, coor_id, coor_type
    integer(i1b)                               :: kcode
    integer(i4b), dimension(:),    pointer     :: time_list, eof_list
!
    real(stnd), dimension(:),      allocatable  :: dat, scal, stdpc
    real(stnd), dimension(:,:),    allocatable  :: mean, std, eof, pc
    real(stnd), dimension(:,:,:),  allocatable  :: temp
    real(stnd)                                  :: tmp, scalfac, offset
!
    real(extd), dimension(:),     allocatable  :: record, record2
!
    logical(lgl)                               :: use_tlimited, use_x, use_y, use_z, use_svd, norm_pc,       &
                                                  create_hdf5, missing_att, scale_factor, add_offset
    logical(lgl), dimension(nf90_max_var_dims)  :: use_coor_variable
    logical(lgl), dimension(:,:,:), allocatable :: mask
!
    character(len=nchar)                       :: name, history
    character(len=nf90_max_name)               :: fic_name, var_name, clim_name, mask_name,    &
                                                  eof_fic_name, eof_name, proj_name, att_name 
    character(len=nf90_max_name), dimension(nf90_max_var_dims) :: dim_names
    character(len=14)                                          :: fmt
    character                                                  :: chr, grid_type
!
#ifdef _PARALLEL_READ
    integer(i4b)  :: ncid2, varid2
!
    logical  :: test_par
#endif
!
!
! EXECUTABLE STATEMENTS
! =====================
!
!   GET THE NUMBER OF COMMAND LINE ARGUMENTS.
!
    narg = iargc()
!
!   CHECK THE NUMBER OF COMMAND LINE ARGUMENTS.
!
    if ( narg==0 ) then
!
        write (prtunit,*)
        write (prtunit,*) ' Purpose : '
        write (prtunit,*)
        write (prtunit,*) ' Project a fourdimensional variable extracted from a NetCDF dataset'
        write (prtunit,*) ' onto an orthogonal set of eigenvectors from an EOF or SVD analysis '
        write (prtunit,*)
        write (prtunit,*) ' Usage : '
        write (prtunit,*)
        write (prtunit,*) ' comp_project_eof_4d  -f=input_netcdf_file'
        write (prtunit,*) '                      -v=netcdf_variable'
        write (prtunit,*) '                      -fe=eof_netcdf_file'
        write (prtunit,*) '                      -ve=eof_netcdf_variable (without _eof or _svd extension)'
        write (prtunit,*) '                      -m=input_mesh_mask_netcdf_file'
        write (prtunit,*) '                      -se=selected_eof                 (optional)'
        write (prtunit,*) '                      -g=grid_type                     (optional : n, t, u, v, w, f)'
        write (prtunit,*) '                      -r=resolution                    (optional : r2, r4)'
        write (prtunit,*) '                      -b=nlon_orca, nlat_orca          (optional)'
        write (prtunit,*) '                      -x=lon1,lon2                     (optional)'
        write (prtunit,*) '                      -y=lat1,lat2                     (optional)'
        write (prtunit,*) '                      -z=level1,level2                 (optional)'
        write (prtunit,*) '                      -t=time1,time2                   (optional)'
        write (prtunit,*) '                      -l=selected_time_period          (optional)'
        write (prtunit,*) '                      -a=type_of_analysis              (optional : scp, cov, cor)'
        write (prtunit,*) '                      -c=input_climatology_netcdf_file (optional)'
        write (prtunit,*) '                      -d=type_of_distance              (optional : dist2, ident)'
        write (prtunit,*) '                      -o=output_pc_netcdf_file         (optional)'
        write (prtunit,*) '                      -normpc                          (optional)'
        write (prtunit,*) '                      -svd                             (optional)'
        write (prtunit,*) '                      -double                          (optional)'
#ifdef _USE_NETCDF36
        write (prtunit,*) '                      -bigfile                         (optional)'
#endif
#ifdef _USE_NETCDF4
        write (prtunit,*) '                      -hdf5                            (optional)'
#endif
        write (prtunit,*) '                      -tlimited                        (optional)'
        write (prtunit,*)
        write (prtunit,*) ' By default : '
        write (prtunit,*)
        write (prtunit,*) '-se= all the eigenvectors or singular vectors stored in eof_netcdf_file'
        write (prtunit,*) '-g=n'
        write (prtunit,*) '-r=r2'
        write (prtunit,*) '-b= by default, nlon_orca and nlat_orca are determined from the -r='
        write (prtunit,*) '    argument'
        write (prtunit,*) '-x= the whole longitude domain associated with the netcdf_variable'
        write (prtunit,*) '-y= the whole latitude domain associated with the netcdf_variable'
        write (prtunit,*) '-z= the whole vertical resolution associated with the netcdf_variable'
        write (prtunit,*) '-t= the whole time period associated with the netcdf_variable'
        write (prtunit,*) '-l= all the time steps as specified by the -t= argument'
        write (prtunit,*) '-a=scp'
        write (prtunit,*) '-c= is not needed if -a=scp'
        write (prtunit,*) '-d=dist3'
        write (prtunit,*) '-o=proj_netcdf_variable.nc'
        write (prtunit,*) '-normpc   : by default, the computed PC (or SV) time series are not normalized'
        write (prtunit,*) '-svd      : by default, the projection is computed from EOF patterns'
        write (prtunit,*) '-double   : by default, the results are stored as single floating numbers'
#ifdef _USE_NETCDF36
        write (prtunit,*) '-bigfile  : by default, a NetCDF classical format file is created'
#endif
#ifdef _USE_NETCDF4
        write (prtunit,*) '-hdf5     : by default, a NetCDF classical format file is created'
#endif
        write (prtunit,*) '-tlimited : by default, the time dimension is defined as unlimited'
        write (prtunit,*)
#ifdef _OPENMP
        write (prtunit,*) 'This procedure is parallelized with OpenMP'
#endif
        write (prtunit,*)
!
        stop
!
    else if ( narg<5 .or. narg>24 ) then
!
        call merror( name_proc//"Wrong number of line arguments" )
!
    end if
!
    fic_name     = " "
    var_name     = " "
    eof_fic_name = " "
    eof_name     = " "
    clim_name    = " "
    mask_name    = " "
    proj_name    = " "
!
    history = "comp_project_eof_4d"
!
!   DEFAULT VALUES FOR COMMAND LINE ARGUMENTS.
!
    output_mode = nf90_clobber
    output_type = nf90_real4
    nlat_def    = nlat_def2
    nlon_def    = nlon_def2
    nlevel_def  = ndepth_def2
    method      = 1
    distance    = 2
    ntime_list  = 0
    neof_list   = 0
!
    grid_type  = "n"
!
    nullify( time_list, eof_list )
!
    lon1      = 1
    lon2      = huge( lon2 )
    lat1      = 1
    lat2      = huge( lat2 )
    level1    = 1
    level2    = huge( level2 )
    time1     = 1
    time2     = huge( time2 )
!
    use_x        = false
    use_y        = false
    use_z        = false
    norm_pc      = false
    use_svd      = false
    create_hdf5  = false
    use_tlimited = false
!
!   GET THE COMMAND LINE ARGUMENTS.
!
    do i = 1, narg
!
        call getarg( i, name )
!
        history = trim(history)//" "//trim(name)
!
        call case_change( name(:3), tolower )
!
        select case ( name(1:3 ) )
!
            case ('-f=')
!
!               GET THE NAME OF THE NETCDF DATASET.
!
                fic_name = name(4:)
!
            case ('-v=')
!
!               GET THE NAME OF THE NETCDF VARIABLE IN THE DATASET.
!
                var_name = name(4:)
!
            case ('-g=')
!
!               GET THE GRID-TYPE.
!
                call case_change( name(4:), tolower )
!
                select case ( name(4:) )
                    case ( 'n', 'u', 'v', 'w', 't', 'f' )
                        grid_type = name(4:4)
                    case default
                        call merror( name_proc//"Unknown grid type" )
                end select
!
            case ('-r=')
!
!               DETERMINE THE RESOLUTION FOR THE ORCA GRID.
!
                call case_change( name(4:), tolower )
!
                select case ( name(4:) )
                    case ( cresolution(1) )
                        nlat_def   = nlat_def2
                        nlon_def   = nlon_def2
                        nlevel_def = ndepth_def2
                   case ( cresolution(2) )
                        nlat_def   = nlat_def4
                        nlon_def   = nlon_def4
                        nlevel_def = ndepth_def4
                    case default
                        call merror( name_proc//"Unknown resolution option" )
                end select
!
            case ('-b=')
!
!               GET THE DIMENSIONS OF THE ORCA GRID.
!
                call get_listarg( name_proc, name(4:),                                        &
                                  "nlon_orca, nlat_orca and nlevel_orca are integer values",  &
                                  "Wrong ORCA grid definition ",                              &
                                  nlon_def, nlat_def, nlevel_def )
!
            case ('-x=')
!
!               GET THE DOMAIN DEFINITION (Longitude).
!
                call get_listarg( name_proc, name(4:),                    &
                                  "lon1 and lon2 are integer values",     &
                                  "Wrong domain definition (longitude)",  &
                                  lon1, lon2 )
!
                use_x = true
!
            case ('-y=')
!
!               GET THE DOMAIN DEFINITION (latitude).
!
                call get_listarg( name_proc, name(4:),                    &
                                  "lat1 and lat2 are integer values",     &
                                  "Wrong domain definition (latitude)",   &
                                  lat1, lat2 )
!
                use_y = true
!
            case ('-z=')
!
!               GET THE LEVEL DEFINITION.
!
                call get_listarg( name_proc, name(4:),                    &
                                  "level1 and level2 are integer values", &
                                  "Wrong level definition",               &
                                  level1, level2 )
!
                use_z = true
!
            case ('-t=')
!
!               GET THE PERIOD DEFINITION.
!
                call get_listarg( name_proc, name(4:),                    &
                                  "time1 and time2 are integer values",   &
                                  "Wrong period definition",              &
                                  time1, time2 )
!
            case ('-m=')
!
!               GET THE NAME OF THE NETCDF MESH-MASK DATASET.
!
                mask_name = name(4:)
!
            case ('-c=')
!
!               GET THE NAME OF THE NETCDF CLIMATOLOGY DATASET.
!
                clim_name = name(4:)
!
            case ('-a=')
!
!               DETERMINE THE METHOD OF ANALYSIS.
!
                call case_change( name(4:), tolower )
!
                select case ( name(4:) )
                    case ( cmethod(1) )
                        method = 1
                    case ( cmethod(2) )
                        method = 2
                    case ( cmethod(3) )
                        method = 3
                    case default
                        call merror( name_proc//"Unknown analysis option" )
                end select
!
            case ('-d=')
!
!               DETERMINE THE DISTANCE FOR THE ANALYSIS.
!
                call case_change( name(4:), tolower )
!
                select case ( name(4:) )
                    case ( cdistance(1) )
                        distance = 1
                    case ( cdistance(2) )
                        distance = 2
                    case ( cdistance(3) )
                        distance = 3
                    case default
                        call merror( name_proc//"Unknown distance option" )
                end select
!
            case ('-o=')
!
!               GET THE NAME OF THE NEW NETCDF DATASET FOR STORING THE (STANDARDIZED) VARIABLE.
!
                proj_name = name(4:)
!
            case ('-l=')
!
                if ( ntime_list/=0 ) deallocate( time_list )
!
                call get_int_list(  name_proc, name, 4, ntime_list, time_list )
!
            case default
!
                select case ( name(1:4 ) )
!
                    case ('-se=')
!
                        if ( neof_list/=0 ) deallocate( eof_list )
!
                        call get_int_list(  name_proc, name, 5_i4b, neof_list , eof_list )
!
                    case ('-fe=')
!
!                       GET THE NAME OF THE EOF NETCDF DATASET.
!
                        eof_fic_name = name(5:)
!
                    case ('-ve=')
!
!                       GET THE NAME OF THE EOF NETCDF VARIABLE IN THE DATASET.
!
                        eof_name = name(5:)
!
                    case default
!
                        call case_change( name(4:), tolower )
!
                        select case ( name(1:) )
!
                            case ('-normpc')
!
!                               NORMALIZE THE PC TIME SERIES.
!
                                norm_pc = true
!
                            case ('-svd')
!
!                               COMPUTE THE PROJECTION FROM SINGULAR VECTORS.
!
                                use_svd = true
!
                            case ('-double')
!
!                               STORE RESULTS AS DOUBLE PRECISION NUMBERS.
!
                                output_type = nf90_real8
!
#ifdef _USE_NETCDF36
                            case ('-bigfile')
!
!                               REDEFINE THE CREATION MODE FLAG FOR THE OUTPUT FILE.
!
                                output_mode = nf90_64bit_offset
#endif
!
#ifdef _USE_NETCDF4
                            case ('-hdf5')
!
!                               REDEFINE THE CREATION MODE FLAG FOR THE OUTPUT FILE.
!
                                output_mode = nf90_hdf5
                                create_hdf5 = true
#endif
!
                            case ('-tlimited')
!
!                               DEFINE THE TIME DIMENSION AS LIMITED.
!
                                use_tlimited = true
!
                            case default
!
!                               WRONG ARGUMENT.
!
                                call merror( name_proc//"Wrong argument" )
!
                        end select
!
                end select
!
        end select
!
    end do
!
!   CHECK THE COMMAND LINE ARGUMENTS.
!
    if ( fic_name==" " )                                                  &
    call merror( name_proc//"Input NETCDF file is missing" )
!
    if ( eof_fic_name==" " )                                              &
    call merror( name_proc//"Input EOF or SVD NETCDF file is missing" )
!
    if ( var_name==" " )                                                  &
    call merror( name_proc//"Input NETCDF variable is missing" )
!
    if ( eof_name==" " )                                                  &
    call merror( name_proc//"Input EOF or SVD NETCDF variable is missing" )
!
    if ( clim_name==" " .and. method>=2 )                                 &
    call merror( name_proc//"Climatology NETCDF file is missing" )
!
    if ( mask_name==" " )                                                 &
    call merror( name_proc//"Input mesh-mask NETCDF file is missing" )
!
    if ( proj_name==" " )                                                 &
    proj_name="proj_"//trim(var_name)//".nc"
!
!   OPEN THE NETCDF DATASETS.
!
    if ( ntime_list==0 ) then
        i = nf90_nowrite
    else
        i = nf90_share
    end if
!
    k = nf90_sizehint_default
    status = nf90_open( trim(fic_name), i, ncid, chunksize=k )
    if ( status/=nf90_noerr )                                            &
    call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_open( trim(eof_fic_name), nf90_nowrite, ncid_eof )
    if ( status/=nf90_noerr )                                            &
    call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_open( trim(mask_name), nf90_nowrite, ncid_mask )
    if ( status/=nf90_noerr )                                            &
    call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    if ( method>=2 ) then
!
        k = nf90_sizehint_default
        status = nf90_open( trim(clim_name), nf90_nowrite, ncid_clim, chunksize=k )
        if ( status/=nf90_noerr )                                        &
        call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    end if
!
!   GET THE VARIABLE VARIABLE ID FROM ITS NAME.
!   FIND THE DIMENSION LENGTHS AND NAMES FOR THE NETCDF VARIABLE.
!   GET THE COORDINATE VARIABLES IDS FROM THEIR NAMES, IF ANY.
!   FIND THE EXTERNAL TYPE, THE DIMENSION ID AND THE NUMBER OF ATTRIBUTES 
!   FOR THE COORDINATE VARIABLES.
!
!    call get_variable_dims( name_proc, trim(var_name), ncid, varid, dimids, dimlens )
!
   call get_variable_dimcoords(   name_proc, trim(var_name), ncid,                    &
                                  varid,  dimids_out=dimids, dim_names_out=dim_names, &
                                  dimlens_out=dimlens, coor_id_out=coor_id,           &
                                  coor_natts_out=coor_natts, coor_type_out=coor_type, &
                                  use_coor_variable_out=use_coor_variable             )
!
    if ( count(dimlens/=0)/=4 )           &
    call merror( name_proc//"Input NETCDF variable is not fourdimensional" )
!
    nlon   = dimlens(1)
    nlat   = dimlens(2)
    nlevel = dimlens(3)
    nt     = dimlens(4)
!
    chr = merge( 't', grid_type, grid_type=='w' )
!
!   GET THE MASK ID FROM ITS NAME.
!   VERIFY THE DIMENSION LENGTHS FOR THE NETCDF MASK VARIABLE.
!
    call get_variable_dims( name_proc, chr//"mask", ncid_mask, maskid, dimids, dimlens )
!
    if ( dimlens(1)/=nlon .or. dimlens(2)/=nlat .or. dimlens(3)/=nlevel )    &
    call merror( name_proc//"Shape disagreement with the mask" )
!
!   GET THE BOUNDARIES OF THE GEOGRAPHICAL DOMAIN FROM THE MESH-MASK DATASET.
!
    if ( .not.use_x ) then
!
        status = nf90_get_att( ncid_mask, maskid, "lon1_Eastern_limit", i )
        if ( status==nf90_noerr ) lon1 = i
!
        status = nf90_get_att( ncid_mask, maskid, "lon2_Western_limit", i )
        if ( status==nf90_noerr ) lon2 = i
!
    end if
!
    if ( .not.use_y ) then
!
        status = nf90_get_att( ncid_mask, maskid, "lat1_Southern_limit", i )
        if ( status==nf90_noerr ) lat1 = i
!
        status = nf90_get_att( ncid_mask, maskid, "lat2_Northern_limit", i )
        if ( status==nf90_noerr ) lat2 = i
!
    end if
!
    if ( .not.use_z ) then
!
        status = nf90_get_att( ncid_mask, maskid, "level1_First_level", i )
        if ( status==nf90_noerr ) level1 = i
!
        status = nf90_get_att( ncid_mask, maskid, "level2_Last_level", i )
        if ( status==nf90_noerr ) level2 = i
!
    end if
!
!   CHECK THE BOUNDARIES OF THE GEOGRAPHICAL DOMAIN.
!   CHECK THE GEOGRAPHICAL DOMAIN.
!   REMOVE DUPLICATE POINTS FROM THE ORCA_R2 GRID.
!
    if ( grid_type/='n' ) then
!
        if ( nlon>nlon_def )                                                     &
        call merror( name_proc//"Input NETCDF variable is not on the ORCA GRID (longitude)" )
!
        if ( nlat>nlat_def )                                                     &
        call merror( name_proc//"Input NETCDF variable is not on the ORCA GRID (latitude)" )
!
        if ( nlevel>nlevel_def )    &
        call merror( name_proc//"Input NETCDF variable is not on the ORCA GRID (level)" )
!
!       A TRICK TO DISABLE THE ELIMINATION AND DUPLICATION IN THE HIGH LATITUDES OF THE ORCA GRID
!       IF THE RESOLUTION IS NOT R2 OR R4.
!
        if ( ( nlat_def/=nlat_def2 .or. nlon_def/=nlon_def2 ) .and. ( nlat_def/=nlat_def4 .or. nlon_def/=nlon_def4 ) )    &
        nlat_def = nlat_def + 1
!
    end if
!
    call check_lon_domain( grid_type, nlon, nlon_def, lon1, lon2, lon3 )
!
    if ( lon3<=0  )  call merror( name_proc//"Wrong domain definition (longitude)" )
!
    lat1 = max( lat1, 1 )
    lat2 = min( lat2, nlat )
    lat3 = lat2 - lat1 + 1
    if ( lat3<=0  )  call merror( name_proc//"Wrong domain definition (latitude)" )
!
!   CHECK THE LEVEL DEFINITION.
!
    level1 = max( level1, 1 )
    level2 = min( level2, nlevel )
    level3 = level2 - level1 + 1
    if ( level3<=0  ) call merror( name_proc//"Wrong vertical definition" )
!
!   CHECK THE PERIOD DEFINITION.
!
    time1 = max( time1, 1 )
    time2 = min( time2, nt )
    time3 = time2 - time1 + 1
    if ( time3<=0  ) call merror( name_proc//"Wrong time definition" )
!
!   CHECK THE TIME COORDINATES LIST.
!
    if ( ntime_list==0 ) then
!
        ntime_list = time3
!
!       ALLOCATE VARIABLE FOR THE TIME COORDINATES LIST IF NOT DONE BEFORE.
!
        allocate( time_list(ntime_list), stat=status )
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
        time_list(1:ntime_list) = arth( 1_i4b, 1_i4b, ntime_list )
!
    else
!
        if ( any( time_list(1:ntime_list)>time3 ) )          &
        call merror( name_proc//"-t= and -l= options mismatch" )
!
    end if
!
!   ALLOCATE WORK ARRAYS.
!
    allocate( temp(nlon,nlat,nlevel), mask(nlon,nlat,nlevel), stat=status )
    if ( status/=0 ) call merror( name_proc//allocate_error )
!
!   GET THE MASK.
!
    status = nf90_get_var( ncid_mask, maskid, temp )     
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!   DETERMINE THE GRID POINTS USED IN THE EOF ANALYSIS FROM THE MASK.
!
    mask(:nlon,:nlat,:nlevel) = temp(:nlon,:nlat,:nlevel)==one
!
!   SELECT THE DOMAIN.
!
    call select_grid_points_3d( grid_type, lon1, lon2, lat1, lat2, level1, level2, &
                                nlon_def, nlat_def, mask, j1, j2  )
!
    if ( lon1<0 ) lon3 = nlon
!
    nvar = count( mask )
    if ( nvar<=0  )  call merror( name_proc//"Wrong domain definition : domain is empty" )
!
!   GET THE EOF VARIABLE ID FROM ITS NAME.
!   FIND THE DIMENSION LENGTHS AND NAMES FOR THE NETCDF VARIABLE.
!   GET THE COORDINATE VARIABLES IDS FROM THEIR NAMES, IF ANY.
!   FIND THE EXTERNAL TYPE, THE DIMENSION ID AND THE NUMBER OF ATTRIBUTES 
!   FOR THE COORDINATE VARIABLES.
!
    if ( use_svd ) then
        name = trim(eof_name)//"_svd"
    else
        name = trim(eof_name)//"_eof"
    end if
!
    call get_variable_dims( name_proc, trim(name), ncid_eof, eofid, dimids, dimlens )
!
!    call get_variable_dimcoords(  name_proc, trim(name), ncid_eof,                    &
!                                  eofid,  dimids_out=dimids, dim_names_out=dim_names, &
!                                  dimlens_out=dimlens, coor_id_out=coor_id,           &
!                                  coor_natts_out=coor_natts, coor_type_out=coor_type, &
!                                  use_coor_variable_out=use_coor_variable             )
!
    if ( count(dimlens/=0)/=4 )           &
    call merror( name_proc//"Input EOF (or SVD) NETCDF variable is not fourdimensional" )
!
    if ( dimlens(1)/=nlon .or. dimlens(2)/=nlat .or. dimlens(3)/=nlevel )               &
    call merror( name_proc//"Shape disagreement between the input NETCDF variables" )
!
    neof = dimlens(4)
!
!   CHECK THE EOF LIST.
!
    if ( neof_list==0 ) then
!
        neof_list = neof
!
!       ALLOCATE VARIABLE FOR THE TIME COORDINATES LIST IF NOT DONE BEFORE.
!
        allocate( eof_list(neof_list), stat=status )
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
        eof_list(1:neof_list) = arth( 1_i4b, 1_i4b, neof_list )
!
    else
!
        if ( any( eof_list(1:neof_list)>neof ) )          &
        call merror( name_proc//"Wrong -se= argument for the selection of EOFs (or SVDs)" )
!
    end if
!
!   ALLOCATE WORK ARRAY FOR THE EIGENVECTORS.
!
    allocate( eof(nvar,neof_list), stat=status )
    if ( status/=0 ) call merror( name_proc//allocate_error )
!
!   GET THE EIGENVECTORS OF THE EOF ANALYSIS.
!
    startlec(1:3)  = (/ j1, lat1, level1 /)
    countlec(1:4)  = (/ lon3, lat3, level3, 1_i4b /)
!
    do i = 1, neof_list
!
        startlec(4) = eof_list(i)
!
        status = nf90_get_var( ncid_eof, eofid, temp(:lon3,:lat3,:level3), &
                               start=startlec, count=countlec      )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        eof(:nvar, i) = pack( temp(:lon3,:lat3,:level3), mask(j1:j2,lat1:lat2,level1:level2) )
!
    end do
!
!   FIND THE MISSING VALUE ATTRIBUTE ASSOCIATED WITH THE NETCDF EOF VARIABLE, IF ANY.
!
    status = nf90_get_att( ncid_eof, eofid, "missing_value", tmp )
    missing_att = status==nf90_noerr
!
    if ( .not.missing_att ) then
        status = nf90_get_att( ncid_eof, eofid, "_FillValue", tmp )
        missing_att = status==nf90_noerr
    end if
!
    if ( missing_att ) then
        if ( any( eof(:nvar,:neof_list)==tmp ) )    &
        call merror( name_proc//"Missing values in the EOF or (SVD) patterns" )
    end if
!
!   NORMALIZE THE EIGENVECTORS OR SINGULAR VECTORS IF REQUIRED.
!
    if ( norm_pc .or. .not.use_svd ) then
!
        allocate( stdpc(neof), stat=status )
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
        if ( use_svd ) then
            name = trim(eof_name)//"_SV_STDs"
        else
            name = trim(eof_name)//"_sing"
        end if
!
!       GET THE SINGULAR VALUES VARIABLE ID FROM ITS NAME.
!
        status = nf90_inq_varid( ncid_eof, trim(name), stdid )
        if ( status/=nf90_noerr )                                            &
        call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_get_var( ncid_eof, stdid, stdpc(:neof) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!       SCALE THE EIGENVECTORS OR SINGULAR VECTORS.
!
        do i = 1, neof_list
 !
            j   = eof_list(i)
            tmp = stdpc(j)
!
            if ( norm_pc .and. .not.use_svd ) then
                eof(:nvar,i) = (eof(:nvar,i)/tmp)/tmp
            else
                eof(:nvar,i) = eof(:nvar,i)/tmp
            end if
!
        end do
!
        deallocate( stdpc, stat=status )
!
    end if
!
!   COMPUTE THE DISTANCE.
!
    if ( distance<=2 ) then
!
!       THE DISTANCE IS ASSOCIATED WITH THE (HORIZONTAL) GRID.
!
!       ALLOCATE WORK ARRAY.
!
        allocate( scal(nvar), mean(lon3,lat3), stat=status )
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
!       GET THE SCALE FACTORS IDs FROM THEIR NAMES.
!       VERIFY THE DIMENSION LENGTHS FOR THE SCALE FACTOR VARIABLES.
!
        call get_variable_dims( name_proc, "e1"//chr, ncid_mask,    &
                                e1id, dimids, dimlens )
!
        if ( dimlens(1)/=nlon .or. dimlens(2)/=nlat )               &
        call merror( name_proc//"Shape disagreement with the first scale factor" )
!
        call get_variable_dims( name_proc, "e2"//chr, ncid_mask,    &
                                e2id, dimids, dimlens )
!
        if ( dimlens(1)/=nlon .or. dimlens(2)/=nlat )               &
        call merror( name_proc//"Shape disagreement with the second scale factor" )
!
!       GET THE SCALE FACTORS.
!
        status = nf90_get_var( ncid_mask, e1id, mean(:lon3,:lat3), start=startlec(1:2), &  
                               count=countlec(1:2)  )     
        if ( status/=nf90_noerr )                                                     &
        call merror( name_proc//trim( nf90_strerror( status ) ) )
        temp(:lon3,:lat3,1) = mean(:lon3,:lat3)
!
        status = nf90_get_var( ncid_mask, e2id, mean(:lon3,:lat3), start=startlec(1:2), &
                               count=countlec(1:2)  )     
        if ( status/=nf90_noerr )                                                     &
        call merror( name_proc//trim( nf90_strerror( status ) ) )
        mean(:lon3,:lat3) = mean(:lon3,:lat3)*temp(:lon3,:lat3,1)

        temp(:lon3,:lat3,:level3) = spread( mean(:lon3,:lat3), dim=3, ncopies=level3 )
        scal(:nvar) = pack( temp(:lon3,:lat3,:level3), mask(j1:j2,lat1:lat2,level1:level2) )
!
        if ( distance==2 ) then
!
!           ALLOCATE WORK ARRAY.
!
            allocate( dat(level3), stat=status )
            if ( status/=0 ) call merror( name_proc//allocate_error )
!
!           THE DISTANCE IS ASSOCIATED WITH THE WHOLE 3D GRID.
!
            chr = merge( grid_type, 't', grid_type=='w' .or. grid_type=='n' )
!
!           GET THE THIRD SCALE FACTOR ID FROM ITS NAME.
!           VERIFY THE DIMENSION LENGTHS FOR THE THIRD SCALE FACTOR VARIABLE.
!
            call get_variable_dims( name_proc, "e3"//chr, ncid_mask,    &
                                    e3id, dimids, dimlens )
!
            if ( dimlens(1)/=1 .or. dimlens(2)/=1 .or. dimlens(3)/=nlevel )             &
            call merror( name_proc//"Shape disagreement with the third scale factor" )
!
!           GET THE THIRD SCALE FACTOR VARIABLE.
!
            status = nf90_get_var( ncid_mask, e3id, dat,                             &
                                   start=(/1_i4b,1_i4b,level1/), count=(/1_i4b,1_i4b,level3/) )    
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           CHECK THE THIRD SCALE FACTOR.
!
            if ( dat(1)<zero )                        &
    call merror( name_proc//" First level with -d=dist3 are not allowed for mesh-mask "//trim(mask_name) )
!
            if ( dat(level3)<zero )                   &
    call merror( name_proc//" Last level with -d=dist3 are not allowed for mesh-mask "//trim(mask_name) )
!
            temp(:lon3,:lat3,:level3) =                                                 &
            spread( spread( dat(:level3), dim=1, ncopies=lat3), dim=1, ncopies=lon3 )
            scal(:nvar) =                                                               &
            scal(:nvar)*pack( temp(:lon3,:lat3,:level3), mask(j1:j2,lat1:lat2,level1:level2) )
!
!           DEALLOCATE WORK ARRAY.
!
            deallocate( dat )
!
        end if
!
!       COMPUTE THE DIAGONAL METRIC ASSOCIATED WITH THE (HORIZONTAL) GRID.
!
        scal(:nvar) = scal(:nvar)*(one/maxval(scal(:nvar)))
!
!       SCALE THE EIGENVECTORS IF REQUIRED.
!
        do i = 1, neof_list
!
            eof(:nvar,i) = scal(:nvar)*eof(:nvar,i)
!
        enddo
!
!       DEALLOCATE WORK ARRAY.
!
        deallocate( scal, mean )
!
    end if
!
!   FIND THE SCALE_FACTOR ATTRIBUTE ASSOCIATED WITH THE NETCDF VARIABLE, IF ANY.
!
    status = nf90_get_att( ncid, varid, "scale_factor", scalfac )
    if ( status/=nf90_noerr ) then
        scale_factor = false
        scalfac      = one
    else
        scale_factor = true
    end if
!
!   FIND THE ADD_OFFSET ATTRIBUTE ASSOCIATED WITH THE NETCDF VARIABLE, IF ANY.
!
    status = nf90_get_att( ncid, varid, "add_offset", offset )
    if ( status/=nf90_noerr ) then
        add_offset = false
        offset     = zero
    else
        add_offset = true
    end if
!
!   READ CLIMATOLOGY AND STANDARD-DEVIATIONS IF REQUIRED.
!
    if ( method>1 ) then
!
!       GET THE MEAN FIELD ID FROM ITS NAME.
!       FIND THE DIMENSION LENGTHS FOR THE NETCDF MEAN VARIABLE.
!
        call get_variable_dims( name_proc, trim(var_name)//"_mean", ncid_clim,    &
                                meanid, dimids, dimlens )
!
        if ( count(dimlens/=0)/=4 )     &
        call merror( name_proc//"Input climatology is not fourdimensional (mean field)" )
!
        if ( dimlens(1)/=nlon .or. dimlens(2)/=nlat .or. dimlens(3)/=nlevel )    &
        call merror( name_proc//"Shape disagreement with the climatology (mean field)" )
!
        period = dimlens(4)
!
!       ALLOCATE WORK ARRAY FOR THE MEANS.
!
        allocate( mean(nvar,period), stat=status )
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
!       GET THE MEANS.
!
        do j = 1, period
!
            startlec(4) = j
!
            status = nf90_get_var( ncid_clim, meanid, temp(:lon3,:lat3,:level3), &   
                                   start=startlec, count=countlec                )     
            if ( status/=nf90_noerr )                                            &
            call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           PACK THE MEAN FIELD.
!
            mean(:nvar,j) = pack( temp(:lon3,:lat3,:level3), mask(j1:j2,lat1:lat2,level1:level2) )
!
        end do
!
!       FIND THE MISSING VALUE ATTRIBUTE ASSOCIATED WITH THE MEAN VARIABLE, IF ANY.
!
        status = nf90_get_att( ncid_clim, meanid, "missing_value", tmp )
        missing_att = status==nf90_noerr
!
        if ( .not.missing_att ) then
            status = nf90_get_att( ncid_clim, meanid, "_FillValue", tmp )
            missing_att = status==nf90_noerr
        end if
!
        if ( missing_att ) then
            if ( any( mean(:nvar,:period)==tmp ) )    &
            call merror( name_proc//"Missing values in the input climatology" )
        end if
!
!       FIND THE SCALE_FACTOR ATTRIBUTE ASSOCIATED WITH THE MEAN VARIABLE, IF ANY.
!
        status = nf90_get_att( ncid_clim, meanid, "scale_factor", tmp )
        if ( status==nf90_noerr ) mean(:nvar,:period) = mean(:nvar,:period)*tmp
!
!       FIND THE ADD_OFFSET ATTRIBUTE ASSOCIATED WITH THE MEAN VARIABLE, IF ANY.
!
        status = nf90_get_att( ncid_clim, meanid, "add_offset", tmp )
        if ( status==nf90_noerr ) mean(:nvar,:period) = mean(:nvar,:period) + tmp
!
        if ( method>2 ) then
!
!           GET THE STD ID FROM ITS NAME.
!           FIND THE DIMENSION LENGTHS FOR THE NETCDF STD VARIABLE.
!
            call get_variable_dims( name_proc, trim(var_name)//"_std", ncid_clim,    &
                                    stdid, dimids, dimlens )
!
            if ( count(dimlens/=0)/=4 )          &
            call merror( name_proc//"Input climatology is not fourdimensional (std field)" )
!
            if ( dimlens(1)/=nlon .or. dimlens(2)/=nlat .or. dimlens(3)/=nlevel .or. dimlens(4)/=period )    &
            call merror( name_proc//"Shape disagreement with the climatology (std field)" )
!
!           ALLOCATE WORK ARRAYS.
!
            allocate( std(nvar,period), stat=status )
            if ( status/=0 ) call merror( name_proc//allocate_error )
!
!           GET THE STDS.
!
            do j = 1, period
!
                startlec(4) = j
!
                status = nf90_get_var( ncid_clim, stdid, temp(:lon3,:lat3,:level3),     & 
                                       start=startlec, count=countlec           )     
                if ( status/=nf90_noerr )                                       &
                call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!               PACK THE STD FIELD.
!
                std(:nvar,j) = pack( temp(:lon3,:lat3,:level3), mask(j1:j2,lat1:lat2,level1:level2) )
!
            end do
!
!           FIND THE MISSING VALUE ATTRIBUTE ASSOCIATED WITH THE STD VARIABLE, IF ANY.
!
            status = nf90_get_att( ncid_clim, stdid, "missing_value", tmp )
            missing_att = status==nf90_noerr
!
            if ( .not.missing_att ) then
                status = nf90_get_att( ncid_clim, stdid, "_FillValue", tmp )
                missing_att = status==nf90_noerr
             end if
!
            if ( missing_att ) then
                if ( any( std(:nvar,:period)==tmp ) )    &
                call merror( name_proc//"Missing values in the input standard-deviations field" )
            end if
!
!           FIND THE SCALE_FACTOR ATTRIBUTE ASSOCIATED WITH THE STD VARIABLE, IF ANY.
!
            status = nf90_get_att( ncid_clim, stdid, "scale_factor", tmp )
            if ( status==nf90_noerr ) std(:nvar,:period) = std(:nvar,:period)*tmp
!
!           FIND THE ADD_OFFSET ATTRIBUTE ASSOCIATED WITH THE STD VARIABLE, IF ANY.
!
            status = nf90_get_att( ncid_clim, stdid, "add_offset", tmp )
            if ( status==nf90_noerr ) std(:nvar,:period) = std(:nvar,:period) + tmp
!
!           STANDARDIZED VALUES ARE REQUESTED.
!
            tmp = lamch('s')
!
            where ( std(:nvar,:period)>tmp )
                 std(:nvar,:period) = one/std(:nvar,:period)
            elsewhere
                 std(:nvar,:period) = zero
            end where
!
        end if
!
    end if
!
!   ALLOCATE WORK ARRAYS.
!
    allocate( pc(neof_list,ntime_list), stat=status )
    if ( status/=0 ) call merror( name_proc//allocate_error )
!
!   COMPUTE THE OFFSET FOR THE RECORD NUMBER.
!
    ioffset = time1 - 1
!
!   COMPUTE THE PROJECTIONS ONTO THE EIGENVECTORS.
!
#ifdef _PARALLEL_READ
!
    i = omp_get_num_procs()
    j = omp_get_max_threads()
    test_par = i>1 .and. j>1 .and. ntime_list*nvar>omp_limit .and. ntime_list>=j
!
    if ( test_par ) then
!
!       DEALLOCATE WORK ARRAY.
!
        deallocate( temp )
!
!$OMP PARALLEL PRIVATE(i,j,k,status,startlec,ncid2,varid2,temp,dat)   &
!$OMP         ,SHARED(mean,std,eof,pc,mask,offset,scalfac,countlec,   &
!$OMP                 j1,j2,lon3,lat1,lat2,lat3,level1,level2,level3, &
!$OMP                 nvar,ioffset,period,                            &
!$OMP                 time_list,ntime_list,neof_list,method)
!
!       ALLOCATE WORK ARRAYS.
!
        allocate( temp(lon3,lat3,level3), dat(nvar), stat=status )
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
!$OMP CRITICAL (opennc)
!
!       OPEN THE NETCDF DATASET.
!
        status = nf90_open( trim(fic_name), nf90_share, ncid2  )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!       GET THE VARIABLE ID FROM ITS NAME.
!
        status = nf90_inq_varid( ncid2, trim(var_name), varid2 )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!$OMP END CRITICAL (opennc)
!
        startlec(1)  = j1
        startlec(2)  = lat1
        startlec(3)  = level1
!
!$OMP DO SCHEDULE(STATIC)
!
        do i = 1, ntime_list
!
            j = time_list(i) + ioffset
            startlec(4) = j
!
!           FILL THE WORK ARRAY WITH A RECORD FROM THE NETCDF VARIABLE IN THE INPUT FILE.
!
            status = nf90_get_var( ncid2, varid2, temp(:lon3,:lat3,:level3), start=startlec, count=countlec )     
            if ( status/=nf90_noerr )   call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           PACK THE DATA.
!
            dat(:nvar) = pack( temp(:lon3,:lat3,:level3), mask(j1:j2,lat1:lat2,level1:level2) )
!
!           MULTIPLY THE NETCDF VARIABLE BY THE SCALE FACTOR IF REQUIRED.
!
            if ( scale_factor ) dat(:nvar) = dat(:nvar)*scalfac
!
!           ADD OFFSET TO THE NETCDF VARIABLE IF REQUIRED.
!
            if ( add_offset ) dat(:nvar) = dat(:nvar) + offset
!
!           TRANSFORM THE DATA IF REQUIRED.
!
            if ( method>1 ) then
!
                k = mod( j, period )
                if ( k==0 ) k = period
!
                if ( method==2 ) then
!
                    dat(:nvar) = dat(:nvar) - mean(:nvar,k)
!
                else
!
                    dat(:nvar) = (dat(:nvar) - mean(:nvar,k))*std(:nvar,k)
!
                end if
!
            end if        
!
!           EOF PROJECTION.
!
            pc(:neof_list,i) = matmul( dat(:nvar), eof(:nvar,:neof_list) )
!
        end do
!
!$OMP END DO
!
!       DEALLOCATE WORK ARRAYS.
!
        deallocate( temp, dat )
!
!$OMP CRITICAL (closenc)
!
!       CLOSE THE NETCDF DATASET.
!
        status = nf90_close( ncid2 )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!$OMP END CRITICAL (closenc)
!
!$OMP END PARALLEL
!
    else
!
#endif
!
!       ALLOCATE WORK ARRAY.
!
        allocate( dat(nvar), stat=status )
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
        do i = 1, ntime_list
!
            j = time_list(i) + ioffset
            startlec(4) = j
!
!           FILL THE WORK ARRAY WITH A RECORD FROM THE NETCDF VARIABLE IN THE INPUT FILE.
!
            status = nf90_get_var( ncid, varid, temp(:lon3,:lat3,:level3), start=startlec, count=countlec )     
            if ( status/=nf90_noerr )   call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           PACK THE DATA.
!
            dat(:nvar) = pack( temp(:lon3,:lat3,:level3), mask(j1:j2,lat1:lat2,level1:level2) )
!
!           MULTIPLY THE NETCDF VARIABLE BY THE SCALE FACTOR IF REQUIRED.
!
            if ( scale_factor ) dat(:nvar) = dat(:nvar)*scalfac
!
!           ADD OFFSET TO THE NETCDF VARIABLE IF REQUIRED.
!
            if ( add_offset ) dat(:nvar) = dat(:nvar) + offset
!
!           TRANSFORM THE DATA IF REQUIRED.
!
            if ( method>1 ) then
!
                k = mod( j, period )
                if ( k==0 ) k = period
!
                if ( method==2 ) then
!
                    dat(:nvar) = dat(:nvar) - mean(:nvar,k)
!
                else
!
                    dat(:nvar) = (dat(:nvar) - mean(:nvar,k))*std(:nvar,k)
!
                end if
!
            end if        
!
!           EOF OR SVD PROJECTION.
!
            pc(:neof_list,i) = matmul( dat(:nvar), eof(:nvar,:neof_list) )
!
        end do
!
!       DEALLOCATE WORK ARRAYS.
!
        deallocate( temp, dat )
!
#ifdef _PARALLEL_READ
!
    end if
!
#endif
!
!   CREATE THE NEW NETCDF DATASET FOR THE EOF OR SVD PROJECTION.
!
    if ( create_hdf5 ) then
        status = nf90_create( proj_name, cmode=output_mode, ncid=ncid_proj )
    else
        i = nf90_sizehint_default
        status = nf90_create( proj_name, cmode=output_mode, ncid=ncid_proj, chunksize=i )
    end if
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_set_fill( ncid_proj, nf90_nofill, i )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!   DEFINE THE DIMENSIONS FOR THE NEW NETCDF DATASET.
!
    dimlens(1)   = neof_list
    if ( use_svd ) then
        dim_names(1) = "sv_number"
    else
        dim_names(1) = "pc_number"
    end if
    coor_type(1) = nf90_int
!
    if ( use_tlimited ) then
        dimlens(2)   = ntime_list
    else
        dimlens(2)   = nf90_unlimited
    end if
    dim_names(2) = dim_names(4)
    coor_type(2) = coor_type(4)
!
    do i = 1, 2
!
        status = nf90_def_dim( ncid_proj, name=trim(dim_names(i)),                     &
                               len=dimlens(i), dimid=dimids(i) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim(nf90_strerror(status)) )
!
    end do
!
!   DEFINE THE VARIABLES FOR THE NEW NETCDF DATASET.
!
    do i = 1, 2
!
        status = nf90_def_var( ncid_proj, trim(dim_names(i)), coor_type(i),           &
                               dimids=dimids(i), varid=coor_id2(i) )
        if ( status/=nf90_noerr )                                                     &
        call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end do
!
!   DEFINE PROJECTION VARIABLE.
!
    if ( use_svd ) then
        name = trim(var_name)//"_sv"
    else
        name = trim(var_name)//"_pc"
    end if
!
    status = nf90_def_var( ncid_proj, trim(name), output_type, dimids=dimids(1:2), varid=projid )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!   ASSIGN ATTRIBUTES FOR THE NEW NETCDF DATASET.
!
    do i = 1, coor_natts(4)
!
        status = nf90_inq_attname( ncid, coor_id(4), i, att_name )
        if ( status/=nf90_noerr )                                                     &
        call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!       COPY ATTRIBUTES FOR THE COORDINATE VARIABLES.
!
        status = nf90_copy_att(  ncid, coor_id(4), att_name, ncid_proj, coor_id2(2) )
        if ( status/=nf90_noerr )                                                     &
        call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    end do
!
    if ( use_svd ) then
!
        status = nf90_put_att( ncid_proj, coor_id2(1), "long_name", "Singular Vector number" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_proj, coor_id2(1), "short_name", "SVDs" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_proj, projid, "long_name", "Singular Variables" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_proj, projid, "short_name", "SVs" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        name = trim(eof_name)//" singular vectors"
!
    else
!
        status = nf90_put_att( ncid_proj, coor_id2(1), "long_name", "Empirical Orthogonal Function number" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_proj, coor_id2(1), "short_name", "EOFs" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_proj, projid, "long_name", "Principal Components" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_proj, projid, "short_name", "PCs" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        name = trim(eof_name)//" eigenvectors"
!
    end if
!
!   SET GLOBAL ATTRIBUTES.
!
    status = nf90_put_att( ncid_proj, nf90_global, "title", trim(var_name)//" projected onto "//trim(name) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_proj, nf90_global, "distance", cdistance(distance) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_proj, nf90_global, "method", cmethod(method) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    if ( norm_pc ) then
        i = 1
    else
        i = 0
    end if
!
    status = nf90_put_att( ncid_proj, nf90_global, "normalization", i )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    if ( use_svd ) then
        i = 1
    else
        i = 0
    end if
!
    status = nf90_put_att( ncid_proj, nf90_global, "use_svd", i )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_proj, nf90_global, "grid_type", grid_type )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_proj, nf90_global, "history", trim(history) )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!   LEAVE DEFINE MODE FOR THE NEW NETCDF DATASET.
!
    status = nf90_enddef( ncid_proj )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!   WRITE VARIABLES FOR THE NEW  NETCDF DATASET.
!
    status = nf90_put_var( ncid_proj, coor_id2(1), eof_list(1:neof_list) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    if ( use_coor_variable(4) ) then
!
!       ALLOCATE WORK ARRAYS.
!
        allocate( record(time3), record2(ntime_list),stat=status )
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
        status = nf90_get_var( ncid, coor_id(4), record(1:time3), start=(/ time1 /), count=(/ time3 /) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        record2(1:ntime_list) = record( time_list(1:ntime_list) )
!
        status = nf90_put_var( ncid_proj, coor_id2(2), record2(1:ntime_list) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        deallocate( record, record2 )
!
    else
!
        ioffset = time1 - 1
!
        time_list(1:ntime_list) =  time_list(1:ntime_list) + ioffset
!
        status = nf90_put_var( ncid_proj, coor_id2(2), time_list(1:ntime_list) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end if
!
    status = nf90_put_var( ncid_proj, projid, pc(1:neof_list,1:ntime_list) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!   DEALLOCATE WORK ARRAYS.
!
    select case ( method )
!
        case ( 1 )
!
            deallocate( eof, pc, mask, eof_list, time_list )
!
        case ( 2 )
!
            deallocate( eof, pc, mask,  eof_list, time_list,  mean )
!
        case ( 3 )
!
            deallocate( eof, pc, mask,  eof_list, time_list,  mean, std )
!
    end select
!
!   CLOSE THE NETCDF DATASETS.
!
    status = nf90_close( ncid_eof )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_close( ncid )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_close( ncid_mask )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    if ( method>=2 ) then
        status = nf90_close( ncid_clim )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
    end if
!
    status = nf90_close( ncid_proj )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
! END OF PROGRAM comp_project_eof_4d
! ==================================
!
end program comp_project_eof_4d
