program comp_reg_3d
!! #############################################################################
!!
!! Copyright 2018 IRD
!!
!! This file is part of NCSTAT.
!!
!! NCSTAT is free software: you can redistribute it and/or modify
!! it under the terms of the GNU Lesser General Public License as 
!! published by the Free Software Foundation, either version 3 of 
!! the License, or (at your option) any later version.
!!
!! NCSTAT is distributed in the hope that it will be useful,
!! but WITHOUT ANY WARRANTY; without even the implied warranty of
!! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!! GNU Lesser General Public License for more details.
!!
!! You can find a copy of the GNU Lesser General Public License
!! in the NCSTAT/doc directory.
!!
!! #############################################################################
!
!+
!
! .. _comp_reg_3d:
!
! ===========
! comp_reg_3d
! ===========
!
! Authors
! =======
!
! Pascal Terray (LOCEAN/IPSL)
!
! Latest revision
! ===============
!
! 13/09/2018
!
! Purpose
! =======
!
! Estimate polynomial trends and regression models from time series extracted from a 
! tridimensional variable in a NetCDF dataset and, optionally, remove
! the linear terms from the data by using linear least squares estimation
! and/or store the residuals and/or the predictions in a NetCDF dataset.
!
! Optionally, regression diagnostics and statistical tests to diagnose the quality of the
! fitted regression model may also be computed and stored [Rusta]_ [Rustb]_ .   
!
! Finally, if the NetCDF variable is fourdimensional use :ref:`comp_reg_4d` instead of comp_reg_3d
! and if the NetCDF variable is unidimensional use :ref:`comp_reg_1d` instead of comp_reg_3d.
!
! This procedure is parallelized if OpenMP is used.
!
! Further Details
! ===============
!
! Usage
! -----
!
! ::
!
!       $ comp_reg_3d \
!         -f=input_netcdf_file \
!         -v=netcdf_variable \
!         -m=input_mesh_mask_netcdf_file           (optional) \
!         -g=grid_type                             (optional : n, t, u, v, w, f) \
!         -x=lon1,lon2                             (optional) \
!         -y=lat1,lat2                             (optional) \
!         -t=time1,time2                           (optional) \
!         -p=periodicity                           (optional) \
!         -a=type_of_analysis                      (optional : reg, residual, predict, all) \
!         -o=output_netcdf_file                    (optional) \
!         -to=time_origin                          (optional) \
!         -dg=polynomial_degree                    (optional) \
!         -fi=input_index_netcdf_file              (optional) \
!         -vi=index_netcdf_variable                (optional) \
!         -ti=itime1,itime2                        (optional) \
!         -pi=iperiodicity,istep                   (optional) \
!         -ni=index_for_2d_index_netcdf_variable   (optional) \
!         -sm=smoothing_factor                     (optional) \
!         -dv=dv_time1,dv_time2                    (optional) \
!         -mi=missing_value                        (optional) \
!         -use_eps=tol                             (optional) \
!         -comp_min_norm                           (optional) \
!         -add_mean                                (optional) \
!         -rsquare                                 (optional) \
!         -adjrsquare                              (optional) \
!         -stderr                                  (optional) \
!         -ftest                                   (optional) \
!         -ttest                                   (optional) \
!         -double                                  (optional) \
!         -bigfile                                 (optional) \
!         -hdf5                                    (optional) \
!         -tlimited                                (optional)
!
! By default
! ----------
!
!        *-m=* 
!         an *input_mesh_mask_netcdf_file* is not used
!        *-g=*
!         the *grid_type* is set to ``n`` which means that the 2-D grid-mesh associated with
!         the input *netcdf_variable* is assumed to be regular or Gaussian
!        *-x=*
!         the whole longitude domain associated with the *netcdf_variable*
!        *-y=*
!         the whole latitude domain associated with the *netcdf_variable*
!        *-t=*
!         the whole time period associated with the *netcdf_variable*
!        *-p=*
!         the *periodicity* is set to ``1``
!        *-a=*
!         *type_of_analysis* is set to ``reg``
!        *-o=*
!         *output_netcdf_file* is set to ``reg_``\ *netcdf_variable*\ ``.``\ *index_netcdf_variable*\ ``.nc``
!         if the **-vi=** argument is used and to ``reg_``\ *netcdf_variable*\ ``.poly_trend.nc`` otherwise
!        *-to=*
!         If a polynomial trend is estimated, the origin
!         for the time scale is set to ``0``
!        *-dg=*
!         the *polynomial_degree* is set ``1``\ . 
!         This means that a linear trend is estimated if the **-vi=** argument is not used
!        *-fi=*
!         a polynomial trend is estimated if the **-vi=** argument is not used,
!         otherwise the **-fi=** argument may be used to specified the NetCDF dataset
!         for extracting the *index_netcdf_variable*
!        *-vi=*
!         a polynomial trend is estimated if the **-vi=** argument is not used
!        *-ti=*
!         the whole time period associated with the *index_netcdf_variable* if the
!         **-vi=**\ *index_netcdf_variable* is specified
!        *-pi=*
!         this parameter is not used
!        *-ni=*
!         if the *index_netcdf_variable* is bidimensional, the first time series is used
!        *-sm=*
!         no smoothing is applied to the *index_netcdf_variable* if the
!         **-vi=** argument is used
!        *-dv=*
!         a dummy variable is not used
!        *-mi=*
!         the *missing_value* is set to ``1.e+20`` for the NetCDF variables in the *output_netcdf_file*
!        *-use_eps=*
!         no tolerance is used for solving the linear least square problem associated with the specified
!         regression model. If **-use_eps=** is used, the specified tolerance *tol* is used for solving the
!         linear least square problem
!        *-comp_min_norm*
!         the minimal norm solution is not computed. If **-comp_min_norm** is activated, the
!         minimal norm solution of the  linear least square problem is computed
!        *-add_mean*
!         the means are not added to the residuals. If **-add_mean** is activated, the
!         means are added to the residuals
!        *-rsquare*
!         the coefficient of determination is not computed. If **-rsquare** is activated, the
!         coefficient of determination is computed
!        *-adjrsquare*
!         the adjusted coefficient of determination is not computed. If **-adjrsquare** is activated, the
!         adjusted coefficient of determination is computed
!        *-stderr*
!         the standard errors of the estimates are not computed. If **-stderr** is activated, the
!         standard errors of the estimates are computed
!        *-ftest*
!         a F-test for the regression model is not computed. If **-ftest** is activated, a
!         a F-test is computed
!        *-ttest*
!         Student-tests for the regression coefficients are not computed. If **-ttest** is activated, the
!         Student-tests are computed
!        *-double*
!         the results are stored as single-precision floating
!         point numbers in the output NetCDF file. If **-double** is activated, the
!         results are stored as double-precision floating point numbers 
!        *-bigfile*
!         a NetCDF classical format file is created. If **-bigfile** is activated,
!         the output NetCDF file is a 64-bit offset format file
!        *-hdf5*
!         a NetCDF classical format file is created. If **-hdf5** is activated,
!         the output NetCDF file is a NetCDF-4/HDF5 format file
!        *-tlimited*
!         the time dimension is defined as unlimited in the output NetCDF file.
!         However, if **-tlimited** is activated, the time dimension is defined
!         as limited in the output NetCDF file
!
! Remarks
! -------
!
!   #) The **-v=**\ *netcdf_variable* argument specifies the NetCDF variable for which a
!      regression analysis must be computed and the **-f=**\ *input_netcdf_file* argument
!      specifies that this NetCDF variable must be extracted from the NetCDF file,
!      *input_netcdf_file*.
!
!   #) The optional argument **-m=**\ *input_mesh_mask_netcdf_file* specifies the land-sea mask
!      to apply to *netcdf_variable* for transforming this tridimensional NetCDF variable as a
!      rectangular matrix of observed variables before estimating the regression model. By default,
!      it is assumed that each cell in the 2-D grid-mesh associated with the input tridimensional
!      NetCDF variable is a valid time series (e.g. missing values are not present).
!
!      The geographical shapes of the *netcdf_variable* (in the *input_netcdf_file*) and the mask
!      (in the *input_mesh_mask_netcdf_file*) must agree if an *input_mesh_mask_netcdf_file* is used.
!
!      Refer to :ref:`comp_clim_3d` or :ref:`comp_mask_3d` for creating a valid
!      *input_mesh_mask_netcdf_file* NetCDF file for regular or gaussian grids before
!      using comp_reg_3d.
!
!   #) If **-g=** is set to ``t``, ``u``, ``v``, ``w`` or ``f``, it is assumed that
!      the input NetCDF variable is from an experiment with the ORCA model (R2, R4 or R05 resolutions).
!      This argument is also used to determine the name of the mesh_mask variable if
!      an *input_mesh_mask_netcdf_file* is used.
!
!   #) If the **-x=**\ *lon1,lon2* and **-y=**\ *lat1,lat2* arguments are missing, the whole
!      geographical domain associated with the *netcdf_variable* is used.
!
!      The longitude or latitude range must be a vector of two integers specifying the first
!      and last selected indices along each dimension. The indices are relative to ``1``\ .
!      Negative values are allowed for *lon1*. In this case the longitude domain
!      is from ``nlon``\  +\  *lon1*\  +\  ``1`` to *lon2* where ``nlon`` is the number of longitude
!      points in the grid associated with the NetCDF variable and it is assumed that the grid
!      is periodic.
!
!      Refer to :ref:`comp_mask_3d` for transforming geographical coordinates as indices or
!      generating an appropriate mesh-mask before using comp_reg_3d.
!
!   #) If the **-t=**\ *time1,time2* argument is missing, data in the whole time period
!      associated with the *netcdf_variable* is taken into account.
!      The selected time period is a vector of two integers specifying the first
!      and last time observations. The indices are relative to ``1``\ .
!
!      The selected time period (e.g.  *time2* - *time1* + ``1``) must be a whole multiple of
!      the *periodicity* if the **-p=** argument is specified.
!
!   #) The **-p=**\ *periodicity* argument gives the periodicity of the input data for the
!      *netcdf_variable*. For example, with monthly data **-p=**\ ``12`` should be specified,
!      with yearly data **-p=**\ ``1`` may be used, etc.
!
!      If the **-p=**\ *periodicity* argument is specified, the regression models are 
!      computed by taking into account the periodicity of the data. This means that *periodicity*
!      regression models are estimated for each each cell in the 2-D grid-mesh associated with
!      the input tridimensional NetCDF variable.
!
!   #) The **-a=**\ argument specifies the statistics which must be computed and if the residuals from
!      the trend or regression models are stored in the output NetCDF file. If:
!
!      * **-a=**\ ``reg``, the default, the residuals or predictions are not computed and only the
!        regression and intercept coefficients are computed and stored
!
!      * **-a=**\ ``residual``, the residuals are computed and stored, in addition of the
!        regression and intercept coefficients
!
!      * **-a=**\ ``predict``,  the predictions are computed and stored, in addition of the
!        regression and intercept coefficients
!
!      * **-a=**\ ``all``, the residuals and predictions are computed and stored, in addition of the
!        regression and intercept coefficients.
!
!   #) The **-to=**\ *time_origin* argument specifies the origin for the time scale if a polynomial
!      regression model is used (e.g. when the **-vi=** argument is not used).
!
!      By default, the origin for the time scale is set to ``0``\ . This shift of the zero
!      point for the time scale makes the estimate for the intercept(s) in the polynomial
!      model equal to the model prediction(s) for the first observation at the beginning of the
!      record.
!
!   #) The **-dg=**\ *polynomial_degree* argument specifies the degree of the polynomial if
!      a polynomial trend regression model is used  (e.g. when the **-vi=** argument is not used).
!
!      If **-dg=**\ ``1``  a linear trend is used, if **-dg=**\ ``2`` a quadratic trend is used, etc......
!
!   #) The **-vi=**\ *index_netcdf_variable* specifies a predictor time series for the regression
!      model (e.g. an independent variable).
!
!      If the **-vi=**\ *index_netcdf_variable* is present, the **-fi=** argument
!      must also be present and this argument specifies the NetCDF dataset
!      which contains the *index_netcdf_variable*. However, if the NetCDF dataset, which
!      contains the *index_netcdf_variable*, is the same as the NetCDF dataset specified
!      by the **-f=** argument, it is not necessary to specify the **-fi=** argument.
!
!      If the **-vi=**\ *index_netcdf_variable* is not specified a regression model with
!      a polynomial trend is assumed and the **-dg=**  argument specifies the degree of the polynomial :
!      if **-dg=**\ ``1``  a linear trend is used, if **-dg=**\ ``2`` a quadratic trend is used, etc......
!
!   #) If the **-ti=**\ *itime1,itime2* argument is missing, data in the whole time period
!      associated with the *index_netcdf_variable* is taken into account.
!      The selected time period is a vector of two integers specifying the first
!      and last time observations. The indices are relative to ``1``\ .
!      If the **-vi=** argument is not present, this argument is not used.
!
!   #) The selected time periods for the *netcdf_variable* and *index_netcdf_variable*
!      must agree. This means that the following equality must be verified
!
!      ``(time2 - time1 + 1)/periodicity = ceiling((itime2 - itime1 - istep + 2)/iperiodicity)``,
!
!      otherwise, an error message will be issued and the program will stop.
!
!   #) The **-pi=** argument gives the periodicity and selects the time step for the
!      *index_netcdf_variable*. For example, to compute regression models with the January
!      monthly time series extracted from the *index_netcdf_variable*, which is assumed
!      to be sampled every month, **-pi=**\ ``12,1`` should be specified, with yearly data
!      **-pi=**\ ``1,1`` may be used, etc.
!
!      If the **-vi=** argument is not present, this argument is not used.
!
!   #) The **-ni=** argument specifies the index (e.g. an integer) for selecting the time series if
!      the *index_netcdf_variable* specified in the **-vi=** argument is a 2D NetCDF variable.
!
!   #) The **-sm=**\ *smoothing_factor* means that the time series associated with the 
!      *index_netcdf_variable* (e.g. the **-vi=** argument) must be smoothed with
!      a moving average of approximately ``2``\ .\ *smoothing_factor*\  + ``1`` terms
!      before estimating the regression parameters for predicting the *netcdf_variable*
!      (e.g. the **-v=** argument ) from the *index_netcdf_variable*.
!      *smoothing_factor* must be a strictly positive integer.
!
!      If the **-vi=** argument is not present, this argument is not used.
!
!   #) If the **-dv=**\ *dv_time1,dv_time2* argument is specified, a dummy variable is also included
!      in the regression model. The dummy variable is an absence/presence variable (e.g. with values
!      ``0``  or ``1`` ) and the time observations where the dummy variable is equal to ``1`` is
!      specified by the *dv_time1* and *dv_time2*  integers. The *dv_time1* and *dv_time2* integers specify
!      the first and last time observations of the selected time period in which the dummy variable is set
!      to ``1``\ .
!
!      These time indices are counted from the start of the (selected) time
!      period (e.g. *time1* in the **-t=**\ *time1,time2* argument or ``1`` if this argument 
!      is missing) and must take into account the periodicity of the data if the **-p=** argument
!      is specified.            
!
!   #) The **-mi=**\ *missing_value* argument specifies the missing value indicator associated with
!      the NETCDF variables in the *output_netcdf_file*.
!      If the **-mi=** argument is not specified, *missing_value* is set to ``1.e+20``\ .
!
!   #) The **-use_eps=**\ *tol* argument is used to determine the effective rank of the linear least squares
!      problem. *tol* must be set to the relative precision of the elements in 
!      the NETCDF variables specify by the **-v=** and **-vi=** arguments.
!      If each element is correct to, say, ``5`` digits then *tol* =\ ``0.00001`` should be used.
!      *tol*  must not be greater or equal to ``1`` or less than ``0`` , otherwise an error message
!      is printed and the program stops. If the **-use_eps=** argument is not used, the numerical
!      rank is determined.
!
!   #) If **-comp_min_norm** is specified, a complete orthogonal factorization
!      of the coefficient matrix and the minimun 2-norm solutions are computed.
!
!   #) If **-add_mean** is specified, the means are added to the residuals of the regression
!      model. This option has an effect only if  **-a=**\ ``residual`` or **-a=**\ ``all`` (e.g.
!      if the residuals are computed and stored in the output_netcdf_file).
!      By default, the means of the residuals in the output NetCDF file are zero.
!
!   #) If **-rsquare** is specified, the coefficient of determination of the specified model is computed.
!
!   #) If **-adjrsquare** is specified, the adjusted coefficient of determination of the specified
!      model is computed.
!
!   #) If **-stderr** is specified, the standard errors of the estimated regression coefficients
!      are computed (unless the specified model is not of full rank).
!
!   #) If **-ftest** is specified, a F-test is performed to test the null hypothesis that all the
!      regression coefficients are zero (excepted the intercept).
!
!   #) If **-ttest** is specified, Student-tests are performed to test the null hypothesis that the
!      regression coefficients are zero (independently of the other regression coefficients).
!
!   #) The **-double** argument specifies that the results are stored as double-precision floating
!      point numbers in the output NetCDF file.
!      By default, the results are stored as single-precision floating point numbers
!      in the output NetCDF file.
!
!   #) The **-bigfile** argument is allowed only if the NCSTAT software has been compiled with the
!      _USE_NETCDF36 or _USE_NETCDF4 CPP macros (e.g. ``-D_USE_NETCDF36`` or ``-D_USE_NETCDF4``) and linked
!      to the NetCDF 3.6 library or higher.
!      If this argument is specified, the *output_netcdf_file* will be a 64-bit offset format file
!      instead of a NetCDF classic format file. However, this argument is recognized in the procedure
!      only if the NCSTAT software has been built with the _USE_NETCDF36 or _USE_NETCDF4 CPP macros.
!
!   #) The **-hdf5** argument is allowed only if the NCSTAT software has been
!      compiled with the _USE_NETCDF4 CPP macro (e.g. ``-D_USE_NETCDF4``) and linked
!      to the NetCDF 4 library or higher. 
!      If this argument is specified, the *output_netcdf_file* will be a NetCDF-4/HDF5
!      format file instead of a NetCDF classic format file. However, this
!      argument is recognized in the procedure only if the NCSTAT software has
!      been built with the _USE_NETCDF4 CPP macro.
!
!   #) The **-tlimited** argument specifies that the time dimension must be defined as limited in
!      the output NetCDF file.
!      By default, this time dimension is defined as unlimited in the output NetCDF file.
!
!   #) Duplicate parameters are allowed, but this is always the last occurrence of a parameter
!      which will be used for the computations. Moreover, the number of specified parameters
!      must not be greater than the total number of allowed parameters.
!
!   #) It is assumed that the data has no missing values excepted those associated with a constant
!      land-sea mask if the **-m=** argument is used.
!
!   #) For more details on regression analysis in the climate literature see
!
!      * "Fitting nature s basic functions Part I: polynomials and linear least squares",
!        by Rust, B.W., Computing in Science and Engineering, Vol. 3, no 5, 84-89, 2001. doi: `10.1109/MCISE.2001.947111`_
!
!      * "Fitting nature s basic functions Part II: estimating uncertainties and testing hypotheses",
!        by Rust, B.W., Computing in Science and Engineering, Vol. 3, no 6, 60-64, 2001. doi: `10.1109/5992.963429`_
!
!      * "Fitting nature s basic functions Part III: exponentials, sinusoids, and nonlinear least squares",
!        by Rust, B.W., Computing in Science and Engineering, Vol. 4, no 4, 72-77, 2002. doi: `10.1109/MCISE.2002.1014982`_
!
!      * "Statistical Analysis in Climate Research",
!        by von Storch, H., and Zwiers, F.W., Cambridge University press,
!        Cambridge, UK, Chapter 8, 484 pp., 2002. ISBN: `9780521012300`_
!
! .. _10.1109/MCISE.2001.947111: http://dx.doi.org/10.1109/MCISE.2001.947111
! .. _10.1109/5992.963429: http://dx.doi.org/10.1109/5992.963429
! .. _10.1109/MCISE.2002.1014982: http://dx.doi.org/10.1109/MCISE.2002.1014982
! .. _9780521012300: http://www.cambridge.org/gb/academic/subjects/earth-and-environmental-science/climatology-and-climate-change/statistical-analysis-climate-research?format=PB
!
! Outputs
! -------
!
!   comp_reg_3d creates an output NetCDF file that contains the regression statistics and statistical tests
!   associated with these coefficients, taking into account eventually the periodicity of the data as determined 
!   by the **-p=**\ *periodicity* argument.
!
!   If the **-vi=**\ *index_netcdf_variable* is specified, the output NetCDF data set will have *periodicity* time
!   observations and may contain the following NetCDF variables depending on the arguments used in calling the
!   procedure (in the description below, nlat and nlon are the spatial dimensions of the input NetCDF variable) :
!
!   #) *netcdf_variable*\ _\ *index_netcdf_variable*\ _reg0\ ``(periodicity,nlat,nlon)`` : the intercept coefficients
!      in the regression models for predicting each grid-point in the time series of the 2-D grid-mesh associated with the input
!      NetCDF variable *netcdf_variable* by the *index_netcdf_variable* time series.
!
!   #) *netcdf_variable*\ _\ *index_netcdf_variable*\ _reg1\ ``(periodicity,nlat,nlon)`` : the regression coefficients between 
!      each grid-point in the time series of the 2-D grid-mesh associated with the input NetCDF variable *netcdf_variable* and the
!      *index_netcdf_variable* time series.
!
!      By default, the regression coefficients are expressed in units of the input NetCDF variable
!      *netcdf_variable* by unit of the *index_netcdf_variable* time series.
!
!   #) *netcdf_variable*\ _\ *index_netcdf_variable*\ _stderr0\ ``(periodicity,nlat,nlon)`` : the standard-errors of the
!      intercept coefficients in the regression models for predicting each grid-point in the time series of the 2-D grid-mesh
!      associated with the input NetCDF variable *netcdf_variable* by the *index_netcdf_variable* time series.
!
!   #) *netcdf_variable*\ _\ *index_netcdf_variable*\ _stderr1\ ``(periodicity,nlat,nlon)`` : the standard-errors of the
!      regression coefficients between each grid-point in the time series of the 2-D grid-mesh associated with the input NetCDF
!      variable *netcdf_variable* and the *index_netcdf_variable* time series.
!
!   #) *netcdf_variable*\ _\ *index_netcdf_variable*\ _tprob0\ ``(periodicity,nlat,nlon)`` : the Student t-test probabilities
!      associated with the intercept coefficients in the regression models for predicting each grid-point in the time series of
!      the 2-D grid-mesh associated with the input NetCDF variable *netcdf_variable* by the *index_netcdf_variable* time series.
!
!   #) *netcdf_variable*\ _\ *index_netcdf_variable*\ _tprob1\ ``(periodicity,nlat,nlon)`` : the Student t-test probabilities
!      associated with the regression coefficients between each grid-point in the time series of the 2-D grid-mesh associated with
!      the input NetCDF variable *netcdf_variable* and the *index_netcdf_variable* time series.
!
!   #) *netcdf_variable*\ _\ *index_netcdf_variable*\ _dv\ ``(periodicity,nlat,nlon)`` : the regression coefficients between 
!      each grid-point in the time series of the 2-D grid-mesh associated with the input NetCDF variable *netcdf_variable* and the
!      dummy variable time series if a dummy variable is included in the regression models.
!
!      This variable is stored only if the **-dv=** argument has been specified when calling
!      comp_reg_3d.
!
!   #) *netcdf_variable*\ _\ *index_netcdf_variable*\ _dv_stderr\ ``(periodicity,nlat,nlon)`` : the standard-errors of the
!      regression coefficients between each grid-point in the time series of the 2-D grid-mesh associated with the input NetCDF
!      variable *netcdf_variable* and the dummy variable time series if a dummy variable is included in the regression models.
!
!      This variable is stored only if the **-dv=** argument has been specified when calling
!      comp_reg_3d.
!
!   #) *netcdf_variable*\ _\ *index_netcdf_variable*\ _dv_tprob\ ``(periodicity,nlat,nlon)`` : the Student t-test probabilities
!      associated with the regression coefficients between each grid-point in the time series of the 2-D grid-mesh associated
!      with the input NetCDF variable *netcdf_variable* and the dummy variable time series if a dummy variable is included in
!      the regression models.
!
!      This variable is stored only if the **-dv=** argument has been specified when calling
!      comp_reg_3d.
!
!   #) *netcdf_variable*\ _\ *index_netcdf_variable*\ _r2\ ``(periodicity,nlat,nlon)`` : the r-square statistics
!      associated with the regression models for predicting each grid-point in the time series of the 2-D grid-mesh
!      associated with the input NetCDF variable *netcdf_variable* by the *index_netcdf_variable* time series.
!
!   #) *netcdf_variable*\ _\ *index_netcdf_variable*\ _adjr2\ ``(periodicity,nlat,nlon)`` : the adjusted r-square statistics
!      associated with the regression models for predicting each grid-point in the time series of the 2-D grid-mesh
!      associated with the input NetCDF variable *netcdf_variable* by the *index_netcdf_variable* time series.
!
!   #) *netcdf_variable*\ _\ *index_netcdf_variable*\ _fprob\ ``(periodicity,nlat,nlon)`` : the F-test probabilities
!      associated with the regression models for predicting each grid-point in the time series of the 2-D grid-mesh
!      associated with the input NetCDF variable *netcdf_variable* by the *index_netcdf_variable* time series.
!
!   #) *netcdf_variable*\ _\ *index_netcdf_variable*\ _predict\ ``(ntime,nlat,nlon)`` : the predictions
!      associated with the regression models for predicting each grid-point in the time series of the 2-D grid-mesh
!      associated with the input NetCDF variable *netcdf_variable* by the *index_netcdf_variable* time series.
!
!   #) *netcdf_variable*\ _\ *index_netcdf_variable*\ _resid\ ``(ntime,nlat,nlon)`` : the residuals
!      associated with the regression models for predicting each grid-point in the time series of the 2-D grid-mesh
!      associated with the input NetCDF variable *netcdf_variable* by the *index_netcdf_variable* time series.
!
!   All these statistics are packed in tridimensional variables whose first and second dimensions
!   are exactly the same as those associated with the input NetCDF variable *netcdf_variable*,
!   even if you restrict the geographical domain with the **-x=**\  and **-y=** arguments. However,
!   outside the selected domain, the output NetCDF variables are filled with missing values.
!
!   If the **-vi=**\ *index_netcdf_variable* is not specified when calling the procedure, similar statistics will be produced
!   and each component of the selected polynomial trend regression model will have regression coefficients, standard-errors and 
!   Student t-test probabilities associated with it. As an illustration, the intercept and the regression coefficients (e.g. the
!   slope) in a linear trend regression model will be stored in NetCDF variables *netcdf_variable*\ _poly_trend_reg0 and
!   *netcdf_variable*\ _poly_trend_reg1, respectively. For a quadratic trend regression model, in addition to these variables, the
!   regression coefficients associated with the quadratic component will be stored in a NetCDF variable *netcdf_variable*\ _poly_trend_reg2.
!   The same naming conventions are used for the standard-errors and Student t-test probabilities associated with each component of the
!   selected polynomial trend regression model.
!
! Examples
! --------
!
!   #)   For linear detrending bimonthly data from a tridimensional NetCDF
!        variable ``mslp`` in the NetCDF file ``mslp.seas.mean.nc``
!        and store the results in a NetCDF file named
!        ``reg_mslp_seas_ncep2.nc``, use the following
!        command (note that cyclostationarity is assumed for the ``mslp`` variable
!        since  **-p**\ =\ ``6`` is specified ) :
!
!        ::
!
!          $ comp_reg_3d \
!            -f=mslp.seas.mean.nc \
!            -v=mslp \
!            -m=mesh_mask_mslp_ncep2.nc \
!            -p=6 \
!            -a=residual \
!            -o=reg_mslp_seas_ncep2.nc
!
!   #)   For computing bimonthly lag regressions and residuals from a tridimensional NetCDF
!        variable ``mslp`` in the NetCDF file ``mslp.seas.mean.nc``
!        and a February-March Nino34 SST index in the NetCDF file ``sst_nino34_7901_seas.nc`` 
!        and store the results in a NetCDF file named
!        ``reg_mslp_seas_ncep2_nino34_23.nc``, use the following
!        command (note that cyclostationarity is assumed for both the ``mslp`` variable
!        since  **-p**\ =\ ``6`` is specified, and the index variable since **-pi**\ =\ ``6,1`` ) :
!
!        ::
!
!          $ comp_reg_3d \
!            -f=mslp.seas.mean.nc \
!            -v=mslp \
!            -m=mesh_mask_mslp_ncep2.nc \
!            -p=6 \
!            -fi=sst_nino34_7901_seas.nc \
!            -vi=sst \
!            -pi=6,1 \
!            -a=residual \
!            -o=reg_mslp_seas_ncep2_nino34_23.nc
!
!-
! ================================================================================================
!
#ifndef _OPENMP
#undef _PARALLEL_READ
#endif
!
#ifdef _USE_NETCDF4
#define _USE_NETCDF36
#endif
!
! USED MODULES 
! ============
!
#ifdef _USE_NAGWARE
    use f90_unix_env,     only : iargc, getarg
#endif
!
#ifdef _PARALLEL_READ
    use netcdf,   only : nf90_share
    use Statpack, only : omp_limit
    use omp_lib,  only : omp_get_num_procs, omp_get_max_threads
#endif
!
#ifdef _MATMUL
    use Statpack, only : matmul=>matmul2
#endif
!
#ifdef _USE_NETCDF36
    use netcdf,   only : nf90_64bit_offset
#endif
!
#ifdef _USE_NETCDF4
    use netcdf,   only : nf90_hdf5
#endif
!
    use netcdf,   only : nf90_max_name, nf90_max_var_dims, nf90_clobber, nf90_real4, nf90_real8,    &
                         nf90_int, nf90_sizehint_default, nf90_open, nf90_close, nf90_inq_varid,    &
                         nf90_nowrite, nf90_noerr, nf90_get_att, nf90_get_var, nf90_strerror,       &
                         nf90_create, nf90_set_fill, nf90_nofill, nf90_def_dim, nf90_unlimited,     &
                         nf90_def_var, nf90_inq_attname, nf90_copy_att, nf90_put_att, nf90_enddef,  &
                         nf90_global, nf90_put_var
!
    use Statpack, only : i1b, i4b, stnd, extd, lgl, true, false, zero, one, kfix, kexp,          & 
                         kint, tolower, case_change, string_to_val, val_to_string,               &
                         allocate_error, merror, arth, llsq_qr_solve2, comp_smooth, comp_mvs,    &
                         probf, probstudent, comp_triang_inv, comp_uut_ltl, outerprod
!
    use NCSTAT_Utilities, only : prtunit, nchar, sp, dp, def_missing,                            &
                                 get_listarg, get_variable_dims, get_variable_dimcoords,         &
                                 write_file_3d
!
!
! STRONG TYPING IMPOSED 
! =====================
!
    implicit none
!
!
! PARAMETERS 
! ==========
!
!
    character(len=*), parameter :: name_proc='comp_reg_3d : ',        &
!
                            cmethod(4)=(/"reg     ","residual","predict ","all     "/),     &
!
                            cregtype(2)=(/"poly_trend","variable  "/)         
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
!
#ifndef _USE_NAGWARE
    integer(i4b)  :: iargc
#endif
    integer(i4b)  :: narg, status, ncid, ncidi, ncid_mask, ncid_reg,                          &
                     varid, maskid, varidi, residid, predictid, rsquareid, adjrsquareid,      &
                     fprobid, i, i1, i2, j, j1, j2, k, l, krank,                              &
                     nlon, nlat, nt, nidim, nti, nvar, nn, nreg, ndim2, dg,                   &
                     lon1, lon2, lon3, lat1, lat2, lat3,                                      &
                     time1, time2, time3, itime1, itime2, itime3, dvtime1, dvtime2, dvtime3,  &
                     period, nperiod, iperiod, istep, niperiod, iindex, torigin,              &
                     method, regtype, smooth_factor, output_type, output_mode
    integer(i4b), dimension(:),   allocatable  :: regid, stderrid, tprobid
    integer(i4b), dimension(4)                 :: coor_id2
    integer(i4b), dimension(3)                 :: startlec, countlec
    integer(i4b), dimension(nf90_max_var_dims) :: dimids, dimlens, coor_natts, coor_id, coor_type
    integer(i1b)                               :: kcode
!
    real(stnd), dimension(:,:),   allocatable  :: temp, b, x, mat, mat2, stderr, tstat, tprob
    real(stnd), dimension(:),     allocatable  :: bmean, brsquare, bssr, bstd, fprob, diagr
    real(stnd)                                 :: missing, tmp, eps
    real(extd), dimension(:),     allocatable  :: record
!
    logical(lgl)                               :: use_tlimited, poly_trend, use_x, use_y, use_dv,          &
                                                  comp_resid, comp_predict, comp_min_norm, comp_rsquare,   &
                                                  comp_adjrsquare, comp_stat, comp_ssr, comp_ftest,        &
                                                  comp_ttest, comp_stderr, add_mean, use_mask, use_miss,   &
                                                  create_hdf5
    logical(lgl), dimension(nf90_max_var_dims) :: use_coor_variable
    logical(lgl), dimension(:,:), allocatable  :: mask
!
    character(len=nchar)                       :: name, name2, name3, name4, history
    character(len=nf90_max_name)               :: fic_name, var_name, mask_name, fici_name, vari_name,  &
                                                  reg_name, att_name 
    character(len=nf90_max_name), dimension(nf90_max_var_dims) :: dim_names
    character(len=14)                                          :: fmt
    character                                                  :: chr, grid_type
!
#ifdef _PARALLEL_READ
    integer(i4b)  :: ncid2, varid2, ioffset
!
    logical  :: test_par
#endif
!
!
! EXECUTABLE STATEMENTS
! =====================
!
!   GET THE NUMBER OF COMMAND LINE ARGUMENTS.
!
    narg = iargc()
!
!   CHECK THE NUMBER OF COMMAND LINE ARGUMENTS.
!
    if ( narg==0 ) then
!
        write (prtunit,*)
        write (prtunit,*) ' Purpose : '
        write (prtunit,*)
        write (prtunit,*) ' Estimates polynomial trends and regression models from time series extracted'
        write (prtunit,*) ' from a tridimensional variable in a NetCDF dataset and, optionally, removed'
        write (prtunit,*) ' the linear terms from the data by using linear least squares estimation'
        write (prtunit,*) ' and stored the residuals and/or the predictions in a Netcdf dataset.'
        write (prtunit,*)
        write (prtunit,*) ' Usage : '
        write (prtunit,*)
        write (prtunit,*) ' comp_reg_3d  '
        write (prtunit,*) '        -f=input_netcdf_file'
        write (prtunit,*) '        -v=netcdf_variable'
        write (prtunit,*) '        -m=input_mesh_mask_netcdf_file          (optional)'
        write (prtunit,*) '        -g=grid_type                            (optional : n, t, u, v, w, f)'
        write (prtunit,*) '        -x=lon1,lon2                            (optional)'
        write (prtunit,*) '        -y=lat1,lat2                            (optional)'
        write (prtunit,*) '        -t=time1,time2                          (optional)'
        write (prtunit,*) '        -p=periodicity                          (optional)'
        write (prtunit,*) '        -a=type_of_analysis                     (optional : reg, residual, predict, all)'
        write (prtunit,*) '        -o=output_netcdf_file                   (optional)'
        write (prtunit,*) '        -to=time_origin                         (optional)'
        write (prtunit,*) '        -dg=polynomial_degree                   (optional)'
        write (prtunit,*) '        -fi=input_index_netcdf_file             (optional)'
        write (prtunit,*) '        -vi=index_netcdf_variable               (optional)'
        write (prtunit,*) '        -ti=itime1,itime2                       (optional)'
        write (prtunit,*) '        -pi=iperiodicity,istep                  (optional)'
        write (prtunit,*) '        -ni=index_for_2d_index_netcdf_variable  (optional)'
        write (prtunit,*) '        -sm=smoothing_factor                    (optional)'
        write (prtunit,*) '        -dv=dv_time1,dv_time2                   (optional)'
        write (prtunit,*) '        -mi=missing_value                       (optional)'
        write (prtunit,*) '        -use_eps=tol                            (optional)'
        write (prtunit,*) '        -comp_min_norm                          (optional)'
        write (prtunit,*) '        -add_mean                               (optional)'
        write (prtunit,*) '        -rsquare                                (optional)'
        write (prtunit,*) '        -adjrsquare                             (optional)'
        write (prtunit,*) '        -stderr                                 (optional)'
        write (prtunit,*) '        -ftest                                  (optional)'
        write (prtunit,*) '        -ttest                                  (optional)'
        write (prtunit,*) '        -double                                 (optional)'
#ifdef _USE_NETCDF36
        write (prtunit,*) '        -bigfile                                (optional)'
#endif
#ifdef _USE_NETCDF4
        write (prtunit,*) '        -hdf5                                   (optional)'
#endif
        write (prtunit,*) '        -tlimited                               (optional)'
        write (prtunit,*)
        write (prtunit,*) ' By default : '
        write (prtunit,*)
        write (prtunit,*) '-m= by default, an input_mesh_mask_netcdf_file is not used'
        write (prtunit,*) '-g=n'
        write (prtunit,*) '-x=the whole longitude domain associated with the netcdf_variable'
        write (prtunit,*) '-y=the whole latitude domain associated with the netcdf_variable'
        write (prtunit,*) '-t=the whole time period  associated with the netcdf_variable'
        write (prtunit,*) '-p=1'
        write (prtunit,*) '-a=reg'
        write (prtunit,*) '-o=reg_netcdf_variable.index_netcdf_variable.nc if the -vi= argument'
        write (prtunit,*) '   is used and reg_netcdf_variable.polytrend.nc otherwise'        
        write (prtunit,*) '-to=by default, the origin for the time scale is set to zero'
        write (prtunit,*) '-dg=1 : by default, a linear trend is estimated'
        write (prtunit,*) '-fi=by default, a polynomial trend is estimated'
        write (prtunit,*) '-vi=by default, a polynomial trend is estimated'
        write (prtunit,*) '-ti=the whole time period  associated with the index_netcdf_variable'
        write (prtunit,*) '-pi=1,1 : by default, this parameter is not used'
        write (prtunit,*) '-ni=1   : by default, if the index_netcdf_variable is bidimensional,'
        write (prtunit,*) '          the first time series is used'
        write (prtunit,*) '-sm=by default, no smoothing is applied to the index_netcdf_variable'
        write (prtunit,*) '-dv=by default, a dummy variable is not used'
        write (prtunit,*) '-mi=by default, the missing_value is equal to 1.e+20'
        write (prtunit,*) '-use_eps=by default, no tolerance is used for solving the llsq problem'
        write (prtunit,*) '-comp_min_norm : by default, the minimal norm solution is not computed'
        write (prtunit,*) '-add_mean      : by default, the means are not added to the residuals'
        write (prtunit,*) '-rsquare       : by default, the coefficient of determination is not computed'
        write (prtunit,*) '-adjrsquare    : by default, the adjusted coefficient of determination is'
        write (prtunit,*) '                 not computed'
        write (prtunit,*) '-stderr        : by default, the standard errors of the estimates are not computed'
        write (prtunit,*) '-ftest         : by default, a F-test is not computed'
        write (prtunit,*) '-ttest         : by default, Student-tests are not computed'
        write (prtunit,*) '-double        : by default, the results are stored as single floating numbers'
#ifdef _USE_NETCDF36
        write (prtunit,*) '-bigfile : by default, a netCDF classical format file is created'
#endif
#ifdef _USE_NETCDF4
        write (prtunit,*) '-hdf5     : by default, a NetCDF classical format file is created'
#endif
        write (prtunit,*) '-tlimited : by default, the time dimension is defined as unlimited'
#ifdef _OPENMP
        write (prtunit,*)
        write (prtunit,*) 'This procedure is parallelized with OpenMP'
#endif
        write (prtunit,*)
!
        stop
!
    else if ( narg<2 .or. narg>32 ) then
!
        call merror( name_proc//"Wrong number of line arguments" )
!
    end if
!
    fic_name   = " "
    var_name   = " "
    fici_name  = " "
    vari_name  = " "
    mask_name  = " "
    reg_name   = " "
!
    history = "comp_reg_3d"
!
!   DEFAULT VALUES FOR COMMAND LINE ARGUMENTS.
!
    output_mode   = nf90_clobber
    output_type   = nf90_real4
    method        = 1
    regtype       = 1
    period        = 1
    nperiod       = 0
    smooth_factor = 0
    dg            = 1
    torigin       = 0
!
    eps           = zero
    missing       = def_missing
!
    grid_type     = "n"
!
    lon1      = 1
    lon2      = huge( lon2 )
    lat1      = 1
    lat2      = huge( lat2 )
    time1     = 1
    time2     = huge( time2 )
!
    itime1    = 1
    itime2    = huge( itime2 )
    iperiod   = 1
    istep     = 1
    niperiod  = 0
    iindex    = 1
!
    dvtime1   = 0
    dvtime2   = 0
!
    poly_trend      = true
    use_dv          = false
    comp_resid      = false
    comp_predict    = false
    comp_min_norm   = false
    add_mean        = false
    comp_rsquare    = false
    comp_adjrsquare = false
    comp_stderr     = false
    comp_ftest      = false
    comp_ttest      = false
!
    use_mask        = false
    use_x           = false
    use_y           = false
    create_hdf5     = false
    use_tlimited    = false
!
!   GET THE COMMAND LINE ARGUMENTS.
!
    do i = 1, narg
!
        call getarg( i, name )
!
        history = trim(history)//" "//trim(name)
!
        call case_change( name(:3), tolower )
!
        select case ( name(1:3 ) )
!
            case ('-f=')
!
!               GET THE NAME OF THE NETCDF DATASET.
!
                fic_name = name(4:)
!
            case ('-v=')
!
!               GET THE NAME OF THE NETCDF VARIABLE IN THE DATASET.
!
                var_name = name(4:)
!
            case ('-g=')
!
!               GET THE GRID-TYPE.
!
                call case_change( name(4:), tolower )
!
                select case ( name(4:) )
                    case ( 'n', 'u', 'v', 'w', 't', 'f' )
                        grid_type = name(4:4)
                    case default
                        call merror( name_proc//"Unknown grid type" )
                end select
!
            case ('-x=')
!
!               GET THE DOMAIN DEFINITION (Longitude).
!
                call get_listarg( name_proc, name(4:),                    &
                                  "lon1 and lon2 are integer values",     &
                                  "Wrong domain definition (longitude)",  &
                                  lon1, lon2 )
!
                use_x = true
!
            case ('-y=')
!
!               GET THE DOMAIN DEFINITION (latitude).
!
                call get_listarg( name_proc, name(4:),                    &
                                  "lat1 and lat2 are integer values",     &
                                  "Wrong domain definition (latitude)",   &
                                  lat1, lat2 )
!
                use_y = true
!
            case ('-t=')
!
!               GET THE PERIOD DEFINITION.
!
                call get_listarg( name_proc, name(4:),                    &
                                  "time1 and time2 are integer values",   &
                                  "Wrong period definition",              &
                                  time1, time2 )
!
            case ('-m=')
!
!               GET THE NAME OF THE NETCDF MESH-MASK DATASET.
!
                mask_name = name(4:)
!
                use_mask = true
!
            case ('-o=')
!
!               GET THE NAME OF THE NEW NETCDF DATASET FOR STORING THE RESULTS.
!
                reg_name = name(4:)
!
            case ('-p=')
!
!               GET THE PERIODICITY.
!
                call string_to_val( name(4:), kcode, fmt )
!
                if ( kcode==kint ) then
                    read( name(4:), fmt ) period
                end if
!
                if ( kcode/=kint .or. period<=0 ) &
                call merror( name_proc//"The -p= parameter is a strictly positive integer" )
!
            case ('-a=')
!
!               DETERMINE IF RESIDUALS OR PREDICTIONS ARE REQUIRED.
!
                call case_change( name(4:), tolower )
!
                select case ( name(4:) )
                    case ( cmethod(1) )
                        method = 1
                    case ( cmethod(2) )
                        method     = 2
                        comp_resid = true
                    case ( cmethod(3) )
                        method     = 3
                        comp_predict = true
                    case ( cmethod(4) )
                        method     = 4
                        comp_resid   = true
                        comp_predict = true
                    case default
                        call merror( name_proc//"Unknown analysis option" )
                end select
!
            case default
!
                select case ( name(1:4 ) )
!
                    case ('-fi=')
!
!                       GET THE NAME OF THE NETCDF DATASET FOR THE INDEX VARIABLE.
!
                        fici_name = name(5:)
!
                    case ('-vi=')
!
!                       GET THE NAME OF THE NETCDF INDEX VARIABLE IN THE DATASET.
!
                        vari_name = name(5:)
!
                        poly_trend = false
                        regtype    = 2
!
                    case ('-ti=')
!
!                       GET THE PERIOD DEFINITION FOR THE INDEX VARIABLE.
!
                        call get_listarg( name_proc, name(5:),                            &
                                          "itime1 and itime2 are integer values",         &
                                          "Wrong period definition for index variable",   &
                                          itime1, itime2 )
!
                    case ('-pi=')
!
!                       GET THE PERIODICITY AND THE TIME STEP FOR THE INDEX.
!
                        call get_listarg( name_proc, name(5:),                                 &
                                          "iperiod and istep are integer values",              &
                                          "Wrong periodicity definition for index variable",   &
                                          iperiod, istep )
!
                        if ( iperiod<=0 .or. istep<=0 .or. istep>iperiod )   &
            call merror( name_proc//"The -pi= parameters are positive integers with istep<=iperiod" )
!
                    case ('-ni=')
!
!                       GET THE INDEX FOR A 2D INDEX VARIABLE.
!
                        call string_to_val( name(5:), kcode, fmt )
                        if ( kcode==kint ) then
                            read( name(5:), fmt ) iindex
                        end if
                        if ( kcode/=kint .or. iindex<=0  )                           &
             call merror( name_proc//"The index for a 2D index variable is a positive integer value" )
!
                    case ('-sm=')
!
!                       DETERMINE THE SMOOTHING FACTOR.
!
                        call string_to_val( name(5:), kcode, fmt )
                        if ( kcode==kint ) then
                            read( name(5:), fmt ) smooth_factor
                        end if
                        if ( smooth_factor<=0 )        &
         call merror( name_proc//"The -sm= argument is a strictly positive integer value " )
!
                    case ('-to=')
!
!                       DETERMINE THE ORIGIN FOR THE TIME SCALE.
!
                        call string_to_val( name(5:), kcode, fmt )
                        if ( kcode==kint ) then
                            read( name(5:), fmt ) torigin
                        else
         call merror( name_proc//"The -to= argument is an integer value " )
                        end if
!
                    case ('-dg=')
!
!                       DETERMINE THE POLYNOMIAL DEGREE.
!
                        call string_to_val( name(5:), kcode, fmt )
                        if ( kcode==kint ) then
                            read( name(5:), fmt ) dg
                        end if
                        if ( dg<=0 .or. kcode/=kint )        &
         call merror( name_proc//"The -dg= argument is a strictly positive integer value " )
!
                    case ('-dv=')
!
!                       GET THE DUMMY VARIABLE DEFINITION.
!
                        call get_listarg( name_proc, name(5:),                         &
                                          "dv_time1 and dv_time2 are integer values",  &
                                          "Wrong dummy variable definition",           &
                                          dvtime1, dvtime2 )
!
                        use_dv = true
!
                    case ('-mi=')
!
!                       GET THE MISSING VALUE INDICATOR.
!
                        call string_to_val( name(5:), kcode, fmt )
                        if ( kcode/=kfix .and. kcode/=kexp )                &
                    call merror( name_proc//"The -mi= argument must be a real number " )
                        read( name(5:), fmt ) missing
!
                    case default
!
                        call case_change( name(4:), tolower )
!
                        select case ( name(1:9) )
!
                            case ('-use_eps=')
!
!                               DETERMINE THE TOLERANCE FOR SOLVING LINEAR LEAST SQUARES PROBLEMS.
!
                                call string_to_val( name(10:), kcode, fmt )
                                if ( kcode==kfix .or. kcode==kexp ) then
                                    read( name(10:), fmt ) eps
                                end if
                                if ( (kcode/=kfix .and. kcode/=kexp) .or.     &
                                     eps<zero                        .or.     &
                                     eps>=one )                               &
      call merror( name_proc//"The -use_eps= argument must be a real number and verify 0<=use_eps<1 " )
!
                            case default
!
                                select case ( name(1:) )
!
                                      case ('-comp_min_norm')
!
                                          comp_min_norm = true
!
                                      case ('-add_mean')
!
                                          add_mean = true
!
                                      case ('-rsquare')
!
                                          comp_rsquare = true
!
                                      case ('-adjrsquare')
!
                                          comp_adjrsquare = true
!
                                      case ('-stderr')
!
                                          comp_stderr = true
!
                                      case ('-ftest')
!
                                          comp_ftest = true
!
                                      case ('-ttest')
!
                                          comp_ttest = true
!
                                      case ('-double')
!
!                                         STORE RESULTS AS DOUBLE PRECISION NUMBERS.
!
                                          output_type = nf90_real8
!
#ifdef _USE_NETCDF36
                                      case ('-bigfile')
!
!                                         REDEFINE THE CREATION MODE FLAG FOR THE OUTPUT FILE.
!
                                          output_mode = nf90_64bit_offset
#endif
!
#ifdef _USE_NETCDF4
                                      case ('-hdf5')
!
!                                         REDEFINE THE CREATION MODE FLAG FOR THE OUTPUT FILE.
!
                                          output_mode = nf90_hdf5
                                          create_hdf5 = true
#endif
!
                                      case ('-tlimited')
!
!                                         DEFINE THE TIME DIMENSION AS LIMITED.
!
                                          use_tlimited = true
!
                                      case default
!
!                                         WRONG ARGUMENT.
!
                                          call merror( name_proc//"Wrong argument" )
!
                                  end select
!
                          end select
!
                end select
!
        end select
!
    end do
!
!   CHECK THE COMMAND LINE ARGUMENTS.
!
    if ( fic_name==" " )                                                 &
    call merror( name_proc//"Input NETCDF file is missing" )
!
    if ( var_name==" " )                                                 &
    call merror( name_proc//"Input NETCDF variable is missing" )
!
    if ( poly_trend )   then
!
        if ( reg_name==" " )                                                  &
        reg_name="reg_"//trim(var_name)//".poly_trend.nc"
!
    else
!
!       RESET THE DEFAULT VALUE OF dg VARIABLE IN CASE IT HAS BEEN CHANGED BY THE USER.
!
        dg = 1
!
        if ( fici_name==" " )  fici_name = fic_name
!
        if ( reg_name==" " )                                                  &
        reg_name="reg_"//trim(var_name)//"."//trim(vari_name)//".nc"
!
    end if
!
!   SET LOGICAL FLAGS FOR THE REQUIRED COMPUTATIONS.
!
    add_mean  = add_mean .and. comp_resid
!
    comp_ssr = comp_rsquare .or. comp_adjrsquare .or. comp_ftest
!
    comp_stat = add_mean .or. comp_ssr
!
    comp_ssr = comp_ssr .or. comp_stderr .or. comp_ttest
!
!   OPEN THE NETCDF DATASETS.
!
    k = nf90_sizehint_default
    status = nf90_open( trim(fic_name), nf90_nowrite, ncid, chunksize=k )
    if ( status/=nf90_noerr )                                             &
    call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    if ( use_mask ) then
!
        status = nf90_open( trim(mask_name), nf90_nowrite, ncid_mask )
        if ( status/=nf90_noerr )                                             &
        call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    end if
!
    if ( .not.poly_trend ) then
!
        if ( fici_name/=fic_name ) then
            status = nf90_open( trim(fici_name), nf90_nowrite, ncidi )
            if ( status/=nf90_noerr )                                     &
            call merror( name_proc//trim( nf90_strerror( status ) ) )
        else
            ncidi = ncid
        end if
!
    end if
!
!   GET THE VARIABLE ID FROM ITS NAME.
!   FIND THE DIMENSION LENGTHS AND NAMES FOR THE NETCDF VARIABLE.
!   GET THE COORDINATE VARIABLES IDS FROM THEIR NAMES, IF ANY.
!   FIND THE EXTERNAL TYPE, THE DIMENSION ID AND THE NUMBER OF ATTRIBUTES 
!   FOR THE COORDINATE VARIABLES.
!
    call get_variable_dimcoords(  name_proc, trim(var_name), ncid,                    &
                                  varid,  dimids_out=dimids, dim_names_out=dim_names, &
                                  dimlens_out=dimlens, coor_id_out=coor_id,           &
                                  coor_natts_out=coor_natts, coor_type_out=coor_type, &
                                  use_coor_variable_out=use_coor_variable             )
!
    if ( count(dimlens/=0)/=3 )           &
    call merror( name_proc//"Input NETCDF variable is not tridimensional" )
!
    nlon   = dimlens(1)
    nlat   = dimlens(2)
    nt     = dimlens(3)
!
    if ( use_mask ) then
!
        chr = merge( 't', grid_type, grid_type=='w' )
!
!       GET THE MASK ID FROM ITS NAME.
!       VERIFY THE DIMENSION LENGTHS FOR THE NETCDF MASK VARIABLE.
!
        call get_variable_dims( name_proc, chr//"mask", ncid_mask, maskid, dimids, dimlens )
!
        if ( dimlens(1)/=nlon .or. dimlens(2)/=nlat )                      &
        call merror( name_proc//"Shape disagreement with the mask" )
!
!       GET THE BOUNDARIES OF THE GEOGRAPHICAL DOMAIN FROM THE MESH-MASK DATASET.
!
        if ( .not.use_x ) then
!
            status = nf90_get_att( ncid_mask, maskid, "lon1_Eastern_limit", i )
            if ( status==nf90_noerr ) lon1 = i
!
            status = nf90_get_att( ncid_mask, maskid, "lon2_Western_limit", i )
            if ( status==nf90_noerr ) lon2 = i
!
        end if
!
        if ( .not.use_y ) then
!
            status = nf90_get_att( ncid_mask, maskid, "lat1_Southern_limit", i )
            if ( status==nf90_noerr ) lat1 = i
!
            status = nf90_get_att( ncid_mask, maskid, "lat2_Northern_limit", i )
            if ( status==nf90_noerr ) lat2 = i
!
        end if
!
    end if
!
!   CHECK THE BOUNDARIES OF THE GEOGRAPHICAL DOMAIN.
!
    if ( lon1<0 ) then
        lon2 = max( lon2, 1 )
        lon3 = nlon + lon1 - lon2 + 1
    else
        lon1 = max( lon1, 1 )
        lon2 = min( lon2, nlon )
        lon3 = lon2 - lon1 + 1
    end if
!
    if ( lon3<=0  )  call merror( name_proc//"Wrong domain definition (longitude)" )
!
    if ( lon1<0 ) then
       lon3 = nlon
       j1   = 1
       j2   = nlon
    else
       j1 = lon1
       j2 = lon2
    end if
!
    lat1 = max( lat1, 1 )
    lat2 = min( lat2, nlat )
    lat3 = lat2 - lat1 + 1
    if ( lat3<=0  )  call merror( name_proc//"Wrong domain definition (latitude)" )
!
!   CHECK THE TIME DEFINITION.
!
    time1 = max( time1, 1 )
    time2 = min( time2, nt )
    time3 = time2 - time1 + 1
    if ( time3<=0  ) call merror( name_proc//"Wrong time definition" )
!
!   CHECK THE PERIOD DEFINITION.
!
    if ( period>time3 )  call merror( name_proc//"-t=, -p= options mismatch" )
!
    nperiod = time3/period
    if ( time3/=nperiod*period )      &
    call merror( name_proc//"The selected time period is not a multiple of the periodicity" )
!
!   CHECK THE DUMMY VARIABLE DEFINITION.
!
    if ( use_dv ) then
!
        dvtime1 = max( dvtime1, 1 )
        dvtime2 = min( dvtime2, nperiod )
        dvtime3 = dvtime2 - dvtime1 + 1
        if ( dvtime3<=0 .or. dvtime3==nperiod  )       &
        call merror( name_proc//"Wrong dummy variable definition" )  
!
    end if
!
!   ALLOCATE WORK ARRAYS.
!
    allocate( temp(nlon,nlat), mask(nlon,nlat), stat=status )
    if ( status/=0 ) call merror( name_proc//allocate_error )
!
    if ( use_mask ) then
!
!       GET THE MASK.
!
        status = nf90_get_var( ncid_mask, maskid, temp )     
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!       DETERMINE THE GRID POINTS USED IN THE REGRESSION ANALYSIS FROM THE MASK.
!
        mask(:nlon,:nlat) = temp(:nlon,:nlat)==one
!
    else
!
!       CONSTRUCT A DUMMY MASK.
!
        mask(:nlon,:nlat) = true
!
    end if
!
!   SELECT THE DOMAIN.
!
    if ( lon1>0 ) then
        mask(1:lon1-1,:nlat)    = false
        mask(lon2+1:nlon,:nlat) = false
    else
        mask(lon2+1:nlon+lon1,:nlat) = false
    end if
!
    mask(:nlon,1:lat1-1)    = false
    mask(:nlon,lat2+1:nlat) = false
!   
    nvar = count( mask )
    if ( nvar<=0  )  call merror( name_proc//"Wrong domain definition : domain is empty" )
!
!   ALLOCATE WORK ARRAYS.
!
    nreg = dg + 1
    if ( use_dv ) nreg = nreg + 1
    nn = period*nvar
!
    allocate( b(nperiod,nn), x(nreg,nn), mat(nperiod,nreg), regid(nreg), stat=status   )
    if ( status/=0 ) call merror( name_proc//allocate_error )
!
    mat(1:nperiod,1) = one
!
    if ( poly_trend ) then
!
        tmp = real( torigin, stnd )
        b(1:nperiod,1) = arth( tmp, one, nperiod )
!
        do i = 2, dg+1
            mat(1:nperiod,i) = mat(1:nperiod,i-1)*b(1:nperiod,1)
        end do
!
    else
!
!       GET THE INDEX VARIABLE ID FROM ITS NAME.
!       FIND THE DIMENSION LENGTHS FOR THE NETCDF INDEX VARIABLE.
!
        call get_variable_dims( name_proc, trim(vari_name), ncidi,    &
                                varidi, dimids, dimlens )
!
        nidim = count(dimlens/=0)
        if ( nidim/=1 .and. nidim/=2 )     &
        call merror( name_proc//"Input index NETCDF variable is not uni or bidimensional" )
!
        if ( nidim==2 ) then
            if ( iindex>dimlens(1) )     &
            call merror( name_proc//"Selected index for 2D index_variable is out of range " )
            nti = dimlens(2)
        else 
            nti = dimlens(1)
        end if
!
!       CHECK THE TIME DEFINITION FOR THE INDEX VARIABLE.
!
        itime1 = max( itime1, 1 )
        itime1 = itime1 + istep - 1
        itime2 = min( itime2, nti )
        itime3 = itime2 - itime1 + 1
        if ( itime3<=0  ) call merror( name_proc//"Wrong time definition for index NETCDF variable" )
!
!       CHECK THE PERIOD DEFINITION FOR THE INDEX VARIABLE.
!
        if ( iperiod>itime3 )  call merror( name_proc//"-ti=, -pi= options mismatch" )
!
        tmp      = real(itime3,stnd)/real(iperiod,stnd)
        niperiod = ceiling( tmp )
!
        if ( niperiod/=nperiod )  call merror( name_proc//"-t=, -p=, -ti=, -pi= options mismatch" )
!
!       GET INDEX VARIABLE.
!
        if ( nidim==1 ) then
            status = nf90_get_var( ncidi, varidi, mat(1:nperiod,2),                               &   
                                   start=(/ itime1 /), count=(/ nperiod /), stride=(/ iperiod /) )
        else 
            status = nf90_get_var( ncidi, varidi, mat(1:nperiod,2),                               &   
                                   start=(/ iindex, itime1 /), count=(/ 1, nperiod /), stride=(/ 1, iperiod /) )
        end if
!
        if ( status/=nf90_noerr )                                     &
        call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!       FIND THE MISSING VALUE ATTRIBUTE ASSOCIATED WITH THE INDEX NETCDF VARIABLE, IF ANY.
!
        status = nf90_get_att( ncidi, varidi, "missing_value", tmp )
        use_miss = status==nf90_noerr
!
        if ( .not.use_miss ) then
            status = nf90_get_att( ncidi, varidi, "_FillValue", tmp )
            use_miss = status==nf90_noerr
        end if
!
        if ( use_miss ) then
            if ( any( mat(1:nperiod,2)==tmp ) )      &
            call merror( name_proc//"Missing values are not allowed" )
        end if
!
!       FIND THE SCALE_FACTOR ATTRIBUTE ASSOCIATED WITH THE INDEX NETCDF VARIABLE, IF ANY.
!
        status = nf90_get_att( ncidi, varidi, "scale_factor", tmp )
        if ( status==nf90_noerr ) mat(1:nperiod,2) = mat(1:nperiod,2)*tmp
!
!       FIND THE ADD_OFFSET ATTRIBUTE ASSOCIATED WITH THE INDEX NETCDF VARIABLE, IF ANY.
!
        status = nf90_get_att( ncidi, varidi, "add_offset", tmp )
        if ( status==nf90_noerr ) mat(1:nperiod,2) = mat(1:nperiod,2) + tmp
!
!       SMOOTH THE INDEX TIME SERIES IF REQUIRED.
!
        if ( smooth_factor/=0 ) then
            call comp_smooth( mat(1:nperiod,2), smooth_factor )
        end if
!
    end if
!
!   DEFINE DUMMY VARIABLE IF REQUIRED.
!
    if ( use_dv ) then
        mat(1:nperiod,nreg)       = zero
        mat(dvtime1:dvtime2,nreg) = one
    end if
!
!   SAVE PREDICTORS IF REQUIRED.
!
    if ( comp_predict ) then
!
        allocate( mat2(nperiod,nreg), stat=status   )
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
        mat2(1:nperiod,1:nreg) = mat(1:nperiod,1:nreg)

    end if
!
!   FIRST READ THE NETCDF VARIABLE.
!
    countlec(1:3)  = (/ lon3, lat3, 1 /)
!
#ifdef _PARALLEL_READ
!
    i = omp_get_num_procs()
    j = omp_get_max_threads()
    test_par = i>1 .and. j>1 .and. time3*nvar>omp_limit .and. nperiod>1
!
    if ( test_par ) then
!
        ioffset = time1 - 1
!
!       DEALLOCATE WORK ARRAY.
!
        deallocate( temp )
!
!$OMP PARALLEL PRIVATE(i,l,k,i1,i2,status,startlec,ncid2,varid2,temp)          &
!$OMP         ,SHARED(b,mask,ioffset,j1,j2,lat1,lat3,lon3,nvar,period,countlec)
!
!       ALLOCATE WORK ARRAY.
!
        allocate( temp(lon3,lat3), stat=status )
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
!$OMP CRITICAL (opennc)
!
!       OPEN THE NETCDF DATASET.
!
        k = nf90_sizehint_default
        status = nf90_open( trim(fic_name), nf90_share, ncid2, chunksize=k  )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!       GET THE VARIABLE ID FROM ITS NAME.
!
        status = nf90_inq_varid( ncid2, trim(var_name), varid2 )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!$OMP END CRITICAL (opennc)
!
        startlec(1)  = j1
        startlec(2)  = lat1
!
!$OMP DO SCHEDULE(STATIC)
        do i = 1, nperiod
!
            l = ioffset + (i-1)*period
            i2 = 0
!
            do k = 1, period
!
                i1          = i2 + 1
                i2          = i2 + nvar
                l           = l + 1
                startlec(3) = l
!
!               FILL THE WORK ARRAY WITH A RECORD FROM THE NETCDF VARIABLE IN THE INPUT FILE.
!
                status = nf90_get_var( ncid2, varid2, temp(:lon3,:lat3), start=startlec, count=countlec )
                                           
                if ( status/=nf90_noerr )   call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!               PACK THE DATA.
!
                b(i,i1:i2) = pack( temp(:lon3,:lat3), mask(j1:j2,lat1:lat2) )
!
            end do
!
        end do
!$OMP END DO
!
!       DEALLOCATE WORK ARRAY.
!
        deallocate( temp )
!
!$OMP CRITICAL (closenc)
!
!       CLOSE THE NETCDF DATASET.
!
        status = nf90_close( ncid2 )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!$OMP END CRITICAL (closenc)
!
!$OMP END PARALLEL
!
    else
!
#endif
        l             = time1 - 1
        startlec(1:2) = (/ j1, lat1 /) 
!
        do i = 1, nperiod
!
            i2 = 0
!
            do k = 1, period
!
                i1          = i2 + 1
                i2          = i2 + nvar
                l           = l + 1
                startlec(3) = l
!
!               FILL THE WORK ARRAY WITH A RECORD FROM THE NETCDF VARIABLE IN THE INPUT FILE.
!
                status = nf90_get_var( ncid, varid, temp(:lon3,:lat3),                         &
                                       start=startlec, count=countlec )     
                if ( status/=nf90_noerr )   call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!               PACK THE DATA.
!
                b(i,i1:i2) = pack( temp(:lon3,:lat3), mask(j1:j2,lat1:lat2) )
!
            end do
!
        end do
!
!       DEALLOCATE WORK ARRAY.
!
        deallocate( temp )
!
#ifdef _PARALLEL_READ
!
    end if
!
#endif
!
!   FIND THE SCALE_FACTOR ATTRIBUTE ASSOCIATED WITH THE NETCDF VARIABLE, IF ANY.
!
    status = nf90_get_att( ncid, varid, "scale_factor", tmp )
    if ( status==nf90_noerr ) b(1:nperiod,1:nn) = b(1:nperiod,1:nn)*tmp
!
!   FIND THE ADD_OFFSET ATTRIBUTE ASSOCIATED WITH THE NETCDF VARIABLE, IF ANY.
!
    status = nf90_get_att( ncid, varid, "add_offset", tmp )
    if ( status==nf90_noerr )  b(1:nperiod,1:nn) = b(1:nperiod,1:nn) + tmp
!
!   COMPUTE MEANS AND CORRECTED SUMS OF SQUARES IF REQUIRED.
!
    if ( comp_stat ) then
!
!       ALLOCATE WORK ARRAYS.
!
        allocate( bmean(nn), brsquare(nn), bstd(nn), stat=status   )
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
!       COMPUTE MEANS AND CORRECTED SUMS OF SQUARES.
!
        call comp_mvs( b(1:nperiod,1:nn), true, false, bmean(1:nn), brsquare(1:nn), bstd(1:nn), dimvar=2 )
!
    end if
!
    if ( comp_ssr ) then
!
!       COMPUTE THE REGRESSION MODEL WITH A LINEAR LEAST SQUARE SOLVER.
!
        if ( comp_stderr .or. comp_ttest ) then
!
!           ALLOCATE WORK ARRAYS.
!
            if ( comp_ttest ) then
!
                allocate( bssr(nn), diagr(nreg), stderrid(nreg), stderr(nreg,nn),      &
                          tprobid(nreg), tstat(nreg,nn), tprob(nreg,nn), stat=status   )
!
            else
!
                allocate( bssr(nn), diagr(nreg), stderrid(nreg), stderr(nreg,nn), stat=status   )
!
            end if
!
            if ( status/=0 ) call merror( name_proc//allocate_error )
!
            krank = 0
!
            call llsq_qr_solve2( mat(1:nperiod,1:nreg), b(1:nperiod,1:nn),x(1:nreg,1:nn),              &
                                 rnorm=bssr(1:nn), comp_resid=comp_resid, krank=krank, tol=eps,        &
                                 min_norm=comp_min_norm, diagr=diagr(:nreg), ip=stderrid(:nreg)        )
!
            if ( krank==nreg ) then
!
!               COMPUTE THE STANDARD ERRORS OF THE ESTIMATE IF THE REGRESSION MODEL IS OF FULL RANK.
!
!               RESTORE THE DIAGONAL ELEMENTS OF THE TRIANGULAR FACTOR R OF THE QR DECOMPOSITION OF mat.
!
                do i=1, nreg
                    mat(i,i) = diagr(i)
                end do
!
!               COMPUTE UPPER PART OF INVERSE OF THE TRIANGULAR FACTOR R.
!
                call comp_triang_inv( mat(1:nreg,1:nreg) )
!
!               COMPUTE THE PRODUCT (R-1)(R-1)t.
!
                call comp_uut_ltl( mat(1:nreg,1:nreg), fill=false )
!
                tmp = one/real( nperiod-nreg, stnd )
!
                do i=1, nreg
                    stderr(i,1) = sqrt( tmp*mat(i,i) )
                end do
!
!               RE-ORDER THE DIAGONAL OF THE PRODUCT (R-1)(R-1)t  TO COMPENSATE FOR THE COLUMN INTERCHANGES.
!
                diagr(stderrid(1:nreg)) = stderr(1:nreg,1)
!
!               COMPUTE THE STANDARD ERRORS OF THE ESTIMATES.
!
                stderr(1:nreg,:nn) = outerprod( diagr(1:nreg), bssr(1:nn) )
!
                if ( comp_ttest ) then
!
!                   COMPUTE THE T-RATIO AND ASSESS ITS SIGNIFICANCE.
!
                    where ( stderr(1:nreg,1:nn)/=zero )
                        tstat(1:nreg,1:nn) = x(1:nreg,1:nn)/stderr(1:nreg,1:nn)
                    elsewhere
                        tstat(1:nreg,1:nn) = zero
                    end where
!
                    tmp = real( nperiod-nreg, stnd )
                    tprob(1:nreg,1:nn) = probstudent( tstat(1:nreg,1:nn), tmp )
!
                    where ( stderr(1:nreg,1:nn)==zero )  tprob(1:nreg,1:nn) = missing
!
                end if
!
            else
!
                stderr(:nreg,:nn) = missing
!
                if ( comp_ttest ) then
!
                    tprob(:nreg,:nn) = missing
!
                end if
!
            end if
!
       else
!
!          ALLOCATE WORK ARRAY.
!
           allocate( bssr(nn), stat=status   )
           if ( status/=0 ) call merror( name_proc//allocate_error )
!
            call llsq_qr_solve2( mat(1:nperiod,1:nreg), b(1:nperiod,1:nn),x(1:nreg,1:nn),                 &
                                 rnorm=bssr(1:nn), comp_resid=comp_resid, min_norm=comp_min_norm, tol=eps )
!
        end if
!
    else
!
!       COMPUTE THE REGRESSION MODEL WITH A LINEAR LEAST SQUARE SOLVER.
!
        call llsq_qr_solve2( mat(1:nperiod,1:nreg), b(1:nperiod,1:nn),x(1:nreg,1:nn),      &
                             comp_resid=comp_resid, min_norm=comp_min_norm, tol=eps        )
!
    end if
!
    if ( add_mean ) then
!
!       ADD THE MEANS TO THE RESIDUAL TIME SERIES.
!
        do i=1, nn
            b(1:nperiod,i) = b(1:nperiod,i) + bmean(i)
        end do
!
    end if
!
    if ( comp_rsquare .or. comp_adjrsquare .or. comp_ftest ) then
!
!       COMPUTE THE COEFFICIENT OF DETERMINATION OF THE MODEL.
!
        where( brsquare(1:nn)/=zero)
            brsquare(1:nn) = one - ((bssr(1:nn)**2)/brsquare(1:nn))
        elsewhere
            brsquare(1:nn) = missing
        end where
!
        if ( comp_ftest ) then
!
!           ALLOCATE WORK ARRAY.
!
            allocate( fprob(nn), stat=status   )
            if ( status/=0 ) call merror( name_proc//allocate_error )
!
!           COMPUTE A F-STATISTIC TO TEST THE NULLITY OF THE REGRESSION COEFFICIENTS.
!
            tmp = real( nperiod-nreg, stnd )/real( nreg-1 , stnd )
!
            where( brsquare(1:nn)/=missing .and. brsquare(1:nn)/=one )
                bmean(1:nn) = tmp*(brsquare(1:nn)/(one-brsquare(1:nn)))
            elsewhere
                bmean(1:nn) = zero
            end where
!
            i1 = nreg - 1
            i2 = nperiod - nreg
!
            fprob(1:nn) = probf( bmean(1:nn), i1, i2, true )
!
            where( brsquare(1:nn)/=missing )
                fprob(1:nn) = merge( fprob(1:nn), zero, brsquare(1:nn)/=one )
            elsewhere
                fprob(1:nn) = missing
            end where
!
        end if
!
    end if
!
!   CREATE THE NEW NETCDF DATASET FOR THE REGRESSION ANALYSIS.
!
    if ( create_hdf5 ) then
        status = nf90_create( trim(reg_name), cmode=output_mode, ncid=ncid_reg )
    else
        k = nf90_sizehint_default
        status = nf90_create( trim(reg_name), cmode=output_mode, ncid=ncid_reg, chunksize=k )
    end if
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_set_fill( ncid_reg, nf90_nofill, i )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!   DEFINE THE DIMENSIONS FOR THE NEW NETCDF DATASET.
!
    dimlens(1)   = nlon
    dimlens(2)   = nlat
!
    if ( comp_resid .or. comp_predict ) then
!
        dimlens(3)   = period
!
        if ( use_tlimited ) then
            dimlens(4)   = time3
        else
            dimlens(4)   = nf90_unlimited
        end if
!
        dim_names(4)         = dim_names(3)
        use_coor_variable(4) = use_coor_variable(3)
        coor_id(4)           = coor_id(3)
        coor_type(4)         = coor_type(3)
        coor_natts(4)        = coor_natts(3)
!
        ndim2 = 4
!
    else
!
        if ( use_tlimited ) then
            dimlens(3)   = period
        else
            dimlens(3)   = nf90_unlimited
        end if
!
        ndim2 = 3
!
    end if
!
    dim_names(3)         = "period"
    coor_type(3)         = nf90_int
    use_coor_variable(3) = false
!
    do i = 1, ndim2
!
        status = nf90_def_dim( ncid_reg, name=trim(dim_names(i)),                         &
                               len=dimlens(i), dimid=dimids(i) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim(nf90_strerror(status)) )
!
    end do
!
!   DEFINE THE VARIABLES FOR THE NEW NETCDF DATASET.
!
    do i = 1, ndim2
!
        status = nf90_def_var( ncid_reg, trim(dim_names(i)), coor_type(i),                &
                               dimids=dimids(i), varid=coor_id2(i) )
        if ( status/=nf90_noerr )                                                         &
        call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end do
!
    if ( poly_trend ) then
        name = trim(var_name)//"_poly_trend"
    else
        name = trim(var_name)//"_"//trim(vari_name)
    end if
!
    name2 = trim(name)//"_reg"
!
    do i = 1, dg+1
!
        call val_to_string( i-1, fmt, j )
        name3 = trim(name2)//fmt(1:j)
!
        status = nf90_def_var( ncid_reg, trim(name3), output_type, dimids=dimids(:3), varid=regid(i) )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        if ( comp_stderr ) then
!
            name4 = trim(name3)//"_stderr"
!
            status = nf90_def_var( ncid_reg, trim(name4), output_type, dimids=dimids(:3), varid=stderrid(i) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end if
!
        if ( comp_ttest ) then
!
            name4 = trim(name3)//"_tprob"
!
            status = nf90_def_var( ncid_reg, trim(name4), output_type, dimids=dimids(:3), varid=tprobid(i) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end if
!
    end do
!
    if ( use_dv ) then
!
        status = nf90_def_var( ncid_reg, trim(name)//"_dv", output_type, dimids=dimids(:3), varid=regid(nreg) )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        if ( comp_stderr ) then
!
            status = nf90_def_var( ncid_reg, trim(name)//"_dv_stderr", output_type, dimids=dimids(:3), varid=stderrid(nreg) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end if
!
        if ( comp_ttest ) then
!
            status = nf90_def_var( ncid_reg, trim(name)//"_dv_tprob", output_type, dimids=dimids(:3), varid=tprobid(nreg) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end if
!
    end if
!
    if ( comp_rsquare ) then
        status = nf90_def_var( ncid_reg, trim(name)//"_r2", output_type, dimids=dimids(:3), varid=rsquareid )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
    end if
!
    if ( comp_adjrsquare ) then
        status = nf90_def_var( ncid_reg, trim(name)//"_adjr2", output_type, dimids=dimids(:3), varid=adjrsquareid )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
    end if
!
    if ( comp_ftest ) then
        status = nf90_def_var( ncid_reg, trim(name)//"_fprob", output_type, dimids=dimids(:3), varid=fprobid )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
    end if
!
    if ( comp_resid ) then
        dimids(3) = dimids(4)
        status = nf90_def_var( ncid_reg, trim(name)//"_resid", output_type, dimids=dimids(:3), varid=residid )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
    end if
!
    if ( comp_predict ) then
        dimids(3) = dimids(4)
        status = nf90_def_var( ncid_reg, trim(name)//"_predict", output_type, dimids=dimids(:3), varid=predictid )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
    end if
!
!   ASSIGN ATTRIBUTES FOR THE NEW NETCDF DATASET.
!
    do j = 1, ndim2
!
        if ( .not.use_coor_variable(j) ) cycle
!
        do i = 1, coor_natts(j)
!
            status = nf90_inq_attname( ncid, coor_id(j), i, att_name )
            if ( status/=nf90_noerr )                                                     &
            call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           COPY ATTRIBUTES FOR THE COORDINATE VARIABLES.
!
            status = nf90_copy_att(  ncid, coor_id(j), att_name, ncid_reg, coor_id2(j) )
            if ( status/=nf90_noerr )                                                     &
            call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end do
!
    end do
!
    status = nf90_put_att( ncid_reg, coor_id2(3), "long_name", "Period axis" )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_reg, coor_id2(3), "short_name", "Period" )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    if ( ( comp_resid .or. comp_predict ) .and. .not.use_coor_variable(4) ) then
!
        status = nf90_put_att( ncid_reg, coor_id2(4), "long_name", "Time axis" )
        if ( status/=nf90_noerr )                                                  &
        call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_reg, coor_id2(4), "short_name", "Time" )
        if ( status/=nf90_noerr )                                                  &
        call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end if
!
    if ( poly_trend ) then
!
        name = trim(var_name)//" by poly_trend"
!
    else
!
        if ( nidim==1 ) then
            name = trim(var_name)//" by "//trim(vari_name)
        else
            call val_to_string( iindex, fmt, i )
            name = trim(var_name)//" by "//trim(vari_name)//"("//fmt(1:i)//")"
        end if
!
    end if
!
    status = nf90_put_att( ncid_reg, regid(1), "long_name", "Intercept of "//trim(name)  )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_reg, regid(1), "short_name", "Intercept"  )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    do i = 2, dg+1
!
        status = nf90_put_att( ncid_reg, regid(i), "long_name", "Regression of "//trim(name)  )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_reg, regid(i), "short_name", "Regression"  )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        if ( poly_trend ) then
!
            j = i - 1
!
            status = nf90_put_att( ncid_reg, regid(i), "degree_term", j  )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end if
!
    end do
!
    if ( use_dv ) then
!
        status = nf90_put_att( ncid_reg, regid(nreg), "long_name", "Dummy variable" )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_reg, regid(nreg), "short_name", "Dum_var"  )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_reg, regid(nreg), "dv_time1", dvtime1  )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_reg, regid(nreg), "dv_time2", dvtime2  )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    end if
!
    if ( comp_stderr ) then
!
        do i = 1, nreg
!
            status = nf90_put_att( ncid_reg, stderrid(i), "long_name", "Standard error of estimate" )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_reg, stderrid(i), "short_name", "STD_error"  )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end do
!
    end if
!
    if ( comp_ttest ) then
!
        do i = 1, nreg
!
            status = nf90_put_att( ncid_reg, tprobid(i), "long_name", "Significance probability of T-ratio" )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_reg, tprobid(i), "short_name", "T-ratio Probability"  )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end do
!
    end if
!
    if ( comp_rsquare ) then
!
        status = nf90_put_att( ncid_reg, rsquareid, "long_name",               &
                               "Coefficient of determination of "//trim(name)  )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_reg, rsquareid, "short_name", "R2"  )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    end if
!
    if ( comp_ftest ) then
!
        status = nf90_put_att( ncid_reg, fprobid, "long_name",               &
                               "Significance probability of F-ratio"//trim(name)  )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_reg, fprobid, "short_name", "F-ratio Probability"  )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    end if
!
    if ( comp_resid ) then
!
        status = nf90_put_att( ncid_reg, residid, "long_name", "Residuals of "//trim(name)  )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_reg, residid, "short_name", "Residuals"  )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    end if
!
    if ( comp_predict ) then
!
        status = nf90_put_att( ncid_reg, predictid, "long_name", "Predictions of "//trim(name)  )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_reg, predictid, "short_name", "Predictions"  )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    end if
!
    if (  output_type==nf90_real8 ) then
!
        do i = 1, nreg
!
            status = nf90_put_att( ncid_reg, regid(i), "missing_value", real( missing, dp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_reg, regid(i), "_FillValue", real( missing, dp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            if ( comp_stderr ) then
!
                status = nf90_put_att( ncid_reg, stderrid(i), "missing_value", real( missing, dp ) )
                if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
                status = nf90_put_att( ncid_reg, stderrid(i), "_FillValue", real( missing, dp ) )
                if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            end if
!
            if ( comp_ttest ) then
!
                status = nf90_put_att( ncid_reg, tprobid(i), "missing_value", real( missing, dp ) )
                if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
                status = nf90_put_att( ncid_reg, tprobid(i), "_FillValue", real( missing, dp ) )
                if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            end if
!
        end do
!
        if ( comp_rsquare ) then
!
            status = nf90_put_att( ncid_reg, rsquareid, "missing_value", real( missing, dp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_reg, rsquareid, "_FillValue", real( missing, dp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_reg, rsquareid, "valid_min", real( zero, dp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_reg, rsquareid, "valid_max", real( one, dp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end if
!
        if ( comp_adjrsquare ) then
!
            status = nf90_put_att( ncid_reg, adjrsquareid, "missing_value", real( missing, dp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_reg, adjrsquareid, "_FillValue", real( missing, dp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end if
!
        if ( comp_ftest ) then
!
            status = nf90_put_att( ncid_reg, fprobid, "missing_value", real( missing, dp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_reg, fprobid, "_FillValue", real( missing, dp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end if
!
        if ( comp_resid ) then
!
            status = nf90_put_att( ncid_reg, residid, "missing_value", real( missing, dp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_reg, residid, "_FillValue", real( missing, dp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end if
!
        if ( comp_predict ) then
!
            status = nf90_put_att( ncid_reg, predictid, "missing_value", real( missing, dp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_reg, predictid, "_FillValue", real( missing, dp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end if
!
    else
!
        do i = 1, nreg
!
            status = nf90_put_att( ncid_reg, regid(i), "missing_value", real( missing, sp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_reg, regid(i), "_FillValue", real( missing, sp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            if ( comp_stderr ) then
!
                status = nf90_put_att( ncid_reg, stderrid(i), "missing_value", real( missing, sp ) )
                if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
                status = nf90_put_att( ncid_reg, stderrid(i), "_FillValue", real( missing, sp ) )
                if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            end if
!
            if ( comp_ttest ) then
!
                status = nf90_put_att( ncid_reg, tprobid(i), "missing_value", real( missing, sp ) )
                if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
                status = nf90_put_att( ncid_reg, tprobid(i), "_FillValue", real( missing, sp ) )
                if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            end if
!
        end do
!
        if ( comp_rsquare ) then
!
            status = nf90_put_att( ncid_reg, rsquareid, "missing_value", real( missing, sp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_reg, rsquareid, "_FillValue", real( missing, sp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_reg, rsquareid, "valid_min", real( zero, sp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_reg, rsquareid, "valid_max", real( one, sp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end if
!
        if ( comp_adjrsquare ) then
!
            status = nf90_put_att( ncid_reg, adjrsquareid, "missing_value", real( missing, sp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_reg, adjrsquareid, "_FillValue", real( missing, sp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end if
!
        if ( comp_ftest ) then
!
            status = nf90_put_att( ncid_reg, fprobid, "missing_value", real( missing, sp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_reg, fprobid, "_FillValue", real( missing, sp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end if
!
        if ( comp_resid ) then
!
            status = nf90_put_att( ncid_reg, residid, "missing_value", real( missing, sp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_reg, residid, "_FillValue", real( missing, sp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end if
!
        if ( comp_predict ) then
!
            status = nf90_put_att( ncid_reg, predictid, "missing_value", real( missing, sp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_reg, predictid, "_FillValue", real( missing, sp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end if
!
    end if
!
    name2(1:nchar) = " "
    status = nf90_get_att( ncid, varid, "units", name2 )
    if ( status==nf90_noerr ) then
!
        status = nf90_put_att( ncid_reg, regid(1), "units", trim(name2)  )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        if ( comp_stderr ) then
!
            status = nf90_put_att( ncid_reg, stderrid(1), "units", trim(name2)  )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end if
!
        if ( .not.poly_trend ) then
!
            name3(1:nchar) = " "
            status = nf90_get_att( ncidi, varidi, "units", name3 )
!
            if ( status==nf90_noerr ) then
!
                status = nf90_put_att( ncid_reg, regid(2), "units", trim(name2)//" by "//trim(name3)  )
                if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
                if ( comp_stderr ) then
!
                    status = nf90_put_att( ncid_reg, stderrid(2), "units", trim(name2)//" by "//trim(name3)  )
                    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
                end if
!
            end if
!
        end if
!
        if ( use_dv ) then
            status = nf90_put_att( ncid_reg, regid(nreg), "units", trim(name2)  )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
        end if
!
        if ( comp_resid ) then
            status = nf90_put_att( ncid_reg, residid, "units", trim(name2)  )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
        end if
!
        if ( comp_predict ) then
            status = nf90_put_att( ncid_reg, predictid, "units", trim(name2)  )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
        end if
!
    end if
!
!   SET GLOBAL ATTRIBUTES.
!
    status = nf90_put_att( ncid_reg, nf90_global, "title", "Regression analysis of "//trim(name) )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_reg, nf90_global, "grid_type", grid_type )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    if ( poly_trend ) then
!
        status = nf90_put_att( ncid_reg, nf90_global, "time_origin", torigin )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_reg, nf90_global, "polynomial_degree", dg )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    else
!
        status = nf90_put_att( ncid_reg, nf90_global, "iperiod", iperiod )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_reg, nf90_global, "istep", istep )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_reg, nf90_global, "smoothing_factor", smooth_factor )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end if
!
    if ( comp_resid ) then
!
        if ( add_mean ) then
            i = 1
        else
            i = 0
        end if
!
        status = nf90_put_att( ncid_reg, nf90_global, "add_mean", i )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end if
!
    status = nf90_put_att( ncid_reg, nf90_global, "regtype", cregtype(regtype) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_reg, nf90_global, "method", cmethod(method) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_reg, nf90_global, "history", trim(history) )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!   LEAVE DEFINE MODE FOR THE NEW NETCDF DATASET.
!
    status = nf90_enddef( ncid_reg )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!   WRITE VARIABLES FOR THE NEW  NETCDF DATASET.
!
!
!   ALLOCATE WORK ARRAY.
!
    i = max( nlon, nlat, period )
!
    allocate( record(i), stat=status )
    if ( status/=0 ) call merror( name_proc//allocate_error )
!
    dimlens(3) = period
!
    do j = 1, 3   
!
        j2 = dimlens(j)
!
        if ( use_coor_variable(j) ) then
            status = nf90_get_var( ncid, coor_id(j), record(:j2), count=(/ j2 /) )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_var( ncid_reg, coor_id2(j), record(:j2), count=(/ j2 /) )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
        else
            status = nf90_put_var( ncid_reg, coor_id2(j), arth(1_i4b,1_i4b,j2 ) )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
        end if
!
    end do
!
!   DEALLOCATE WORK ARRAY.
!
    deallocate( record )
!
    if ( comp_resid .or. comp_predict ) then
!
!       WRITE TIME VARIABLE FOR THE NEW  NETCDF DATASET, IF REQUIRED.
!
        if ( use_coor_variable(4) ) then
!
!           ALLOCATE WORK ARRAY.
!
            allocate( record(time3), stat=status )
            if ( status/=0 ) call merror( name_proc//allocate_error )
!
            status = nf90_get_var( ncid, coor_id(4), record, start=(/ time1 /) )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_var( ncid_reg, coor_id2(4), record )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!           DEALLOCATE WORK ARRAY.
!
            deallocate( record )
!
        else
!
            status = nf90_put_var( ncid_reg, coor_id2(4), arth(time1,1_i4b,time3 ) )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        end if
!
    end if
!
!   WRITE THE REGRESSION COEFFICIENTS.
!
    do j = 1, nreg
!
        call write_file_3d( name_proc, x(j,:nn), mask(:nlon,:nlat), period, missing, ncid_reg, regid(j) )
!
    end do
!
    if ( comp_stderr ) then
!
!       WRITE STANDARD-ERRORS OF THE REGRESSION COEFFICIENTS.
!
        do j = 1, nreg
!
            call write_file_3d( name_proc, stderr(j,:nn), mask(:nlon,:nlat), period, missing, ncid_reg, stderrid(j) )
!
        end do    
!
!       DEALLOCATE WORK ARRAYS.    
!
        deallocate( stderr, stderrid, diagr )
!
    end if
!
    if ( comp_ttest ) then
!
!       WRITE T-TEST PROBABILITY.
!
        do j = 1, nreg
!
            call write_file_3d( name_proc, tprob(j,:nn), mask(:nlon,:nlat), period, missing, ncid_reg, tprobid(j) )
!
        end do    
!
!       DEALLOCATE WORK ARRAYS.    
!
        if ( comp_stderr ) then
            deallocate( tstat, tprob, tprobid  )
        else
            deallocate( tstat, tprob, tprobid, stderr, diagr, stderrid )
        end if
!
    end if
!
    if ( comp_rsquare ) then
!
!       WRITE COEFFICIENTS OF DETERMINATION.
!
        call write_file_3d( name_proc, brsquare(:nn), mask(:nlon,:nlat), period, missing, ncid_reg, rsquareid )
!
    end if
!
    if ( comp_adjrsquare ) then
!
!       COMPUTE ADJUSTED COEFFICIENTS OF DETERMINATION.
!
        tmp = one/real( nperiod-nreg, stnd )
!
        where( brsquare(1:nn)/=missing )    &
           brsquare(1:nn) = tmp*((real(nperiod-1,stnd)*brsquare(1:nn)) - real(nreg-1,stnd))
!
!       WRITE ADJUSTED COEFFICIENTS OF DETERMINATION.
!
        call write_file_3d( name_proc, brsquare(:nn), mask(:nlon,:nlat), period, missing, ncid_reg, adjrsquareid )
!
    end if
!
    if ( comp_ftest ) then
!
!       WRITE F-TEST PROBABILITY.
!
        call write_file_3d( name_proc, fprob(:nn), mask(:nlon,:nlat), period, missing, ncid_reg, fprobid )
!
!       DEALLOCATE WORK ARRAY.
!
        deallocate( fprob )
!
    end if
!
    if ( comp_resid ) then
!
!       WRITE THE RESIDUALS IF REQUIRED.
!
        call write_file_3d( name_proc, b(:nperiod,:nn), mask(:nlon,:nlat), period, missing, ncid_reg, residid )
!
    end if
!
    if ( comp_predict ) then
!
!       COMPUTE THE PREDICTIONS IF REQUIRED.
!
        b(1:nperiod,1:nn) = matmul( mat2(1:nperiod,1:nreg), x(1:nreg,1:nn) )
!
!       WRITE THE PREDICTIONS.
!
        call write_file_3d( name_proc, b(:nperiod,:nn), mask(:nlon,:nlat), period, missing, ncid_reg, predictid )
!
!       DEALLOCATE WORK ARRAYS.
!
        deallocate( mask, x, b, mat, mat2, regid )
!
    else
!
!       DEALLOCATE WORK ARRAYS.
!
        deallocate( mask, x, b, mat, regid )
!
    end if
!
    if ( comp_stat ) then
!
!       DEALLOCATE WORK ARRAYS.
!
        deallocate( bmean, brsquare, bstd )
!
    end if
!
    if ( comp_ssr ) then
!
!       DEALLOCATE WORK ARRAY.
!
        deallocate( bssr )
!
    end if
!
!   CLOSE THE NETCDF DATASETS.
!
    status = nf90_close( ncid )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    if ( .not.poly_trend ) then
!
        if ( ncid/=ncidi ) then
            status = nf90_close( ncidi )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
        end if
!
    end if
!
    if ( use_mask ) then
        status = nf90_close( ncid_mask )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
    end if
!
    status = nf90_close( ncid_reg )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
! END OF PROGRAM comp_reg_3d
! ==========================
!
end program comp_reg_3d
