program comp_season_3d
!! #############################################################################
!!
!! Copyright 2021 IRD
!!
!! This file is part of NCSTAT.
!!
!! NCSTAT is free software: you can redistribute it and/or modify
!! it under the terms of the GNU Lesser General Public License as 
!! published by the Free Software Foundation, either version 3 of 
!! the License, or (at your option) any later version.
!!
!! NCSTAT is distributed in the hope that it will be useful,
!! but WITHOUT ANY WARRANTY; without even the implied warranty of
!! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!! GNU Lesser General Public License for more details.
!!
!! You can find a copy of the GNU Lesser General Public License
!! in the NCSTAT/doc directory.
!!
!! #############################################################################
!
!+
!
! .. _comp_season_3d:
!
! ==============
! comp_season_3d
! ==============
!
! Authors
! =======
!
! Pascal Terray (LOCEAN/IPSL)
!
! Latest revision
! ===============
!
! 18/02/2021
!
! Purpose
! =======
!
! Compute time (i.e. seasonal, daily, ...) averages from a tridimensional variable extracted from
! a NetCDF dataset and write the results in an output NetCDF dataset.
!
! The time averages are computed for each point in the time series of the 2-D
! grid-mesh associated with the input NetCDF variable and the time series are assumed to be
! strictly periodic with a periodicity given by the **-p=** argument (this means, in particular,
! that comp_season_3d cannot be used to compute monthly time series from observed daily data).
! Note that the date information associated with each time average in the output NetCDF dataset
! is the date of the first contributing input observation in each time average (if this date
! information is available in the input NetCDF dataset).
!
! If your data contains missing values (excepted those associated with a constant land-sea mask)
! use :ref:`comp_season_miss_3d` instead of comp_season_3d to estimate time averages from your
! gappy dataset. Finally, if the NetCDF variable is fourdimensional use :ref:`comp_season_4d`
! instead of comp_season_3d.
!
! Further Details
! ===============
!
! Usage
! -----
!
! ::
!
!       $ comp_season_3d \
!         -f=input_netcdf_file \
!         -v=netcdf_variable \
!         -p=periodicity_of_data \
!         -s=length_of_season \
!         -o=output_seasonal_netcdf_file \
!         -x=lon1,lon2                      (optional) \
!         -y=lat1,lat2                      (optional) \
!         -t=time1,time2                    (optional) \
!         -m=input_mesh_mask_netcdf_file    (optional) \
!         -g=grid_type                      (optional : n, t, u, v, w, f) \
!         -compact                          (optional) \
!         -bigfile                          (optional) \
!         -hdf5                             (optional) \
!         -tlimited                         (optional)
!
! By default
! ----------
!
!        *-x=*
!         the whole longitude domain associated with the *netcdf_variable*
!        *-y=*
!         the whole latitude domain associated with the *netcdf_variable*
!        *-t=*
!         the whole time period associated with the *netcdf_variable*
!        *-m=*
!         an *input_mesh_mask_netcdf_file* is not used. In this case, it is assumed
!         that the *netcdf_variable* does not have any missing values
!        *-g=*
!         the *grid_type* is set to ``n`` which means that the 2-D grid-mesh associated with
!         the input NetCDF variable is assumed to be regular or Gaussian
!        *-compact*
!         the output NetCDF dataset is not compacted
!        *-bigfile*
!         a NetCDF classical format file is created. If **-bigfile** is activated,
!         the output NetCDF file is a 64-bit offset format file
!        *-hdf5*
!         a NetCDF classical format file is created. If **-hdf5** is activated,
!         the output NetCDF file is a NetCDF-4/HDF5 format file
!        *-tlimited*
!         the time dimension is defined as unlimited in the output NetCDF file.
!         However, if **-tlimited** is activated, the time dimension is defined
!         as limited in the output NetCDF file
!
! Remarks
! -------
!
!   #) The **-v=**\ *netcdf_variable* argument specifies the NetCDF variable which must be
!      averaged and the **-f=**\ *input_netcdf_file* argument specifies that this NetCDF variable
!      must be extracted from the NetCDF file, *input_netcdf_file*.
!
!   #) The **-p=**\ *periodicity_of_data* argument gives the periodicity of the input data for the
!      *netcdf_variable*. For example, with monthly data **-p=**\ ``12`` should be specified.
!
!   #) The **-s=**\ *length_of_season* argument gives the length of the season or the time interval for averaging the
!      input data (i.e. the number of time observations used to compute the time averages in the output NetCDF file).
!      The specified number for the **-s=** argument should be an exact divisor of the number specified
!      in the **-p=** argument. For example, to construct a seasonal (averages of four months)
!      dataset from a monthly dataset, **-p=**\ ``12`` and **-s=**\ ``4`` should be specified.
!
!   #) If the **-x=**\ *lon1,lon2* and **-y=**\ *lat1,lat2* arguments are missing the whole geographical
!      domain associated with the *netcdf_variable* is used to construct the output NetCDF file.
!
!      The longitude or latitude range must be a vector of two integers specifying the first
!      and last selected indices along each dimension. The indices are relative to ``1``\ .
!      Negative values are not allowed for *lon1*.
!
!      Refer to :ref:`comp_mask_3d` for transforming geographical coordinates as indices or
!      generating an appropriate mesh-mask before using comp_season_3d.
!
!   #) If the **-t=**\ *time1,time2* argument is missing the whole time period
!      associated with the *netcdf_variable* is used to construct the output NetCDF file.
!      The selected time period is not necessarily a multiple of the number of observations
!      used to compute the time averages (e.g. the **-s=** argument). If this is not the case,
!      the last time average is computed with less observations (i.e. less observations
!      than the number specified in the **-s=** argument).
!
!   #) If **-g=** is set to ``t``, ``u``, ``v``, ``w`` or ``f`` it is assumed that the NetCDF variable
!      is from an experiment with the ORCA (R2, R4 or R05 resolutions) model.
!
!      The **-g=** argument is used only if a mesh-mask NetCDF dataset is specified with the **-m=** argument.
!      This argument is also used to determine the name of the NetCDF mesh_mask variable if an 
!      *input_mesh_mask_netcdf_file* is used as specified with the **-m=** argument.
!
!   #) The geographical shapes of the *netcdf_variable* (in the *input_netcdf_file*) and the mask
!      (in the *input_mesh_mask_netcdf_file*) must agree if the **-m=** argument is used.
!
!   #) It is assumed that the data have no missing values, excepted those associated with a constant
!      land-sea mask (if a mesh-mask NetCDF file is used). If it is the case, use :ref:`comp_season_miss_3d`
!      instead of comp_season_3d.
!
!   #) If the **-compact** argument is specified and a domain is selected (with the **-x=** and **-y=**
!      arguments) then only the data for the selected domain will be written in the output NetCDF file.
!      By default, the whole grid is stored (with missing values outside the selected domain) in the
!      output NetCDF file.
!
!   #) The **-bigfile** argument is allowed only if the NCSTAT software has been compiled with the
!      _USE_NETCDF36 or _USE_NETCDF4 macros (e.g. ``-D_USE_NETCDF36`` or ``-D_USE_NETCDF4``) and linked
!      to the NetCDF 3.6 library or higher.
!      If this argument is specified, the *output_netcdf_file* will be a 64-bit offset format file
!      instead of a NetCDF classic format file. However, this argument is recognized in the procedure
!      only if the NCSTAT software has been built with the _USE_NETCDF36 or _USE_NETCDF4 CPP macros.
!
!   #) The **-hdf5** argument is allowed only if the NCSTAT software has been
!      compiled with the _USE_NETCDF4 macro (e.g. ``-D_USE_NETCDF4``) and linked
!      to the NetCDF 4 library or higher. 
!      If this argument is specified, the *output_netcdf_file* will be a NetCDF-4/HDF5
!      format file instead of a NetCDF classic format file. However, this
!      argument is recognized in the procedure only if the NCSTAT software has
!      been built with the _USE_NETCDF4 CPP macro.
!
!   #) Duplicate parameters are allowed, but this is always the last occurrence of a parameter
!      which will be used for the computations. Moreover, the number of specified parameters
!      must not be greater than the total number of allowed parameters.
!
! Outputs
! -------
!
!   comp_season_3d creates an output NetCDF file that contains time averages computed from the time series
!   associated with the input NetCDF variable and the coordinate NetCDF variables of the input NetCDF dataset
!   *input_netcdf_file* .
!   This NetCDF variable will have the same dimensions and name as the input NetCDF variable
!   in the file *input_netcdf_file* (in the description below, nlat and nlon are the length
!   of the spatial dimensions of the input NetCDF variable) and *nseason* time observations (corresponding to
!   the number of time averages which have been computed from the input time series) :
!
!   #) *netcdf_variable* ``(nseason,nlat,nlon)`` : the computed time averages for
!      each point in the time series of the 2-D grid-mesh associated with the input NetCDF variable.
!
!   By default, the whole grid associated with the input NetCDF variable is stored (with missing values outside
!   the selected domain). Note, however, that if the argument **-compact** is used, the geographical dimensions of
!   the output NetCDF variable will be reduced to the selected domain as specified by the **-x=** and **-y=** arguments
!   (e.g. in this case ``nlat``\ =\ *lat2*\ -\ *lat1*\ +\ ``1`` and ``nlon``\ =\ *lon2*\ -\ *lon1*\ +\ ``1``  ).
!
!   The number of time steps written in the output NetCDF file (e.g. ``ntime``\ ) is determined from the **-t=**, **-s=**
!   and **-p=** arguments.
!
! Examples
! --------
!
!   #)   For computing seasonal averages (i.e. time averages of 4 months) from a monthly tridimensional
!        NetCDF variable ``sosstsst`` in the NetCDF file ``ST7_1m_00101_20012_grid_T_sosstsst.nc``
!        and store the results in a NetCDF file named ``ST7_4m_00101_20012_grid_T_sosstsst.nc``
!        use the following command (note that cyclostationarity is assumed for the ``sosstsst``
!        variable since  **-p**\ =\ ``12`` is specified) :
!
!        ::
!
!          $ comp_season_3d \
!            -f=ST7_1m_0101_20012_grid_T_sosstsst.nc \
!            -v=sosstsst \
!            -p=12 \
!            -s=4 \
!            -m=mask_orca2.nc \
!            -o=sST7_4m_0101_20012_grid_T_sosstsst.nc
!
!-
! ================================================================================================
!
#ifdef _USE_NETCDF4
#define _USE_NETCDF36
#endif
!
! USED MODULES 
! ============
!
#ifdef _USE_NAGWARE
    use f90_unix_env,     only : iargc, getarg
#endif
!
#ifdef _USE_NETCDF36
    use netcdf,    only : nf90_64bit_offset
#endif
!
#ifdef _USE_NETCDF4
    use netcdf,    only : nf90_hdf5
#endif
!
    use netcdf,   only : nf90_max_var_dims, nf90_max_name, nf90_clobber,                               &
                         nf90_sizehint_default, nf90_open, nf90_nowrite, nf90_noerr,                   &
                         nf90_strerror, nf90_inq_varid, nf90_inquire_variable, nf90_inquire_dimension, &
                         nf90_get_att, nf90_create, nf90_def_dim, nf90_def_var, nf90_real8,            &
                         nf90_int, nf90_short, nf90_byte,                                              &
                         nf90_unlimited, nf90_inq_attname, nf90_copy_att, nf90_put_att, nf90_global,   &
                         nf90_enddef, nf90_get_var, nf90_put_var, nf90_close, nf90_set_fill, nf90_nofill
!
    use Statpack, only : i1b, i4b, stnd, lgl, one, true, false,                 &
                         allocate_error, merror, arth,                          &
                         kint, tolower, case_change, string_to_val
!
    use NCSTAT_Utilities, only : prtunit, nchar, dp, def_missing, get_listarg,  &
                                 get_variable_dims
!
!
! STRONG TYPING IMPOSED 
! =====================
!
    implicit none
!
!
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='comp_season_3d : '
!
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
!
#ifndef _USE_NAGWARE
    integer(i4b)  :: iargc
#endif
    integer(i4b)  :: narg, status, ncid, ncid_season, ncid_mask, varid, meanid, maskid, seasonid,     & 
                     nlon, nlat, nt, np, nperiod, ns, nseason, natts, xtype, i, j, j1, j2, j3, k,     &
                     lon1, lon2, lon3, lat1, lat2, lat3, time1, time2, time3, time2b, output_mode
    integer(i4b),              dimension(3)                 :: coor_type, coor_id2, startlec, stridelec, startwri,     &
                                                               coor_natts, coor_id
    integer(i4b),              dimension(nf90_max_var_dims) :: dimids, dimlens
    integer(i4b), allocatable, dimension(:)                 :: period
!
    integer(i1b)  :: kcode
!
    real(stnd)                                :: missing, invs
    real(stnd), allocatable, dimension(:,:,:) :: data
    real(stnd), allocatable, dimension(:,:)   :: mean
    real(stnd), allocatable, dimension(:)     :: temp
!
    logical(lgl)                                 :: use_missing, use_fillvalue, use_tlimited,           &
                                                    use_x, use_y, use_compact, use_short, create_hdf5
    logical(lgl), dimension(3)                   :: use_coor_variable
    logical(lgl), allocatable, dimension(:,:,:)  :: mask
!
    character                                  :: chr, grid_type
    character(len=14)                          :: fmt
    character(len=nchar)                       :: name, history
    character(len=nf90_max_name)               :: fic_name, var_name, season_name, mask_name, att_name
    character(len=nf90_max_name), dimension(3) :: dim_names
!
!
! EXECUTABLE STATEMENTS
! =====================
!
!   GET THE NUMBER OF COMMAND LINE ARGUMENTS.
!
    narg = iargc()
!
!   CHECK THE NUMBER OF COMMAND LINE ARGUMENTS.
!
    if ( narg==0 ) then
!
        write (prtunit,*)
        write (prtunit,*) ' Purpose : '
        write (prtunit,*)
        write (prtunit,*) ' Compute seasonal averages from a tridimensional variable '
        write (prtunit,*) ' extracted from a NetCDF dataset and write the results in '
        write (prtunit,*) ' a NetCDF dataset'
        write (prtunit,*)
        write (prtunit,*) ' Usage : '
        write (prtunit,*)
        write (prtunit,*) ' comp_season_3d -f=input_netcdf_file'
        write (prtunit,*) '                -v=netcdf_variable'
        write (prtunit,*) '                -p=periodicity'
        write (prtunit,*) '                -s=length of the season'
        write (prtunit,*) '                -o=output_seasonal_netcdf_file'
        write (prtunit,*) '                -x=lon1,lon2                      (optional)'
        write (prtunit,*) '                -y=lat1,lat2                      (optional)'
        write (prtunit,*) '                -t=time1,time2                    (optional)'
        write (prtunit,*) '                -m=input_mesh_mask_netcdf_file    (optional)'
        write (prtunit,*) '                -g=grid_type                      (optional : n, t, u, v, w, f)'
        write (prtunit,*) '                -compact                          (optional)'
#ifdef _USE_NETCDF36
        write (prtunit,*) '                -bigfile                          (optional)'
#endif
#ifdef _USE_NETCDF4
        write (prtunit,*) '                -hdf5                             (optional)'
#endif
        write (prtunit,*) '                -tlimited                         (optional)'
        write (prtunit,*)
        write (prtunit,*) ' By default : '
        write (prtunit,*)
        write (prtunit,*) '-x= the whole longitude domain associated with the netcdf_variable'
        write (prtunit,*) '-y= the whole latitude domain associated with the netcdf_variable'
        write (prtunit,*) '-t= the whole time period  associated with the netcdf_variable'
        write (prtunit,*) '-m= a mesh-mask NetCDF file is not used'
        write (prtunit,*) '-g= the grid type is set to n'
        write (prtunit,*) '-compact  : by default, the output NetCDF file is not compacted'
#ifdef _USE_NETCDF36
        write (prtunit,*) '-bigfile  : by default, a NetCDF classical format file is created'
#endif
#ifdef _USE_NETCDF4
        write (prtunit,*) '-hdf5     : by default, a NetCDF classical format file is created'
#endif
        write (prtunit,*) '-tlimited : by default, the time dimension is defined as unlimited'
        write (prtunit,*)
!
        stop
!
    else if ( narg<5 .or. narg>14 ) then
!
        call merror( name_proc//"Wrong number of line arguments" )
!
    end if
!
    fic_name    = " "
    var_name    = " "
    season_name = " "
    mask_name   = " "
!
    history = "comp_season_3d"
!
!   DEFAULT VALUES FOR COMMAND LINE ARGUMENTS.
!
    output_mode = nf90_clobber
!
    nperiod     = 0
    nseason     = 0
!
    grid_type   = "n"
!
    lon1      = 1
    lon2      = huge( lon2 )
    lat1      = 1
    lat2      = huge( lat2 )
    time1     = 1
    time2     = huge( time2 )
!
    use_x        = false
    use_y        = false
    use_compact  = false
    create_hdf5  = false
    use_tlimited = false
!
!   CHECK THE COMMAND LINE ARGUMENTS.
!
    do i = 1, narg
!
        call getarg( i, name )
!
        history = trim(history)//" "//trim(name)
!
        call case_change( name(:3), tolower )
!
        select case ( name(1:3 ) )
!
            case ('-f=')
!
!               GET THE NAME OF THE NETCDF DATASET.
!
                fic_name = name(4:)
!
            case ('-v=')
!
!               GET THE NAME OF THE NETCDF VARIABLE IN THE DATASET.
!
                var_name = name(4:)
!
            case ('-p=')
!
!               GET THE PERIODICITY.
!
                call string_to_val( name(4:), kcode, fmt )
                if ( kcode/=kint ) then
                    call merror( name_proc//"The periodicity is an integer value" )
                end if
                read( name(4:), fmt ) nperiod
!
            case ('-s=')
!
!               GET THE LENGTH OF THE SEASON.
!
                call string_to_val( name(4:), kcode, fmt )
                if ( kcode/=kint ) then
                    call merror( name_proc//"The length of the season is an integer value" )
                end if
                read( name(4:), fmt ) nseason
!
            case ('-o=')
!
!               GET THE NAME OF THE NEW NETCDF DATASET FOR THE SEASONAL DATA.
!
                season_name = name(4:)
!
            case ('-x=')
!
!               GET THE DOMAIN DEFINITION (Longitude).
!
                call get_listarg( name_proc, name(4:),                    &
                                  "lon1 and lon2 are integer values",     &
                                  "Wrong domain definition (longitude)",  &
                                  lon1, lon2 )
!
                use_x = true
!
            case ('-y=')
!
!               GET THE DOMAIN DEFINITION (latitude).
!
                call get_listarg( name_proc, name(4:),                    &
                                  "lat1 and lat2 are integer values",     &
                                  "Wrong domain definition (latitude)",   &
                                  lat1, lat2 )
!
                use_y = true
!
            case ('-t=')
!
!               GET THE PERIOD DEFINITION.
!
                call get_listarg( name_proc, name(4:),                    &
                                  "time1 and time2 are integer values",   &
                                  "Wrong period definition",              &
                                  time1, time2 )
!
            case ('-m=')
!
!               GET THE NAME OF THE OPTIONAL MASK NETCDF DATASET.
!
                mask_name = name(4:)
!
            case ('-g=')
!
!               GET THE GRID-TYPE.
!
                call case_change( name(4:), tolower )
!
                select case ( name(4:) )
                    case ( 'n', 'u', 'v', 'w', 't', 'f' )
                        grid_type = name(4:4)
                    case default
                        call merror( name_proc//"Unknown grid type" )
                end select
!
            case default
!
                call case_change( name(4:), tolower )
!
                select case ( name(1:) )
!
                    case ('-tlimited')
!
!                       DEFINE THE TIME DIMENSION AS LIMITED.
!
                        use_tlimited = true
!
#ifdef _USE_NETCDF36
                    case ('-bigfile')
!
!                       REDEFINE THE CREATION MODE FLAG FOR THE OUTPUT FILE.
!
                        output_mode = nf90_64bit_offset
#endif
!
#ifdef _USE_NETCDF4
                    case ('-hdf5')
!
!                       REDEFINE THE CREATION MODE FLAG FOR THE OUTPUT FILE.
!
                        output_mode = nf90_hdf5
                        create_hdf5 = true
#endif
!
                    case ('-compact')
!
!                       COMPACT THE OUTPUT FILE.
!
                        use_compact = true
!
                    case default
!
!                       WRONG ARGUMENT.
!
                        call merror( name_proc//"Wrong argument" )
!
                end select
!
        end select
!
    end do
!
!   CHECK THE COMMAND LINE ARGUMENTS.
!
    if ( fic_name==" " ) call merror( name_proc//"Input NETCDF file is missing" )
!
    if ( var_name==" " ) call merror( name_proc//"Input NETCDF variable is missing" )
!
    if ( season_name==" " ) call merror( name_proc//"Output NETCDF file is missing" )
!
    if ( nperiod<=0  ) call merror( name_proc//"The periodicity argument is missing or invalid" )
!
    if ( nseason<=0  ) call merror( name_proc//"The length of the season argument is missing or invalid" )
!
    ns = nperiod/nseason
    if ( ns*nseason/=nperiod  )      &
    call merror( name_proc//"The periodicity is not a multiple of the length of the season" )
!
!   OPEN THE NETCDF DATASET.
!
    k = nf90_sizehint_default
    status = nf90_open( trim(fic_name), nf90_nowrite, ncid, chunksize=k )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!   GET THE VARIABLE ID FROM ITS NAME.
!
    status = nf90_inq_varid( ncid, trim(var_name), varid )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!   FIND THE EXTERNAL TYPE, THE DIMENSION LENGTHS AND NAMES, 
!   THE NUMBER OF ATTRIBUTS FOR THE NETCDF VARIABLE.
!
    status = nf90_inquire_variable( ncid, varid, xtype=xtype, ndims=i, dimids=dimids, natts=natts )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    if ( i/=3 ) call merror( name_proc//"Input NETCDF variable is not tridimensional" )
!
    do i = 1, 3
!
        status = nf90_inquire_dimension( ncid, dimids(i), name=dim_names(i), len=dimlens(i) )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    end do
!
    nlon = dimlens(1)
    nlat = dimlens(2)
    nt   = dimlens(3)
!
    do i = 1, 3
!
!       GET THE COORDINATE VARIABLES IDS FROM THEIR NAME, IF ANY.
!
        status = nf90_inq_varid( ncid, trim(dim_names(i)), coor_id(i) )
        if ( status==nf90_noerr ) then
!
!           FIND THE EXTERNAL TYPE, THE DIMENSION ID AND THE NUMBER OF ATTRIBUTES 
!           FOR THE COORDINATE VARIABLES.
!
            status = nf90_inquire_variable( ncid, coor_id(i), xtype=coor_type(i), ndims=k,    &
                                            dimids=dimlens, natts=coor_natts(i) )
            if ( status/=nf90_noerr )                                                         &
            call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            use_coor_variable(i) = k==1 .and. dimlens(1)==dimids(i)
        else
            use_coor_variable(i) = false
        end if
!
        if ( .not.use_coor_variable(i) ) coor_type(i) = nf90_int
!
    end do
!
!   FIND THE MISSING INDICATOR VALUE ASSOCIATED WITH THE NETCDF VARIABLE, IF ANY.
!
    status = nf90_get_att( ncid, varid, "missing_value", missing )
    use_missing = status==nf90_noerr
!
    if ( .not.use_missing ) then
!
!       TRY THE FILLVALUE INDICATOR IF THE MISSING ATTRIBUT IS MISSING.
!
        status = nf90_get_att( ncid, varid, "_FillValue", missing )
        use_fillvalue = status==nf90_noerr
!
        if ( .not.use_fillvalue ) then
            missing = def_missing
            if ( mask_name/=" " )  xtype = nf90_real8
        end if
    else
        use_fillvalue = false
    end if
!
    if ( mask_name/=" " ) then
!
!       OPEN THE NETCDF MESH-MASK DATASET.
!
        status = nf90_open( trim(mask_name), nf90_nowrite, ncid_mask )
        if ( status/=nf90_noerr )                                            &
        call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        chr = merge( 't', grid_type, grid_type=='w' )
!
!       GET THE MASK ID FROM ITS NAME.
!       VERIFY THE DIMENSION LENGTHS FOR THE NETCDF MASK VARIABLE.
!
        call get_variable_dims( name_proc, chr//"mask", ncid_mask, maskid, dimids, dimlens )
!
        if ( dimlens(1)/=nlon .or. dimlens(2)/=nlat )                      &
        call merror( name_proc//"Shape disagreement with the mask" )
!
!       GET THE LONGITUDE BOUNDS, IF REQUIRED.
!
        if ( .not.use_x ) then
!
            status = nf90_get_att( ncid_mask, maskid, "lon1_Eastern_limit", i )
            if ( status==nf90_noerr ) then
!
                if ( i>0 ) then
!
                    lon1 = i
!
                    status = nf90_get_att( ncid_mask, maskid, "lon2_Western_limit", i )
                    if ( status==nf90_noerr ) lon2 = i
!
                end if
!
            end if
!
        end if
!
!       GET THE LATITUDE BOUNDS, IF REQUIRED.
!
        if ( .not.use_y ) then
!
            status = nf90_get_att( ncid_mask, maskid, "lat1_Southern_limit", i )
            if ( status==nf90_noerr ) lat1 = i
!
            status = nf90_get_att( ncid_mask, maskid, "lat2_Northern_limit", i )
            if ( status==nf90_noerr ) lat2 = i
!
        end if
!
    end if
!
!   CHECK THE BOUNDARIES OF THE GEOGRAPHICAL DOMAIN.
!
    lat1 = max( lat1, 1 )
    lat2 = min( lat2, nlat )
    lat3 = lat2 - lat1 + 1
    if ( lat3<=0  )  call merror( name_proc//"Wrong domain definition (latitude)" )
!
    if ( lon1<=0  )  call merror( name_proc//"Wrong domain definition (longitude)" )
    lon2 = min( lon2, nlon )
    lon3 = lon2 - lon1 + 1
    if ( lon3<=0  )  call merror( name_proc//"Wrong domain definition (longitude)" )
!
!   CHECK THE PERIOD DEFINITION.
!
    time1 = max( time1, 1 )
    time2 = min( time2, nt )
    time3 = time2 - time1 + 1
    if ( time3<=0 ) call merror( name_proc//"Wrong time definition" )
!
    np = time3/nseason
    if ( np*nseason/=time3 ) np = np + 1
!
!   CREATE THE NEW NETCDF DATASET FOR THE SEASONAL DATA.
!
    if ( create_hdf5 ) then
        status = nf90_create( trim(season_name), cmode=output_mode, ncid=ncid_season )
    else
        k = nf90_sizehint_default
        status = nf90_create( trim(season_name), cmode=output_mode, ncid=ncid_season, chunksize=k )
    end if
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    if ( use_compact ) then
        status = nf90_set_fill( ncid_season, nf90_nofill, i )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
    end if
!
!   DEFINE THE DIMENSIONS FOR THE SEASONAL NETCDF DATASET.
!
    if ( use_compact ) then
        startlec(1:3) = (/ lon1, lat1, time1 /)
        dimlens(1:2)  = (/ lon3, lat3 /)
    else
        startlec(1:3) = (/ 1_i4b, 1_i4b, time1 /)
        dimlens(1:2)  = (/ nlon, nlat /)
    end if
!
    if ( use_tlimited ) then
        dimlens(3)   = np
    else
        dimlens(3)   = nf90_unlimited
    end if
!
    stridelec(1:3) = (/ 1_i4b, 1_i4b, nseason /)
!
    do i = 1, 3
!
        status = nf90_def_dim( ncid_season, name=trim(dim_names(i)), len=dimlens(i), dimid=dimids(i) )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    end do
!
!   DEFINE THE VARIABLES FOR THE SEASONAL NETCDF DATASET.
!
    do i = 1, 3
!
        status = nf90_def_var( ncid_season, trim(dim_names(i)), coor_type(i),              &
                               dimids=dimids(i), varid=coor_id2(i) )
        if ( status/=nf90_noerr )                                                          &
        call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end do
!
    status = nf90_def_var( ncid_season, "season", nf90_int, dimids=dimids(3), varid=seasonid )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_def_var( ncid_season, trim(var_name), xtype, dimids=dimids(:3), varid=meanid )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!   ASSIGN ATTRIBUTES FOR THE SEASONAL NETCDF DATASET.
!
    do j = 1, 3
!
        if ( .not.use_coor_variable(j) ) cycle
!
        do i = 1, coor_natts(j)
!
            status = nf90_inq_attname( ncid, coor_id(j), i, att_name )
            if ( status/=nf90_noerr )                                                     &
            call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           COPY ATTRIBUTES FOR THE COORDINATE VARIABLES.
!
            status = nf90_copy_att(  ncid, coor_id(j), att_name, ncid_season, coor_id2(j) )
            if ( status/=nf90_noerr )                                                     &
            call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end do
!
    end do
!
!   CREATE ATTRIBUTES FOR THE TIME AXIS VARIABLE IF REQUIRED.
!
    if ( .not.use_coor_variable(3) ) then
!
        status = nf90_put_att( ncid_season, coor_id2(3), "long_name", "Time axis" )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_season, coor_id2(3), "short_name", "Time" )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    end if
!
!   CREATE ATTRIBUTES FOR THE SEASONAL AXIS VARIABLE.
!
    status = nf90_put_att( ncid_season, seasonid, "long_name", "Seasonal axis" )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_season, seasonid, "short_name", "Season" )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!   COPY USEFUL ATTRIBUTES FOR THE NETCDF VARIABLE.
!
    do i = 1, natts
!
        status = nf90_inq_attname( ncid, varid, i, att_name )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        select case ( att_name )
!
            case ( 'units', 'scale_factor', 'add_offset', 'long_name', 'short_name',  &
                   'missing_value', '_FillValue'  )
!
                status = nf90_copy_att(  ncid, varid, att_name, ncid_season, meanid )
                if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end select
!
    end do
!
    if ( .not.use_missing .and. .not.use_fillvalue .and. mask_name/=" " ) then
!
        status = nf90_put_att( ncid_season, meanid, "missing_value", real( missing, dp ) )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_season, meanid, "_FillValue", real( missing, dp ) )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    end if
!
!   SET GLOBAL ATTRIBUTES.
!
    status = nf90_put_att( ncid_season, nf90_global, "title", trim(var_name)//" Seasonal dataset" )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_season, nf90_global, "history", trim(history) )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!   LEAVE DEFINE MODE FOR THE SEASONAL NETCDF DATASET.
!
    status = nf90_enddef( ncid_season )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!   ALLOCATE WORK ARRAYS.
!
    dimlens(3) = np
    i = maxval( dimlens(1:3) )
!
    allocate( data(lon3,lat3,nseason),           &
              mean(lon3,lat3),                   &
              period(np),                        &
              temp(i),                           &
              stat=status )
    if ( status/=0 ) call merror( name_proc//allocate_error )
!
!   WRITE VARIABLES FOR THE SEASONAL NETCDF DATASET.
!
!   WRITE COORDINATE VARIABLES.
!
    do j = 1, 3    
!
        j2 = dimlens(j)
!
        if ( use_coor_variable(j) ) then
!
            j1 = startlec(j)
            j3 = stridelec(j)
!
            status = nf90_get_var( ncid, coor_id(j), temp(:j2), start= (/ j1 /), count=(/ j2 /), stride=(/ j3 /) )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_var( ncid_season, coor_id2(j), temp(:j2), count=(/ j2 /) )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        else
!
            status = nf90_put_var( ncid_season, coor_id2(j), arth(1_i4b,1_i4b,j2 ) )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        end if
!
    end do
!
!   CONSTRUCT SEASONAL AXIS VARIABLE.
!
    j = 1
!
    do i = 1, np
!
        period(i) = j
!
        if ( j==ns ) then
            j = 1
        else
            j = j + 1
        end if
!
    end do
!
!   WRITE SEASONAL AXIS VARIABLE.
!
    status = nf90_put_var( ncid_season, seasonid, period )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    invs = one/real( nseason, stnd )
    time2b = time2 - nseason + 1
!
    startlec(1:2) = (/ lon1, lat1 /)
!
    if ( use_compact ) then
        startwri(1:2) = 1
    else
        startwri(1:2) = startlec(1:2)
    end if
!
    startwri(3) = 0
    use_short = xtype == nf90_int .or. xtype == nf90_short .or. xtype == nf90_byte
!
!   WRITE THE SEASONAL NETCDF VARIABLE.
!
    if ( mask_name/=" " ) then
!
!       THE NETCDF VARIABLE CONTAINS MISSING VALUES.
!
!       ALLOCATE WORK ARRAYS.
!
        allocate( mask(lon3,lat3,nseason), stat=status )
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
!       GET THE MASK.
!
        status = nf90_get_var( ncid_mask, maskid, mean, start=startlec(1:2)  )     
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!       DETERMINE THE GRID POINTS USED FROM THE MASK.
!
        mask(:lon3,:lat3,:nseason) = spread( mean(:lon3,:lat3)==one, dim=3, ncopies=nseason )
!
        do i = time1, time2b, nseason
!
            startlec(3) = i
!
!           FILL THE WORK ARRAY WITH THE NETCDF VARIABLE.
!
            status = nf90_get_var( ncid, varid, data, start=startlec )     
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           COMPUTE SEASONAL MEANS FOR THE NETCDF VARIABLE.
!
            mean(:lon3,:lat3) = sum( data, dim=3, mask=mask )
!
            where ( mask(:lon3,:lat3,1) )
                mean(:lon3,:lat3) = mean(:lon3,:lat3)*invs
            elsewhere
                mean(:lon3,:lat3) = missing
            end where
!
            startwri(3) = startwri(3) + 1
!
            if ( use_short ) then 
               status = nf90_put_var( ncid_season, meanid, nint(mean), start=startwri )
            else
               status = nf90_put_var( ncid_season, meanid, mean, start=startwri )
            end if
!
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end do
!
        j2 = time2 - i + 1
!
        if ( j2>0 ) then
!
!           COMPUTE THE MEANS FOR THE REMAINING OBSERVATIONS.
!
            invs = one/real( j2, stnd )
!
            startlec(3) = i
!
!           FILL THE WORK ARRAY WITH THE NETCDF VARIABLE.
!
            status = nf90_get_var( ncid, varid, data(:lon3,:lat3,:j2), &
                                   start=startlec, count=(/ lon3, lat3, j2 /) )     
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           COMPUTE SEASONAL MEANS FOR THE NETCDF VARIABLE.
!
            mean(:lon3,:lat3) = sum( data(:lon3,:lat3,:j2), dim=3, mask=mask(:lon3,:lat3,:j2) )
!
            where ( mask(:lon3,:lat3,1) )
                mean(:lon3,:lat3) = mean(:lon3,:lat3)*invs
            elsewhere
                mean(:lon3,:lat3) = missing
            end where
!
            startwri(3) = startwri(3) + 1
!
            if ( use_short ) then 
               status = nf90_put_var( ncid_season, meanid, nint(mean), start=startwri )
            else
               status = nf90_put_var( ncid_season, meanid, mean, start=startwri )
            end if
!
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!        
        end if
!
!       DEALLOCATE WORK ARRAYS.
!
        deallocate( data, mean, period, temp, mask  )
!
!       CLOSE THE NETCDF DATASET.
!
        status = nf90_close( ncid_mask )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    else
!
!       THE NETCDF VARIABLE DOES NOT CONTAIN MISSING VALUES.
!
        do i = time1, time2b, nseason
!
            startlec(3) = i
!
!           FILL THE WORK ARRAY WITH THE NETCDF VARIABLE.
!
            status = nf90_get_var( ncid, varid, data, start=startlec )     
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           COMPUTE SEASONAL MEANS FOR THE NETCDF VARIABLE.
!
            mean(:lon3,:lat3) = sum( data, dim=3 )*invs
!
            startwri(3) = startwri(3) + 1
!
            if ( use_short ) then 
               status = nf90_put_var( ncid_season, meanid, nint(mean), start=startwri )
            else
               status = nf90_put_var( ncid_season, meanid, mean, start=startwri )
            end if
!
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end do
!
        j2 = time2 - i + 1
!
        if ( j2>0 ) then
!
!           COMPUTE THE MEANS FOR THE REMAINING OBSERVATIONS.
!
            invs = one/real( j2, stnd )
!
            startlec(3) = i
!
!           FILL THE WORK ARRAY WITH THE NETCDF VARIABLE.
!
            status = nf90_get_var( ncid, varid, data(:lon3,:lat3,:j2), &
                                   start=startlec, count=(/ lon3, lat3, j2 /) )     
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           COMPUTE SEASONAL MEANS FOR THE NETCDF VARIABLE.
!
            mean(:lon3,:lat3) = sum( data(:lon3,:lat3,:j2), dim=3 )*invs
!
            startwri(3) = startwri(3) + 1
!
            if ( use_short ) then 
               status = nf90_put_var( ncid_season, meanid, nint(mean), start=startwri )
            else
               status = nf90_put_var( ncid_season, meanid, mean, start=startwri )
            end if
!
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )        
!
        end if
!
!       DEALLOCATE WORK ARRAYS.
!
        deallocate( data, mean, period, temp  )
!
    end if
!
!   CLOSE THE NETCDF DATASETS.
!
    status = nf90_close( ncid )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_close( ncid_season )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!
! END OF PROGRAM comp_season_3d
! =============================
!
end program comp_season_3d
