program comp_spectrum_ratio_1d
!! #############################################################################
!!
!! Copyright 2018 IRD
!!
!! This file is part of NCSTAT.
!!
!! NCSTAT is free software: you can redistribute it and/or modify
!! it under the terms of the GNU Lesser General Public License as 
!! published by the Free Software Foundation, either version 3 of 
!! the License, or (at your option) any later version.
!!
!! NCSTAT is distributed in the hope that it will be useful,
!! but WITHOUT ANY WARRANTY; without even the implied warranty of
!! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!! GNU Lesser General Public License for more details.
!!
!! You can find a copy of the GNU Lesser General Public License
!! in the NCSTAT/doc directory.
!!
!! #############################################################################
!
!+
!
! .. _comp_spectrum_ratio_1d:
!
! ======================
! comp_spectrum_ratio_1d
! ======================
!
!
! Authors
! =======
!
! Pascal Terray (LOCEAN/IPSL)
!
! Latest revision
! ===============
!
! 13/09/2018
!
! Purpose
! =======
!
! Compute the ratio of two Power Spectral Density (PSD) estimates and a pointwise tolerance interval for this ratio
! under the assumption that the two "true" underlying spectra are the same.
!
! How to compare quantitatively two estimated spectra is an important question solved by comp_spectrum_ratio_1d.
! Suppose that *f1*\ (\ ``w``\ ) and *f2*\ (\ ``w``\ ) are spectral estimates (where ``w`` is a Fourier frequency)
! with ``df1`` and ``df2`` degrees of freedom, respectively. To answer this question, spectral analysis theory [Diggle]_
! suggests to examine the spectral ratio
!
! *R*\ (\ ``w``\ ) = *f1*\ (\ ``w``\ ) / *f2*\ (\ ``w``\ )
!
! , which may be assumed to follow an F-distribution with numerator and denominator degrees of
! freedom ``df1`` and ``df2``, respectively. This result can be used to calculate pointwise
! confidence (or tolerance) intervals for the spectral ratios at the Fourier frequencies [Diggle]_ [Masson_etal]_ [Terray_etalb]_.
!
! For additional details on these spectral computations, see the second reference cited below.
!
! It is assumed that the two estimated spectra have been computed with calls to :ref:`comp_spectrum_1d`
! with the **-conflim** argument.
!
! The pointwise tolerance interval and the ratio of the two estimated spectra (if the **-nv=** and
! **-dv=** arguments are specified) are stored in an output NetCDF dataset.
!
! Further Details
! ===============
!
! Usage
! -----
!
! ::
!
!       $ comp_spectrum_ratio_1d \
!              -nf=numerator_netcdf_file \
!              -df=denominator_netcdf_file       (optional) \
!              -nv=numerator_netcdf_variable     (optional) \
!              -dv=denominator_netcdf_variable   (optional) \
!              -np=numerator_period              (optional) \
!              -dp=denominator_period            (optional) \
!              -t=freq1,freq2                    (optional) \
!              -o=output_netcdf_file             (optional) \
!              -pro=probability_interval         (optional : 0.0 > 1.) \
!              -mi=missing_value                 (optional) \
!              -logratio                         (optional) \
!              -double                           (optional) \
!              -hdf5                             (optional) \
!              -tlimited                         (optional)
!
! By default
! ----------
!
!        *-df=*
!         the *denominator_netcdf_file* is the same as the *numerator_netcdf_file*
!        *-nv=*
!         a *numerator_netcdf_variable* is not used and only the upper and lower critical
!         ratios for each frequency are computed
!        *-dv=*
!         a *denominator_netcdf_variable* is not used and only the upper and lower critical
!         ratios for each frequency  are computed
!        *-np=*
!         the *numerator_period* argument is not used
!        *-dp=*
!         the *denominator_period* argument is not used
!        *-t=*
!         the whole set of frequencies in the two input NetCDF files
!        *-o=*
!         the *output_netcdf_file* is named ``spectrum_ratio.nc``
!        *-pro=*
!         the *probability_interval* is set to ``0.90``, which means that a ``90``\ % tolerance interval
!         for the ratio of the estimated spectra is computed
!        *-mi=*
!         the *missing_value* for the output NetCDF variables is set to ``1.e+20``
!        *-logratio*
!         the ratios and their associated upper and lower critical ratios are not Log transformed.
!         If **-logratio** is specified, the ratios, upper and lower critical ratios are Log transformed
!        *-double*
!         the results are stored as single-precision floating
!         point numbers in the output NetCDF file. If **-double** is activated, the
!         results are stored as double-precision floating point numbers 
!        *-hdf5*
!         a NetCDF classical format file is created. If **-hdf5** is activated,
!         the output NetCDF file is a NetCDF-4/HDF5 format file
!        *-tlimited*
!         the time dimension (e.g. the frequency axis) is defined as unlimited in the output NetCDF file.
!         However, if **-tlimited** is activated, the time dimension is defined
!         as limited in the output NetCDF file
!
! Remarks
! -------
!
!   #) The **-nf=**\ *numerator_netcdf_file* argument specifies the NetCDF file containing the spectrum
!      estimates and associated degrees of freedom of the numerator of the spectral ratio.
!
!      It is assumed that this NetCDF file has been created by NCSTAT procedures performing spectral computations
!      like :ref:`comp_spectrum_1d` (e.g. this NetCDF file must contain two NetCDF
!      variables named edof and freq).
!
!   #) The **-df=**\ *denominator_netcdf_file* argument specifies the NetCDF file containing the spectrum
!      estimates and associated degrees of freedom of the denominator of the spectral ratio.
!      If this argument is not specified, it is assumed that the *denominator_netcdf_file* is 
!      the same as the *numerator_netcdf_file*.
!
!      It is assumed that this NetCDF file has been created by NCSTAT procedures performing spectral computations
!      like :ref:`comp_spectrum_1d` (e.g. this NetCDF file must contain two NetCDF
!      variables named edof and freq).
!
!   #) The **-nv=**\ *numerator_netcdf_variable* and **-dv=**\ *denominator_netcdf_variable* arguments specify
!      the NetCDF variables containing the power spectrum estimates for the numerator and denominator
!      of the spectral ratio, respectively.
!
!      If one or two of these optional arguments are absent, comp_spectrum_ratio_1d computes only the upper
!      and lower critical ratios associated with the ( *probability_interval*\  * ``100`` )\ %
!      tolerance interval (*probability_interval* is the value of the **-pro=** argument described below).
!
!   #) If the **-np=**\ *numerator_period* argument is specified, the power spectrum estimates in the *numerator_netcdf_file*
!      have been computed for different periods separately (as determined by the value of the **-p=**\ *period_length* argument
!      in :ref:`comp_spectrum_1d` for example). In that case, the **-np=** argument selects the period in the 
!      *numerator_netcdf_file* which must be used for computing the ratio of the PSD estimates (e.g. this argument is
!      only used and useful, if both the **-nv=** and **-dv=** arguments are also specified).
!
!   #) If the **-dp=**\ *denominator_period* argument is specified, the power spectrum estimates in the *denominator_netcdf_file* 
!      have been computed for different periods separately (as determined by the value of the **-p=**\ *period_length* argument
!      in :ref:`comp_spectrum_1d` for example). In that case, the **-dp=** argument selects the period in the 
!      *denominator_netcdf_file* which must be used for computing the ratio of the PSD estimates (e.g. this argument is
!      only used and useful if both the **-nv=** and **-dv=** arguments are also specified).
!
!   #) If the **-t=**\ *freq1,freq2*  argument is missing, the ratios are computed for the whole set of
!      frequencies in the *numerator_netcdf_file* and *denominator_netcdf_file* files (e.g. the spectra must
!      have been estimated for the same set of frequencies).
!
!      The selected frequency interval is a vector of two integers specifying the first and last 
!      frequencies for which the ratios and the associated tolerance intervals must be computed.
!      The indices are relative to ``1``.
!
!   #) The **-pro=**\ *probability_interval* argument specifies that a ( *probability_interval*\  * ``100`` )\ %
!      tolerance interval must be computed. This argument is used to determine the upper and lower critical ratios
!      of the computed tolerance interval.
!      *probability_interval* must be greater than ``0`` and less than ``1``. The default value is ``0.90``.
!
!   #) The **-mi=**\ *missing_value* argument specifies the missing value indicator for
!      the output NetCDF variables in the *output_netcdf_file*. 
!
!      If the **-mi=** argument is not specified, the *missing_value* is set
!      to ``1.e+20``.
!
!   #) The **-logratio** argument specifies that the logarithms of the ratios must be computed and stored
!      in the output NetCDF file.
!
!   #) The **-double** argument specifies that the results are stored as double-precision floating
!      point numbers in the output netCDF file.
!      By default, the results are stored as single-precision floating point numbers 
!      in the output NetCDF file.
!
!   #) The **-hdf5** argument is allowed only if the NCSTAT software has been
!      compiled with the _USE_NETCDF4 CPP macro (e.g. ``-D_USE_NETCDF4``) and linked
!      to the NetCDF 4 library or higher. 
!      If this argument is specified, the *output_netcdf_file* will be a NetCDF-4/HDF5
!      format file instead of a NetCDF classic format file. However, this
!      argument is recognized in the procedure only if the NCSTAT software has
!      been built with the _USE_NETCDF4 CPP macro.
!
!   #) Duplicate parameters are allowed, but this is always the last occurrence of a parameter
!      which will be used for the computations. Moreover, the number of specified parameters
!      must not be greater than the total number of allowed parameters.
!
!   #) For more details on power spectrum analysis, theory for computing a pointwise tolerance
!      interval for the ratio of two estimated spectra and examples in the climate literature, see
!
!      * "Fourier analysis of time series- An introduction",
!        by Bloomfield, P., John Wiley and Sons, New York, Chapter 6, 1976. http://as.wiley.com/WileyCDA/WileyTitle/productCd-0471889482.html
!
!      * "Time series: a biostatistical introduction",
!        by P.J. Diggle, Clarendon Press, Oxford, Chap. 4, 1990. http://www.oupcanada.com/catalog/9780198522263.html
!
!      * "Impact of intra-daily SST variability on ENSO characteristics in a coupled model"
!        by S. Masson et al., 2012, Climate Dynamics, Vol. 39:681-707, doi: `10.1007/s00382-011-1247-2`_
!
!      * "The role of the intra-daily SST variability in the Indian monsoon variability in a global coupled model"
!        by P. Terray et al., 2012, Climate Dynamics, Vol. 39:729-754, doi: `10.1007/s00382-011-1240-9`_
!
!      * "Southern Hemisphere extra-tropical forcing: A new paradigm for El Nino-Southern Oscillation"
!        by P. Terray, 2011, Climate Dynamics, Vol. 36:2171-2199, doi: `10.1007/s00382-010-0825-z`_
!
! .. _`10.1007/s00382-011-1247-2` : http://dx.doi.org/10.1007/s00382-011-1247-2
! .. _`10.1007/s00382-011-1240-9` : http://dx.doi.org/10.1007/s00382-011-1240-9
! .. _`10.1007/s00382-010-0825-z` : http://dx.doi.org/10.1007/s00382-010-0825-z
!
! Outputs
! -------
!
!   comp_spectrum_ratio_1d creates an output NetCDF file that contains the upper and lower critical spectral
!   ratios associated with the specified pointwise tolerance interval and, eventually, the spectral ratios of
!   two estimated spectra, if both the **-nv=** and **-dv=** arguments have been used in the call to comp_spectrum_ratio_1d.
!   The output NetCDF dataset contains the following NetCDF variables (in the description below, nfreq is the number
!   of frequencies for which upper and lower critical (and eventually estimated) spectral ratios are computed as determined
!   by the **-t=** and **-p=** arguments) :
!
!   #) frequency\ ``(nfreq)`` : the frequencies at which the spectral ratios are computed.
!
!   #) periodicity\ ``(nfreq)`` : the corresponding periods in number of time observations.
!
!   #) lower_critical_ratio\ ``(nfreq)`` : the lower critical spectral ratios associated with the
!      specified pointwise tolerance interval.
!
!      This variable is stored only if the **-logratio** argument has not been specified when calling
!      comp_spectrum_1d.
!
!   #) upper_critical_ratio\ ``(nfreq)`` : the upper critical spectral ratios associated with the
!      specified pointwise tolerance interval.
!
!      This variable is stored only if the **-logratio** argument has not been specified when calling
!      comp_spectrum_1d.
!
!   #) *numerator_netcdf_variable*\ _\ *denominator_netcdf_variable*\ _ratio\ ``(nfreq)`` : the spectral ratios
!      computed from the spectra specified with the two input NetCDF variables *numerator_netcdf_variable*
!      and *denominator_netcdf_variable*.
!
!      This variable is stored only if the two **-nv=** and **-dv=** arguments have been specified and if the **-logratio**
!      argument has not been specified when calling comp_spectrum_1d.
!
!   #) lower_critical_logratio\ ``(nfreq)`` : the lower critical spectral ratios associated with the
!      specified pointwise tolerance interval.
!
!      This variable is stored only if the **-logratio** argument has been specified when calling
!      comp_spectrum_1d.
!
!   #) upper_critical_logratio\ ``(nfreq)`` : the upper critical spectral ratios associated with the
!      specified pointwise tolerance interval.
!
!      This variable is stored only if the **-logratio** argument has been specified when calling
!      comp_spectrum_1d.
!
!   #) *numerator_netcdf_variable*\ _\ *denominator_netcdf_variable*\ _logratio\ ``(nfreq)`` : the spectral ratios
!      computed from the spectra specified with the two input NetCDF variables *numerator_netcdf_variable*
!      and *denominator_netcdf_variable*.
!
!      This variable is stored only if both the **-nv=** and **-dv=** arguments have been specified and if the **-logratio**
!      argument has not been specified when calling comp_spectrum_1d.
!
! Examples
! --------
!
!   #)   For computing the upper and lower critical spectral ratios associated with a ``90``\ % confidence interval
!        of two spectra stored, respectively, in the files ``spectrum_sst.monthly.nino34.1979_2005.nc`` and
!        ``spectrum_sst.monthly.nino34.1950_1978.nc``, use the following command :
!
!        ::
!
!          $ comp_spectrum_ratio_1d  -fn=spectrum_sst.monthly.nino34.1979_2005.nc \
!                                    -fd=spectrum_sst.monthly.nino34.1950_1978.nc \
!                                    -o=spectrum_ratio_sst_nino34.nc
!
!-
! ================================================================================================
!
!
! USED MODULES 
! ============
!
#ifdef _USE_NAGWARE
    use f90_unix_env,     only : iargc, getarg
#endif
!
#ifdef _USE_NETCDF4
    use netcdf,   only : nf90_hdf5
#endif
!
    use netcdf,   only : nf90_max_name, nf90_max_var_dims, nf90_clobber, nf90_real4, nf90_real8,        &
                         nf90_sizehint_default, nf90_open,                                              &
                         nf90_nowrite, nf90_noerr, nf90_get_var, nf90_strerror,                         &
                         nf90_create, nf90_set_fill, nf90_nofill, nf90_def_dim, nf90_unlimited,         &
                         nf90_def_var, nf90_put_att, nf90_enddef, nf90_global, nf90_put_var, nf90_close
!
    use Statpack, only : i1b, i4b, stnd, lgl, true, false, zero, one, kfix, kexp, kint, tolower,     & 
                         case_change, string_to_val, allocate_error, merror, spctrm_ratio
!
    use NCSTAT_Utilities, only : prtunit, nchar, sp, dp, def_missing,      &
                                 get_listarg, get_variable_dims
!
!
! STRONG TYPING IMPOSED
! =====================
!
    implicit none
!
!
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='comp_spectrum_ratio_1d : '
!
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
!
#ifndef _USE_NAGWARE
    integer(i4b)  :: iargc
#endif
    integer(i4b)  :: narg, status, ncid_ratio, coor_id, periodid,                  &
                     ratio_id, upratio_id, loratio_id, output_type, output_mode,   &
                     i, j, nfic, ndim, freq1, freq2, freq3, nf 
    integer(i4b), dimension(2)                 :: ncid, freqid, dofid, varid, nfreq, period
    integer(i4b), dimension(4)                 :: vecid
    integer(i4b), dimension(nf90_max_var_dims) :: dimids, dimlens
    integer(i1b)                               :: kcode
!
    real(stnd), dimension(:),   allocatable    :: lwr_ratio, upr_ratio, spect_ratio
    real(stnd), dimension(:,:), allocatable    :: freq, edof, spect
    real(stnd)                                 :: missing, probtol
!
    logical(lgl)                               :: use_tlimited, logratio, comp_ratio
    logical(lgl), dimension(2)                 :: use_period
!
    character(len=nchar)                       :: name, name2, history
    character(len=nf90_max_name)               :: ratio_name, dim_name
    character(len=nf90_max_name), dimension(2) :: fic_name, var_name
    character(len=14)                          :: fmt
!
!
! EXECUTABLE STATEMENTS
! =====================
!
!   GET THE NUMBER OF COMMAND LINE ARGUMENTS.
!
    narg = iargc()
!
!   CHECK THE NUMBER OF COMMAND LINE ARGUMENTS.
!
    if ( narg==0 ) then
!
        write (prtunit,*)
        write (prtunit,*) ' Purpose : '
        write (prtunit,*)
        write (prtunit,*) ' Compute the ratio of two estimated spectra and a pointwise tolerance interval'
        write (prtunit,*) ' for this ratio under the assumption that the two "true" underlying spectra are'
        write (prtunit,*) ' the same'
        write (prtunit,*)
        write (prtunit,*) ' Usage : '
        write (prtunit,*)
        write (prtunit,*) ' comp_spectrum_ratio_1d  '
        write (prtunit,*) '        -nf=numerator_netcdf_file'
        write (prtunit,*) '        -df=denominator_netcdf_file       (optional)'
        write (prtunit,*) '        -nv=numerator_netcdf_variable     (optional)'
        write (prtunit,*) '        -dv=denominator_netcdf_variable   (optional)'
        write (prtunit,*) '        -np=numerator_period              (optional)'
        write (prtunit,*) '        -dp=denominator_period            (optional)'
        write (prtunit,*) '        -t=freq1,freq2                    (optional)'
        write (prtunit,*) '        -o=output_netcdf_file             (optional)'
        write (prtunit,*) '        -pro=probability_interval         (optional : 0. > 1.)'
        write (prtunit,*) '        -mi=missing_value                 (optional)'
        write (prtunit,*) '        -logratio                         (optional)'
        write (prtunit,*) '        -double                           (optional)'
#ifdef _USE_NETCDF4
        write (prtunit,*) '        -hdf5                             (optional)'
#endif
        write (prtunit,*) '        -tlimited                         (optional)'
        write (prtunit,*)
        write (prtunit,*) ' By default : '
        write (prtunit,*)
        write (prtunit,*) '-df=         : by default, the denominator_netcdf_file is the same as the numerator_netcdf_file'
        write (prtunit,*) '-nv=         : by default, only the upper and lower critical ratios are computed'
        write (prtunit,*) '-dv=         : by default, only the upper and lower critical ratios are computed'
        write (prtunit,*) '-np=         : by default, the numerator_period argument is not used'
        write (prtunit,*) '-dp=         : by default, the denominator_period argument is not used'
        write (prtunit,*) '-t=the whole set of frequencies in the two input NetCDF files'
        write (prtunit,*) '-o=spectrum_ratio.nc'
        write (prtunit,*) '-pro=0.90    : by default, a 90% tolerance interval is computed'
        write (prtunit,*) '-mi=1.e+20   : by default, the missing_value is equal to 1.e+20'
        write (prtunit,*) '-logratio    : by default, the ratios are not Log transformed'
        write (prtunit,*) '-double      : by default, the results are stored as single floating numbers'
#ifdef _USE_NETCDF4
        write (prtunit,*) '-hdf5        : by default, a NetCDF classical format file is created'
#endif
        write (prtunit,*) '-tlimited    : by default, the time dimension is defined as unlimited'
        write (prtunit,*)
!
        stop
!
    else if ( narg<1 .or. narg>14 ) then
!
        call merror( name_proc//"Wrong number of line arguments" )
!
    end if
!
    fic_name(:) = " "
    var_name(:) = " "
    ratio_name  = " "
!
    history = "comp_spectrum_ratio_1d"
!
!   DEFAULT VALUES FOR COMMAND LINE ARGUMENTS.
!
    output_mode  = nf90_clobber
    output_type  = nf90_real4
!
    probtol      = 0.90_stnd
!
    missing = def_missing
!
    freq1 = 1
    freq2 = huge( freq2 )
!
    period(:2)     = -1
    use_period(:2) = false
!
    logratio     = false
    use_tlimited = false
!
!   GET THE COMMAND LINE ARGUMENTS.
!
    do i = 1, narg
!
        call getarg( i, name )
!
        history = trim(history)//" "//trim(name)
!
        call case_change( name(:3), tolower )
!
        select case ( name(1:3 ) )
!
            case ('-t=')
!
!               GET THE FREQUENCY INTERVAL.
!
                call get_listarg( name_proc, name(4:),                    &
                                  "freq1 and freq2 are integer values",   &
                                  "Wrong frequency interval",             &
                                  freq1, freq2 )
!
            case ('-o=')
!
!               GET THE NAME OF THE NEW NETCDF DATASET FOR STORING THE RESULTS.
!
                ratio_name = name(4:)
!
            case default
!
                select case ( name(1:4 ) )
!
                    case ('-nf=')
!
!                       GET THE NAME OF THE NUMERATOR NETCDF DATASET.
!
                        fic_name(1) = name(5:)
!
                    case ('-nv=')
!
!                       GET THE NAME OF THE NUMERATOR NETCDF VARIABLE.
!
                        var_name(1) = name(5:)
!
                    case ('-np=')
!
!                       GET THE NUMERATOR PERIOD.
!
                        call string_to_val( name(5:), kcode, fmt )
                        if ( kcode/=kint ) &
                        call merror( name_proc//"The numerator_period is an integer value" )
!
                        read( name(5:), fmt ) period(1)
!
                        if ( period(1)<=0 ) &
                        call merror( name_proc//"The -np= parameter is a positive integer" )
!
                        use_period(1) = true
!
                    case ('-df=')
!
!                       GET THE NAME OF THE DENOMINATOR NETCDF DATASET.
!
                        fic_name(2) = name(5:)
!
                    case ('-dv=')
!
!                       GET THE NAME OF THE DENOMINATOR NETCDF VARIABLE.
!
                        var_name(2) = name(5:)
!
                    case ('-dp=')
!
!                       GET THE DENOMINATOR PERIOD.
!
                        call string_to_val( name(5:), kcode, fmt )
                        if ( kcode/=kint ) &
                        call merror( name_proc//"The denominator_period is an integer value" )
!
                        read( name(5:), fmt ) period(2)
!
                        if ( period(2)<=0 ) &
                        call merror( name_proc//"The -dp= parameter is a positive integer" )
!
                        use_period(2) = true
!
                    case ('-mi=')
!
!                       GET THE MISSING VALUE INDICATOR.
!
                        call string_to_val( name(5:), kcode, fmt )
                        if ( kcode/=kfix .and. kcode/=kexp )                &
                        call merror( name_proc//"The -mi= argument must be a real number " )
                        read( name(5:), fmt ) missing
!
                    case default
!
                        call case_change( name(4:), tolower )
!
                        select case ( name(1:5) )
!
                            case ('-pro=')
!
!                               DETERMINE THE PROBABILITY FOR THE TOLERANCE INTERVAL.
!
                                call string_to_val( name(6:), kcode, fmt )
                                if ( kcode==kfix .or. kcode==kexp ) then
                                    read( name(6:), fmt ) probtol
                                end if
                                if ( (kcode/=kfix .and. kcode/=kexp) .or.     &
                                     probtol<=zero                   .or.     &
                                     probtol>=one )                           &
      call merror( name_proc//"The -pro= argument must be a real number and verify 0.<pro<1. " )
!
                            case default
!
                                select case ( name(1:) )
!
                                    case ('-logratio')
!
!                                       COMPUTE THE LOG RATIOS.
!
                                        logratio = true
!
#ifdef _USE_NETCDF4
                                    case ('-hdf5')
!
!                                       REDEFINE THE CREATION MODE FLAG FOR THE OUTPUT FILE.
!
                                        output_mode = nf90_hdf5
#endif
!
                                    case ('-tlimited')
!
!                                       DEFINE THE TIME DIMENSION AS LIMITED.
!
                                        use_tlimited = true
!
                                    case ('-double')
!
!                                       STORE RESULTS AS DOUBLE PRECISION NUMBERS.
!
                                        output_type = nf90_real8
!
                                    case default
!
!                                       WRONG ARGUMENT.
!
                                        call merror( name_proc//"Wrong argument" )
!
                                end select
!
                        end select
!
                end select
!
        end select
!
    end do
!
!   CHECK THE COMMAND LINE ARGUMENTS.
!
    if ( fic_name(1)==" " )                                                 &
    call merror( name_proc//"Input numerator NETCDF file is missing" )
!
    if ( fic_name(2)==" " )   fic_name(2) = fic_name(1)
!
    comp_ratio = var_name(1)/=" " .and. var_name(2)/=" "
!
    if ( ratio_name==" " ) ratio_name="spectrum_ratio.nc"
!
    nfic = merge( 1, 2, fic_name(1)==fic_name(2) )
!
    name = "(numerator)"
!
    do j = 1, 2
!
        if ( j<=nfic ) then
!
!           OPEN THE NETCDF DATASETS.
!
            i = nf90_sizehint_default
            status = nf90_open( trim(fic_name(j)), nf90_nowrite, ncid(j), chunksize=i )
!
            if ( status/=nf90_noerr )                                                &
            call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           GET THE VARIABLES ID FROM THEIR NAME.
!
!           FIND THE DIMENSION LENGTH AND ID FOR THE NETCDF VARIABLES edof
!           AND frequency IN THE NETCDF FILES.
!
            call get_variable_dims(  name_proc, "frequency", ncid(j), freqid(j),        &
                                     dimids_out=dimids, dimlens_out=dimlens  )
!
            ndim = count(dimlens/=0)
            if ( ndim/=1  )     &
            call merror( name_proc//"Input NETCDF variable frequency "//trim(name)//" is not unidimensional" )
!
            nfreq(j) = dimlens(1)
            i        = dimids(1)
!
            call get_variable_dims(  name_proc, "edof", ncid(j), dofid(j),      &
                                     dimids_out=dimids, dimlens_out=dimlens     )
!
            ndim = count(dimlens/=0)
            if ( ndim/=1  )     &
            call merror( name_proc//"Input NETCDF variable edof "//trim(name)//" is not unidimensional" )
!
            if ( dimids(1)/=i  )     &
            call merror( name_proc//"Incorrect "//trim(name)//" NETCDF file" )
!
        else
!
!           THE TWO INPUT NETCDF DATASETS ARE IDENTICAL.
!
            ncid(2)  = ncid(1)
            nfreq(2) = nfreq(1)
!
        end if
!
        if ( comp_ratio ) then
!
!           GET THE VARIABLE IDS FROM THEIR NAME.
!           FIND THE DIMENSION LENGTHS AND IDS FOR THE NETCDF VARIABLES.
!
            call get_variable_dims(  name_proc, trim(var_name(j)), ncid(j),                 &
                                     varid(j),  dimids_out=dimids, dimlens_out=dimlens      )
!
            ndim = count(dimlens/=0)
!
            if ( use_period(j) ) then
!
                if ( ndim/=2  )     &
                call merror( name_proc//"Input NETCDF variable "//trim(name)//" is not bidimensional" )
!
!               CHECK THE PERIODS.
!
                if ( period(j)>dimlens(2) )      &
                call merror( name_proc//"period "//trim(name)//" is out of bounds" )
!
            else
!
                if ( ndim/=1  )     &
                call merror( name_proc//"Input NETCDF variable "//trim(name)//" is not unidimensional" )
!
            end if
!
            if ( dimids(1)/=i )     &
            call merror( name_proc//"Incorrect "//trim(name)//" NETCDF file" )
!
        end if
!
        name = "(denominator)"
!
    end do
!
    if ( nfreq(1)/=nfreq(2) )        &
    call merror( name_proc//"Frequencies are not the same in the two input NETCDF files" )
!
    nf = nfreq(1)
!
!   CHECK THE FREQUENCY INTERVAL.
!
    freq1 = max( freq1, 1 )
    freq2 = min( freq2, nf )
    freq3 = freq2 - freq1 + 1
!
!   ALLOCATE WORK ARRAYS FOR SPECTRUM, DEGREES OF FREEDOM AND FREQUENCY AXIS.
!
    if ( comp_ratio ) then
        allocate( spect(freq3,2), spect_ratio(freq3), lwr_ratio(freq3),          &
                  upr_ratio(freq3), freq(freq3,2), edof(freq3,2), stat=status    )
    else
        allocate( lwr_ratio(freq3), upr_ratio(freq3), freq(freq3,2),             &
                  edof(freq3,2), stat=status    )
    end if
!
    if ( status/=0 ) call merror( name_proc//allocate_error )
!
!   GET THE DATA.
!
    do j = 1, 2
!
        if ( j<=nfic ) then
!
!           GET THE FREQUENCY AXES.
!
            status = nf90_get_var( ncid(j), freqid(j), freq(:freq3,j), start=(/ freq1 /), count=(/ freq3 /)   )
!
            if ( status/=nf90_noerr )                                     &
            call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           GET THE DEGREES OF FREEDOM FOR THE TWO SPECTRA.
!
            status = nf90_get_var( ncid(j), dofid(j), edof(:freq3,j), start=(/ freq1 /), count=(/ freq3 /)   )
!
            if ( status/=nf90_noerr )                                     &
            call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        else
!
            freq(:freq3,2) = freq(:freq3,1)
            edof(:freq3,2) = edof(:freq3,1)
!
        end if
!
        if ( comp_ratio ) then
!
!           GET SPECTRA IF REQUIRED.
!
            if ( use_period(j) ) then
!
                status = nf90_get_var( ncid(j), varid(j), spect(:freq3,j),     &   
                                       start=(/ freq1, period(j) /), count=(/ freq3, 1_i4b /)     )
!
            else
!
                status = nf90_get_var( ncid(j), varid(j), spect(:freq3,j),     &   
                                       start=(/ freq1 /), count=(/ freq3 /)     )
!
            end if
!
            if ( status/=nf90_noerr )                                     &
            call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end if
!
    end do
!
    if ( nfic/=1 ) then
!
        if ( any( freq(:freq3,1)/=freq(:freq3,2) )   )       &
        call merror( name_proc//"Frequencies are not the same in the two input NETCDF files" )
!
    end if
!
!   COMPUTE THE UPPER AND LOWER CRITICAL RATIOS.
!
    call spctrm_ratio( edof(:freq3,1), edof(:freq3,2), lwr_ratio(:freq3),    &
                       upr_ratio(:freq3), pinterval=probtol                  )
!
    if ( logratio ) then
!
!       COMPUTE THE LOG OF THE TWO CRITICAL RATIOS IF REQUIRED.
!
        where( lwr_ratio(:freq3)>zero )
            lwr_ratio( :freq3) = log( lwr_ratio( :freq3) )
        elsewhere
            lwr_ratio( :freq3) = missing
        end where
!
        where( upr_ratio(:freq3)>zero )
            upr_ratio( :freq3) = log( upr_ratio( :freq3) )
        elsewhere
            upr_ratio( :freq3) = missing
        end where        
!
    end if
!
    if ( comp_ratio ) then
!
        if ( logratio ) then
!
!           COMPUTE THE LOG RATIO OF THE TWO SPECTRAL ESTIMATES.
!
            where( spect(:freq3,2)>zero .and. spect(:freq3,1)>zero )
                spect_ratio( :freq3) = log( spect(:freq3,1)/spect(:freq3,2) )
            elsewhere
                spect_ratio( :freq3) = missing
            end where
!
        else
!
!           COMPUTE THE RATIO OF THE TWO SPECTRAL ESTIMATES.
!
            where( spect(:freq3,2)/=zero )
                spect_ratio( :freq3) = spect(:freq3,1)/spect(:freq3,2)
            elsewhere
                spect_ratio( :freq3) = missing
            end where
!
        end if
!
    end if
!
!   CREATE THE NEW NETCDF DATASET FOR THE FREQUENCY ANALYSIS.
!
    status = nf90_create( ratio_name, cmode=output_mode, ncid=ncid_ratio )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_set_fill( ncid_ratio, nf90_nofill, i )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!   DEFINE THE DIMENSIONS FOR THE NEW NETCDF DATASET.
!
    if ( use_tlimited ) then
        dimlens(1)   = freq3
    else
        dimlens(1)   = nf90_unlimited
    end if
!
    dim_name = "frequency"
!
    status = nf90_def_dim( ncid_ratio, name=trim(dim_name),                           &
                           len=dimlens(1), dimid=dimids(1) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim(nf90_strerror(status)) )
!
!   DEFINE THE VARIABLES FOR THE NEW NETCDF DATASET.
!
    status = nf90_def_var( ncid_ratio, trim(dim_name), output_type,                   &
                           dimids=dimids(1), varid=coor_id )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_def_var( ncid_ratio, "periodicity",  output_type,                   &
                           dimids=dimids(1), varid=periodid )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    if ( logratio ) then
        name = "logratio"
    else
        name = "ratio"
    end if
!
    status = nf90_def_var( ncid_ratio, "lower_critical_"//trim(name),  output_type,      &
                           dimids=dimids(1), varid=loratio_id     )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_def_var( ncid_ratio, "upper_critical_"//trim(name),  output_type,      &
                           dimids=dimids(1), varid=upratio_id  )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    if ( comp_ratio ) then
!
        name2 = trim(var_name(1))//"_"//trim(var_name(2))//"_"//trim(name)
!
        status = nf90_def_var( ncid_ratio, trim(name2),  output_type,   &
                               dimids=dimids(1), varid=ratio_id )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end if
!
!   ASSIGN ATTRIBUTES FOR THE NEW NETCDF DATASET.
!
    status = nf90_put_att( ncid_ratio, coor_id, "long_name", "Frequency axis" )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_ratio, coor_id, "short_name", "Frequency" )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_ratio, coor_id, "units", "cycles per unit of time" )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_ratio, periodid, "long_name", "Periodicity axis" )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_ratio, periodid, "short_name", "Periodicity" )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_ratio, periodid, "units", "Number of observations" )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_ratio, loratio_id, "long_name",                           &
                               "Lower probability_interval*100% critical "//trim(name)  )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_ratio, loratio_id, "short_name", "Lower critical "//trim(name)  )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_ratio, loratio_id, "units", "none"  )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_ratio, upratio_id, "long_name",                           &
                               "Upper probability_interval*100% critical "//trim(name)  )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_ratio, upratio_id, "short_name", "Upper critical "//trim(name)  )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_ratio, upratio_id, "units", "none"  )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    if ( comp_ratio ) then
!
        if ( logratio ) then
            name2 = "Log ratio of Power Spectral Estimates of"
        else
            name2 = "Ratio of Power Spectral Estimates of"
        end if
!
        name2 = trim(name2)//" "//trim(var_name(1))//" and "//trim(var_name(2))
!
        status = nf90_put_att( ncid_ratio, ratio_id, "long_name", trim(name2)  )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_ratio, ratio_id, "short_name", "PSD "//trim(name)  )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_ratio, ratio_id, "units", "none"  )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        ndim = 4
        vecid(:4) = (/ coor_id, loratio_id, upratio_id, ratio_id  /)
!
    else
!
        ndim = 3
        vecid(:3) = (/ coor_id, loratio_id, upratio_id  /)
!
    end if
!
!   SET missing_value AND _FillValue ATTRIBUTES.
!
    if (  output_type==nf90_real8 ) then
!
        do j=1, ndim
!
            status = nf90_put_att( ncid_ratio, vecid(j), "missing_value", real( missing, dp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_ratio, vecid(j), "_FillValue", real( missing, dp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end do
!
    else
!
        do j=1, ndim
!
            status = nf90_put_att( ncid_ratio, vecid(j), "missing_value", real( missing, sp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_ratio, vecid(j), "_FillValue", real( missing, sp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end do
!
    end if
!
!   SET GLOBAL ATTRIBUTES.
!
    if ( comp_ratio ) then
        name2 = "Power spectrum ratio analysis of "//trim(var_name(1))//" (numerator) and "//   &
                trim(var_name(2))//" (denominator)"
    else
        name2 = "Power spectrum ratio analysis"
    end if
!
    status = nf90_put_att( ncid_ratio, nf90_global, "title", trim(name2) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_ratio, nf90_global, "probability_interval", probtol )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    if ( logratio ) then
        i = 1
    else
        i = 0
    end if
!
    status = nf90_put_att( ncid_ratio, nf90_global, "log_transform", i )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    if ( comp_ratio ) then
!
        name = "numerator"
!
        do j = 1, 2
!
            if ( use_period(j) ) then
!
                status = nf90_put_att( ncid_ratio, nf90_global, trim(name)//"_period", period(j) )
                if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
            end if            
!
            name = "denominator"
!
        end do
!
    end if
!
    status = nf90_put_att( ncid_ratio, nf90_global, "history", trim(history) )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!   LEAVE DEFINE MODE FOR THE NEW NETCDF DATASET.
!
    status = nf90_enddef( ncid_ratio )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!   WRITE VARIABLES FOR THE NEW NETCDF DATASET.
!
!   WRITE FREQUENCY AND PERIOD AXES.
!
    status = nf90_put_var( ncid_ratio, coor_id, freq(:freq3,1) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!   BUILD UP PERIODICITY VECTOR.
!
    where( freq(:freq3,1)/=zero )
        freq(:freq3,2) = one/freq(:freq3,1)
    elsewhere
        freq(:freq3,2) = missing
    end where
!
    status = nf90_put_var( ncid_ratio, periodid, freq(:freq3,2) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!   WRITE THE CRITICAL RATIOS.
!
    status = nf90_put_var( ncid_ratio, loratio_id, lwr_ratio( :freq3) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_var( ncid_ratio, upratio_id, upr_ratio( :freq3) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    if ( comp_ratio ) then
!
!       WRITE THE RATIO OF THE PSD ESTIMATES.
!
        status = nf90_put_var( ncid_ratio, ratio_id, spect_ratio( :freq3)  )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!       DEALLOCATE WORK ARRAYS.
!
        deallocate( spect, spect_ratio, lwr_ratio, upr_ratio, freq, edof  )
!
    else
!
!       DEALLOCATE WORK ARRAYS.
!
        deallocate( lwr_ratio, upr_ratio, freq, edof  )
!
    end if
!
!   CLOSE THE NETCDF DATASETS.
!
    status = nf90_close( ncid(1) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    if ( ncid(1)/=ncid(2) ) then
        status = nf90_close( ncid(2) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
    end if
!
    status = nf90_close( ncid_ratio )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!
! END OF PROGRAM comp_spectrum_ratio_1d
! =====================================
!
end program comp_spectrum_ratio_1d
