program comp_svd_3d
!! #############################################################################
!!
!! Copyright 2021 IRD
!!
!! This file is part of NCSTAT.
!!
!! NCSTAT is free software: you can redistribute it and/or modify
!! it under the terms of the GNU Lesser General Public License as 
!! published by the Free Software Foundation, either version 3 of 
!! the License, or (at your option) any later version.
!!
!! NCSTAT is distributed in the hope that it will be useful,
!! but WITHOUT ANY WARRANTY; without even the implied warranty of
!! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!! GNU Lesser General Public License for more details.
!!
!! You can find a copy of the GNU Lesser General Public License
!! in the NCSTAT/doc directory.
!!
!! #############################################################################
!
!+
!
! .. _comp_svd_3d:
!
! ===========
! comp_svd_3d
! ===========
!
! Authors
! =======
!
! Pascal Terray (LOCEAN/IPSL) and Eric Maisonnave (CERFACS)
!
! Latest revision
! ===============
!
! 06/05/2021
!
! Purpose
! =======
!
! Compute a Singular Values Decomposition (SVD) analysis, also known as Maximum Covariance Analysis (MCA),
! of a covariance (or correlation) matrix between (selected) time series associated with two input
! NetCDF variables (specified with the **-v=** and **-v2=** arguments) extracted from one or two NetCDF
! datasets (specified with the **-f=** and **-f2=** arguments).
!
! SVD analysis or MCA can be considered as a generalization of Empirical Orthogonal Function (EOF) analysis
! [Bjornsson_Venegas]_ [Bretherton_etal]_ [vonStorch_Zwiers]_. It aims at estimating the covariance matrix between
! two fields and at computing the SVD of this covariance matrix for defining pairs of spatial patterns, which
! describe (maximize) a fraction of the total Square Covariance (SCF) between the two fields. Optionally, the
! temporal covariance matrix between the two fields may be weighted by the surface (or volume) associated
! with each cell in the two grids so that equal areas (or volumes) carry equal weights in the results of the
! SVD analysis (see the **-d=** and **-d2=** arguments description).
!
! The procedure first repacks the first (or left) input tridimensional NetCDF variable (specified with
! the **-v=** argument) and the second (or right) input tri or fourdimensional NetCDF variable (specified
! with the **-v2=** argument) as a *ntime* by *nv1* rectangular matrix, **X**\ , and a *ntime* by
! *nv2* rectangular matrix, **Y**\  respectively. The procedure then computes the covariance (or
! correlation and sum of squares and cross-products at the user option) matrix between **X**\  and **Y**\  ,
! e.g. the rectangular matrix (transpose(**X**\ ).\ **Y**\ )/\ *ntime*.
! In the following discussion, the **X**\  and **Y**\  matrices will be called the left and right fields,
! respectively.
!
! The second step of the SVD analysis is to compute the leading *k* terms of the SVD decomposition
! of the covariance matrix between the left and right fields, given by
!
!           (transpose(**X**\ ).\ **Y**\ )/\ *ntime*  = **U**\ **S**\ **V**
!
! where
!
!     * **U**
!       is a *nv1* by *k* matrix with orthonormal columns (the left singular vectors stored columnwise)
!     * **S**
!       is a square *k* by *k* matrix with nonnegative elements on its principal diagonal and zeros
!       elsewhere (the diagonal elements of **S** are the singular values of the covariance matrix)
!     * **V**
!       is a *k* by *nv2* matrix with orthonormal rows (the right singular vectors stored rowwise)
!
! In a third step, the first *k* standardized left and right "Singular Variables" (SV) time series are
! computed by projecting the left and right fields onto the first *k* left and right singular
! vectors, respectively.
! These SV time series play a similar role as principal component time series in an
! Empirical Orthogonal Function (EOF) analysis. Refer to :ref:`comp_eof_3d` or :ref:`comp_eof_4d`
! for more details on EOF analysis.
!
! Finally, from the *k* left and right SV time series, two types of regression maps are generated for each
! field: the *k*\ th homogeneous vector, which is the regression map between a given data field and its *k*\ th
! standardized SV time series, and the *k*\ th heterogeneous vector, which is the regression map between a given
! data field and the *k*\ th standardized SV time series of the other field. The *k*\ th heterogeneous vector
! indicates how well the time series of one field can be predicted from the *k*\ th SV time series of the
! other field.
!
! Simple statistics associated with each singular triplet (e.g. a singular value and the associated left and
! right singular vectors) of the SVD of the covariance matrix are also computed. These include the Square 
! Covariance Fraction (SCF) coefficient, which is a simple measure of the relative importance of each singular
! triplet in the linear relationship between the two fields, the correlation coefficients between the *k*\ th
! left and right SV time series of the two fields and the Normalized root-mean-square Covariance (NC) coefficient.
! See [Bretherton_etal]_ [Zhang_etal]_ for a discussion of the relative merits of these coefficients for determining how strongly
! related the coupled patterns described by a singular triplet are. Confidence levels for the SCF, NC and correlation
! coefficients can be estimated by a moving block bootstrap algorithm in which these statistics are recomputed many times
! after replacing the right field by a random field constructed by resampling randomly blocks of observations from
! the original right field. The **-nb=**\ , **-bl=**\ , **-bp=**\  and **-cb=**\  arguments allow the user to
! determine the exact form of the blockwise bootstrap algorithm.
! This moving block bootstrap algorithm is formally similar to the one described in :ref:`comp_cor_3d` or
! :ref:`comp_cor_4d` for testing the significance of a correlation coefficient when **-a=**\ ``bootstrap``\ .
! Refer to :ref:`comp_cor_3d` for further details on this moving block bootstrap algorithm.
!
! Two output NetCDF datasets containing the singular values, the left and right singular vectors,
! the corresponding left and right standardized SV time series and, the homogeneous and heterogeneous 
! vectors for each field are created. The left and right singular vectors, and the homogeneous
! and heterogeneous vectors for each field, are repacked onto the original grids of the two
! input NetCDF variables in the output NetCDF datasets. In addition, if the confidence levels for
! the SCF, NC and correlation coefficients are estimated, these probabilities are also included in
! the output NetCDF datasets.
!
! This procedure is parallelized if OpenMP is used and will be also much faster if an optimized BLAS
! library is specified at compilation with the _BLAS CPP macro. Moreover, this procedure may use a partial SVD
! algorithm which is highly efficient on huge covariance matrix if you are interested only in the few
! leading terms of the SVD of the covariance matrix between the **X**\  and **Y**\  fields.
!
! Further Details
! ===============
!
! Usage
! -----
!
! ::
!
!       $ comp_svd_3d \
!         -f=input_netcdf_file \
!         -v=netcdf_variable \
!         -m=input_mesh_mask_netcdf_file \
!         -a=type_of_analysis                             (optional : scp, cov, cor) \
!         -n=number_of_sing_triplets                      (optional) \
!         -g=grid_type                                    (optional : n, t, u, v, w, f) \
!         -r=resolution                                   (optional : r2, r4) \
!         -b=nlon_orca, nlat_orca                         (optional) \
!         -x=lon1,lon2                                    (optional) \
!         -y=lat1,lat2                                    (optional) \
!         -t=time1,time2                                  (optional) \
!         -c=input_climatology_netcdf_file                (optional) \
!         -d=type_of_distance                             (optional : dist2, ident) \
!         -o=output_svd_netcdf_file_left_field            (optional) \
!         -f2=input_netcdf_file_right_field               (optional) \
!         -v2=netcdf_variable_right_field                 (optional) \
!         -m2=input_mesh_mask_netcdf_file_right_field     (optional) \
!         -g2=grid_type_right_field                       (optional : n, t, u, v, w, f) \
!         -r2=resolution_right_field                      (optional : r2, r4) \
!         -b2=nlon_orca, nlat_orca                        (optional) \
!         -x2=lon1_right_field,lon2_right_field           (optional) \
!         -y2=lat1_right_field,lat2_right_field           (optional) \
!         -z2=level1_right_field,level2_right_field       (optional) \
!         -t2=time1_right_field,time2_right_field         (optional) \
!         -c2=input_climatology_netcdf_file_right_field   (optional) \
!         -d2=type_of_distance_right_field                (optional : dist2, dist3, ident) \
!         -o2=output_svd_netcdf_file_right_field          (optional) \
!         -alg=algorithm                                  (optional : svd, inviter, deflate) \
!         -cb=bootstrap_statistic_significativity_type    (optional : values, vector) \
!         -nb=number_of_shuffles                          (optional) \
!         -bp=bootstrap_periodicity                       (optional) \
!         -bl=bootstrap_block_length                      (optional) \
!         -mi=missing_value                               (optional) \
!         -double                                         (optional) \
!         -bigfile                                        (optional) \
!         -hdf5                                           (optional) \
!         -tlimited                                       (optional)
!
! By default
! ----------
!
!        *-a=*
!         the *type_of_analysis* is set to ``scp``. This means that the singular vectors
!         and singular values are computed from the sums of squares and cross-products matrix
!         between the left and right fields
!        *-n=*
!         *number_of_sing_triplets* is set to ``4``\ . This means that the first ``4`` singular
!         triplets of the covariance matrix between the left and right fields are computed and
!         stored in the output NetCDF files *output_svd_netcdf_file_left_field* and
!         *output_svd_netcdf_file_right_field*
!        *-g=*
!         the *grid_type* is set to ``n`` which means that the 2-D grid-mesh associated with
!         the left field extracted from the input NetCDF variable *input_netcdf_file* is
!         assumed to be regular or Gaussian
!        *-r=*
!         if the input NetCDF variable *netcdf_variable* is from the NEMO or ORCA model (e.g.
!         if **-g=** argument is not set to ``n``) the resolution is assumed to be ``r2``
!        *-b=*
!         if **-g=** is not set to ``n``\ , the dimensions of the 2-D grid-mesh, *nlon_orca* and
!         *nlat_orca*, of the left input NetCDF variable *netcdf_variable* are determined from
!         the **-r=** argument. However, you may override this choice by default with the **-b=**
!         argument
!        *-x=*
!         the whole longitude domain associated with the *netcdf_variable*
!        *-y=*
!         the whole latitude domain associated with the *netcdf_variable*
!        *-t=*
!         the whole time period associated with the *netcdf_variable*
!        *-c=*
!         this argument is not used. This argument is required only if the *type_of_analysis* is
!         set to ``cov`` or ``cor`` and is used to specify the climatology NetCDF file for
!         computing anomalies or standardized anomalies for the left field
!        *-d=*
!         the *type_of_distance* is set to ``dist2``. This means that distances and scalar products
!         for the left field in the SVD analysis are computed with the diagonal metric associated
!         with the 2-D grid-mesh associated with the input NetCDF variable *netcdf_variable*
!        *-o=*
!         the *output_svd_netcdf_file_left_field* is named ``svd_left_``\ *netcdf_variable*\ ``.nc``
!        *-f2=*
!         this argument is not used. It is required only if the right field
!         is not stored in the same file as the left NetCDF variable
!        *-v2=*
!         this argument is not used. It is required only if the right field
!         is not extracted from the same input NetCDF variable than the left field
!        *-m2=*
!         this argument will take the same value as the **-m=** argument. It is required only
!         if the right field is not extracted from the same input NetCDF variable as the left field
!        *-g2=*
!         same as the **-g=** argument if the **-v2=** argument is omitted and **-g2=**\ ``n`` otherwise
!        *-r2=*
!         same as the **-r=** argument if **-v2=** is omitted and **-r2=**\ ``r2`` otherwise
!        *-b2=*
!         if **-g2=** is not set to ``n``\ , the dimensions of the 2-D grid-mesh, *nlon_orca*
!         and *nlat_orca*, of the right field extracted from the input NetCDF variable
!         *netcdf_variable_right_field* are determined from the **-r2=** argument.
!         However, you may override this choice by default with the **-b2=** argument
!        *-x2=*
!         the whole longitude domain associated with the *netcdf_variable_right_field*
!        *-y2=*
!         the whole latitude domain associated with the *netcdf_variable_right_field*
!        *-z2=*
!         the whole level associated with the *netcdf_variable_right_field*
!        *-t2=*
!         the whole time period associated with the *netcdf_variable_right_field*
!        *-c2=*
!         this argument is not used. This argument is required only if the *type_of_analysis* is
!         set to ``cov`` or ``cor`` and is used to specify the climatology NetCDF file for
!         computing anomalies or standardized anomalies for the right field if this field is not extracted
!         from the same input NetCDF variable as the left field
!        *-d2=*
!         same as **-d=** if **-v2=** is omitted, **-d2=**\ ``dist2`` if the *netcdf_variable_right_field*
!         is a 3D variable and **-d2=**\ ``dist3`` if the *netcdf_variable_right_field* is a 4D variable
!        *-o2=*
!         the *output_svd_netcdf_file_right_field* is named
!         ``svd_right_``\ *netcdf_variable_right_field*\ ``.nc``
!        *-alg=*
!         the *algorithm* option is set to ``inviter``\ . This means that the SVD analysis is
!         computed by a partial SVD of the covariance matrix using an inverse iteration algorithm
!        *-cb=*
!         *bootstrap_statistic_significativity_type* is set to ``values``\ . This means
!         that only the SCF and NC coefficients are tested by the moving block bootstrap
!         algorithm. This saves computing time because this requires only the computation
!         of singular values in the moving block bootstrap algorithm
!        *-nb=*
!         *number_of_shuffles* is set to ``99``\ . This means that ``99`` bootstrap
!         samples are generated in the moving block bootstrap algorithm
!         for testing the significance of the singular triplets
!        *-bp=*
!         the time series are assumed to be stationary and *bootstrap_periodicity* is set to ``1``
!         in the moving block bootstrap procedure for testing the significance
!         of the singular triplets. This means that the blocks in the bootstrap algorithm are not
!         forced to begin at specific observations. Use this parameter if the time series are
!         cyclostationary, see the remarks below for further details
!        *-bl=*
!         *bootstrap_block_length* is set to *bootstrap_periodicity*\ .\ ``2``
!        *-mi=*
!         the *missing_value* attribute in the output NetCDF files is set to ``1.e+20``
!        *-double*
!         the results of the SVD analysis are stored as single-precision floating
!         point numbers in the output NetCDF files. If **-double** is activated, the
!         results are stored as double-precision floating point numbers 
!        *-bigfile*
!         NetCDF classical format files are created. If **-bigfile** is activated,
!         the output NetCDF files are 64-bit offset format files
!        *-hdf5*
!         NetCDF classical format files are created. If **-hdf5** is activated,
!         the output NetCDF files are NetCDF-4/HDF5 format files
!        *-tlimited*
!         the time dimension is defined as unlimited in the output NetCDF files.
!         However, if **-tlimited** is activated, the time dimension is defined
!         as limited in the output NetCDF files
!
! Remarks
! -------
!
!   #) The **-v=**\ *netcdf_variable* argument specifies the NetCDF variable from which
!      the left field for the SVD analysis must be extracted and the **-f=**\ *input_netcdf_file*
!      argument specifies that this NetCDF variable must be extracted from the NetCDF file,
!      *input_netcdf_file*.
!
!   #) The **-v2=**\ *netcdf_variable_right_field* argument specifies the NetCDF variable
!      from which the right field of the SVD analysis must be extracted and the
!      **-f2=input_netcdf_file_right_field** argument specifies that this NetCDF variable
!      must be extracted from the NetCDF file, *input_netcdf_file_right_field*.
!
!   #) If the **-x=**\ *lon1,lon2* and **-y=**\ *lat1,lat2* arguments are missing, the geographical
!      domain used for defining the left field in the SVD analysis is determined from the attributes of
!      the input mesh mask NetCDF variable named *grid_type*\ mask (e.g. lon1_Eastern_limit, lon2_Western_limit,
!      lat1_Southern_limit and lat2_Northern_limit ) which is read from the input NetCDF file
!      *input_mesh_mask_netcdf_file*\ .
!      If these attributes are missing and the **-x=** and **-y=** arguments are also not specified,
!      the whole geographical domain associated with the *netcdf_variable* is used for defining the
!      left field in the SVD analysis.
!
!      The longitude or latitude range must be a vector of two integers specifying the first
!      and last selected indices along each dimension. The indices are relative to ``1``\ .
!      Negative values are allowed for *lon1*. In this case the longitude domain
!      is from ``nlon``\ +\ *lon1*\ +\ ``1`` to *lon2* where ``nlon`` is the number of longitude
!      points in the grid associated with the NetCDF variable and it is assumed that the grid
!      is periodic.
!
!      This remark applies also for the **-x2=**, **-y2=** and **-z2=** arguments used for defining
!      the right field in the SVD analysis.
!
!      Refer to :ref:`comp_mask_3d` for transforming geographical coordinates as indices or generating an
!      appropriate mesh-mask before using comp_svd_3d.
!
!   #) If the **-t=**\ *time1,time2* argument is missing, the whole time period
!      associated with the *netcdf_variable* is used to estimate the covariance matrix
!      between the left and right fields.
!
!      The selected time period is a vector of two integers specifying the first
!      and last time observations. The indices are relative to ``1``\ . Note that the
!      SV time series in the output NetCDF files will have ``ntime`` = *time2* - *time1* + ``1``
!      time observations.
!
!      This remark applies also for **-t2=** argument used to define the time dimension
!      of the right field.
!
!   #) If **-g=** is set to ``t``, ``u``, ``v``, ``w`` or ``f`` it is assumed that the
!      NetCDF variable *netcdf_variable* is from an experiment with the NEMO or ORCA model.
!      In this case, the duplicate points from the ORCA grid are removed when extracting the left field
!      of the SVD analysis, as far as possible, and, in particular,
!      if the 2-D grid-mesh of the input NetCDF variable covers the whole globe. On output, the duplicate
!      points are restored when writing the SVD results (e.g. the singular, homogeneous and heterogeneous vectors),
!      if the geographical domain of the input NetCDF variable *netcdf_variable* is the whole globe.
!
!      If **-g=** is set to ``n``, it is assumed that the 2-D grid-mesh associated with *netcdf_variable* is
!      regular or Gaussian and as such has no duplicate points.
!
!      The **-g=** argument is also used to determine the name of the NetCDF variables which
!      contain the 2-D mesh-mask and the scale factors in the *input_mesh_mask_netcdf_file* (e.g.
!      these variables are named *grid_type*\ mask, e1\ *grid_type* and e2\ *grid_type*\ ,
!      respectively). This *input_mesh_mask_netcdf_file* may be created by :ref:`comp_clim_3d` if the
!      2-D grid-mesh is regular or gaussian.
!
!      This remark applies also for **-g2=** argument used to define the grid type of the right field.
!
!   #) If **-g=** is set to ``t``, ``u``, ``v``, ``w`` or ``f`` (e.g. if the NetCDF variable
!      is from an experiment with the NEMO or ORCA model), the **-r=** argument gives the 
!      resolution used. If:
!
!      * **-r=**\ ``r2`` the NetCDF variable is from an experiment with the ORCA R2 model
!
!      * **-r=**\ ``r4`` the NetCDF variable is from an experiment with the ORCA R4 model.
!
!      This remark applies also for **-g2=** argument used to define the grid type of the right field.
!
!   #) If the NetCDF variable *netcdf_variable* is from an experiment with the ORCA model,
!      but the resolution is not ``r2`` or ``r4``\ , the dimensions of the ORCA grid
!      must be specified explicitly with the **-b=** argument.

!      This remark applies also for **-b2=** argument used to define the grid type of the right field.
!
!   #) The **-a=** argument specifies if the left and right fields are centered or standardized
!      with an input climatology (specified with the **-c=** and **-c2=** arguments) before
!      computing the covariance matrix between the two fields. If:
!
!      * **-a=**\ ``scp``, the SVD analysis is done on the raw data of the two fields
!
!      * **-a=**\ ``cov``, the SVD analysis is done on the anomalies of the two fields
!
!      * **-a=**\ ``cor``, the SVD analysis is done on the standardized anomalies
!        of the two fields
!
!   #) The *input_climatology_netcdf_file* and *input_climatology_netcdf_file_right_field*
!      specified, respectively,  with the **-c=** and **-c2=** arguments are needed only 
!      if **-a=**\ ``cov`` or **-a=**\ ``cor``.
!
!   #) If **-a=**\ ``cov`` or **-a=**\ ``cor``, the selected time periods for the left and right fields
!      specified, respectively, with the **-t=** and **-t2=** arguments, must agree with the climatologies.
!
!      This means that the first selected time observation (*time1* if the **-t=** argument
!      is present) must correspond to the first day, month, season of the climatology
!      specified with the **-c=** argument for the left field.
!      This remark also applies for the right field and the **-t2=** and **-c2=** arguments.
!
!   #) The geographical shapes of the *netcdf_variable* (in the *input_netcdf_file*), the mask
!      (in the *input_mesh_mask_netcdf_file*), the scale factors (in the *input_mesh_mask_netcdf_file*),
!      and the climatology (in the *input_climatology_netcdf_file*) must agree.
!
!      Similarly, for the right field, the geographical shapes of the *netcdf_variable_right_field*
!      (in the *input_netcdf_file_right_field*), the mask (in the *input_mesh_mask_netcdf_file_right_field*),
!      the scale factors (in the *input_mesh_mask_netcdf_file_right_field*), and the climatology
!      (in the *input_climatology_netcdf_file_right_field*) must agree.
!
!   #) The **-n=**\ *number_of_sing_triplets* argument specifies the number of singular triplets of the SVD
!      of the covariance matrix between the left and right fields, which must be stored (and also computed
!      if **-alg=**\ ``inviter``\  or **-alg=**\ ``deflate``\ is specified) in the output NetCDF files given
!      in the **-o=** and **-o2=** arguments. The default value is ``4``\ .
!
!   #) The **-d=** argument specifies the metric and scalar product used for the left field
!      in the SVD analysis. If:
!
!      * **-d=**\ ``dist2``, the SVDF analysis is done with the diagonal distance
!        associated with the horizontal 2-D grid-mesh of the left field (e.g. each grid point
!        is weighted accordingly to the surface associated with it)
!
!      * **-d=**\ ``ident``, the EOF analysis is done with the identity metric : the
!        Euclidean distance and the usual scalar product is used in the SVD analysis.
!
!      This remark applies also for the **-d2=** argument used to define the distance and
!      scalar product of the right field in the SVD analysis. 
!      If the second NetCDF variable *netcdf_variable_right_field*, used to define the right
!      field in the SVD analysis, has ``4`` dimensions, the following value is also allowed for 
!      the **-d2=** argument:
!
!      * **-d=**\ ``dist3`` meaning that the SVD analysis is done with the diagonal distance 
!        associated with the whole 3D grid of the right field (e.g. each grid point is weighted
!        accordingly to the volume or weight associated with it).
!
!   #) The **-alg=** argument determines how the singular values and singular vectors of the
!      covariance matrix between the left and right fields are computed. If:
!
!      * **-alg=**\ ``svd``, a full SVD of the rectangular covariance matrix is computed
!
!      * **-alg=**\ ``inviter``, a partial SVD of the rectangular covariance matrix
!        is computed by inverse iteration
!
!      * **-alg=**\ ``deflate``, a partial SVD of the rectangular covariance matrix
!        is computed by a deflation technique.
!
!      All algorithms are parallelized if OpenMP is used. The default is **-alg=**\ ``inviter``
!      since computing a partial SVD is generally much faster than computing a full SVD. But,
!      **-alg=**\ ``deflate`` is generally as fast as **-alg=**\ ``inviter``.
!
!   #) If any of the **-nb=**, **-bl=**, **-bp=** and **-cb=** arguments is specified, a moving 
!      block bootstrap algorithm is used to test the significativity of the SCF, NC and, eventually,
!      correlation coefficients associated with each singular triplet of the SVD analysis.
!
!   #) The **-cb=**\ *bootstrap_statistic_significativity_type* argument specifies which
!      statistics are bootstrapped. If **-cb=**\ ``values`` (this is the default),
!      the Square Covariance Fraction (SCF) and the Normalized root-mean-square
!      Covariance (NC) coefficients are tested with the moving block bootstrap algorithm.
!      If **-cb=**\ ``vector``, the correlations between the Singular Variable time series of
!      the left and right fields are also tested in addition of the SCF and NC coefficients.
!      This may require much more computer time since singular vectors of the bootstrap
!      versions of the covariance matrix are needed to estimate the Singular Variable
!      time series.
!      By default, bootstrap tests of these coefficients are not performed.
!
!   #) The **-nb=**\ *number_of_shuffles* argument specifies the number of shuffles for the
!      bootstrap tests of the SCF, NC and correlation coefficients (by default ``99``\ ).
!
!   #) The **-bp=**\ *bootstrap_periodicity* argument specifies that the index, ``i``\ , of the first
!      observation of each selected block in the moving block bootstrap algorithm verifies the condition
!      ``i`` = ``1``\  +\  *bootstrap_periodicity*\ .\ ``j`` where ``j`` is a random positive integer.
!      *bootstrap_periodicity* must be greater than zero and less than the length of the time series.
!      This parameter is useful if the time series are cyclostationary instead of stationary.
!      By default, *bootstrap_periodicity* is set to ``1``\ .
!
!   #) The **-bl=**\ *bootstrap_block_length* argument specifies the size of the blocks in the moving
!      block bootstrap algorithm. *bootstrap_block_length* must be greater than zero and less than the
!      length of the time series.
!      By default, *bootstrap_block_length* is set to *bootstrap_periodicity*\ .\ ``2``\ .
!
!   #) The **-bigfile** argument is allowed only if the NCSTAT software has been compiled with the
!      _USE_NETCDF36 or _USE_NETCDF4 CPP macros (e.g. ``-D_USE_NETCDF36`` or ``-D_USE_NETCDF4``) and linked
!      to the NetCDF 3.6 library or higher.
!      If this argument is specified, the output NetCDF files will be 64-bit offset format files
!      instead of NetCDF classic format files. However, this argument is recognized in the procedure
!      only if the NCSTAT software has been built with the _USE_NETCDF36 or _USE_NETCDF4 CPP macros.
!
!   #) The **-hdf5** argument is allowed only if the NCSTAT software has been
!      compiled with the _USE_NETCDF4 CPP macro (e.g. ``-D_USE_NETCDF4``) and linked
!      to the NetCDF 4 library or higher. 
!      If this argument is specified, the output NetCDF files will be NetCDF-4/HDF5
!      format files instead of NetCDF classic format files. However, this
!      argument is recognized in the procedure only if the NCSTAT software has
!      been built with the _USE_NETCDF4 CPP macro.
!
!   #) The **-mi=**\ *missing_value* argument specifies the missing value indicator associated with
!      the NetCDF variables in the *output_netcdf_file* and *output_netcdf_file_right_field*.
!      If the **-mi=** argument is not specified *missing_value* is set to ``1.e+20``.
!
!   #) The **-double** argument specifies that the results are stored as double-precision floating
!      point numbers in the output NetCDF files.
!
!      By default, the results are stored as single-precision floating point numbers
!      in the output NetCDF files.
!
!   #) It is assumed that the data has no missing values.
!
!   #) Duplicate parameters are allowed, but this is always the last occurrence of a parameter
!      which will be used for the computations. Moreover, the number of specified parameters
!      must not be greater than the total number of allowed parameters.
!
!   #) For more details on SVD analysis in the climate literature, see
!
!      * "A manual for EOF and SVD analyses of climate data",
!        by Bjornsson, H., and Venegas, S.A., McGill University, CCGCR Report No. 97-1,
!        Montral, Qubec, 52pp, 1997. https://www.jsg.utexas.edu/fu/files/EOFSVD.pdf
!
!      * "An intercomparison of methods for finding coupled patterns in climate data",
!        by Bretherton, Smith, C., and Wallace, J.M.,
!        Journal of Climate, Vol. 5, 541-560 pp, 1992. doi: `10.1175/1520-0442(1992)005<0541:AIOMFF>2.0.CO;2`_
!
!      * "Seasonality of large scale atmosphere-ocean interaction over the North Pacific",
!        by Zhang, Y.,  Norris, J.R., and  Wallace, J.M.,
!        Journal of Climate, Vol. 11, 2473-2481 pp, 1998. doi: `10.1175/1520-0442(1998)011<2473:SOLSAO>2.0.CO;2`_
!
! .. _`10.1175/1520-0442(1992)005<0541:AIOMFF>2.0.CO;2` : https://doi.org/10.1175/1520-0442(1992)005<0541:AIOMFF>2.0.CO;2
! .. _`10.1175/1520-0442(1998)011<2473:SOLSAO>2.0.CO;2` : https://doi.org/10.1175/1520-0442(1998)011<2473:SOLSAO>2.0.CO;2
!
! Outputs
! -------
!
!   comp_svd_3d creates two output NetCDF files. The first output file (specified in the
!   **-o=**\ *output_svd_netcdf_file_left_field* argument) contains all the SVD statistics associated with
!   the left field (specified in the **-v=**\ *netcdf_variable* argument) and the second output file
!   (specified in the **-o2=**\ *output_svd_netcdf_file_right_field* argument) the SVD statistics
!   associated with the right field (specified in the **-v2=**\ *netcdf_variable_right_field* argument).
!
!   The *output_svd_netcdf_file_left_field* contains the singular values and left singular vectors of the covariance matrix,
!   the left homogeneous and heterogeneous vectors, and the left SV time series of the SVD analysis. The number
!   of SV time series, regression vectors, singular vectors and singular values stored in the output NetCDF
!   dataset is determined by the **-n=**\ *number_of_sing_triplets* argument. The output NetCDF dataset contains
!   the following NetCDF variables (in the description below, nlat and nlon are the length of the dimensions of
!   the first input NetCDF variable *netcdf_variable*) :
!
!   #) *netcdf_variable*\ _svd\ ``(number_of_sing_triplets,nlat,nlon)`` : the *number_of_sing_triplets* leading left
!      singular vectors of the sums of squares and cross-products (**-a=**\ ``scp``), covariance (**-a=**\ ``cov``)
!      or correlation (**-a=**\ ``cor``) matrix between the left and right fields. The singular vectors are sorted
!      by descending order of the associated singular values.
!
!      The left singular vectors are packed in a tridimensional variable whose first and second dimensions
!      are exactly the same as those associated with the input NetCDF variable *netcdf_variable*
!      even if you restrict the geographical domain with the **-x=** and **-y=** arguments. However,
!      outside the selected domain, the output NetCDF variable is filled with missing values. If
!      this is a problem, you can use :ref:`comp_norm_3d` for restricting the geographical
!      domain in the input dataset before using comp_svd_3d.
!
!   #) *netcdf_variable*\ _hom\ ``(number_of_sing_triplets,nlat,nlon)`` : the selected left homogeneous vectors of the
!      sums of squares and cross-products (**-a=**\ ``scp``), covariance (**-a=**\ ``cov``) or correlation
!      (**-a=**\ ``cor``) matrix. The left homogeneous vectors are sorted by descending
!      order of the associated singular values. These vectors are scaled such that they give the scalar 
!      products (**-a=**\ ``scp``), covariances (**-a=**\ ``cov``) or correlations (**-a=**\ ``cor``) 
!      between the original observed time series in the left field and the left SV time series.
!
!      The left homogeneous vectors are packed in a tridimensional variable whose first and second dimensions
!      are exactly the same as those associated with the input NetCDF variable *netcdf_variable*
!      even if you restrict the geographical domain with the **-x=** and **-y=** arguments. However,
!      outside the selected domain, the output NetCDF variable is filled with missing values.
!
!   #) *netcdf_variable*\ _het\ ``(number_of_sing_triplets,nlat,nlon)`` : the selected left heterogeneous vectors
!      of the sums of squares and cross-products (**-a=**\ ``scp``), covariance (**-a=**\ ``cov``) or correlation
!      (**-a=**\ ``cor``) matrix. The left heterogeneous vectors are sorted by descending
!      order of the associated singular values. These vectors are scaled such that they give the scalar 
!      products (**-a=**\ ``scp``), covariances (**-a=**\ ``cov``) or correlations (**-a=**\ ``cor``) 
!      between the original observed time series in the left field and the right SV time series stored
!      in the other output NetCDF file.
!
!      The left heterogeneous vectors are packed in a tridimensional variable whose first and second dimensions
!      are exactly the same as those associated with the input NetCDF variable *netcdf_variable*
!      even if you restrict the geographical domain with the **-x=** and **-y=** arguments. However,
!      outside the selected domain, the output NetCDF variable is filled with missing values.
!
!   #) *netcdf_variable*\ _sv\ ``(ntime,number_of_sing_triplets)`` : the left SV time series
!      sorted by descending order of the singular values.
!
!      The SV time series are always standardized to unit variance.
!
!   #) *netcdf_variable*\ _SV_STDs\ ``(number_of_sing_triplets)`` : the standard-deviations of the left SV time series
!      sorted by descending order of the singular values.
!
!   #) *netcdf_variable*\ _SV_EXPVARs\ ``(number_of_sing_triplets)`` : the proportion of variance of the left field
!      explained by the left SV time series sorted by descending order of the singular values.
!
!   #) *netcdf_variable*\ _CORSV\ ``(number_of_sing_triplets)`` : the symmetric correlation matrix between the left
!      SV time series, only the upper triangle of this symmetric matrix is written in the output file.
!
!   #) *netcdf_variable*\ _RAW_VAR\ ``(1)`` : the raw variance (or inertia if **-a=**\ ``scp``) of the left field
!      averaged over the selected domain.
!
!   #) *Sing_vals*\ ``(number_of_sing_triplets)`` : the singular values of the sums of
!      squares and cross-products (if **-a=**\ ``scp``) or covariance (if **-a=**\ ``cov``) or
!      correlation (if **-a=**\ ``cor``) matrix between the left and right fields sorted in
!      decreasing order.
!
!   #) *SCFs*\ ``(number_of_sing_triplets)`` : the Squared Covariance Fractions (SCF) described by each of the
!      leading singular triplets of the squares and cross-products (if **-a=**\ ``scp``) or covariance
!      (if **-a=**\ ``cov``) or correlation (if **-a=**\ ``cor``) matrix between the left and right fields.
!
!   #) *NCs*\ ``(number_of_sing_triplets)`` : the Normalized root-mean-square Covariance (NC) coefficients associated
!      with each of the leading singular triplets of the squares and cross-products (if **-a=**\ ``scp``) or covariance
!      (if **-a=**\ ``cov``) or correlation (if **-a=**\ ``cor``) matrix between the left and right fields.
!
!   #) *Corrs*\ ``(number_of_sing_triplets)`` : the correlation coefficients between the corresponding left 
!      and right SV time series.
!
!   #) *SCF_stat_sign*\ ``(number_of_sing_triplets)`` : the critical probabilities associated with the
!      Squared Covariance Fractions (SCF) coefficients associated with each of the leading singular triplets
!      estimated by a moving block bootstrap procedure. Large values indicate significant SCF coefficients.
!      This NetCDF variable is computed and stored only if one of the **-nb=**, **-bl=**, **-bp=** or
!      **-cb=** arguments is specified when calling the procedure.
!
!   #) *NC_stat_sign*\ ``(number_of_sing_triplets)`` : the critical probabilities associated with the Normalized
!      root-mean-square Covariance (NC) coefficients associated with each of the leading singular triplets
!      estimated by a moving block bootstrap procedure. Large values indicate significant NC coefficients.
!      This NetCDF variable is computed and stored only if one of the **-nb=**, **-bl=**, **-bp=** or
!      **-cb=** arguments is specified when calling the procedure.
!
!   #) *Corr_stat_sign*\ ``(number_of_sing_triplets)`` : the critical probabilities associated with the correlation
!      coefficients between the corresponding left and right SV time series estimated by a moving block bootstrap procedure.
!      Large values indicate significant correlation coefficients between the corresponding left and right SV time series.
!      This NetCDF variable is computed and stored only if **-cb=**\ ``vector`` is specified when calling the procedure.
!
!   The *output_svd_netcdf_file_right_field* contains the singular values and right singular vectors of the covariance
!   matrix, the right homogeneous and heterogeneous vectors, and the right SV time series of the SVD analysis. The number
!   of SV time series, regression vectors and singular vectors and singular values stored in this output NetCDF
!   dataset is also determined by the **-n=**\ *number_of_sing_triplets* argument. This output NetCDF dataset contains
!   exactly the same NetCDF variables than the first NetCDF output file, but for the statistics of the right field
!   instead of the left field. Refer to the description above for the content and definition of the NetCDF variables
!   in the file *output_svd_netcdf_file_right_field*.
!
! Examples
! --------
!
!   #)   For computing an SVD analysis from two NetCDF variables ``sst`` and ``slp`` stored,
!        respectively, in the NetCDF files ``HadISST1_1m_1979_2005_sst.nc`` and
!        ``hadslp_1m_1979_2005_slp.nc`` use the following command (note that the analysis
!        is done on the anomalies after removing the annual cycle for each variable since
!        **-a=**\ ``cov``\  is specified) :
!
!        ::
!
!          $ comp_svd_3d \
!            -a=cov \
!            -n=5 \
!            -f=HadISST1_1m_1979_2005_sst.nc  \
!            -v=sst \
!            -c=clim_HadISST1_1m_1979_2005_sst.nc  \
!            -x=111,330 \
!            -y=101,140 \
!            -m=mask_HadISST1_sst.nc \
!            -o=svd_HadISST1_1m_1979_2005_sst_oiatl.nc \
!            -f2=hadslp_1m_1979_2005_slp.nc \
!            -v2=slp \
!            -c=clim_hadslp_1m_1979_2005_slp.nc \
!            -x2=-14,31 \
!            -y2=21,33 \
!            -m=mask_hadslp_slp.nc \
!            -o2=svd_hadslp_1m_1979_2005_slp_oiatl.nc
!
!
!-
! ================================================================================================
!
#ifndef _OPENMP
#undef _PARALLEL_READ
#endif
!
#ifdef _USE_NETCDF4
#define _USE_NETCDF36
#endif
!
! USED MODULES 
! ============
!
#ifdef _USE_NAGWARE
    use f90_unix_env,     only : iargc, getarg
#endif
!
#ifdef _PARALLEL_READ
    use netcdf,           only : nf90_share, nf90_open
#endif
!
#ifdef _OPENMP
    use Statpack,         only : omp_limit
    use omp_lib,          only : omp_get_num_procs, omp_get_max_threads
#endif
!
#ifdef _BLAS
    use BLAS_interfaces,  only : gemm
#endif
!
#ifdef _MATMUL
    use Statpack,         only : matmul=>matmul2
#endif
!
#ifdef _TRANSPOSE
    use Utilities,        only : transpose=>transpose2
#endif
!
#ifdef _USE_NETCDF36
    use netcdf,    only : nf90_64bit_offset
#endif
!
#ifdef _USE_NETCDF4
    use netcdf,    only : nf90_hdf5
#endif
!
    use netcdf,           only : nf90_max_var_dims, nf90_max_name, nf90_get_att, nf90_noerr, nf90_get_var, &
                                 nf90_create, nf90_clobber, nf90_strerror, nf90_set_fill, nf90_nofill,     &
                                 nf90_def_dim, nf90_def_var, nf90_real8, nf90_real4, nf90_inq_attname,     &
                                 nf90_copy_att, nf90_put_att, nf90_enddef, nf90_global, nf90_unlimited,    &
                                 nf90_inq_varid, nf90_put_var, nf90_close, nf90_sizehint_default
!
    use Statpack,         only : i1b, i4b, stnd, extd, lgl, true, false, zero, one, c100,                 & 
                                 kint, kfix, kexp, tolower, case_change, string_to_val,                   &
                                 allocate_error, merror, arth, norm, lassq, triangle,                     &
                                 svd_cmp, select_singval_cmp2, bd_inviter2, bd_deflate2, comp_pc_mca,     &
                                 random_number_, random_seed_, lamch
!
    use NCSTAT_Utilities, only : prtunit, nchar, sp, dp, nlon_def2, nlat_def2, ndepth_def2,              &
                                 nlon_def4, nlat_def4, def_missing,                                      &
                                 get_listarg, open_2_files, get_variables_dimcoords, get_variable_dims,  &
                                 check_lon_domain, select_grid_points_2d, select_grid_points_3d,         &
                                 duplicate_orca_points_2d, duplicate_orca_points_3d 
!
!
! STRONG TYPING IMPOSED 
! =====================
!
    implicit none
!
!
! PARAMETERS 
! ==========
!
    integer(i4b), parameter ::                                      &
!
!  maxiter IS THE NUMBER OF INVERSE ITERATIONS PERFORMED WHEN COMPUTING A
!  PARTIAL SVD OF THE COVARIANCE MATRIX WITH THE INVERSE ITERATION METHOD.
!
                                   maxiter=2,                       &
!
!  max_qr_steps IS THE NUMBER OF QR SWEEPS PERFORMED AS A BACKUP PROCEDURE IF DEFLATION FAILED
!  WHEN COMPUTING A PARTIAL SVD OF THE COVARIANCE OR CORRELATION MATRIX WITH THE DEFLATION METHOD.
!
                                   max_qr_steps=4,                  &
!
!  blksz IS THE BLOCK SIZE FOR COMPUTING THE COVARIANCE MATRIX IF A BLAS LIBRARY
!  IS NOT USED.
!
                                   blksz=60         
!
    logical(lgl), parameter ::                                      &
!
!  ortho DETERMINES IF THE ORTHOGONALITY OF THE SINGULAR VECTORS IS AUTOMATICALLY PRESERVED
!  WHEN COMPUTING A PARTIAL SVD OF THE COVARIANCE MATRIX WITH THE DEFLATION METHOD.
!
                                   ortho=false
!
    character(len=*), parameter :: name_proc='comp_svd_3d : ',                  &
!
                                   cmethod(3)=(/"scp","cov","cor"/),            &          
!
                                   cdistance(3)=(/"dist2","dist3","ident"/),    &   
!
                                   cresolution(2)=(/"r2","r4"/),                &
!
                                   csignificance(2)=(/"values","vector"/),      &        
!
                                   calgorithm(3)=(/"inviter","deflate","svd    "/)
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
#ifndef _USE_NAGWARE
    integer(i4b)  :: iargc
#endif
    integer(i4b)                 :: narg, status, i, i1, i2, j, j1, j2, k, l, l1, l2, m, nm, mm, nsvd, nsing, alg,  &
                                    method, nb, bl, cb, bp, sb, var_nb, nb_block, nb_block_max, rec, offset_rec,    &
                                    j1_var, j2_var, lon1_var, lon2_var, lon3_var,                                   &
                                    lat1_var, lat2_var, lat3_var, level1_var, level2_var, level3_var,               &
                                    ntime, output_type, output_mode, eid, meanid, stdid
    integer(i4b), dimension(2)   :: ncid_data, ncid_clim, ncid_mask, ncid_svd,                                      &
                                    varid, maskid, sing_valid, rawvarid, singvarid, ncid, corrsvid, pcvarid,        &
                                    varexpid, stat_sig_singvarid, stat_sig_ncid, stat_sig_corrid,                   &
                                    svdid, homid, hetid, pcid, pccorpid,                                            &
                                    nvar, distance, time1, time2, time3,                                            &
                                    lon1, lon2, lon3,  lat1, lat2, lat3, level1, level2, level3,                    &
                                    nlon_def, nlat_def, nlevel_def, dumm, dim_tmp
    integer(i4b), dimension(4)                   :: startlec, countlec
    integer(i4b), dimension(4,2)                 :: ndim
    integer(i4b), dimension(5,2)                 :: coor_id2
    integer(i4b), dimension(nf90_max_var_dims,2) :: dimids, dimlens, coor_id, coor_natts, coor_type
    integer(i4b), dimension(:),  allocatable     :: index
!
    integer(i1b)  :: kcode
!
    real(stnd)                                          :: tmp, scl, ssq, ssqcov, missing, safmin, abstol
    real(stnd), dimension(:),     allocatable           :: d, e, tauq, taup,                                &
                                                           sing_val, sing_valb, singvar, singvarb,          &
                                                           nc, ncb, corrsv, corrsvb,                        &
                                                           stat_sig_singvar, stat_sig_nc, stat_sig_corr
    real(stnd), dimension(:),     allocatable, target   :: scal_1, scal_2
    real(stnd), dimension(:),                  pointer  :: pt_scal
    real(stnd), dimension(:,:),   allocatable           :: temp_1, temp_2, mean, std, mat_cov, varexp,      &
                                                           right_vecb, left_vecb, pcvar, pcvarb, pccorp
    real(stnd), dimension(:,:),   allocatable, target   :: mat_1, mat_2, right_vec, left_vec,               &
                                                           pccor_1, pccor_2, pccor_1_2, pccor_2_1
    real(stnd), dimension(:,:),                pointer  :: pt_mat
    real(stnd), dimension(:,:,:), allocatable           :: temp_2_3d, pc
!
    real(extd)                              :: var1, var2
    real(extd), dimension(2)                :: raw_var
    real(extd), dimension(:),  allocatable  :: record
!
    logical(lgl)                                         :: failure, first, last, do_bootstrap,       &
                                                            set_d2, use_tlimited, create_hdf5
    logical(lgl), dimension(2)                           :: use_x, use_y, use_z
    logical(lgl), dimension(nf90_max_var_dims,2)         :: use_coor_variable
    logical(lgl), dimension(:,:),   allocatable, target  :: mask_1, mask_2
    logical(lgl), dimension(:,:),                pointer :: pt_mask
    logical(lgl), dimension(:,:,:), allocatable          :: mask_2_3d
!
    character(len=14)                            :: fmt
    character,                    dimension(2)   :: chr, grid_type
    character(len=nchar)                         :: name, temp_name, history
    character(len=nf90_max_name)                 :: att_name
    character(len=nf90_max_name), dimension(2)   :: fic_name, var_name, clim_name,  &
                                                    mask_name, svd_name
    character(len=nf90_max_name), dimension(nf90_max_var_dims,2) :: dim_names
!
#ifdef _PARALLEL_READ
    integer(i4b)  :: ncid2, varid2
#endif
!
#ifdef _OPENMP
    logical(lgl) :: test_par, parallel_read
#endif
!
!
! EXECUTABLE STATEMENTS
! =====================
!
!   GET THE NUMBER OF COMMAND LINE ARGUMENTS.
!
    narg = iargc()
!
!   CHECK THE NUMBER OF COMMAND LINE ARGUMENTS.
!
    if ( narg==0 ) then
!
        write (prtunit,*)
        write (prtunit,*) ' Purpose : '
        write (prtunit,*)
        write (prtunit,*) ' Compute an SVD analysis from a tridimensional variable and a tri or   '
        write (prtunit,*) ' four dimensional variable extracted from one or two NetCDF datasets.'
        write (prtunit,*)
        write (prtunit,*) ' Usage : '
        write (prtunit,*)
        write (prtunit,*) ' comp_svd_3d  -f=input_netcdf_file'
        write (prtunit,*) '              -v=netcdf_variable'
        write (prtunit,*) '              -m=input_mesh_mask_netcdf_file'
        write (prtunit,*) '              -a=type_of_analysis                            (optional : scp, cov, cor)'
        write (prtunit,*) '              -n=number_of_sing_triplets                     (optional)'
        write (prtunit,*) '              -g=grid_type                                   (optional : n, t, u, v, w, f)'
        write (prtunit,*) '              -r=resolution                                  (optional : r2, r4)'
        write (prtunit,*) '              -b=nlon_orca, nlat_orca                        (optional)'
        write (prtunit,*) '              -x=lon1,lon2                                   (optional)'
        write (prtunit,*) '              -y=lat1,lat2                                   (optional)'
        write (prtunit,*) '              -t=time1,time2                                 (optional)'
        write (prtunit,*) '              -c=input_climatology_netcdf_file               (optional)'
        write (prtunit,*) '              -d=type_of_distance                            (optional : dist2, ident)'
        write (prtunit,*) '              -o=output_svd_netcdf_file_left_field           (optional)'
        write (prtunit,*) '              -f2=input_netcdf_file_right_field              (optional)'
        write (prtunit,*) '              -v2=netcdf_variable_right_field                (optional)'
        write (prtunit,*) '              -m2=input_mesh_mask_netcdf_file_right_field    (optional)'     
        write (prtunit,*) '              -g2=grid_type_right_field                      (optional : n, t, u, v, w, f)'
        write (prtunit,*) '              -r2=resolution_right_field                     (optional : r2, r4)'
        write (prtunit,*) '              -b2=nlon_orca, nlat_orca                       (optional)'
        write (prtunit,*) '              -x2=lon1_right_field,lon2_right_field          (optional)'
        write (prtunit,*) '              -y2=lat1_right_field,lat2_right_field          (optional)'
        write (prtunit,*) '              -z2=level1_right_field,level2_right_field      (optional)'
        write (prtunit,*) '              -t2=time1_right_field,time2_right_field        (optional)'
        write (prtunit,*) '              -c2=input_climatology_netcdf_file_right_field  (optional)'
        write (prtunit,*) '              -d2=type_of_distance_right_field               (optional : dist2, dist3, ident)'
        write (prtunit,*) '              -o2=output_svd_netcdf_file_right_field         (optional)'
        write (prtunit,*) '              -alg=algorithm                                 (optional : inviter, deflate, svd)'
        write (prtunit,*) '              -cb=bootstrap_statistic_significance_type      (optional : values, vector)'
        write (prtunit,*) '              -nb=number_of_shuffles                         (optional)'
        write (prtunit,*) '              -bp=bootstrap_periodicity                      (optional)'
        write (prtunit,*) '              -bl=bootstrap_block_length                     (optional)'
        write (prtunit,*) '              -mi=missing_value                              (optional)'
        write (prtunit,*) '              -double                                        (optional)'
#ifdef _USE_NETCDF36
        write (prtunit,*) '              -bigfile                                       (optional)'
#endif
#ifdef _USE_NETCDF4
        write (prtunit,*) '              -hdf5                                          (optional)'
#endif
        write (prtunit,*) '              -tlimited                                      (optional)'
        write (prtunit,*)
        write (prtunit,*) ' By default : '
        write (prtunit,*)
        write (prtunit,*) '-a=scp   : the SVD analysis is done on the raw data'
        write (prtunit,*) '-n=4     : the four leading singular triplets are computed'
        write (prtunit,*) '-g=n     : the grid of the left field is regular or gaussian'
        write (prtunit,*) '-r=r2    : if the grid_type is not n, orca2 configuration is assumed'
        write (prtunit,*) '-b= by default, nlon_orca and nlat_orca are determined from the -r='
        write (prtunit,*) '    argument'
        write (prtunit,*) '-d=dist2 : distances in the left field are computed with the metric associated with the grid-mesh'
        write (prtunit,*) '-x= the whole longitude domain associated with the netcdf_variable'
        write (prtunit,*) '-y= the whole latitude domain associated with the netcdf_variable'
        write (prtunit,*) '-t= the whole time period associated with the netcdf_variable' 
        write (prtunit,*) '-o=svd_left_netcdf_variable.nc'
        write (prtunit,*) '-f2= same as the -f= argument'
        write (prtunit,*) '-v2= same as the -v= argument'
        write (prtunit,*) '-m2= same as the -m= argument'
        write (prtunit,*) '-g2= same as -g= if -v2= is omitted and -g2=n otherwise'
        write (prtunit,*) '-r2= same as -r= if -v2= is omitted and -r2=r2 otherwise'
        write (prtunit,*) '-b2= by default, nlon_orca and nlat_orca are determined from the -r2='
        write (prtunit,*) '     argument'
        write (prtunit,*) '-x2= the whole longitude domain associated with the netcdf_variable_right_field'
        write (prtunit,*) '-y2= the whole latitude domain associated with the netcdf_variable_right_field'
        write (prtunit,*) '-z2= the whole level associated with the netcdf_variable_right_field'
        write (prtunit,*) '-t2= the whole time period associated with the netcdf_variable_right_field' 
        write (prtunit,*) '-c2= only required if -a=cov or -a=cor'
        write (prtunit,*) '-d2= same as -d= if -v2= is omitted, -d2=dist2 if the netcdf_variable_right_field'
        write (prtunit,*) '     is a 3D variable, -d2=dist3 if the netcdf_variable_right_field is a 4D variable'
        write (prtunit,*) '-o2=svd_right_netcdf_variable_right_field.nc'
        write (prtunit,*) '-alg=inviter : the singular triplets are computed by inverse iteration'
        write (prtunit,*) '-cb=values : only SCF and NC coefficients are tested by the bootstrap algorithm'
        write (prtunit,*) '-nb=99 : the number_of_shuffles in the moving block bootstrap algorithm is set to 99 '
        write (prtunit,*) '-bp=1 : this means that no bootstrap_periodicity is used in the bootstrap algorithm'
        write (prtunit,*) '-bl= the block length in the moving block bootstrap algorithm is set to'
        write (prtunit,*) '     bootstrap_periodicity*2'
        write (prtunit,*) '-mi= the missing_value is equal to 1.e+20'
        write (prtunit,*) '-double   : by default, the results are stored as single floating numbers'
#ifdef _USE_NETCDF36
        write (prtunit,*) '-bigfile : by default, a NetCDF classical format file is created'
#endif
#ifdef _USE_NETCDF4
        write (prtunit,*) '-hdf5     : by default, a NetCDF classical format file is created'
#endif
        write (prtunit,*) '-tlimited : by default, the time dimension is defined as unlimited'
        write (prtunit,*)
#ifdef _OPENMP
        write (prtunit,*) 'This procedure is parallelized with OpenMP'
#endif
        write (prtunit,*)
!
        stop
!
    else if ( narg<3 .or. narg>37 ) then
!
        call merror( name_proc//"Wrong number of line arguments" )
!
    end if
!
    fic_name(:)  = " "
    var_name(:)  = " "
    mask_name(:) = " "
    clim_name(:) = " "
    svd_name(:)  = " "
    temp_name(:) = " "
!
    history = "comp_svd_3d"
!
!   DEFAULT VALUES FOR COMMAND LINE ARGUMENTS.
!
    output_mode   = nf90_clobber
    output_type   = nf90_real4
    grid_type(:)  = "n"
    nlat_def(:)   = nlat_def2
    nlon_def(:)   = nlon_def2
    nlevel_def(1) = 1
    nlevel_def(2) = ndepth_def2
!
    distance(:)  = 1
    method       = 1
    alg          = 1
    nsvd         = 4
!
    cb           = 1
    nb           = 99
    bl           = 0
    bp           = 0
!
    missing     = def_missing
!
    lon1(:)      = 1
    lon2(:)      = huge( lon2 )
    lat1(:)      = 1
    lat2(:)      = huge( lat2 )
    level1(:)    = 1
    level2(:)    = huge( level2 )
    time1(:)     = 1
    time2(:)     = huge( time2 )
!
    use_x(:)     = false
    use_y(:)     = false
    use_z(:)     = false
    set_d2       = false
    do_bootstrap = false
    create_hdf5  = false
    use_tlimited = false
!
!   GET THE COMMAND LINE ARGUMENTS.
!
    do i = 1, narg
!
        call getarg( i, name )
!
        history = trim(history)//" "//trim(name)
!
        call case_change( name(:3), tolower )
!
        if ( name(3:3)=="2" ) then 
          var_nb = 2
          temp_name = name(:2)//name(4:)
          name = temp_name
        else
          var_nb = 1
        end if
!
        select case ( name(1:3 ) )
!
            case ('-f=')
!
!               GET THE NAME OF THE NETCDF DATASET.
!
                fic_name(var_nb) = name(4:)
!
            case ('-v=')
!
!               GET THE NAME OF THE FIRST NETCDF VARIABLE IN THE DATASET.
!
                var_name(var_nb) = name(4:)
!
            case ('-g=')
!
!               GET THE GRID-TYPE.
!
                call case_change( name(4:), tolower )
!
                select case ( name(4:) )
                    case ( 'n', 'u', 'v', 'w', 't', 'f' )
                        grid_type(var_nb) = name(4:4)
                    case default
                        call merror( name_proc//"Unknown grid type" )
                end select
!
            case ('-r=')
!
!               DETERMINE THE RESOLUTION FOR THE ORCA GRID.
!
                call case_change( name(4:), tolower )
!
                select case ( name(4:) )
                    case ( cresolution(1) )
                        nlat_def(var_nb)  = nlat_def2
                        nlon_def(var_nb)  = nlon_def2
                    case ( cresolution(2) )
                        nlat_def(var_nb)  = nlat_def4
                        nlon_def(var_nb)  = nlon_def4
                    case default
                        call merror( name_proc//"Unknown resolution option" )
                end select
!
            case ('-b=')
!
!               GET THE DIMENSIONS OF THE ORCA GRID.
!
                call get_listarg( name_proc, name(4:),                                            &
                                  "nlon_orca and nlat_orca are integer values",                   &
                                  "Wrong ORCA grid definition for the first or second variable ", &
                                  nlon_def(var_nb), nlat_def(var_nb) )
!
                if ( var_nb==2 ) then
                    nlevel_def(2) = huge( nlevel_def )
                end if
!
            case ('-x=')
!
!               GET THE DOMAIN DEFINITION (Longitude).
!
                call get_listarg( name_proc, name(4:),                    &
                                  "lon1 and lon2 are integer values",     &
                                  "Wrong domain definition (longitude)",  &
                                  lon1(var_nb), lon2(var_nb) )
!
                use_x(var_nb) = true
!
            case ('-y=')
!
!               GET THE DOMAIN DEFINITION (latitude).
!
                call get_listarg( name_proc, name(4:),                    &
                                  "lat1 and lat2 are integer values",     &
                                  "Wrong domain definition (latitude)",   &
                                  lat1(var_nb), lat2(var_nb) )
!
                use_y(var_nb) = true
!
            case ('-z=')
!
!               GET THE LEVEL DEFINITION.
!
                if ( var_nb==1 )                                          &
                call merror( name_proc//"Wrong level option (left variable is 3d)" )
                call get_listarg( name_proc, name(4:),                    &
                                  "level1 and level2 are integer values", &
                                  "Wrong level definition",               &
                                  level1(var_nb), level2(var_nb) )
!
                use_z(var_nb) = true
!
            case ('-t=')
!
!               GET THE PERIOD DEFINITION.
!
                call get_listarg( name_proc, name(4:),                    &
                                  "time1 and time2 are integer values",   &
                                  "Wrong period definition",              &
                                  time1(var_nb), time2(var_nb) )
!
            case ('-m=')
!
!               GET THE NAME OF THE NETCDF MESH-MASK DATASET.
!
                mask_name(var_nb) = name(4:)
!
            case ('-c=')
!
!               GET THE NAME OF THE NETCDF CLIMATOLOGY DATASET.
!
                clim_name(var_nb) = name(4:)
!
            case ('-a=')
!
!               DETERMINE THE METHOD OF ANALYSIS.
!
                call case_change( name(4:), tolower )
!
                select case ( name(4:) )
                    case ( cmethod(1) )
                        method = 1
                    case ( cmethod(2) )
                        method = 2
                    case ( cmethod(3) )
                        method = 3
                    case default
                        call merror( name_proc//"Unknown analysis option" )
                end select
!
            case ('-d=')
!
!               DETERMINE THE DISTANCE FOR THE ANALYSIS.
!
                call case_change( name(4:), tolower )
!
                select case ( name(4:) )
!
                    case ( cdistance(1) )
                        distance(var_nb) = 1
                    case ( cdistance(2) )
                        if ( var_nb==1 )                                     &
                        call merror( name_proc//"Wrong distance option (left variable is 3d)" )
                        distance(var_nb) = 2
                    case ( cdistance(3) )
                        distance(var_nb) = 3
                    case default
                        call merror( name_proc//"Unknown distance option" )
!
                end select               
!
                if ( var_nb==2 )  set_d2 = true
!
            case ('-o=')
!
!               GET THE NAME OF THE NEW NETCDF DATASET FOR STORING THE SVD ANALYSIS.
!
                svd_name(var_nb) = name(4:)
!
            case ('-n=')
!
!               GET THE NUMBER OF SINGULAR TRIPLETS TO COMPUTE.
!
                call string_to_val( name(4:), kcode, fmt )
                if ( kcode==kint ) then
                    read( name(4:), fmt ) nsvd
                end if
                if ( kcode/=kint .or. nsvd<=0 )                                        &
                call merror( name_proc//"The number of singular triplets is a positive integer value" )
!
            case default
!
                call case_change( name(4:), tolower )
!
                select case ( name(1:5 ) )
!
                    case ('-alg=')
!
!                       DETERMINE THE ALGORITHM FOR THE COMPUTATION OF SVD.
!
                        select case ( name(6:) )
                            case ( calgorithm(1) )
                                alg = 1
                            case ( calgorithm(2) )
                                alg = 2
                            case ( calgorithm(3) )
                                alg = 3
                            case default
                                call merror( name_proc//"Unknown algorithm option" )
                        end select
!
                    case default
!
                        select case ( name(1:4 ) )
!
                            case ('-mi=')
!
!                               GET THE MISSING VALUE INDICATOR.
!
                                call string_to_val( name(5:), kcode, fmt )
                                if ( kcode/=kfix .and. kcode/=kexp )                &
                    call merror( name_proc//"The -mi= argument must be a real number " )
                                read( name(5:), fmt ) missing
!
                            case ('-cb=')
!
!                               BOOTSTRAP ALGORITHM:
!                               DETERMINE WHAT KIND OF STATISTICS ARE TESTED.
!
                                select case ( name(5:) )
!
                                    case ( csignificance(1) )
                                        cb = 1
                                    case ( csignificance(2) )
                                        cb = 2
                                    case default
                                        call merror( name_proc//"Unknown significance tests option" )
!
                                end select
!
                                do_bootstrap = true
!
                            case ('-nb=')
!
!                               BOOTSTRAP ALGORITHM:
!                               DETERMINE THE NUMBER OF SCRAMBLED SVDs.
!
                                call string_to_val( name(5:), kcode, fmt )
                                if ( kcode==kint ) then
                                    read( name(5:), fmt ) nb
                                end if
                                if ( kcode/=kint .or. nb<=0)                                         &
                    call merror( name_proc//"The number of shuffles is a positive integer value" )
!
                                do_bootstrap = true
!
                            case ('-bl=')
!
!                               BOOTSTRAP ALGORITHM:
!                               DETERMINE BLOCK LENGTH.
!
                                call string_to_val( name(5:), kcode, fmt )
                                if ( kcode==kint ) then
                                    read( name(5:), fmt ) bl
                                end if
                                if ( kcode/=kint .or. bl<=0)                                            &
                     call merror( name_proc//"The bootstrap block length is a positive integer value" )
!
                                do_bootstrap = true
!
                            case ('-bp=')
!
!                               BOOTSTRAP ALGORITHM:
!                               DETERMINE BOOTSTRAP PERIODICITY.
!
                                call string_to_val( name(5:), kcode, fmt )
                                if ( kcode==kint ) then
                                    read( name(5:), fmt ) bp
                                end if
                                if ( kcode/=kint .or. bp<=0)                                           &
                     call merror( name_proc//"The bootstrap periodicity is a positive integer value" )
!
                                do_bootstrap = true
!
                            case default
!
                                select case ( name(1:) )
!
                                    case ('-double')
!
!                                       STORE RESULTS AS DOUBLE PRECISION NUMBERS.
!
                                        output_type = nf90_real8
!
#ifdef _USE_NETCDF36
                                    case ('-bigfile')
!
!                                       REDEFINE THE CREATION MODE FLAG FOR THE OUTPUT FILE.
!
                                        output_mode = nf90_64bit_offset
#endif
!
#ifdef _USE_NETCDF4
                                    case ('-hdf5')
!
!                                       REDEFINE THE CREATION MODE FLAG FOR THE OUTPUT FILE.
!
                                        output_mode = nf90_hdf5
                                        create_hdf5 = true
#endif
!
                                    case ('-tlimited')
!
!                                       DEFINE THE TIME DIMENSION AS LIMITED.
!
                                        use_tlimited = true
!
                                    case default
!
!                                       WRONG ARGUMENT.
!
                                        call merror( name_proc//"Wrong argument" )
!
                                end select
!
                        end select
!
                end select
!
        end select
!
    end do
!
!   CHECK THE COMMAND LINE ARGUMENTS.
!
!   CHECK FIRST INPUT FILE NAME.
!
    if ( fic_name(1)==" " )  call merror( name_proc//"Input NETCDF file is missing" )
!
!   CHECK SECOND INPUT FILE NAME AND DEFINE DEFAULT IF NECESSARY.
!
    if ( fic_name(2)==" " )  fic_name(2) = fic_name(1)
!
!   CHECK FIRST VARIABLE NAME.
!
    if ( var_name(1)==" " )  call merror( name_proc//"Input NETCDF variable is missing" )
!
!   CHECK SECOND VARIABLE NAME AND DEFINE DEFAULT IF NECESSARY.
!
    if ( var_name(2)==" " )  var_name(2) = var_name(1)
!
!   RESET GRID TYPE, RESOLUTION, DISTANCE IF NECESSARY.
!
    if ( fic_name(2)==fic_name(1) .and. var_name(2)==var_name(1) )  then
        grid_type(2)  = grid_type(1)
        nlat_def(2)   = nlat_def(1)
        nlon_def(2)   = nlon_def(1)
        nlevel_def(2) = nlevel_def(1)
        if ( .not.set_d2 ) distance(2) = distance(1)
    end if
!
!   CHECK FIRST MASK FILE NAME.
!
    if ( mask_name(1)==" " )  call merror( name_proc//"Input mesh-mask NETCDF file is missing" )
!
!   CHECK SECOND MASK FILE NAME AND DEFINE DEFAULT IF NECESSARY.
!
    if ( mask_name(2)==" " )  mask_name(2) = mask_name(1)
!
!   CHECK FIRST CLIMATOLOGY FILE NAME.
!
    if ( clim_name(1)==" " .and. method>=2 )                                &
    call merror( name_proc//"Climatology NETCDF file for "//var_name(1)//"is missing" )
!
!   CHECK SECOND CLIMATOLOGY FILE NAME AND DEFINE DEFAULT IF NECESSARY.
!
    if ( clim_name(2)==" " .and. method>=2 ) then
        if ( fic_name(1)==fic_name(2) .and. var_name(2)==var_name(1) ) then
            clim_name(2) = clim_name(1)
        else
            call merror( name_proc//"Climatology NETCDF file for "//var_name(2)//"is missing" )
        end if
    end if
!
!   CHECK OUTPUT FILE NAMES AND DEFINE DEFAULTS IF NECESSARY.
!
    if ( svd_name(1)==" " ) svd_name(1) ="svd_left_"//trim(var_name(1))//".nc"
    if ( svd_name(2)==" " ) svd_name(2) ="svd_right_"//trim(var_name(2))//".nc"
!
!   OPEN THE NETCDF DATASETS.
!
    call open_2_files( name_proc, fic_name, ncid_data)
!
    call open_2_files( name_proc, mask_name, ncid_mask)
!
    if( method>=2 ) call open_2_files( name_proc, clim_name, ncid_clim)
!
!   GET THE VARIABLE IDS FROM THEIR NAMES.
!   FIND THE EXTERNAL TYPE, THE DIMENSION LENGTHS AND NAMES FOR THE NETCDF VARIABLES.
!   GET THE COORDINATE VARIABLES IDS FROM THEIR NAME, IF ANY.
!   FIND THE EXTERNAL TYPE, THE DIMENSION ID AND THE NUMBER OF ATTRIBUTES 
!   FOR THE COORDINATE VARIABLES.
!
    call get_variables_dimcoords( name_proc, var_name, ncid_data,                     &
                                  varid, dimids, dim_names, dimlens,                  &
                                  coor_id, coor_natts, coor_type, use_coor_variable   )
!
    dim_tmp(:2) = count( dimlens/=0, dim=1 )
!
    if ( dim_tmp(1)/=3 )                               &
    call merror( name_proc//"Input first NETCDF variable is not tridimensional" )
    if ( dim_tmp(2)/=3 .and. dim_tmp(2)/=4 )           &
    call merror( name_proc//"Input second NETCDF variable is not tridimensional or fourdimensional" )
!
!   RESET DEFAULT DISTANCE IF THE RIGHT VARIABLE IS 4D.
!
    if ( .not.set_d2 .and. dim_tmp(2)==4 )  distance(2) = 2
!
    do var_nb =1, 2
!
        ndim(:4,var_nb) = dimlens(:4,var_nb)
!
        if ( dim_tmp(var_nb)==3 ) then
            ndim(4,var_nb) = ndim(3,var_nb)
            ndim(3,var_nb) = 0
        endif
!
!       CONSTRUCT MASK VARIABLE NAME.
!
        chr(var_nb) = merge( 't', grid_type(var_nb), grid_type(var_nb)=='w' )
        temp_name = chr(var_nb)//"mask"
!
!       GET THE MASK ID FROM ITS NAME.
!       VERIFY THE DIMENSION LENGTHS FOR THE NETCDF MASK VARIABLE.
!
        call get_variable_dims( name_proc, temp_name, ncid_mask(var_nb), maskid(var_nb), &
                                dimids(:,var_nb), dimlens(:,var_nb) )
!
        if ( any( dimlens(:3,var_nb)/=ndim(:3,var_nb) .and. ndim(:3,var_nb)/=0 ) )                 &
        call merror( name_proc//"Shape disagreement with the masks" )
!
!       GET THE BOUNDARIES OF THE GEOGRAPHICAL DOMAIN FROM THE MESH-MASK DATASET.
!
        if ( .not.use_x(var_nb) ) then
!
            status = nf90_get_att( ncid_mask(var_nb), maskid(var_nb), "lon1_Eastern_limit", i )
            if ( status==nf90_noerr ) lon1(var_nb) = i
!
            status = nf90_get_att( ncid_mask(var_nb), maskid(var_nb), "lon2_Western_limit", i )
            if ( status==nf90_noerr ) lon2(var_nb) = i
!
        end if
!
        if ( .not.use_y(var_nb) ) then
!
            status = nf90_get_att( ncid_mask(var_nb), maskid(var_nb), "lat1_Southern_limit", i )
            if ( status==nf90_noerr ) lat1(var_nb) = i
!
            status = nf90_get_att( ncid_mask(var_nb), maskid(var_nb), "lat2_Northern_limit", i )
            if ( status==nf90_noerr ) lat2(var_nb) = i
!
        end if
!
        if ( .not.use_z(var_nb) .and. dim_tmp(var_nb)==4 ) then
!
            status = nf90_get_att( ncid_mask(var_nb), maskid(var_nb), "level1_First_level", i )
            if ( status==nf90_noerr ) level1(var_nb) = i
!
            status = nf90_get_att( ncid_mask(var_nb), maskid(var_nb), "level2_Last_level", i )
            if ( status==nf90_noerr ) level2(var_nb) = i
!
        end if
!
!       CHECK THE BOUNDARIES OF THE GEOGRAPHICAL DOMAIN.
!       CHECK THE GEOGRAPHICAL DOMAIN.
!       REMOVE DUPLICATE POINTS FROM THE ORCA_R2 GRID.
!
        call check_lon_domain( grid_type(var_nb), ndim(1,var_nb), nlon_def(var_nb),    &
                               lon1(var_nb), lon2(var_nb), lon3(var_nb)    )
!
        if ( grid_type(var_nb)=='n' ) cycle
!
        if ( ndim(1,var_nb)>nlon_def(var_nb) )                                                      &
        call merror( name_proc//"Input NETCDF variable(s) is not on the ORCA GRID (longitude)" )
        if ( ndim(2,var_nb)>nlat_def(var_nb) )                                                      &
        call merror( name_proc//"Input NETCDF variable(s) is not on the ORCA GRID (latitude)" )
        if ( ndim(3,var_nb)>nlevel_def(var_nb) )                                                    &
        call merror( name_proc//"Input NETCDF variable(s) is not on the ORCA GRID (level)" )
!
!       A TRICK TO DISABLE THE ELIMINATION AND DUPLICATION IN THE HIGH LATITUDES OF THE ORCA GRID
!       IF THE RESOLUTION IS NOT R2 OR R4.
!
        if ( ( nlat_def(var_nb)/=nlat_def2 .or. nlon_def(var_nb)/=nlon_def2 ) .and.   &
           ( nlat_def(var_nb)/=nlat_def4 .or. nlon_def(var_nb)/=nlon_def4 ) )         &
            nlat_def(var_nb) = nlat_def(var_nb) + 1
!
    end do
!
    if ( any( lon3(:)<=0 )  ) call merror( name_proc//"Wrong domain definition (longitude)" )
!
    lat1(:) = max( lat1(:), 1 )
    lat2(:) = min( lat2(:), ndim(2,:) )
    lat3(:) = lat2(:) - lat1(:) + 1
    if ( any( lat3(:)<=0 )  ) call merror( name_proc//"Wrong domain definition (latitude)" )
!
    if ( dim_tmp(2)==4 ) then 
        level1(2) = max( level1(2), 1 )
        level2(2) = min( level2(2), ndim(3,2) )
        level3(2) = level2(2) - level1(2) + 1
        if ( level3(2)<=0 ) call merror( name_proc//"Wrong vertical definition" )
    end if
!
!   CHECK THE PERIOD DEFINITIONS.
!
    time1(:) = max( time1(:), 1 )
    time2(:) = min( time2(:), ndim(4,:) )
    time3(:) = time2(:) - time1(:) + 1
!
    if ( any(time3(:)<=0) )  call merror( name_proc//"Wrong time definition" ) 
    if ( time3(1)/=time3(2) )       &
        call merror( name_proc//"Time intervals mismatch between the left and right fields" )
    ntime = time3(1)
!
!   CHECK PARAMETERS OF THE MOVING BLOCK BOOTSTRAP ALGORITHM.
!
    if ( do_bootstrap ) then
!
        if ( bp==0 ) then
            bp = 1
        else
            if ( bp>=ntime )     &
            call merror( name_proc//"Bootstrap periodicity must be less than the number of time observations" )
        end if
!
        if ( bl==0 ) then
            bl = 2*bp
        else
            if ( bl<bp )     &
            call merror( name_proc//"Block length must be greater or equal to the bootstrap periodicity" )
        end if
!
        if ( bl>=ntime )     &
        call merror( name_proc//"Block length must be less than the number of time observations" )
!
!       CALCULATE MAXIMUM NUMBER OF BLOCKS AND REAL NUMBER OF BLOCKS.
!
        nb_block_max = real( (ntime - bl + 1_i4b)/bp, stnd )
!
        if ( nb_block_max<=4 )     &
        call merror( name_proc//"Bad choice of bootstrap algorithm parameters" )
!
        nb_block = ntime / bl 
        if ( ( nb_block * bl )/=ntime ) nb_block = nb_block + 1
!
    else
!
        cb = 0
!
    end if
!
!   ALLOCATE MASK ARRAYS.
!
    if ( dim_tmp(2)==3 ) then
        allocate( mask_1(ndim(1,1),ndim(2,1)), mask_2(ndim(1,2),ndim(2,2)),   &
                  stat=status )
    else
        allocate( mask_1(ndim(1,1),ndim(2,1)),                                &
                  mask_2_3d(ndim(1,2),ndim(2,2),ndim(3,2)), stat=status       )
    end if
    if ( status/=0 ) call merror( name_proc//allocate_error )
!
#ifdef _OPENMP
!
#ifdef _PARALLEL_READ
    i = omp_get_num_procs()
    j = omp_get_max_threads()
    parallel_read = i>1 .and. j>1
#else
    parallel_read = false
#endif
!
#endif
!
!   GET THE MASKS.
!
!$OMP PARALLEL DO IF(parallel_read)                                                &
!$OMP            ,PRIVATE(status,var_nb,lon1_var,lon2_var,lat1_var,lat2_var,       &
!$OMP                     level1_var,level2_var,pt_mask,temp_2,temp_2_3d         ) &
!$OMP            ,SHARED(lon1,lon2,lat1,lat2,dim_tmp,ndim,ncid_mask,maskid,        &
!$OMP                    mask_1,mask_2,mask_2_3d,grid_type,nlon_def,nlat_def,nvar) &
!$OMP            ,NUM_THREADS( 2 )
!
    do var_nb = 1, 2
!
        lon1_var = lon1(var_nb)
        lon2_var = lon2(var_nb)
!
        lat1_var = lat1(var_nb)
        lat2_var = lat2(var_nb)
!
        if ( dim_tmp(var_nb)==3 ) then
!
!           THE VARIABLE IS A 3D-FIELD.
!
!           ALLOCATE WORK ARRAY.
!
            allocate( temp_2(ndim(1,var_nb),ndim(2,var_nb)), stat=status )
            if ( status/=0 ) call merror( name_proc//allocate_error )
!
            if ( var_nb==1 ) then
                pt_mask => mask_1
            else
                pt_mask => mask_2
            end if
!
!           GET THE MASK.
!
!$OMP CRITICAL (getvarnc)
            status = nf90_get_var( ncid_mask(var_nb), maskid(var_nb), temp_2(:,:) )     
!$OMP END CRITICAL (getvarnc)
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           DETERMINE THE GRID POINTS USED IN THE SVD ANALYSIS FROM THE MASK.
!
            pt_mask(:,:) = temp_2(:,:)==one
!
!           SELECT THE DOMAIN.
!
            call select_grid_points_2d( grid_type(var_nb), lon1_var, lon2_var, lat1_var, lat2_var,   &
                                        nlon_def(var_nb), nlat_def(var_nb),                          &
                                        pt_mask, lon1(var_nb), lon2(var_nb) )
!
            nvar(var_nb) = count( pt_mask )
!
!           DEALLOCATE WORK ARRAY.
!
            deallocate( temp_2 )
!
        else
!
!           THE VARIABLE IS A 4D-FIELD AND IS THE SECOND VARIABLE.
!
            level1_var = level1(2)
            level2_var = level2(2)
!
!           ALLOCATE WORK ARRAY.
!
            allocate( temp_2_3d(ndim(1,2),ndim(2,2),ndim(3,2)), stat=status )
            if ( status/=0 ) call merror( name_proc//allocate_error )
!
!           GET THE MASK.
!
!$OMP CRITICAL (getvarnc)
            status = nf90_get_var( ncid_mask(2), maskid(2), temp_2_3d(:,:,:) )     
!$OMP END CRITICAL (getvarnc)
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           DETERMINE THE GRID POINTS USED IN THE SVD ANALYSIS FROM THE MASK.
!
            mask_2_3d(:,:,:) = temp_2_3d(:,:,:)==one
!
!           SELECT THE DOMAIN.
!
            call select_grid_points_3d( grid_type(2), lon1_var, lon2_var, lat1_var, lat2_var,  &
                                        level1_var, level2_var, nlon_def(2), nlat_def(2),      &
                                        mask_2_3d, lon1(2), lon2(2)                            )
!
            nvar(2) = count( mask_2_3d )
!
!           DEALLOCATE WORK ARRAY.
!
            deallocate( temp_2_3d )
!
        end if
!
!       IF THE FIRST LONGITUDE BOUND IS NEGATIVE, WE NEED TO READ ALL THE LONGITUDES.
!
        if ( lon1_var<0 )  lon3(var_nb) = ndim(1,var_nb)
!
        if ( nvar(var_nb)<=0  )                                                 &
        call merror( name_proc//"Wrong domain definition : domain is empty" )
!
    end do
!
!$OMP END PARALLEL DO
!
!   ALLOCATE WORK ARRAYS TO READ THE DATA.
!
    allocate( mat_1(nvar(1),ntime), mat_2(nvar(2),ntime), stat=status )
    if ( status/=0 ) call merror( name_proc//allocate_error )
!
!   READ THE DATA IN THE NETCDF FILES.
!
#ifdef _PARALLEL_READ
!
    test_par = parallel_read .and. ntime>=j
!
    if ( test_par ) then
!
!$OMP PARALLEL PRIVATE(k,status,rec,offset_rec,startlec,countlec,ncid2,varid2,temp_2,temp_2_3d, &
!$OMP                  var_nb,lon1_var,lon2_var,lon3_var,lat1_var,lat2_var,lat3_var,            &
!$OMP                  level1_var,level2_var,level3_var,pt_mat,pt_mask)                         &
!$OMP         ,SHARED(lon1,lon2,lon3,lat1,lat2,lat3,level1,level2,level3,time1,time2)
!
        do var_nb = 1, 2
!
            lon1_var = lon1(var_nb)
            lon2_var = lon2(var_nb)
            lon3_var = lon3(var_nb)
!
            lat1_var = lat1(var_nb)
            lat2_var = lat2(var_nb)
            lat3_var = lat3(var_nb)
!
            offset_rec = time1(var_nb) - 1
!
!$OMP CRITICAL (opennc)
!
!           OPEN THE NETCDF DATASET.
!
            status = nf90_open( trim(fic_name(var_nb)), nf90_share, ncid2 )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           GET THE VARIABLE ID FROM ITS NAME.
!
            status = nf90_inq_varid( ncid2, trim(var_name(var_nb)), varid2 )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!$OMP END CRITICAL (opennc)
!
            if ( dim_tmp(var_nb)==3 ) then
!
!               THE VARIABLE IS A 3D-FIELD.
!
                startlec(1:2)  = (/ lon1_var, lat1_var /)
                countlec(1:3)  = (/ lon3_var, lat3_var, 1_i4b /)
!
                if ( var_nb==1 ) then
                    pt_mat  => mat_1
                    pt_mask => mask_1
                else
                    pt_mat  => mat_2
                    pt_mask => mask_2
                end if
!
!               ALLOCATE WORK ARRAY.
!
                allocate( temp_2(lon3_var,lat3_var), stat=status )
                if ( status/=0 ) call merror( name_proc//allocate_error )
!
!$OMP DO SCHEDULE(STATIC)
!
                do k = time1(var_nb), time2(var_nb)
!
                    startlec(3) = k
!
!                   FILL THE WORK ARRAY WITH A RECORD FROM THE NETCDF VARIABLE.
!
                    status = nf90_get_var( ncid2, varid2, temp_2(:lon3_var,:lat3_var),  &
                                           start=startlec(1:3), count=countlec(1:3)   )
                    if ( status/=nf90_noerr )  call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!                   PACK THE DATA.
!
                    rec = k - offset_rec
                    pt_mat(:,rec) = pack( temp_2(:lon3_var,:lat3_var),                    &
                                          pt_mask(lon1_var:lon2_var,lat1_var:lat2_var ) )
                end do
!
!$OMP END DO
!
!               DEALLOCATE WORK ARRAY.
!
                deallocate( temp_2 )
!
            else
!
!               THE VARIABLE IS A 4D-FIELD AND IS THE SECOND VARIABLE.
!
                level1_var = level1(2)
                level2_var = level2(2)
                level3_var = level3(2)
!
                startlec(1:3)  = (/ lon1_var, lat1_var, level1_var /)
                countlec(1:4)  = (/ lon3_var, lat3_var, level3_var, 1_i4b /)
!
                pt_mat  => mat_2
!
!               ALLOCATE WORK ARRAY.
!
                allocate( temp_2_3d(lon3_var,lat3_var,level3_var), stat=status )
                if ( status/=0 ) call merror( name_proc//allocate_error )
!
!$OMP DO SCHEDULE(STATIC)
!
                do k = time1(var_nb), time2(var_nb)
!
                    startlec(4) = k
!
!                   FILL THE WORK ARRAY WITH A RECORD FROM THE NETCDF VARIABLE.
!
                    status = nf90_get_var( ncid2, varid2, temp_2_3d(:lon3_var,:lat3_var,:level3_var),  &
                                           start=startlec(1:4), count=countlec(1:4)                  )     
                    if ( status/=nf90_noerr )  call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!                   PACK THE DATA.
!
                    rec = k - offset_rec
                    pt_mat(:,rec) = pack( temp_2_3d(:lon3_var,:lat3_var,:level3_var),                          &
                                          mask_2_3d(lon1_var:lon2_var,lat1_var:lat2_var,level1_var:level2_var) )
                end do
!
!$OMP END DO
!
!               DEALLOCATE WORK ARRAY.
!
                deallocate( temp_2_3d )
!
            end if
!
!$OMP CRITICAL (closenc)
!
!           CLOSE THE NETCDF DATASET.
!
            status = nf90_close( ncid2 )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!$OMP END CRITICAL (closenc)
!
!$OMP BARRIER
!
        end do
!
!$OMP END PARALLEL
!
    else
!
#endif
!
        do var_nb = 1, 2
!
            lon1_var = lon1(var_nb)
            lon2_var = lon2(var_nb)
            lon3_var = lon3(var_nb)
!
            lat1_var = lat1(var_nb)
            lat2_var = lat2(var_nb)
            lat3_var = lat3(var_nb)
!
            offset_rec = time1(var_nb) - 1
!
            if ( dim_tmp(var_nb)==3 ) then
!
!               THE VARIABLE IS A 3D-FIELD.
!
!               ALLOCATE WORK ARRAY.
!
                allocate( temp_2(lon3_var,lat3_var), stat=status )
                if ( status/=0 ) call merror( name_proc//allocate_error )
!
                startlec(1:2)  = (/ lon1_var, lat1_var /)
                countlec(1:3)  = (/ lon3_var, lat3_var, 1_i4b /)
!
                if ( var_nb==1 ) then
                    pt_mat  => mat_1
                    pt_mask => mask_1
                else
                    pt_mat  => mat_2
                    pt_mask => mask_2
                end if
!
                do k = time1(var_nb), time2(var_nb)
!
                    startlec(3) = k
!
!                   FILL THE WORK ARRAY WITH A RECORD FROM THE NETCDF VARIABLE.
!
                    status = nf90_get_var( ncid_data(var_nb), varid(var_nb), temp_2(:lon3_var,:lat3_var),     &
                                           start=startlec(1:3), count=countlec(1:3)   )
                    if ( status/=nf90_noerr )  call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!                   PACK THE DATA.
!
                    rec = k - offset_rec
                    pt_mat(:,rec) = pack( temp_2(:lon3_var,:lat3_var),                            &
                                          pt_mask(lon1_var:lon2_var,lat1_var:lat2_var ) )
                end do
!
!               DEALLOCATE WORK ARRAY.
!
                deallocate( temp_2 )
!
            else
!
!               THE VARIABLE IS A 4D-FIELD AND IS THE SECOND VARIABLE.
!
                level1_var = level1(2)
                level2_var = level2(2)
                level3_var = level3(2)
!
!               ALLOCATE WORK ARRAY.
!
                allocate( temp_2_3d(lon3_var,lat3_var,level3_var), stat=status )
                if ( status/=0 ) call merror( name_proc//allocate_error )
!
                startlec(1:3)  = (/ lon1_var, lat1_var, level1_var /)
                countlec(1:4)  = (/ lon3_var, lat3_var, level3_var, 1_i4b /)
!
                pt_mat  => mat_2
!
                do k = time1(2), time2(2)
!
                    startlec(4) = k
!
!                   FILL THE WORK ARRAY WITH A RECORD FROM THE NETCDF VARIABLE.
!
                    status = nf90_get_var( ncid_data(2), varid(2), temp_2_3d(:lon3_var,:lat3_var,:level3_var),   &
                                           start=startlec(1:4), count=countlec(1:4) )     
                    if ( status/=nf90_noerr )  call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!                   PACK THE DATA.
!
                    rec = k - offset_rec
                    pt_mat(:,rec) = pack( temp_2_3d(:lon3_var,:lat3_var,:level3_var),                           &
                                          mask_2_3d(lon1_var:lon2_var,lat1_var:lat2_var,level1_var:level2_var) )
                end do
!
!               DEALLOCATE WORK ARRAY.
!
                deallocate( temp_2_3d )
!
            end if
!
        end do
!
#ifdef _PARALLEL_READ
!
    end if
!
#endif
!
!   ALLOCATE WORK ARRAYS FOR THE METRICS.
!
    if ( distance(1)<=2 ) then
         allocate( scal_1(nvar(1)), stat=status )
         if ( status/=0 ) call merror( name_proc//allocate_error )
    end if
!
    if ( distance(2)<=2 ) then
         allocate( scal_2(nvar(2)), stat=status )
         if ( status/=0 ) call merror( name_proc//allocate_error )
    end if
!
!   COMPUTE MACHINE CONSTANTS.
!
    safmin = lamch('s')
    abstol = sqrt( safmin )
!
!   GET THE CLIMATOLOGIES, COMPUTE ANOMALIES AND THE METRICS.
!
!$OMP PARALLEL DO IF(parallel_read)                                                         &
!$OMP            ,PRIVATE(status,var_nb,j,k,i,i1,i2,lon1_var,lon2_var,lon3_var,             &
!$OMP                     lat1_var,lat2_var,lat3_var,level1_var,level2_var,level3_var,      &
!$OMP                     meanid,stdid,eid,nm,startlec,countlec,tmp,mean,std,d,             &
!$OMP                     pt_mat,pt_mask,pt_scal,temp_1,temp_2,temp_2_3d,                   &
!$OMP                     temp_name,failure )                                               &
!$OMP            ,SHARED(lon1,lon2,lon3,lat1,lat2,lat3,level1,level2,level3,dim_tmp,ndim,   &
!$OMP                    mat_1,mask_1,scal_1,mat_2,mask_2,mask_2_3d,scal_2,method,distance, &
!$OMP                    ncid_data,varid,ncid_clim,ncid_mask,dimids,dimlens,nvar,ntime,     &
!$OMP                    chr,safmin) &
!$OMP            ,NUM_THREADS( 2 )
!
    do var_nb = 1, 2
!
        lon1_var = lon1(var_nb)
        lon2_var = lon2(var_nb)
        lon3_var = lon3(var_nb)
!
        lat1_var = lat1(var_nb)
        lat2_var = lat2(var_nb)
        lat3_var = lat3(var_nb)
!
        if ( var_nb==1 ) then
!
            pt_mat  => mat_1
            pt_mask => mask_1
!
        else
!
            pt_mat  => mat_2
!
            if ( dim_tmp(2)==3) then
                pt_mask => mask_2
            else
                level1_var = level1(2)
                level2_var = level2(2)
                level3_var = level3(2)
            end if
!
        end if
!
!       SELECT THE METHOD OF ANALYSIS.
!
        select case ( method )
!
            case ( 1 )
!
!               SVD ANALYSIS IS DONE ON THE RAW DATA.
!
!               MULTIPLY THE NETCDF VARIABLE BY THE SCALE FACTOR IF REQUIRED.
!
                status = nf90_get_att( ncid_data(var_nb), varid(var_nb), "scale_factor", tmp )
                if ( status==nf90_noerr ) pt_mat(:,:) = pt_mat(:,:)*tmp
!
!               ADD OFFSET TO THE NETCDF VARIABLE IF REQUIRED.
!
                status = nf90_get_att( ncid_data(var_nb), varid(var_nb), "add_offset", tmp )
                if ( status==nf90_noerr ) pt_mat(:,:) = pt_mat(:,:) + tmp
!
            case ( 2, 3 )
!
!               SVD ANALYSIS IS DONE ON THE ANOMALIES OR THE STANDARDIZED ANOMALIES.
!
!               GET THE MEAN FIELDS FROM THEIR NAMES.
!
                temp_name = trim(var_name(var_nb))//"_mean"
!
                call get_variable_dims( name_proc, temp_name,                         &
                     ncid_clim(var_nb), meanid, dimids(:,var_nb), dimlens(:,var_nb)   )
!
                if ( dim_tmp(var_nb)/=count( dimlens(:,var_nb)/=0 ) )                                &
                    call merror( name_proc//"Dimension disagreement with the climatologies" )
!
                if ( any( dimlens(:dim_tmp(var_nb)-1,var_nb)/=ndim(:dim_tmp(var_nb)-1,var_nb)  ) )   &
                    call merror( name_proc//"Shape disagreement with the climatologies" )
!
                nm = dimlens(dim_tmp(var_nb),var_nb)
!
                failure = false
!
                status = nf90_get_att( ncid_clim(var_nb), nf90_global, "lon1_"//trim(var_name(var_nb)), i )
                if ( status==nf90_noerr ) failure = lon1_var<i
!
                status = nf90_get_att( ncid_clim(var_nb), nf90_global, "lon2_"//trim(var_name(var_nb)), i )
                if ( status==nf90_noerr .and. .not.failure ) failure = lon2_var>i
!
                status = nf90_get_att( ncid_clim(var_nb), nf90_global, "lat1_"//trim(var_name(var_nb)), i )
                if ( status==nf90_noerr .and. .not.failure ) failure = lat1_var<i
!
                status = nf90_get_att( ncid_clim(var_nb), nf90_global, "lat2_"//trim(var_name(var_nb)), i )
                if ( status==nf90_noerr .and. .not.failure ) failure = lat2_var>i
!
                if ( dim_tmp(var_nb)==4) then
!
                    status = nf90_get_att( ncid_clim(var_nb), nf90_global, "level1_"//trim(var_name(var_nb)), i )
                    if ( status==nf90_noerr .and. .not.failure ) failure = level1_var<i
!
                    status = nf90_get_att( ncid_clim(var_nb), nf90_global, "level2_"//trim(var_name(var_nb)), i )
                    if ( status==nf90_noerr .and. .not.failure ) failure = level2_var>i
!
                end if
!
                if ( failure ) call merror( name_proc//"Missing values in the climatologies " )
!
!               GET THE MEANS.
!
                if ( dim_tmp(var_nb)==3) then
!
!                   ALLOCATE WORK ARRAYS FOR THE MEANS.
!
                    allocate( mean(nvar(var_nb),nm),temp_2(lon3_var,lat3_var), stat=status )
                    if ( status/=0 ) call merror( name_proc//allocate_error )
!
!                   THE FIELD HAS THREE DIMENSIONS.
!
                    startlec(1:2)  = (/ lon1_var, lat1_var /)
                    countlec(1:3)  = (/ lon3_var, lat3_var, 1_i4b /)
!
                    do j = 1, nm
!
                        startlec(3) = j
!
!$OMP CRITICAL (getvarnc)
                        status = nf90_get_var( ncid_clim(var_nb), meanid,                    &
                                               temp_2(:lon3_var,:lat3_var),                  &
                                               start=startlec(1:3),                          &
                                               count=countlec(1:3)     )
!$OMP END CRITICAL (getvarnc)
                        if ( status/=nf90_noerr )                                            &
                        call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!                       PACK THE MEAN FIELD.
!
                        mean(:,j) = pack( temp_2(:lon3_var,:lat3_var),               &
                                          pt_mask(lon1_var:lon2_var,lat1_var:lat2_var)   )
!
                    end do
!
                else
!
!                   ALLOCATE WORK ARRAYS FOR THE MEANS.
!
                    allocate( mean(nvar(var_nb),nm),temp_2_3d(lon3_var,lat3_var,level3_var), stat=status )
                    if ( status/=0 ) call merror( name_proc//allocate_error )
!
!                   THE FIELD HAS FOUR DIMENSIONS.
!
                    startlec(1:3)  = (/ lon1_var, lat1_var, level1_var /)
                    countlec(1:4)  = (/ lon3_var, lat3_var, level3_var, 1_i4b /)
!
                    do j = 1, nm
!
                        startlec(4) = j
!
!$OMP CRITICAL (getvarnc)
                        status = nf90_get_var( ncid_clim(2), meanid,                         &
                                               temp_2_3d(:lon3_var,:lat3_var,:level3_var),   &
                                               start=startlec(1:4),                          &
                                               count=countlec(1:4)                           )
!$OMP END CRITICAL (getvarnc)
!
                        if ( status/=nf90_noerr )                                              &
                        call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!                       PACK THE MEAN FIELD.
!
                        mean(:,j) = pack( temp_2_3d(:lon3_var,:lat3_var,:level3_var),                     &
                                          mask_2_3d(lon1_var:lon2_var,lat1_var:lat2_var,level1_var:level2_var)   )
!
                    end do
!
                endif
!
                if ( method==2 ) then
!
!                   SVD ANALYSIS IS DONE ON THE ANOMALIES.
!
                    k = ( ntime/nm )*nm
!
!                   COMPUTE ANOMALIES.
!
                    do i = 0, k-1, nm
!
                        i1 = i + 1
                        i2 = i + nm
!
                        pt_mat(:,i1:i2) = pt_mat(:,i1:i2) - mean(:,:)
!
                    end do
!
                    if ( k/=ntime ) then
!
                        pt_mat(:,k+1:ntime) = pt_mat(:,k+1:ntime) -  mean(:,1:ntime-k)
!
                    end if
!
!                   DEALLOCATE WORK ARRAYS.
!
                    if ( dim_tmp(var_nb)==3) then
                        deallocate( mean, temp_2 )
                    else
                        deallocate( mean, temp_2_3d )
                    end if
!
!                   MULTIPLY THE NETCDF VARIABLES BY THE SCALE FACTORS IF REQUIRED.
!
                    status = nf90_get_att( ncid_data(var_nb), varid(var_nb), "scale_factor", tmp )
                    if ( status==nf90_noerr )  pt_mat(:,:) = pt_mat(:,:)*tmp
!
                else
!
!                   SVD ANALYSIS IS DONE ON THE STANDARDIZED ANOMALIES.
!
!                   GET THE STD FIELDS FROM THEIR NAMES.
!
                    temp_name = trim(var_name(var_nb))//"_std"
!
                    call get_variable_dims( name_proc, temp_name, ncid_clim(var_nb),  &
                                            stdid, dimids(:,var_nb), dimlens(:,var_nb) )
!
                    if ( dim_tmp(var_nb)/=count( dimlens(:,var_nb)/=0 ) )                              &
                    call merror( name_proc//"Dimension disagreement with the std fields" )
!
                    if ( any( dimlens(:dim_tmp(var_nb)-1,var_nb)/=ndim(:dim_tmp(var_nb)-1,var_nb)  ) )   &
                    call merror( name_proc//"Shape disagreement with the std fields" )
!
                    if ( dimlens(dim_tmp(var_nb),var_nb)/=nm  )            &
                    call merror( name_proc//" Input std fields and climatologies must have the same periodicity")
!
!                   ALLOCATE WORK ARRAY.
!
                    allocate( std(nvar(var_nb),nm), stat=status )
                    if ( status/=0 ) call merror( name_proc//allocate_error )
!
!                   GET THE STD FIELD.
!
                    if ( dim_tmp(var_nb)==3) then
!
!                       THE FIELD HAS THREE DIMENSIONS.
!
                        do j = 1, nm
!
                            startlec(3) = j
!
!$OMP CRITICAL (getvarnc)
                            status = nf90_get_var( ncid_clim(var_nb), stdid,                     &
                                                   temp_2(:lon3_var,:lat3_var),                  &
                                                   start=startlec(1:3),                          &
                                                   count=countlec(1:3)                           )
!$OMP END CRITICAL (getvarnc)
                            if ( status/=nf90_noerr )                                            &
                                  call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!                           PACK THE STD FIELD.
!
                            std(:,j) = pack( temp_2(:lon3_var,:lat3_var),                     &
                                             pt_mask(lon1_var:lon2_var,lat1_var:lat2_var) )
!
                        end do
!
                    else
!
!                       THE STD FIELD HAS FOUR DIMENSIONS.
!
                        do j = 1, nm
!
                             startlec(4) = j
!
!$OMP CRITICAL (getvarnc)
                             status = nf90_get_var( ncid_clim(2), stdid,                         &
                                                    temp_2_3d(:lon3_var,:lat3_var,:level3_var),  &
                                                    start=startlec(1:4),                         &
                                                    count=countlec(1:4)                          )
!$OMP END CRITICAL (getvarnc)
                             if ( status/=nf90_noerr )                                                  &
                             call merror( name_proc//trim( nf90_strerror( status ) ) )
!
                             std(:,j) = pack( temp_2_3d(:lon3_var,:lat3_var,:level3_var),                   &
                                               mask_2_3d(lon1_var:lon2_var,lat1_var:lat2_var,level1_var:level2_var) )
!
                        end do
!
                    endif
!
                    where ( std(:,:)>safmin )
                        std(:,:) = one/std(:,:)
                    elsewhere
                        std(:,:) = zero
                    end where
!
!                   COMPUTE STANDARDIZED ANOMALIES.
!
                    k = ( ntime/nm )*nm
!
                    do i = 0, k-1, nm
!
                        i1 = i + 1
                        i2 = i + nm
!
                        pt_mat(:,i1:i2) = ( pt_mat(:,i1:i2) - mean(:,:) ) * std(:,:)
!
                    end do
!
                    if ( k/=ntime ) then 
!
                        pt_mat(:,k+1:ntime) = ( pt_mat(:,k+1:ntime) - mean(:,1:ntime-k) ) * std(:,1:ntime-k)
!
                    endif
!
!                   DEALLOCATE WORK ARRAYS.
!
                    if ( dim_tmp(var_nb)==3) then
                        deallocate( mean, std, temp_2 )
                    else
                        deallocate( mean, std, temp_2_3d )
                    end if
!
                endif
!
        end select
!
!       COMPUTE THE METRICS.
!
        if ( distance(var_nb)<=2 ) then
!
!           THE DISTANCE OF THE VARIABLE IS ASSOCIATED WITH THE (HORIZONTAL) GRID.
!
            if ( var_nb==1 ) then
                pt_scal => scal_1
            else
                pt_scal => scal_2
            end if
!
!           ALLOCATE WORK ARRAYS.
!
            allocate( temp_1(lon3_var,lat3_var), temp_2(lon3_var,lat3_var), stat=status )
            if ( status/=0 ) call merror( name_proc//allocate_error )
!
!           GET THE FIRST AND SECOND SCALE FACTORS.
!
            startlec(1:2)  = (/ lon1_var, lat1_var /)
            countlec(1:2)  = (/ lon3_var, lat3_var /)
!
            do i = 1, 2
!
                if ( i==1 ) then
                    temp_name ="e1"//chr(var_nb)
                else
                    temp_name ="e2"//chr(var_nb)
                end if
!
!               GET THE FIRST AND SECOND SCALE FACTORS IDs FROM THEIR NAMES.
!
                call get_variable_dims( name_proc, temp_name, ncid_mask(var_nb), eid,         &
                                        dimids(:,var_nb), dimlens(:,var_nb) )
!
                if ( any( dimlens(:2,var_nb )/=ndim(:2,var_nb) ) )                            &
                call merror( name_proc//"Shape disagreement with the scale factors" )
!
!$OMP CRITICAL (getvarnc)
                status = nf90_get_var( ncid_mask(var_nb), eid, temp_1(:lon3_var,:lat3_var),   &
                                       start=startlec(1:2), count=countlec(1:2)  )     
!$OMP END CRITICAL (getvarnc)
                if ( status/=nf90_noerr )  call merror( name_proc//trim( nf90_strerror( status ) ) )
!
                if ( i==2 ) exit
!
                temp_2(:lon3_var,:lat3_var) = temp_1(:lon3_var,:lat3_var)
!
            end do
!
            temp_1(:lon3_var,:lat3_var) = temp_2(:lon3_var,:lat3_var)*temp_1(:lon3_var,:lat3_var)
!
            if ( dim_tmp(var_nb)==3 ) then
!
                pt_scal(:) = pack( temp_1(:lon3_var,:lat3_var), pt_mask(lon1_var:lon2_var,lat1_var:lat2_var) )
!
            else
!
!               ALLOCATE WORK ARRAY.
!
                allocate( temp_2_3d(lon3_var,lat3_var,level3_var), stat=status )
                if ( status/=0 ) call merror( name_proc//allocate_error )
!
                temp_2_3d(:lon3_var,:lat3_var,:level3_var) = spread(temp_1(:lon3_var,:lat3_var), dim=3, ncopies=level3_var )
                pt_scal(:) = pack( temp_2_3d(:lon3_var,:lat3_var,:level3_var),                                    &
                             mask_2_3d(lon1_var:lon2_var,lat1_var:lat2_var,level1_var:level2_var)  )
!
                if ( distance(2)==2 ) then
!
!                   ALLOCATE WORK ARRAY.
!
                    allocate( d(level3_var), stat=status )
                    if ( status/=0 ) call merror( name_proc//allocate_error )
!
!                   THE METRIC IS ASSOCIATED WITH THE WHOLE 3D GRID.
!
                    chr(2) = merge( grid_type(2), 't', grid_type(2)=='w' .or. grid_type(2)=='n' )
!
!                   GET THE THIRD SCALE FACTOR ID FROM ITS NAME.
!
                    temp_name = "e3"//chr(2)
!
                    call get_variable_dims( name_proc, temp_name, ncid_mask(2), eid, dimids(:,2), dimlens(:,2) )
!
                    if ( dimlens(1,2)/=1 .or. dimlens(2,2)/=1 .or. dimlens(3,2)/=level3_var )                      &
                    call merror( name_proc//"Shape disagreement with the third scale factor (second variable)" )
!
!                   GET THE THIRD SCALE FACTOR VARIABLE.
!
!$OMP CRITICAL (getvarnc)
                    status = nf90_get_var( ncid_mask(2), eid, d(:),                                             &
                                           start=(/1,1,level1_var/), count=(/1,1,level3_var/) )    
!$OMP END CRITICAL (getvarnc)
                    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!                   CHECK THE THIRD SCALE FACTOR.
!
                    if ( d(1)<zero )                            &
         call merror( name_proc//" First level with -d=dist3 are not allowed for mesh-mask "//trim(mask_name(2)) )
!
                    if ( d(level3_var)<zero )                    &
         call merror( name_proc//" Last level with -d=dist3 are not allowed for mesh-mask "//trim(mask_name(2)) )
!
                    temp_2_3d(:lon3_var,:lat3_var,:level3_var) =                                                     &
                    spread( spread( d(:level3_var), dim=1, ncopies=lat3_var), dim=1, ncopies=lon3_var )
!
                    pt_scal(:) = pt_scal(:) * pack( temp_2_3d(:lon3_var,:lat3_var,:level3_var),                                   &
                    mask_2_3d(lon1_var:lon2_var,lat1_var:lat2_var,level1_var:level2_var) )
!
!                   DEALLOCATE WORK ARRAY.
!
                    deallocate( d )
!
                end if
!
!               DEALLOCATE WORK ARRAY.
!
                deallocate( temp_2_3d )
!
            end if
!
!           COMPUTE THE DIAGONAL METRIC ASSOCIATED WITH THE (HORIZONTAL) GRID.
!
            pt_scal(:) = sqrt( pt_scal(:)*(one/maxval(pt_scal(:))) )
!
!           DEALLOCATE WORK ARRAYS.
!
            deallocate( temp_1, temp_2 )
!
!           SCALE THE NETCDF VARIABLE WITH THE METRIC.
!
            do i = 1, ntime
                pt_mat(:,i) = pt_mat(:,i)*pt_scal(:)
            end do
!
!            pt_mat(:,:) = pt_mat(:,:)*spread(pt_scal(:),dim=2,ncopies=ntime ) 
!
         end if
!
    end do
!
!$OMP END PARALLEL DO
!
!   CONSTRUCT THE COVARIANCE MATRIX.
!   NOTE THAT THE ROLE OF THE LEFT AND RIGHT FIELDS ARE INTERCHANGED
!   COMPARED TO THE DOCUMENTATION.
!
    i1 = nvar(1)
    i2 = nvar(2)
!
!   ALLOCATE THE COVARIANCE MATRIX.
!
    allocate(  mat_cov(i2,i1),  stat=status )
    if ( status/=0 ) call merror( name_proc//allocate_error )
!
!   COMPUTE THE COVARIANCE MATRIX.
!
#ifdef _BLAS
!
    call gemm('N', 'T', i2, i1, ntime, one, mat_2, i2, mat_1, i1, zero, mat_cov, i2 )
!
#else
!
#ifdef _OPENMP
!
    i = omp_get_num_procs()
    j = omp_get_max_threads()
    test_par = ( (i1+i2)*ntime )>=omp_limit    .and.    &
               i>1 .and. j>1 .and. max(i1,i2)>blksz
!
#endif
!
    i = min( ntime, blksz )    
!
!$OMP PARALLEL DO IF(test_par), SCHEDULE(STATIC), COLLAPSE(2),   &
!$OMP             PRIVATE(l,l2,j,j2,k,mm),                       &
!$OMP             SHARED(ntime,i,i1,i2,mat_cov,mat_1,mat_2)
!
    do l = 1_i4b, i2, blksz
!
        do j = 1_i4b, i1, blksz
!
            l2 = min( i2, l+blksz-1_i4b )
            j2 = min( i1, j+blksz-1_i4b )
!
            mat_cov(l:l2,j:j2) = matmul( mat_2(l:l2,1_i4b:i), transpose( mat_1(j:j2,1_i4b:i) ) )
!            mat_cov(l:l2,j:j2) = mmproduct( mat_2(l:l2,1_i4b:i), transpose( mat_1(j:j2,1_i4b:i) ) )
!
            do k = blksz+1_i4b, ntime, blksz
!
                mm = min( ntime, k+blksz-1_i4b )
!
                mat_cov(l:l2,j:j2) = mat_cov(l:l2,j:j2) + matmul( mat_2(l:l2,k:mm), transpose( mat_1(j:j2,k:mm) ) )
!                mat_cov(l:l2,j:j2) = mat_cov(l:l2,j:j2) + mmproduct( mat_2(l:l2,k:mm), transpose( mat_1(j:j2,k:mm) ) )
!
            end do
!
        end do
!
    end do
!
!$OMP END PARALLEL DO
!
#endif
!
!   CHECK THE nsvd VALUE BEFORE COMPUTING THE (partial) SVD OF THE COVARIANCE MATRIX.
!
    k = min( i2, i1 )
    nsvd = min( nsvd, k, ntime )
!
    if ( alg==1 .or. alg==2 ) then
!
!       COMPUTE MEAN SQUARE COVARIANCE.
!
        ssqcov = norm( mat_cov(:i2,:i1) )**2
!
!       COMPUTE nsvd SINGULAR VALUES AND SINGULAR VECTORS OF THE mat_cov MATRIX.
!
!
!       ALLOCATE WORK ARRAYS FOR THE PARTIAL SVD ANALYSIS.
!
        allocate(  sing_val(k), d(k), e(k), tauq(k), taup(k),  &
                   stat=status )
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
!       COMPUTE SINGULAR VALUES OF mat_cov AND STORE INTERMEDIATE RESULTS
!       WITH SUBROUTINE select_singval_cmp2 :
!       THE INTERMEDIATE BIDIAGONAL FORM OF mat_cov IS STORED IN d AND e.
!       THE ORTHOGONAL MATRICES USED TO REDUCE mat_cov TO BIDIAGONAL FORM d_e
!       ARE STORED IN mat_cov, tauq, taup.
!       THE SINGULAR VALUES ARE STORED IN sing_val IN DECREASING ORDER (sort='d').
!
        call select_singval_cmp2( mat_cov, nsing, sing_val, failure=failure, sort='d', d=d, e=e, &
                                  ls=nsvd, abstol=abstol, tauq=tauq, taup=taup )
!
!       RESET THE NUMBER OF SELECTED SINGULAR TRIPLETS IN CASE OF MULTIPLE SINGULAR VALUES.
!
        nsvd = nsing
!
        if ( .not. failure ) then
!
!           ALLOCATE WORK ARRAYS FOR THE PARTIAL SVD ANALYSIS.
!
            allocate(  left_vec(i2,nsvd), right_vec(i1,nsvd),      &
                       stat=status )
            if ( status/=0 ) call merror( name_proc//allocate_error )
!
            if ( alg==1 ) then
!
!               COMPUTE THE FIRST nsvd LEFT AND RIGHT SINGULAR VECTORS OF mat_cov BY maxiter
!               INVERSE ITERATIONS ON THE INTERMEDIATE BIDIAGONAL MATRIX
!               d_e AND BACK-TRANSFORMATION WITH SUBROUTINE bd_inviter2.
!
                call bd_inviter2( mat_cov, tauq, taup, d, e, sing_val(:nsvd), left_vec,  &
                                  right_vec, failure, maxiter=maxiter                    )
!
                if ( failure ) call merror( name_proc//&
                &"Error in the SVD analysis (inviter algorithm)" )
!
            else
!
!               COMPUTE THE FIRST nsvd LEFT AND RIGHT SINGULAR VECTORS OF mat_cov BY A
!               DEFLATION TECHNIQUE ON THE INTERMEDIATE BIDIAGONAL MATRIX
!               d_e AND BACK-TRANSFORMATION WITH SUBROUTINE bd_deflate2.
!
                call bd_deflate2( mat_cov, tauq, taup, d, e, sing_val(:nsvd),        &
                                  left_vec, right_vec, failure=failure, ortho=ortho  )
!
                if ( failure ) call merror( name_proc//&
                &"Error in the SVD analysis (deflate algorithm)" )
!
            end if
!
        end if
!
!       DEALLOCATE UNUSED WORK ARRAYS.
!
        if ( cb<=1 ) then
!
            deallocate( d, e, tauq, taup )
!
        end if
!
    else
!
!       COMPUTE THE FULL SVD OF mat_cov MATRIX.
!
!
!       ALLOCATE WORK ARRAYS FOR THE FULL SVD ANALYSIS.
!
        allocate(  sing_val(k), left_vec(i2,nsvd), right_vec(i1,k), stat=status )
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
!       COMPUTE THE SINGULAR TRIPLETS BY A FULL SVD OF THE COVARIANCE MATRIX.
!
        call svd_cmp( mat_cov, sing_val, failure, right_vec, sort='d'  )
!
        left_vec(:i2,:nsvd) = mat_cov(:i2,:nsvd)
!
!       COMPUTE MEAN SQUARE COVARIANCE FROM THE SINGULAR VALUES.
!
        ssqcov = sum( sing_val(:k)**2 )
!
    end if
!
    if ( failure ) call merror(  name_proc//"Error in the SVD analysis" )
!
!
!   ALLOCATE WORK ARRAYS.
!
    mm = ( nsvd * ( nsvd + 1 ) ) / 2
!
    allocate( pccor_2(i2,nsvd), pccor_2_1(i2,nsvd),                             &
              pccor_1(i1,nsvd), pccor_1_2(i1,nsvd),                             &
              pc(ntime,nsvd,2), pccorp(mm,2), pcvar(nsvd,2), varexp(nsvd,2),    &
              singvar(nsvd), nc(nsvd), corrsv(nsvd), temp_1(nsvd,nsvd),         &
              stat=status )
!
    if ( status/=0 ) call merror( name_proc//allocate_error )
!
!   COMPUTE LEFT SINGULAR VARIABLES AND HOMOGENEOUS CORRELATIONS.
!
    first = true
    last  = true
!
    call comp_pc_mca( mat_2(:i2,:ntime), left_vec(:i2,:nsvd), first, last,            &
                      pccor_2(:i2,:nsvd), pccorp(:mm,2), pc(:ntime,:nsvd,2), tmp,     &
                      xpcvar=pcvar(:nsvd,2) )    
!
!   COMPUTE RIGHT SINGULAR VARIABLES AND HOMOGENEOUS CORRELATIONS.
!
    call comp_pc_mca( mat_1(:i1,:ntime), right_vec(:i1,:nsvd), first, last,            &
                      pccor_1(:i1,:nsvd), pccorp(:mm,1), pc(:ntime,:nsvd,1), tmp,      &
                      xpcvar=pcvar(:nsvd,1) )    
!
!   COMPUTE STANDARD-DEVIATIONS OF SINGULAR VARIABLES.
!
    pcvar(:nsvd,:2) = sqrt ( pcvar(:nsvd,:2) )
!
!   COMPUTE PERCENTAGES OF EXPLAINED SQUARED COVARIANCE.
!
    tmp = c100 / ssqcov
    singvar(:nsvd) = ( sing_val(:nsvd)**2 ) * tmp
!
!   COMPUTE NORMALIZED ROOT MEAN SQUARE COVARIANCE.
!
    var1 = norm( mat_1(:i1,:ntime) )
    var2 = norm( mat_2(:i2,:ntime) )
    nc(1:nsvd) = sing_val(1:nsvd) * ( c100 / ( var1 * var2 ) )
!
!   COMPUTE PERCENTAGES OF EXPLAINED VARIANCES.
!
    tmp = c100 * real( ntime, stnd )
    varexp(:nsvd,1) = ( ( pcvar(:nsvd,1)/var1 )**2 )*tmp
    varexp(:nsvd,2) = ( ( pcvar(:nsvd,2)/var2 )**2 )*tmp
!
!   COMPUTE RAW VARIANCE FOR THE RIGHT VARIABLE TAKING INTO ACCOUNT THE METRIC.
!
    if ( allocated( scal_1 ) ) then
        raw_var(1) = var1**2/( ntime*sum(scal_1(:i1)**2 ) )
    else
        raw_var(1) = var1**2/( ntime*i1 )
    end if
!
!   COMPUTE RAW VARIANCE FOR THE LEFT VARIABLE TAKING INTO ACCOUNT THE METRIC.
!
    if ( allocated( scal_2 ) ) then
        raw_var(2) = var2**2/( ntime*sum(scal_2(:i2)**2 ) )
    else
        raw_var(2) = var2**2/( ntime*i2 )
    end if
!
!   PRINT PERCENTAGES OF EXPLAINED AND RAW VARIANCES FOR THE FIRST TRIPLET.
!
!    write(*,*)  "left_expvar = ",  varexp(1,2), "left_var = ",  raw_var(2),  sum(scal_2(:i2)**2 ), i2
!    write(*,*)  "right_expvar = ", varexp(1,1), "right_var = ", raw_var(1),  sum(scal_1(:i1)**2 ), i1
!
!   COMPUTE CORRELATIONS BETWEEN SINGULAR VARIABLES.
!
    sing_val(:nsvd) = sing_val(:nsvd) * ( one / real ( ntime , stnd ) )
!
    corrsv(:nsvd) = sing_val(:nsvd) / ( pcvar(:nsvd,1) * pcvar(:nsvd,2) )
!
!   COMPUTE HETEROGENEOUS CORRELATIONS.
!
    pccor_1_2(:i1,:nsvd) = spread( sing_val(:nsvd)/pcvar(:nsvd,2), dim=1, ncopies=i1)*   &
                           right_vec(:i1,:nsvd)
!
    pccor_2_1(:i2,:nsvd) = spread( sing_val(:nsvd)/pcvar(:nsvd,1), dim=1, ncopies=i2)*   &
                           left_vec(:i2,:nsvd)
!
!   ADJUST THE SINGULAR VECTORS AND THE CORRELATIONS FOR THE METRICS.
!
    if ( distance(1)/=3 ) then
!
        scal_1(:i1) = one / scal_1(:i1)
        right_vec(:i1,:nsvd) = right_vec(:i1,:nsvd) * spread(scal_1(:i1),dim=2,ncopies=nsvd )
        pccor_1_2(:i1,:nsvd) = pccor_1_2(:i1,:nsvd) * spread(scal_1(:i1),dim=2,ncopies=nsvd )
        pccor_1(:i1,:nsvd)   = pccor_1(:i1,:nsvd) * spread(scal_1(:i1),dim=2,ncopies=nsvd )
!
        deallocate( scal_1 )
!
    end if
!
    if ( distance(2)/=3 ) then
!
        scal_2(:i2) = one / scal_2(:i2)
        left_vec(:i2,:nsvd)  = left_vec(:i2,:nsvd) * spread(scal_2(:i2),dim=2,ncopies=nsvd )
        pccor_2_1(:i2,:nsvd) = pccor_2_1(:i2,:nsvd) * spread(scal_2(:i2),dim=2,ncopies=nsvd )
        pccor_2(:i2,:nsvd)   = pccor_2(:i2,:nsvd) *  spread(scal_2(:i2),dim=2,ncopies=nsvd )
!
        deallocate( scal_2 )
!
    end if
!
!   NORMALIZE SINGULAR VARIABLES.
!
    pc(:ntime,:nsvd,:2) = pc(:ntime,:nsvd,:2)*spread( one / pcvar(:nsvd,:2), dim=1, ncopies=ntime)
!
!   MOVING BLOCK BOOTSTRAP TESTS.
!
    if ( do_bootstrap ) then
!
!       ALLOCATE WORK ARRAYS.
!
        allocate( stat_sig_singvar(nsvd), stat_sig_nc(nsvd), stat_sig_corr(nsvd), stat=status )    
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
!       INITIALIZE SIGNIFICANCE LEVELS.
!
        stat_sig_singvar(:nsvd) = one
        stat_sig_nc(:nsvd)      = one
        stat_sig_corr(:nsvd)    = one
!
!       INITIALIZE THE RANDOM GENERATOR.
!
        call random_seed_()
!
!       ALLOCATE WORK ARRAYS.
!
        if  ( cb == 2 ) then
!
            i = merge( nsvd, k, alg<=2 )
            j = max( ntime, nsvd,nb_block )
!
            allocate( right_vecb(i1,i),  left_vecb(i2,nsvd),                 &
                      corrsvb(nsvd), pcvarb(nsvd,2), scal_1(j),              &
                      index(nb_block),                                       &
                      sing_valb(k), singvarb(nsvd), ncb(nsvd), stat=status   )
!
        else
!
            allocate( index(nb_block), scal_1(nb_block),                     &
                      sing_valb(k), singvarb(nsvd), ncb(nsvd), stat=status   )
!
        end if  
!
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
        do i = 1, nb
!
!           GENERATE RANDOM BLOCK INDEX.
!
!           GENERATE nb_block UNIFORM RANDOM NUMBERS BETWEEN 0 AND 1.
!
            call random_number_( scal_1(:nb_block) )
!
            index(:nb_block) = int( scal_1(:nb_block) * nb_block_max , i4b )*bp
!
!           COMPUTE COVARIANCE MATRIX BETWEEN THE LEFT FIELD AND THE BLOCKED PERMUTED RIGHT
!           FIELD AND SUM OF SQUARES FOR THE BLOCKED PERMUTED RIGHT FIELD.
!
!           INITIALIZE mat_cov.
!
            mat_cov(:i2,:i1) = zero
!
!            l = 0
!
            l2 = 0
!
            ssq = zero
            scl = one
!
!           PROCESS THE nb_block RANDOM BLOCKS.
!
            do j = 1, nb_block
!
                m  = index(j)
!
                if ( j/=nb_block ) then
                    sb = bl
                else
                    sb = ntime - (nb_block-1)*bl
                end if
!
                j1 = m + 1
                j2 = m + sb
!
                l1 = l2 + 1
                l2 = l2 + sb
!
                call lassq( mat_1(:i1,j1:j2), scl, ssq )
!
#ifdef _BLAS
!
                call gemm('N', 'T', i2, i1, sb, one, mat_2(:i2,l1:l2), i2, mat_1(:i1,j1:j2), i1, one, mat_cov, i2 )
!
#else
!
                mat_cov(:i2,:i1) = mat_cov(:i2,:i1) + matmul( mat_2(:i2,l1:l2), transpose( mat_1(:i1,j1:j2) ) )
!
!                do sb = j1, j2
!
!                    l = l + 1
!
!                    do m = 1, i1
!                        mat_cov(:i2,m) = mat_cov(:i2,m) + mat_1(m,sb)*mat_2(:i2,l)
!                    end do
!
!                end do            
!
#endif
            end do
!
!           COMPUTE MEAN SQUARE COVARIANCE AND SUM OF SQUARE OF THE BOOTSTRAP MATRIX.
!
            ssqcov = norm( mat_cov(:i2,:i1) )**2
!
            var1 = scl*sqrt( ssq )
!
            if ( cb == 1 ) then
!
!               COMPUTE ONLY nsvd SINGULAR VALUES OF mat_cov, SINGULAR VECTORS ARE NOT REQUIRED.
!
                call select_singval_cmp2( mat_cov, nsing, sing_valb, failure=failure, sort='d', ls=nsvd )
!
!                call svd_cmp( mat_cov, sing_valb, failure, sort='d' )
!
                if ( failure ) call merror(  name_proc//"Error in the BOOTSTRAP SVD analysis" )
!
            else
!
                if ( alg==1 .or. alg==2 ) then
!
!                   COMPUTE FIRST nsvd SINGULAR VALUES OF mat_cov.
!
                    call select_singval_cmp2( mat_cov, nsing, sing_valb, failure=failure, sort='d', d=d, e=e, &
                                              ls=nsvd, abstol=abstol, tauq=tauq, taup=taup )
!
!                    call svd_cmp( mat_cov, sing_valb, failure, sort='d', d=d, e=e, tauq=tauq, taup=taup  )
!
                    if ( .not. failure ) then
!
                        if ( alg==1 ) then
!
!                           COMPUTE THE FIRST nsvd LEFT AND RIGHT SINGULAR VECTORS OF mat_cov BY maxiter
!                           INVERSE ITERATIONS ON THE INTERMEDIATE BIDIAGONAL MATRIX
!                           d_e AND BACK-TRANSFORMATION WITH SUBROUTINE bd_inviter2 .
!
                            call bd_inviter2( mat_cov, tauq, taup, d, e, sing_valb(:nsvd), left_vecb,    &
                                              right_vecb, failure, maxiter=maxiter  )

                            if ( failure ) call merror(  name_proc//                                     &
                            &"Error in the BOOTSTRAP SVD analysis (inviter algorithm)" )
!
                        else
!
!                           COMPUTE THE FIRST nsvd LEFT AND RIGHT SINGULAR VECTORS OF mat_cov BY A
!                           DEFLATION TECHNIQUE ON THE INTERMEDIATE BIDIAGONAL MATRIX
!                           d_e AND BACK-TRANSFORMATION WITH SUBROUTINE bd_deflate2.
!
                            call bd_deflate2( mat_cov, tauq, taup, d, e, sing_valb(:nsvd),        &
                                              left_vecb, right_vecb, failure=failure, ortho=ortho  )
!
                            if ( failure ) call merror( name_proc//&
                            &"Error in the SVD analysis (deflate algorithm)" )
!
                        end if
!
                    end if
!
                else        
!
!                   COMPUTE FULL SVD OF mat_cov.
!
                    call  svd_cmp( mat_cov, sing_valb, failure, right_vecb, sort='d'  )
!
                    left_vecb(:i2,:nsvd) = mat_cov(:i2,:nsvd)
!
                end if
!
                if ( failure ) call merror(  name_proc//"Error in the BOOTSTRAP SVD analysis" )
!
!               COMPUTE STANDARD-DEVIATIONS OF SINGULAR VARIABLES OF THE LEFT FIELD. 
!
                do j = 1, nsvd
!
                    scal_1(:ntime) = matmul( left_vecb(:i2,j), mat_2(:i2,:ntime) )
                    pcvarb(j,2)    = sum( scal_1(:ntime)**2 )
!
                end do
!
!               COMPUTE STANDARD-DEVIATIONS OF SINGULAR VARIABLES OF THE RIGHT FIELD. 
!
!               PROCESS THE nb_block RANDOM BLOCKS.
!
                pcvarb(:nsvd,1) = zero
!
                do j = 1, nb_block
!
                    m  = index(j)
!
                    if ( j/=nb_block ) then
                        sb = bl
                    else
                        sb = ntime - (nb_block-1)*bl
                    end if
!
                    j1 = m + 1
                    j2 = m + sb
!
                    do sb = j1, j2
                        scal_1(:nsvd)   = matmul( mat_1(:i1,sb), right_vecb(:i1,:nsvd) )
                        pcvarb(:nsvd,1) = pcvarb(:nsvd,1) + scal_1(:nsvd)**2
                    end do
!
                end do
!
!               COMPUTE PSEUDO CORRELATION BETWEEN SINGULAR VARIABLES OF THE LEFT AND RIGHT FIELDS. 
!
                corrsvb(:nsvd) = sing_valb(:nsvd) / sqrt ( pcvarb(:nsvd,1) * pcvarb(:nsvd,2) )
!
            end if
!
!           COMPUTE PERCENTAGES OF EXPLAINED SQUARED COVARIANCE.
!
            tmp = c100 / ssqcov
            singvarb(:nsvd) = ( sing_valb(:nsvd)**2 ) * tmp
!
!           COMPUTE PERCENTAGES OF MAXIMAL SQUARED COVARIANCE.
!
            ncb(:nsvd) = sing_valb(:nsvd) * ( c100 / ( var1 * var2 ) )
!
!           COMPUTE SIGNIFICANCE OF STATISTICS.
!
            where ( singvarb(:nsvd) < singvar(:nsvd) ) &
                stat_sig_singvar(:nsvd) = stat_sig_singvar(:nsvd) + one
!
            where ( ncb(:nsvd) < nc(:nsvd) ) &
                stat_sig_nc(:nsvd) = stat_sig_nc(:nsvd) + one
!
            if  ( cb == 2 ) then
!
                where ( corrsvb(:nsvd) < corrsv(:nsvd) ) &
                    stat_sig_corr(:nsvd) = stat_sig_corr(:nsvd) + one
!
            end if
!
        end do
!
!       DEALLOCATE WORK ARRAYS.
!
        deallocate( mat_cov, index, scal_1, sing_valb, singvarb , ncb )
!
        if  ( cb == 2 ) then
!
            if ( alg <= 2 ) then
                deallocate( d, e, tauq, taup, right_vecb, left_vecb, corrsvb, pcvarb )
            else
                deallocate( right_vecb, left_vecb, corrsvb, pcvarb )
            end if 
!
        end if   
!
!       COMPUTE SIGNIFICANCE LEVELS FOR THE STATISTICS.
!
        tmp = one / real ( nb + 1 , stnd )
        stat_sig_singvar(:nsvd) = stat_sig_singvar(:nsvd) * tmp
        stat_sig_nc(:nsvd)      = stat_sig_nc(:nsvd) * tmp
        stat_sig_corr(:nsvd)    = stat_sig_corr(:nsvd) * tmp
!
!   END OF BOOTSTRAP CALCULATIONS.
!
    else
!
!       DEALLOCATE WORK ARRAY.
!
        deallocate( mat_cov )
!
    endif
!
!   CREATE THE OUTPUT NETCDF DATASETS OF THE SVD ANALYSIS.
!
    do var_nb = 1, 2
!
        if ( create_hdf5 ) then
            status = nf90_create( svd_name(var_nb), cmode=output_mode, ncid=ncid_svd(var_nb) )
        else
            k = nf90_sizehint_default
            status = nf90_create( svd_name(var_nb), cmode=output_mode, ncid=ncid_svd(var_nb), chunksize=k )
        end if
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_set_fill( ncid_svd(var_nb), nf90_nofill, i )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        nm = dim_tmp(var_nb)
!
!       DEFINE THE DIMENSIONS FOR THE NEW NETCDF DATASETS.
!
        dimlens(:nm-1,var_nb)          = ndim(:nm-1,var_nb)
        dimlens(nm,var_nb)             = nsvd
        dimlens(nm+1,var_nb)           = 1
!
        if ( use_tlimited ) then
            dimlens(nm+2,var_nb)       = ntime
        else
            dimlens(nm+2,var_nb)       = nf90_unlimited
        end if
!
        dim_names(nm+2,var_nb)         = dim_names(nm,var_nb)
        dim_names(nm,var_nb)           = "pc"
        dim_names(nm+1,var_nb)         = "var"
!
        coor_id(nm+2,var_nb)           = coor_id(nm,var_nb)
        coor_natts(nm+2,var_nb)        = coor_natts(nm,var_nb)
        coor_type(nm+2,var_nb)         = coor_type(nm,var_nb)
!
        use_coor_variable(nm+2,var_nb) = use_coor_variable(nm,var_nb)
        use_coor_variable(nm,var_nb)   = false
        use_coor_variable(nm+1,var_nb) = false
!
!
        do i = 1, nm + 2
!
            status = nf90_def_dim( ncid_svd(var_nb), name=trim(dim_names(i,var_nb)),                   &
                                   len=dimlens(i,var_nb), dimid=dimids(i,var_nb) )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        end do
!
        dumm(:) = dimids(nm,var_nb)
!
!       DEFINE THE VARIABLES FOR THE NEW NETCDF DATASETS.
!
!       DEFINE FIRST THE COORDINATE VARIABLES.
!
        do i = 1, nm + 2
!
            if ( i==nm .or. i==nm+1  ) cycle
!
            status = nf90_def_var( ncid_svd(var_nb), trim(dim_names(i,var_nb)), coor_type(i,var_nb),   &
                                   dimids=dimids(i,var_nb), varid=coor_id2(i,var_nb) )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        end do
!
!       DEFINE THE STATISTIC VARIABLES.
!
        status = nf90_def_var( ncid_svd(var_nb), trim(var_name(var_nb))//"_RAW_VAR", nf90_real8,       &
                               dimids=dimids(nm+1,var_nb), varid=rawvarid(var_nb) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_def_var( ncid_svd(var_nb), "Sing_vals", nf90_real8,                              &
                               dimids=dimids(nm,var_nb), varid=sing_valid(var_nb) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_def_var( ncid_svd(var_nb), "SCFs", output_type,                                  &
                               dimids=dimids(nm,var_nb), varid=singvarid(var_nb) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_def_var( ncid_svd(var_nb), "NCs", output_type,                                   &
                               dimids=dimids(nm,var_nb), varid=ncid(var_nb) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_def_var( ncid_svd(var_nb), "Corrs", output_type,                                 &
                               dimids=dimids(nm,var_nb), varid=corrsvid(var_nb) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_def_var( ncid_svd(var_nb), trim(var_name(var_nb))//"_SV_STDs", output_type,      &
                               dimids=dimids(nm,var_nb), varid=pcvarid(var_nb) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_def_var( ncid_svd(var_nb), trim(var_name(var_nb))//"_SV_EXPVARs", output_type,   &
                               dimids=dimids(nm,var_nb), varid=varexpid(var_nb) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_def_var( ncid_svd(var_nb), trim(var_name(var_nb))//"_CORSV", output_type,        &
                               dimids=dumm(:), varid=pccorpid(var_nb) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        if ( do_bootstrap ) then
!
            status = nf90_def_var( ncid_svd(var_nb), "SCF_stat_sign", output_type,                     &
                                   dimids=dimids(nm,var_nb), varid=stat_sig_singvarid(var_nb) )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_def_var( ncid_svd(var_nb), "NC_stat_sign", output_type,                      &
                                   dimids=dimids(nm,var_nb), varid=stat_sig_ncid(var_nb) )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
            if  ( cb == 2 ) then
!
                status = nf90_def_var( ncid_svd(var_nb), "Corr_stat_sign", output_type,                &
                                       dimids=dimids(nm,var_nb), varid=stat_sig_corrid(var_nb) )
                if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
            end if
!
        end if
!
!       DEFINE THE SPATIAL VARIABLES.
!
        status = nf90_def_var( ncid_svd(var_nb), trim(var_name(var_nb))//"_svd", output_type,   &
                               dimids=dimids(1:nm,var_nb), varid=svdid(var_nb) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_def_var( ncid_svd(var_nb), trim(var_name(var_nb))//"_hom", output_type,   &
                               dimids=dimids(1:nm,var_nb), varid=homid(var_nb) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_def_var( ncid_svd(var_nb), trim(var_name(var_nb))//"_het", output_type,   &
                               dimids=dimids(1:nm,var_nb), varid=hetid(var_nb) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!       DEFINE THE SINGULAR VARIABLES (EG THE STANDARDIZED PC TIME SERIES).
!
        dimids(nm+1,var_nb) = dimids(nm,var_nb)
!
        status = nf90_def_var( ncid_svd(var_nb), trim(var_name(var_nb))//"_sv", output_type,    &
                               dimids=dimids(nm+1:nm+2,var_nb), varid=pcid(var_nb) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!       ASSIGN ATTRIBUTES FOR THE NEW NETCDF DATASETS.
!
        do j = 1, nm + 2
!
            if ( .not.use_coor_variable(j,var_nb) ) cycle
!
            do i = 1, coor_natts(j,var_nb)
!
                status = nf90_inq_attname( ncid_data(var_nb), coor_id(j,var_nb), i, att_name )
                if ( status/=nf90_noerr )                                                       &
                call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!               COPY ATTRIBUTES FOR THE COORDINATE VARIABLES.
!
                status = nf90_copy_att(  ncid_data(var_nb), coor_id(j,var_nb), att_name,        &
                                         ncid_svd(var_nb), coor_id2(j,var_nb) )
                if ( status/=nf90_noerr )                                                       &
                call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            end do
!
        end do
!
!       SET ATTRIBUTES FOR THE TIME VARIABLE IF NOT DONE IN THE ORIGINAL DATASET.
!
        if ( .not.use_coor_variable(nm+2,var_nb) ) then
!
            status = nf90_put_att( ncid_svd(var_nb), coor_id2(nm+2,var_nb), "long_name", "Time axis" )
            if ( status/=nf90_noerr )                                                  &
            call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_svd(var_nb), coor_id2(nm+2,var_nb), "short_name", "Time" )
            if ( status/=nf90_noerr )                                                  &
            call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        end if
!
!
        status = nf90_put_att( ncid_svd(var_nb), rawvarid(var_nb), "long_name", "Raw Variance" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_svd(var_nb), rawvarid(var_nb), "short_name", "RawVar" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_svd(var_nb), sing_valid(var_nb), "long_name", "Singular Values" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_svd(var_nb), sing_valid(var_nb), "short_name", "SingVals" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_svd(var_nb), singvarid(var_nb), "long_name", "Squared Covariances Fractions" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_svd(var_nb), singvarid(var_nb), "short_name", "SqCovFracs" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_svd(var_nb), ncid(var_nb), "long_name", "Normalized RMS Covariances" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_svd(var_nb), ncid(var_nb), "short_name", "NormRMSCovs" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_svd(var_nb), corrsvid(var_nb), "long_name", "Correlations" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_svd(var_nb), corrsvid(var_nb), "short_name", "Corrs" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_svd(var_nb), pcvarid(var_nb), "long_name", "STDs of Singular Variables" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_svd(var_nb), pcvarid(var_nb), "short_name", "STDs" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_svd(var_nb), varexpid(var_nb), "long_name", "Explained Variances" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_svd(var_nb), varexpid(var_nb), "short_name", "ExpVars" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_svd(var_nb), pccorpid(var_nb), "long_name", "Correlations of Singular Variables" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_svd(var_nb), pccorpid(var_nb), "short_name", "CorrSVs" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!
        if ( do_bootstrap ) then
!
            status = nf90_put_att( ncid_svd(var_nb), stat_sig_singvarid(var_nb), &
                                   "long_name",  "Stat. Sign. Squared Covariances Fractions" )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_svd(var_nb), stat_sig_singvarid(var_nb), &
                                   "short_name", "PerSqCovFracs" )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_svd(var_nb), stat_sig_ncid(var_nb), &
                                   "long_name", "Stat. Sign. Normalized RMS Covariances" )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_svd(var_nb), stat_sig_ncid(var_nb), &
                                   "short_name", "PerNormRMSCovs" )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
            if  ( cb == 2 ) then
!
                status = nf90_put_att( ncid_svd(var_nb), stat_sig_corrid(var_nb), &
                                       "long_name", "Stat. Sign. Correlations" )
                if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
                status = nf90_put_att( ncid_svd(var_nb), stat_sig_corrid(var_nb), &
                                       "short_name", "PerCorrs" )
                if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
            end if
!
        end if
!
!
        status = nf90_put_att( ncid_svd(var_nb), svdid(var_nb), "long_name", "Singular Vectors" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_svd(var_nb), svdid(var_nb), "short_name", "SVDs" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_svd(var_nb), homid(var_nb), "long_name", "Homogeneous Vectors" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_svd(var_nb), homid(var_nb), "short_name", "HomVecs" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_svd(var_nb), hetid(var_nb), "long_name",  "Heterogeneous Vectors" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_svd(var_nb), hetid(var_nb), "short_name", "HetVecs" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        if (  output_type==nf90_real8 ) then
!
            status = nf90_put_att( ncid_svd(var_nb), pccorpid(var_nb),                          &
                                   "missing_value", real( missing, dp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_svd(var_nb), pccorpid(var_nb),                          &
                                   "_FillValue", real( missing, dp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_svd(var_nb), svdid(var_nb),                             &
                                   "missing_value", real( missing, dp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_svd(var_nb), svdid(var_nb),                             &
                                   "_FillValue", real( missing, dp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_svd(var_nb), homid(var_nb),                             &
                                   "missing_value", real( missing, dp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_svd(var_nb), homid(var_nb),                             &
                                   "_FillValue", real( missing, dp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_svd(var_nb), hetid(var_nb),                             &
                                   "missing_value", real( missing, dp ) )
!
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
            status = nf90_put_att( ncid_svd(var_nb), hetid(var_nb),                             &
                                   "_FillValue", real( missing, dp ) )
!
        else
!
            status = nf90_put_att( ncid_svd(var_nb), pccorpid(var_nb),                          &
                                   "missing_value", real( missing, sp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_svd(var_nb), pccorpid(var_nb),                          &
                                   "_FillValue", real( missing, sp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_svd(var_nb), svdid(var_nb),                             &
                                   "missing_value", real( missing, sp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_svd(var_nb), svdid(var_nb),                             &
                                   "_FillValue", real( missing, sp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_svd(var_nb), homid(var_nb),                             &
                                   "missing_value", real( missing, sp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_svd(var_nb), homid(var_nb),                             &
                                   "_FillValue", real( missing, sp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_svd(var_nb), hetid(var_nb),                             &
                                   "missing_value", real( missing, sp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
            status = nf90_put_att( ncid_svd(var_nb), hetid(var_nb),                             &
                                   "_FillValue", real( missing, sp ) )
!
        end if
!
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!
        status = nf90_put_att( ncid_svd(var_nb), pcid(var_nb), "long_name", "Standardized Singular Variables" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_svd(var_nb), pcid(var_nb), "short_name", "SVs" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!       WRITE GLOBAL ATTRIBUTES.
!
        status = nf90_put_att( ncid_svd(var_nb), nf90_global, "title",    &
                               trim(var_name(var_nb))//" SVD analysis" )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_svd(var_nb), nf90_global, "subtitle", &
                               trim(var_name(1))//" with "//trim(var_name(2)) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_svd(var_nb), nf90_global, "distance", cdistance(distance(var_nb)) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_svd(var_nb), nf90_global, "method", cmethod(method) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_svd(var_nb), nf90_global, "algorithm", calgorithm(alg) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_svd(var_nb), nf90_global, "grid_type", grid_type(var_nb) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_svd(var_nb), nf90_global, "history", trim(history) )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!       LEAVE DEFINE MODE FOR THE NEW NETCDF DATASET.
!
        status = nf90_enddef( ncid_svd(var_nb) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!       WRITE COORDINATE VARIABLES FOR THE NEW  NETCDF DATASETS.
!
        do j = 1, nm + 2 
!
            if ( j==nm .or. j==nm+1 ) cycle
!
            if ( j<nm ) then
                j1_var = 1
                j2_var = ndim(j,var_nb)
            else
                j1_var = time1(var_nb)
                j2_var = ntime
            end if
!
            if ( use_coor_variable(j,var_nb) ) then
!
                allocate( record(j2_var), stat=status )
                if ( status/=0 ) call merror( name_proc//allocate_error )
!
                status = nf90_get_var( ncid_data(var_nb), coor_id(j,var_nb), &
                                       record, start=(/ j1_var /), count=(/ j2_var /) )
                if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
                status = nf90_put_var( ncid_svd(var_nb), coor_id2(j,var_nb), record )
                if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
                deallocate( record )
!
            else
!
                status = nf90_put_var( ncid_svd(var_nb), coor_id2(j,var_nb), &
                                       arth(j1_var,1_i4b,j2_var) )
                if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
            end if
!
        end do
!
!       WRITE RAW VARIANCE.
!
        status = nf90_put_var( ncid_svd(var_nb), rawvarid(var_nb), raw_var(var_nb) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!       WRITE SINGULAR VALUES.
!
        status = nf90_put_var( ncid_svd(var_nb), sing_valid(var_nb), sing_val(:nsvd) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!       WRITE SQUARED COVARIANCE FRACTION (SCF).
!
        status = nf90_put_var( ncid_svd(var_nb), singvarid(var_nb), singvar(:nsvd) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!       WRITE NORMALIZED ROOT-MEAN-SQUARED COVARIANCE (NC).
!
        status = nf90_put_var( ncid_svd(var_nb), ncid(var_nb), nc(:nsvd) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!       WRITE CORRELATIONS BETWEEN SINGULAR VARIABLES OF LEFT AND RIGHT FIELDS.
!
        status = nf90_put_var( ncid_svd(var_nb), corrsvid(var_nb), corrsv(:nsvd) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!       WRITE STDS OF SINGULAR VARIABLES.
!
        status = nf90_put_var( ncid_svd(var_nb), pcvarid(var_nb), pcvar(:nsvd,var_nb) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!       WRITE EXPLAINED VARIANCES.
!
        status = nf90_put_var( ncid_svd(var_nb), varexpid(var_nb), varexp(:nsvd,var_nb) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!       WRITE CORRELATIONS BETWEEN SINGULAR VARIABLES OF EACH FIELD.
!
        temp_1(:,:) = unpack( pccorp(:,var_nb), triangle( true, nsvd, nsvd, extra=1_i4b), field=missing )
!
        status = nf90_put_var( ncid_svd(var_nb), pccorpid(var_nb), temp_1(:,:) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        if ( do_bootstrap ) then
!
!           WRITE STATISTICAL SIGNIFICANCE OF SQUARE COVARIANCE FRACTIONS.
!
            status = nf90_put_var( ncid_svd(var_nb), stat_sig_singvarid(var_nb), stat_sig_singvar(:nsvd) )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!           WRITE STATISTICAL SIGNIFICANCE OF NORMALIZED ROOT MEAN SQUARE COVARIANCES.
!
            status = nf90_put_var( ncid_svd(var_nb), stat_sig_ncid(var_nb), stat_sig_nc(:nsvd) )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
            if  ( cb == 2 ) then
!
!               WRITE STATISTICAL SIGNIFICANCE OF CORRELATIONS.
!
                status = nf90_put_var( ncid_svd(var_nb), stat_sig_corrid(var_nb), stat_sig_corr(:nsvd) )
                if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
            end if
!
        end if
!
!       ALLOCATE WORK ARRAYS.
!
        if ( nm==3 ) then
!
            if ( var_nb==1 ) then
                pt_mask => mask_1
            else
                pt_mask => mask_2
            end if
!
            allocate( temp_2(ndim(1,var_nb),ndim(2,var_nb)), stat=status )
!
        else
!
            allocate( temp_2_3d(ndim(1,2),ndim(2,2),ndim(3,2)), stat=status )
!
        end if
!
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
!       WRITE SINGULAR, HOMOGENEOUS AND HETEROGENEOUS VECTORS FOR THE VARIABLES.
!
        startlec(1:nm-1) = 1
        countlec(1:nm)   = (/ ndim(:nm-1,var_nb) , 1_i4b /)
!
        first = grid_type(var_nb)/='n' .and. ndim(1,var_nb)==nlon_def(var_nb)
        last  = lat2(var_nb)>=nlat_def(var_nb)-1 .and. ndim(2,var_nb)==nlat_def(var_nb)
!
        do j = 1, 3
!
            select case ( j )
!
                case ( 1 )
                    if ( var_nb==1 ) then
                        pt_mat => right_vec(:i1,:nsvd)
                    else
                        pt_mat => left_vec(:i2,:nsvd)
                    end if
                    k = svdid(var_nb)
                case ( 2 )
                    if ( var_nb==1 ) then
                        pt_mat => pccor_1(:i1,:nsvd)
                    else
                        pt_mat => pccor_2(:i2,:nsvd)
                    end if
                    k = homid(var_nb)
                case ( 3 )
                    if ( var_nb==1 ) then
                        pt_mat => pccor_1_2(:i1,:nsvd)
                    else
                        pt_mat => pccor_2_1(:i2,:nsvd)
                    end if
                    k = hetid(var_nb)
!
            end select
!
            if ( nm==3 ) then
!
!               THE VARIABLE IS A 3D-FIELD.
!
                do i = 1, nsvd
!
                    temp_2(:,:) = unpack( pt_mat(:,i), pt_mask, field=missing )
!
                    if ( first ) then
!
                        if ( last ) then
!
!                           ADD DUPLICATE POINTS TO THE ORCA R2 GRID IF THE WHOLE GLOBE IS USED.
!
                            call duplicate_orca_points_2d(grid_type(var_nb), pt_mask, temp_2 )
!
                        else
!
!                           ADD DUPLICATE LONGITUDES LINES TO THE ORCA R2 GRID IF THE WHOLE LONGITUDE
!                           DOMAIN IS USED.
!
                            temp_2(1,:) = temp_2(ndim(1,var_nb)-1,:)
                            temp_2(ndim(1,var_nb),:) = temp_2(2,:)
!
                        end if
!
                    end if
!
                    startlec(3) = i
!
                    status = nf90_put_var( ncid_svd(var_nb), k, temp_2, start=startlec(1:3) , &
                                           count=countlec(1:3) )
                    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
                end do
!
            else
!
!               THE VARIABLE IS A 4D-FIELD AND IS THE SECOND VARIABLE.
!
                do i = 1, nsvd
!
                    temp_2_3d(:,:,:) = unpack( pt_mat(:,i), mask_2_3d, field=missing )
!
                    if ( first ) then
!
                        if ( last ) then
!
!                           ADD DUPLICATE POINTS TO THE ORCA R2 GRID IF THE WHOLE GLOBE IS USED.
!
                            call duplicate_orca_points_3d(grid_type(2), mask_2_3d, temp_2_3d )
!
                        else
!
!                           ADD DUPLICATE LONGITUDES LINES TO THE ORCA R2 GRID IF THE WHOLE LONGITUDE
!                           DOMAIN IS USED.
!
                            temp_2_3d(1,:,:) = temp_2_3d(ndim(1,2)-1,:,:)
                            temp_2_3d(ndim(1,2),:,:) = temp_2_3d(2,:,:)
!
                        end if
!
                    end if
!
                    startlec(4) = i
!
                    status = nf90_put_var( ncid_svd(2), k, temp_2_3d, start=startlec(1:4) , &
                                           count=countlec(1:4) )
                    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
                end do
!
            end if
!
        end do
!
!       WRITE SVD AMPLITUDE TIME SERIES (STANDARDIZED SINGULAR VARIABLES).
!
        status = nf90_put_var( ncid_svd(var_nb), pcid(var_nb), transpose(pc(:ntime,:nsvd,var_nb)) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!       DEALLOCATE WORK ARRAYS.
!
        if ( nm==3 ) then
            deallocate( temp_2 )
        else
            deallocate( temp_2_3d )
        end if
!
!       CLOSE THE NETCDF DATASETS.
!
        if ( var_nb==2 .or. ncid_data(1)/=ncid_data(2) ) then
            status = nf90_close( ncid_data(var_nb) )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
        end if
!
        if ( var_nb==2 .or. ncid_mask(1)/=ncid_mask(2) ) then
            status = nf90_close( ncid_mask(var_nb) )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
        end if
!
        if ( method>=2 .and. ( var_nb==2 .or. ncid_clim(1)/=ncid_clim(2) ) ) then
            status = nf90_close( ncid_clim(var_nb) )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
        end if
!
        status = nf90_close( ncid_svd(var_nb) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end do
!
!   DEALLOCATE WORK ARRAYS.
!
    if ( do_bootstrap ) then
        deallocate( stat_sig_corr, stat_sig_nc, stat_sig_singvar )
    end if
!
    if ( dim_tmp(2)==3 ) then
        deallocate( temp_1, mask_1, mask_2, mat_1, mat_2, right_vec, left_vec,       &
                    pccor_2_1, pccor_1_2, pccor_2, pccor_1, pccorp, pc,              &
                    sing_val, nc, singvar, pcvar, varexp                             )
    else
        deallocate( temp_1, mask_1, mask_2_3d, mat_1, mat_2, right_vec, left_vec,    &
                    pccor_2_1, pccor_1_2, pccor_2, pccor_1, pccorp, pc,              &
                    sing_val, nc, singvar, pcvar, varexp                             )
    end if
!
!
! END OF PROGRAM comp_svd_3d
! ==========================
!
end program comp_svd_3d
