program comp_symlin_filter_1d
!! #############################################################################
!!
!! Copyright 2021 IRD
!!
!! This file is part of NCSTAT.
!!
!! NCSTAT is free software: you can redistribute it and/or modify
!! it under the terms of the GNU Lesser General Public License as 
!! published by the Free Software Foundation, either version 3 of 
!! the License, or (at your option) any later version.
!!
!! NCSTAT is distributed in the hope that it will be useful,
!! but WITHOUT ANY WARRANTY; without even the implied warranty of
!! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!! GNU Lesser General Public License for more details.
!!
!! You can find a copy of the GNU Lesser General Public License
!! in the NCSTAT/doc directory.
!!
!! #############################################################################
!
!+
!
! .. _comp_symlin_filter_1d:
!
! =====================
! comp_symlin_filter_1d
! =====================
!
! Authors
! =======
!
! Pascal Terray (LOCEAN/IPSL)
!
! Latest revision
! ===============
!
! 06/05/2021
!
! Purpose
! =======
!
! Filter a real time series in a selected frequency band by linear symmetric filtering
! (e.g. Lanczos, Hamming, Hanning or "ideal" window filtering in this version of comp_symlin_filter_1d)
! [Bloomfield]_.
!
! The time series is extracted from a uni or bidimensional variable readed from
! a NetCDF dataset and can be also detrended before linear symmetric filtering at the user option.
!
! The number of coefficients used to build the linear symmetric filter can be selected and the
! filter can be applied to the time series in the time or frequency domain, also
! at the user option. This gives to the user some control on the desired end-effects of the filter (e.g.
! applying the filter in the frequency domain assumes implicitly that the time series is part
! of a periodic infinite series whose period is exactly equal to the length of the analyzed time series; on the
! other hand, applying the filter in the time domain implies some loss of data or some distortions of the desired
! response function of the filter at both ends of the filtered time series).
!
! Additionally, the filtering can be done separately for different segments of equal length of the selected
! time series if this time series is not continuous in time.
!
! The frequency response function (e.g. the transfer function) of the selected linear symmetric filter can be computed
! by :ref:`comp_freq_func_1d`. See the references cited below for more details on linear symmetric filtering.
!
! This procedure returns the filtered real time series in a NetCDF dataset. If the NetCDF variable
! is tridimensional or fourdimensional use :ref:`comp_symlin_filter_3d` or :ref:`comp_symlin_filter_4d`,
! respectively, instead of comp_symlin_filter_1d. If the time series has a seasonal (or diurnal) cycle,
! use :ref:`comp_stl_1d` in order to estimate and remove the harmonic components of the time series
! before using comp_symlin_filter_1d.
!
! Further Details
! ===============
!
! Usage
! -----
!
! ::
!
!       $ comp_symlin_filter_1d \
!         -f=input_netcdf_file \
!         -v=netcdf_variable \
!         -t=time1,time2                           (optional) \
!         -o=output_netcdf_file                    (optional) \
!         -ni=index_for_2d_netcdf_variable         (optional) \
!         -p=periodicity                           (optional) \
!         -pl=minimum_period                       (optional) \
!         -ph=maximum_period                       (optional) \
!         -tr=trend_removal                        (optional : 0, 1, 2, 3, -1, -2, -3) \
!         -nfc=number_of_filter_coefficients       (optional) \
!         -mi=missing_value                        (optional) \
!         -hamming                                 (optional) \
!         -win=window_choice                       (optional : 0.5 > 1.) \
!         -notestf                                 (optional) \
!         -usefft                                  (optional) \
!         -hdf5                                    (optional) \
!         -tlimited                                (optional)
!
! By default
! ----------
!
!        *-t=*
!         the whole time period associated with the *netcdf_variable*
!        *-o=*
!         the *output_netcdf_file* is named ``filt_``\ *netcdf_variable*\ ``.nc``
!        *-ni=*
!         if the *netcdf_variable* is bidimensional, the first time series is used
!        *-p=*
!         the *periodicity* is set to ``time2 - time1 + 1``, which means
!         that the time series is considered as continuous with only one time segment
!        *-pl=*
!         the *minimum_period* is set to ``0``, which means that no filtering is done
!         for the shorter periods
!        *-ph=*
!         the *maximum_period* is set to ``0``, which means that no filtering is done
!         for the longer periods
!        *-tr=0*
!         the *trend_removal* is set to ``0``, which means that no detrending is done before
!         filtering
!        *-nfc=*
!         the *number_of_filter_coefficients* is determined in order to optimize 
!         the frequency response function of the selected filter
!        *-mi=*
!         the *missing_value* for the output variable is set to ``1.e+20``
!        *-hamming*
!         a Lanczos window filter is used. If the **-hamming** argument is specified a Hamming window filter is
!         used instead
!        *-win=*
!         a Hamming window (e.g. **-win=**\ ``0.54``) is convolved with the filter response by default if
!         the **-hamming** argument is used, meaning that a Hamming window filter is computed. If the **-hamming**
!         argument is not used, this argument has no effect
!        *-notestf*
!         normally, the value of the **-nfc=** argument must be chosen such that 
!         the full transition bands about the cutoff frequencies ``1/PH`` and ``1/PL``
!         (where ``PH`` and ``PL`` are the values of the **-ph=** and **-pl=**
!         arguments, respectively) of the selected filter are inside the (``0`` ``0.5``) 
!         frequency interval. By using the **-notestf** argument you can get ride of this limitation
!        *-usefft*
!         the linear symmetric filter is applied in the time domain. When you specify the **-usefft** argument
!         the filter will be applied in the frequency domain, using an FFT algorithm and multiplication,
!         instead of a convolution in the time domain
!        *-double*
!         the results are stored as single-precision floating
!         point numbers in the output NetCDF file. If **-double** is activated, the
!         results are stored as double-precision floating point numbers 
!        *-hdf5*
!         a NetCDF classical format file is created. If **-hdf5** is activated,
!         the output NetCDF file is a NetCDF-4/HDF5 format file
!        *-tlimited*
!         the time dimension is defined as unlimited in the output NetCDF file.
!         However, if **-tlimited** is activated, the time dimension is defined
!         as limited in the output NetCDF file
!
! Remarks
! -------
!
!   #) The **-v=**\ *netcdf_variable* argument specifies the NetCDF variable for which a
!      linear filtering operation must be computed and the **-f=**\ *input_netcdf_file* 
!      argument specifies that this NetCDF variable must be extracted from the NetCDF file
!      *input_netcdf_file*.
!
!   #) If the **-t=**\ *time1,time2* argument is missing the whole time period
!      associated with the *netcdf_variable* is used to decompose the time series.
!
!      The selected time period is a vector of two integers specifying the first
!      and last time observations. The indices are relative to ``1``\ . Note that the
!      output NetCDF file will have ``ntime`` = *time2* - *time1* + ``1`` time observations.
!
!   #) The **-ni=**\ *index_for_2d_netcdf_variable* argument specifies the index for selecting the
!      time series if the *netcdf_variable* is a 2D NetCDF variable. By default, the first time series
!      is used, which is equivalent to set *index_for_2d_netcdf_variable* to ``1``\ .
!
!   #) If the **-p=** argument is specified, the filtering is applied separately for each time segment
!      of length *periodicity* (as determined by the value of the **-p=** argument).
!      The whole selected time period (e.g. *time2* - *time1* + ``1`` ) must also be a multiple of 
!      the *periodicity*.
!
!   #) The **-pl=** argument specifies the minimum period of oscillation of the filtered time series. 
!      The *minimum_period* is expressed in number of time observations. 
!
!      Do not use the **-pl=** argument or use **-pl=**\ ``0`` for high-pass filtering 
!      frequencies corresponding to periods shorter than **-ph=**\ ``PH`` .
!
!      The **-pl=** argument is a positive integer equal to ``0`` or greater than ``2``.
!
!   #) The **-ph=** argument specifies the maximum period of oscillation of the filtered time series. 
!      The *maximum_period* is expressed in number of time observations. 
!      Do not use the **-ph=** argument or use **-ph=**\ ``0`` for low-pass filtering 
!      frequencies corresponding to periods longer than **-pl=**\ ``PL`` .
!      For example, **-pl=**\ ``6``  (or ``18``) and **-ph=**\ ``32``  (or ``96``) select periods
!      between ``1.5`` and ``8``  years for quarterly (monthly) time series.
!
!      The **-ph=** argument is a positive integer equal to ``0`` or greater than ``2`` and 
!      less than the length of the time series or the *periodicity* if the **-p=**
!      argument is used.
!
!      The **-ph=** argument must also be greater or equal to the **-pl=** argument if 
!      both are specified.
!
!   #) Setting **-pl=** and **-ph=** to the same value ``P`` is allowed only if the **-hamming**
!      argument is not present (e.g., if Lanczos window filtering is used).
!      In this case, an -ideal- band-pass filter with peak response near one
!      at the single period ``P`` is computed and applied to the time series.
!
!   #) Setting both **-pl=**\ ``0`` and **-ph=**\ ``0`` is also allowed. In that case,
!      no frequencies filtering is done, but the data may be detrended if the **-tr=** argument 
!      is used with a value of ``1``, ``2`` or ``3``.
!
!   #) The **-tr=** argument specifies pre- and post-filtering processing of the time series. If:
!
!      * **-tr=**\ ``+/-1``, the mean of the time series is removed before time filtering
!
!      * **-tr=**\ ``+/-2``, the drift from the time series is removed before time 
!        filtering. The drift for the time series is estimated using the 
!        formula : ``drift = ( tseries(ntime) - tseries(1) )/( ntime - 1 )``
!
!      * **-tr=**\ ``+/-3``, the least-squares line from the time series is removed before
!        time filtering.
!
!      If **-tr=**\ ``-1``, ``-2`` or ``-3``, the mean, drift or least-squares line are reintroduced
!      post-filtering, respectively.
!
!      For other values of the **-tr=** argument, nothing is done before or after 
!      filtering.
!
!      If the **-p=** argument is present, the pre-filtering and post-filtering processing is applied to 
!      each time segment, separately.
!
!      The **-tr=** argument must be an integer and the default value for the **-tr=** argument is ``0``.
!
!   #) The **-nfc=** argument specifies the desired number of symmetric linear filter coefficients 
!      for the filtering of the time series. If **-nfc=** is not specified, an optimal 
!      value is chosen in order to obtain a good frequency response function for the selected filter.
!
!      However, if **-nfc=** is set to ``K``, the first and last ``(K-1)/2`` time observations in the 
!      output NetCDF file will be affected by end effects. Thus, the user must choose the number of 
!      filter terms, ``K``, as a compromise between:
!
!           1)  A sharp cutoff, that is, ``1/K`` small; and
!
!           2)  Minimizing the number of data points lost or affected by the filtering operation
!               (since ``(K-1)/2`` data points will be lost or affected from each end of the series).
!
!      Finally, the **-nfc=** argument must be greater or equal to ``3`` and odd.
!
!   #) The **-mi=**\ *missing_value* argument specifies the missing value indicator for
!      the output variable in the *output_netcdf_file*. 
!
!      If the **-mi=** argument is not specified, the *missing_value* is set
!      to ``1.e+20``.
!
!   #) If the **-hamming** argument is specified, a Hamming window filter is used instead of
!      Lanczos window filter.
!
!   #) The **-win=** argument controls the form of the window which will be convolved with the filter
!      if a Hamming window filter is requested with the **-hamming** argument.
!      By default, a Hamming window is used (e.g. **-win=**\ ``0.54``).
!
!      Set **-win=**\ ``0.5`` for using a Hanning window or **-win=**\ ``1.`` for a rectangular
!      window  (e.g. an "ideal" filter).
!
!      This argument has an effect only if the **-hamming** argument is also specified.
!      The **-win=** argument is a real number greater or equal to ``O.5`` and less or equal to ``1``.
!
!   #) The **-notestf** argument allows to bypass some of the restrictions on the 
!      number of filter coefficients as specified with the **-nfc=** argument.
!
!      By default, the value of the **-nfc=** argument must be chosen such that the 
!      full transition bands about the cutoff frequencies (``1/PH``  and ``1/PL``) of
!      the selected filter are inside the (``0`` ``0.5``) frequency interval.
!
!      When the **-notestf** argument is specified, only the cutoff frequencies (e.g. ``1/PH``
!      and ``1/PL``) of the selected filter must lie in the (``0`` ``0.5``) frequency interval
!      and not the full transition bands around them.
!
!      This allows to diminish the number of filter coefficients and, thus, to minimize the number
!      of data points lost by the filtering operation (if **-nfc=** is set to ``K``, ``(K-1)/2``  data 
!      points will be "lost" or affected by end effects from each end of the series).
!
!   #) The **-usefft** argument specifies that the linear symmetric filter must be applied in the 
!      frequency domain by using a Fast Fourier Transform and the convolution theorem.
!      Moreover, if the **-usefft** argument is specified, the values at the ends of 
!      the output filtered series are computed implicitly by assuming that the input series
!      is part of a periodic sequence.
!
!   #) The **-double** argument specifies that the results are stored as double-precision floating
!      point numbers in the output NetCDF file.
!      By default, the results are stored as single-precision floating point numbers
!      in the output NetCDF file.
!
!   #) The **-hdf5** argument is allowed only if the NCSTAT software has been
!      compiled with the _USE_NETCDF4 CPP macro (e.g. ``-D_USE_NETCDF4``) and linked
!      to the NetCDF 4 library or higher. 
!      If this argument is specified, the *output_netcdf_file* will be a NetCDF-4/HDF5
!      format file instead of a NetCDF classic format file. However, this
!      argument is recognized in the procedure only if the NCSTAT software has
!      been built with the _USE_NETCDF4 CPP macro.
!
!   #) It the time series has a seasonal or diurnal cycle, use :ref:`comp_stl_1d`
!      to remove the pure harmonic components from the time series before filtering.
!
!   #) It is assumed that the data has no missing values.
!
!   #) Duplicate parameters are allowed, but this is always the last occurrence
!      of a parameter which will be used for the computations. Moreover, the
!      number of specified parameters must not be greater than the total number
!      of allowed parameters.
!
!   #) For more details on linear symmetric filtering and examples of use in the climate literature, see
!
!      * "Fourier analysis of time series- An introduction",
!        by Bloomfield, P., John Wiley and Sons, New York, Chapter 6, 1976. http://as.wiley.com/WileyCDA/WileyTitle/productCd-0471889482.html
!
!      * "Lanczos filtering in one and two dimensions",
!        by Duchon, C., Journal of applied meteorology, vol. 18, 1016-1022, 1979. doi: `10.1175/1520-0450(1979)018<1016:LFIOAT>2.0.CO;2`_
!
! .. _`10.1175/1520-0450(1979)018<1016:LFIOAT>2.0.CO;2` : https://doi.org/10.1175/1520-0450(1979)018<1016:LFIOAT>2.0.CO;2
!
! Outputs
! -------
!
!   comp_symlin_filter_1d creates an output NetCDF file that contains the filtered time series estimated from the
!   time series associated with the input NetCDF variable. The output NetCDF dataset contains the
!   following NetCDF variable (in the description below, ntime is the selected number of time observations) :
!
!   #) *netcdf_variable*\ _filt\ ``(ntime)`` : the filtered time series for the time series 
!      associated with the input NetCDF variable.
!
! Examples
! --------
!
!   #)   For Hamming window filtering a real monthly time series between ``18`` and ``30``
!        months (e.g. biennial time scale) from a NetCDF variable called ``sst``
!        extracted from the file ``sst.monthly.nino34.nc``, which includes a monthly time
!        series, and store the results in the NetCDF file ``qbo_sst_nino34.nc``, use the following
!        command :
!
!        ::
!
!          $ comp_symlin_filter_1d \
!            -f=sst.monthly.nino34.nc \
!            -v=sst \
!            -pl=18 \
!            -ph=30 \
!            -hamming \
!            -o=qbo_sst_nino34.nc
!
!-
! ================================================================================================
!
!
! USED MODULES 
! ============
!
#ifdef _USE_NAGWARE
    use f90_unix_env,     only : iargc, getarg
#endif
!
#ifdef _USE_NETCDF4
    use netcdf,    only : nf90_hdf5
#endif
!
    use netcdf,   only : nf90_max_name, nf90_max_var_dims, nf90_clobber, nf90_real4, nf90_real8,   &
                         nf90_sizehint_default, nf90_open,                                         &
                         nf90_nowrite, nf90_noerr, nf90_get_att, nf90_get_var, nf90_strerror,      &
                         nf90_create, nf90_set_fill, nf90_nofill, nf90_def_dim, nf90_unlimited,    &
                         nf90_def_var, nf90_inq_attname, nf90_copy_att, nf90_put_att, nf90_enddef, &
                         nf90_global, nf90_put_var, nf90_close
!
    use Statpack, only : i1b, i4b, stnd, extd, lgl, true, false, one, half, kfix, kexp, kint, tolower,     & 
                         case_change, string_to_val, val_to_string, allocate_error, merror, arth,          &
                         detrend, lp_coef, lp_coef2, hp_coef, hp_coef2, bd_coef, bd_coef2,                 &
                         pk_coef, symlin_filter2, init_fft, end_fft
!
    use NCSTAT_Utilities, only : prtunit, nchar, sp, dp, def_missing,                              &
                                 get_listarg, get_variable_dimcoords
!
!
! STRONG TYPING IMPOSED
! =====================
!
    implicit none
!
!
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='comp_symlin_filter_1d : '
!
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
!
#ifndef _USE_NAGWARE
    integer(i4b)  :: iargc
#endif
    integer(i4b)  :: narg, status, ncid, ncid_filt,                                        &
                     varid, filtid, coor_id2,                                              &
                     i, i1, i2, iindex, ndim, nt, var_natts,                               &
                     time1, time2, time3,                                                  &
                     pl, ph, nfc, trend, method, output_type, output_mode, period, nperiod
    integer(i4b), dimension(nf90_max_var_dims) :: dimids, dimlens, coor_natts, coor_id, coor_type
    integer(i1b)                               :: kcode
!
    real(stnd), dimension(:),   allocatable    :: vec, coef
    real(stnd)                                 :: missing, tmp, freq, win
    real(extd), dimension(:),     allocatable  :: record
!
    logical(lgl)                               :: use_tlimited, detrend_only, notest_freq, use_miss, &
                                                  use_period, use_fft, use_hamming
    logical(lgl), dimension(nf90_max_var_dims) :: use_coor_variable
!
    character(len=nchar)                       :: name, history
    character(len=nf90_max_name)               :: fic_name, var_name, filt_name, att_name 
    character(len=nf90_max_name), dimension(nf90_max_var_dims) :: dim_names
    character(len=14)                                          :: fmt
!
!
! EXECUTABLE STATEMENTS
! =====================
!
!   GET THE NUMBER OF COMMAND LINE ARGUMENTS.
!
    narg = iargc()
!
!   CHECK THE NUMBER OF COMMAND LINE ARGUMENTS.
!
    if ( narg==0 ) then
!
        write (prtunit,*)
        write (prtunit,*) ' Purpose : '
        write (prtunit,*)
        write (prtunit,*) ' Filter a real time series from a uni or bidimensional variable extracted'
        write (prtunit,*) ' from a NetCDF dataset in a selected frequency band by symmetric linear'
        write (prtunit,*) ' filtering'
        write (prtunit,*)
        write (prtunit,*) ' Usage : '
        write (prtunit,*)
        write (prtunit,*) ' comp_symlin_filter_1d  '
        write (prtunit,*) '        -f=input_netcdf_file'
        write (prtunit,*) '        -v=netcdf_variable'
        write (prtunit,*) '        -t=time1,time2                     (optional)'
        write (prtunit,*) '        -o=output_netcdf_file              (optional)'
        write (prtunit,*) '        -ni=index_for_2d_netcdf_variable   (optional)'
        write (prtunit,*) '        -p=periodicity                     (optional)'
        write (prtunit,*) '        -pl=minimum_period                 (optional)'
        write (prtunit,*) '        -ph=maximum_period                 (optional)'
        write (prtunit,*) '        -tr=trend_removal                  (optional : 0,1,2,3,-1,-2,-3)'
        write (prtunit,*) '        -nfc=number_of_filter_coefficients (optional)'
        write (prtunit,*) '        -mi=missing_value                  (optional)'
        write (prtunit,*) '        -win=window_choice                 (optional : 0.5 > 1.)'
        write (prtunit,*) '        -hamming                           (optional)'
        write (prtunit,*) '        -notestf                           (optional)'
        write (prtunit,*) '        -usefft                            (optional)'
        write (prtunit,*) '        -double                            (optional)'
#ifdef _USE_NETCDF4
        write (prtunit,*) '        -hdf5                              (optional)'
#endif
        write (prtunit,*) '        -tlimited                          (optional)'
        write (prtunit,*)
        write (prtunit,*) ' By default : '
        write (prtunit,*)
        write (prtunit,*) '-t= the whole time period  associated with the netcdf_variable'
        write (prtunit,*) '-o=filt_netcdf_variable.nc'
        write (prtunit,*) '-ni=1            : by default if the netcdf_variable is bidimensional,'
        write (prtunit,*) '                   the first time series is used'
        write (prtunit,*) '-p=time2-time1+1 : by default, the periodicity argument is not used'
        write (prtunit,*) '-pl=0            : by default no filtering is done'
        write (prtunit,*) '-ph=0            : by default no filtering is done'
        write (prtunit,*) '-tr=0            : by default, no trend removal is done'
        write (prtunit,*) '-nfc=            : by default, the number of filter coefficients is determined to optimize'
        write (prtunit,*) '                   the frequency response function of the selected filter'
        write (prtunit,*) '-mi=1.e+20       : by default, the missing_value is equal to 1.e+20'
        write (prtunit,*) '-win=0.54        : by default, a Hamming window is convolved with the filter response if'
        write (prtunit,*) '                   -hamming is specified'
        write (prtunit,*) '-hamming         : by default, a Lanczos window filter is used.'
        write (prtunit,*) '-notestf         : by default, the value of the -nfc= argument must be choosen such that the'
        write (prtunit,*) '                   full transition bands about the cutoff frequencies of the selected filter'
        write (prtunit,*) '                   are inside the (0 0.5) interval'
        write (prtunit,*) '-usefft          : by default, the filter is applied in the time domain'
        write (prtunit,*) '-double          : by default, the results are stored as single floating numbers'
#ifdef _USE_NETCDF4
        write (prtunit,*) '-hdf5            : by default, a NetCDF classical format file is created'
#endif
        write (prtunit,*) '-tlimited        : by default, the time dimension is defined as unlimited'
        write (prtunit,*)
!
        stop
!
    else if ( narg<2 .or. narg>18 ) then
!
        call merror( name_proc//"Wrong number of line arguments" )
!
    end if
!
    fic_name   = " "
    var_name   = " "
    filt_name  = " "
!
    history = "comp_symlin_filter_1d"
!
!   DEFAULT VALUES FOR COMMAND LINE ARGUMENTS.
!
    output_mode   = nf90_clobber
    output_type   = nf90_real4
!
    iindex        = 1
    pl            = 0
    ph            = 0
    trend         = 0
    nfc           = 0
    win           = 0.54_stnd
!
    missing = def_missing
!
    time1     = 1
    time2     = huge( time2 )
!
    period    = huge( period )
    nperiod   = 1
!
    detrend_only = false
    use_hamming  = false
    notest_freq  = false
    use_fft      = false
    use_tlimited = false
    use_period   = false
!
!   GET THE COMMAND LINE ARGUMENTS.
!
    do i = 1, narg
!
        call getarg( i, name )
!
        history = trim(history)//" "//trim(name)
!
        call case_change( name(:3), tolower )
!
        select case ( name(1:3 ) )
!
            case ('-f=')
!
!               GET THE NAME OF THE NETCDF DATASET.
!
                fic_name = name(4:)
!
            case ('-v=')
!
!               GET THE NAME OF THE NETCDF VARIABLE IN THE DATASET.
!
                var_name = name(4:)
!
            case ('-t=')
!
!               GET THE PERIOD DEFINITION.
!
                call get_listarg( name_proc, name(4:),                    &
                                  "time1 and time2 are integer values",   &
                                  "Wrong period definition",              &
                                  time1, time2 )
!
            case ('-p=')
!
!               GET THE PERIODICITY.
!
                call string_to_val( name(4:), kcode, fmt )
                if ( kcode/=kint ) &
                call merror( name_proc//"The periodicity is an integer value" )
!
                read( name(4:), fmt ) period
!
                if ( period<=0 ) &
                call merror( name_proc//"The -p= parameter is a positive integer" )
!
                use_period = true
!
            case ('-o=')
!
!               GET THE NAME OF THE NEW NETCDF DATASET FOR STORING THE RESULTS.
!
                filt_name = name(4:)
!
            case default
!
                select case ( name(1:4 ) )
!
                    case ('-ni=')
!
!                       GET THE INDEX FOR A 2D VARIABLE.
!
                        call string_to_val( name(5:), kcode, fmt )
                        if ( kcode==kint ) then
                            read( name(5:), fmt ) iindex
                        end if
                        if ( kcode/=kint .or. iindex<=0  )                           &
                        call merror( name_proc//"The index for a 2D variable is a positive integer value" )
!
                    case ('-pl=')
!
!                       GET THE MINIMUM PERIOD OF OSCILLATION.
!
                        call string_to_val( name(5:), kcode, fmt )
                        if ( kcode==kint ) then
                            read( name(5:), fmt ) pl
                        end if
                        if ( kcode/=kint .or. pl<0 .or. pl==1 .or. pl==2 )                           &
                        call merror( name_proc//"The -pl= argument is an integer equal to 0 or greater than 2" )
!
                    case ('-ph=')
!
!                       GET THE MAXIMUM PERIOD OF OSCILLATION.
!
                        call string_to_val( name(5:), kcode, fmt )
                        if ( kcode==kint ) then
                            read( name(5:), fmt ) ph
                        end if
                        if ( kcode/=kint .or. ph<0 .or. ph==1 .or. ph==2  )                           &
                        call merror( name_proc//"The -ph= argument is an integer equal to 0 or greater than 2" )
!
                    case ('-tr=')
!
!                       DETERMINE PRE- AND POST-FILTERING COMPUTATIONS.
!
                        call string_to_val( name(5:), kcode, fmt )
                        if ( kcode==kint ) then
                            read( name(5:), fmt ) trend
                        else
                            call merror( name_proc//"The -tr= argument is an integer value" )
                        end if
!
                    case ('-mi=')
!
!                       GET THE MISSING VALUE INDICATOR.
!
                        call string_to_val( name(5:), kcode, fmt )
                        if ( kcode/=kfix .and. kcode/=kexp )                &
                        call merror( name_proc//"The -mi= argument is a real number " )
                        read( name(5:), fmt ) missing
!
                    case default
!
                        call case_change( name(4:), tolower )
!
                        select case ( name(1:5) )
!
                            case ('-nfc=')
!
!                               DETERMINE THE NUMBER OF FILTER COEFFICIENTS.
!
                                call string_to_val( name(6:), kcode, fmt )
                                if ( kcode==kint ) then
                                    read( name(6:), fmt ) nfc
                                end if
                               if ( kcode/=kint .or. nfc<3 .or. (nfc/2)*2==nfc  )                  &
                               call merror( name_proc//"The -nfc= argument is an odd integer greater than 2" )
!
                            case ('-win=')
!
!                               DETERMINE THE FORM OF THE HAMMING WINDOW.
!
                                call string_to_val( name(6:), kcode, fmt )
                                if ( kcode==kfix .or. kcode==kexp ) then
                                    read( name(6:), fmt ) win
                                end if
                                if ( (kcode/=kfix .and. kcode/=kexp) .or.     &
                                     win<half                        .or.     &
                                     win>one )                                &
      call merror( name_proc//"The -win= argument must be a real number and verify 0.5<=win<=1 " )
!
                            case default
!
                                select case ( name(1:) )
!
                                    case ('-hamming')
!
!                                       USE A HAMMING WINDOW FILTER.
!
                                        use_hamming = true
!
                                    case ('-notestf')
!
!                                       BYPASS THE TESTS ON THE CUTOFF FREQUENCIES.
!
                                        notest_freq = true
!
                                    case ('-usefft')
!
!                                       APPLY THE FILTER IN THE FREQUENCY DOMAIN.
!
                                        use_fft = true
!
                                    case ('-double')
!
!                                       STORE RESULTS AS DOUBLE PRECISION NUMBERS.
!
                                        output_type = nf90_real8
!
#ifdef _USE_NETCDF4
                                    case ('-hdf5')
!
!                                       REDEFINE THE CREATION MODE FLAG FOR THE OUTPUT FILE.
!
                                        output_mode = nf90_hdf5
#endif
!
                                    case ('-tlimited')
!
!                                       DEFINE THE TIME DIMENSION AS LIMITED.
!
                                        use_tlimited = true
!
                                    case default
!
!                                       WRONG ARGUMENT.
!
                                        call merror( name_proc//"Wrong argument" )
!
                                end select
!
                        end select
!
                end select
!
        end select
!
    end do
!
!   CHECK THE COMMAND LINE ARGUMENTS.
!
    if ( fic_name==" " )                                                 &
    call merror( name_proc//"Input NETCDF file is missing" )
!
    if ( var_name==" " )                                                 &
    call merror( name_proc//"Input NETCDF variable is missing" )
!
    if ( pl==ph .and. use_hamming )                                      &
    call merror( name_proc//"Setting -pl= equal to -ph= with -hamming argument is not allowed" )
!
    if ( filt_name==" " ) filt_name="filt_"//trim(var_name)//".nc"
!
!   CHECK PERIODS ORDERING TO SELECT FREQUENCIES.
!
    if ( ph<pl .and. ph/=0 )     &
    call merror( name_proc//"The -ph= argument must be greater or equal to the -pl= argument" )
!
    if ( max(pl,ph)==0 ) then
!
!       DETREND THE TIME SERIES.
!
        method = 0
!
        if ( trend>=1 .and. trend<=3 ) detrend_only = true
!
    else if ( ph==0 ) then
!
!       LOW-PASS FILTERING OF THE TIME SERIES.
!
        method = 1
!
        if ( nfc==0 ) then
!
!           SELECT THE OPTIMAL NUMBER OF FILTER COEFFICIENTS.
!
            tmp = one/real( pl, stnd )
            i   = ceiling( one/(half-tmp) )
            nfc = max( i, pl+1 )
!
        end if
!
    else if ( pl==0 ) then
!
!       HIGH-PASS FILTERING OF THE TIME SERIES.
!
        method = 2
!
        if ( nfc==0 ) then
!
!           SELECT THE OPTIMAL NUMBER OF FILTER COEFFICIENTS.
!
            tmp = one/real( ph, stnd )
            i   = ceiling( one/(half-tmp) )
            nfc = max( i, ph+1 )
!
        end if
!
    else if ( pl/=ph ) then
!
!       BAND-PASS FILTERING OF THE TIME SERIES.
!
        method = 3
!
        if ( nfc==0 ) then
!
!           SELECT THE OPTIMAL NUMBER OF FILTER COEFFICIENTS, LANCZOS FILTERING IS ASSUMED.
!
            tmp = one/real( pl, stnd )
            i1  = ceiling( one/(half-tmp) )
!
            tmp = tmp - one/real( ph, stnd )
            i2  = ceiling(  2.6/tmp )
!
            nfc = max( i1, i2, ph+1 )
!
        else if ( .not.use_hamming ) then
!
!           CHECK THAT nfc IS LARGE ENOUGH TO ENSURE UNIT RESPONSE AT BAND CENTER.
!
            tmp = one/real( pl, stnd ) - one/real( ph, stnd )   
!
            if ( real( nfc+1, stnd ) < (2.6/tmp) )         &
            call merror( name_proc//"The -nfc= argument is too small to ensure unit response at band center"  )
!
        end if
!
    else
!
!       BAND-PASS FILTERING OF THE TIME SERIES WITH A PEAK RESPONSE NEAR ONE AT THE PERIOD pl=ph.
!
        method = 4
!
        freq = one/real( pl, stnd )
!
        if ( nfc==0 ) then
!
!           SELECT THE OPTIMAL NUMBER OF FILTER COEFFICIENTS.
!
            tmp = 2.3*max( real( pl, stnd ), one/(half-freq) )
            nfc = ceiling( tmp )
!
        end if
!
    end if
!
!   CHECK IF THE NUMBER OF FILTER COEFFICIENTS IS ODD.
!
    if ( (nfc/2)*2==nfc  ) nfc = nfc + 1
!
!   OPEN THE NETCDF DATASET.
!
    i = nf90_sizehint_default
    status = nf90_open( trim(fic_name), nf90_nowrite, ncid, chunksize=i )
    if ( status/=nf90_noerr )                                                &
    call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!   GET THE VARIABLE ID FROM ITS NAME.
!   FIND THE DIMENSION LENGTHS AND NAMES FOR THE NETCDF VARIABLE.
!   GET THE COORDINATE VARIABLES IDS FROM THEIR NAMES, IF ANY.
!   FIND THE EXTERNAL TYPE, THE DIMENSION ID AND THE NUMBER OF ATTRIBUTES 
!   FOR THE COORDINATE VARIABLES.
!
    call get_variable_dimcoords(  name_proc, trim(var_name), ncid,                          &
                                  varid,  dimids_out=dimids, dim_names_out=dim_names,       &
                                  dimlens_out=dimlens, coor_id_out=coor_id,                 &
                                  coor_natts_out=coor_natts, coor_type_out=coor_type,       &
                                  use_coor_variable_out=use_coor_variable, natts=var_natts  )
!
    ndim = count(dimlens/=0)
    if ( ndim/=1 .and. ndim/=2 )     &
    call merror( name_proc//"Input NETCDF variable is not uni or bidimensional" )
!
    if ( ndim==2 ) then
        if ( iindex>dimlens(1) )     &
        call merror( name_proc//"Selected index for 2D input NETCDF variable is out of range " )
        nt = dimlens(2)
    else 
        nt = dimlens(1)
    end if
!
!   CHECK THE TIME DEFINITION AND PERIODICITY DEFINITION.
!
    time1 = max( time1, 1 )
    time2 = min( time2, nt )
    time3 = time2 - time1 + 1
    if ( time3<4  ) call merror( name_proc//"Wrong time definition or not enough time observations" )
!
    if ( use_period ) then
!
        if ( period<4  ) call merror( name_proc//"Not enough time observations in the periods" )
!
        nperiod = time3/period
        if ( nperiod*period/=time3 )    &
        call merror( name_proc//"The selected time period is not a multiple of the periodicity" )
!
    else
!
        period = time3
!
    end if
!
    if ( period<ph)   call merror( name_proc//"Periods longer than signal length cannot be observed" )
!
    if ( period<nfc)   call merror( name_proc//"The number of filter coefficients must be less or equal to signal length" )
!
!   ALLOCATE WORK ARRAYS FOR THE TIME SERIES AND THE FILTER COEFFICIENTS.
!
    if ( detrend_only ) then
        allocate( vec(time3), stat=status )
    else
        allocate( vec(time3), coef(nfc), stat=status )
    end if
!
    if ( status/=0 ) call merror( name_proc//allocate_error )
!
!   COMPUTE THE SYMMETRIC LINEAR FILTER COEFFICIENTS IF NEEDED.
!
    if ( use_hamming ) then
!
!       A HAMMING WINDOW FILTER IS REQUESTED.
!
        select case( method )
!
            case( 1_i4b )
!
!               LOW-PASS FILTERING OF THE TIME SERIES.
!
                coef(:nfc) = lp_coef2( pl, nfc, win=win, notest_fc=notest_freq )
!
            case( 2_i4b )
!
!               HIGH-PASS FILTERING OF THE TIME SERIES.
!
                coef(:nfc) = hp_coef2( ph, nfc, win=win, notest_fc=notest_freq )
!
            case( 3_i4b )
!
!               BAND-PASS FILTERING OF THE TIME SERIES.
!
                coef(:nfc) = bd_coef2( pl, ph, nfc, win=win, notest_fc=notest_freq )
!
        end select
!
    else
!
!       A LANCZOS WINDOW FILTER IS REQUESTED.
!
        select case( method )
!
            case( 1_i4b )
!
!               LOW-PASS FILTERING OF THE TIME SERIES.
!
                coef(:nfc) = lp_coef( pl, nfc, notest_fc=notest_freq )
!
            case( 2_i4b )
!
!               HIGH-PASS FILTERING OF THE TIME SERIES.
!
                coef(:nfc) = hp_coef( ph, nfc, notest_fc=notest_freq )
!
            case( 3_i4b )
!
!               BAND-PASS FILTERING OF THE TIME SERIES.
!
                coef(:nfc) = bd_coef( pl, ph, nfc, notest_fc=notest_freq )
!
            case( 4_i4b )
!
!               BAND-PASS FILTERING OF THE TIME SERIES WITH A PEAK RESPONSE NEAR ONE AT THE PERIOD pl=ph.
!
                coef(:nfc) = pk_coef( freq, nfc, notest_freq=notest_freq )
!
        end select
!
    end if
!
!   GET THE DATA.
!
    if ( ndim==1 ) then
        status = nf90_get_var( ncid, varid, vec(:time3),              &   
                               start=(/ time1 /), count=(/ time3 /)   )
!
    else 
        status = nf90_get_var( ncid, varid, vec(:time3),                          &   
                               start=(/ iindex, time1 /), count=(/ 1, time3 /)    )
!
    end if
!
    if ( status/=nf90_noerr )                                     &
    call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!   FIND THE MISSING VALUE ATTRIBUTE ASSOCIATED WITH THE NETCDF VARIABLE, IF ANY.
!
    status = nf90_get_att( ncid, varid, "missing_value", tmp )
    use_miss = status==nf90_noerr
!
    if ( .not.use_miss ) then
        status = nf90_get_att( ncid, varid, "_FillValue", tmp )
        use_miss = status==nf90_noerr
    end if
!
    if ( use_miss ) then
        if ( any( vec(:time3)==tmp ) )      &
        call merror( name_proc//"Missing values are not allowed" )
    end if
!
!   MULTIPLY THE NETCDF VARIABLE BY THE SCALE FACTOR IF REQUIRED.
!
    status = nf90_get_att( ncid, varid, "scale_factor", tmp )
    if ( status==nf90_noerr ) vec(:time3) = vec(:time3)*tmp
!
!   ADD OFFSET TO THE NETCDF VARIABLE IF REQUIRED.
!
    status = nf90_get_att( ncid, varid, "add_offset", tmp )
    if ( status==nf90_noerr ) vec(:time3) = vec(:time3) + tmp
!
    if ( use_fft ) then
!
!       INITIALIZE THE FFT SUBROUTINE IF REQUIRED.
!
        call init_fft( period )
!
    end if
!
!   FILTER THE TIME SERIES BY PERIODS.
!
    do i = 1, nperiod
!
        i1 = (i-1)*period +1
        i2 = i*period
!
        if ( detrend_only ) then
!
!           DETREND THE TIME SERIES, ONLY.
!
            call detrend( vec(i1:i2), trend )
!
        else if ( method/=0 ) then
!
!           FREQUENCIES FILTERING OF THE REAL TIME SERIES IS REQUIRED.
!
            call symlin_filter2( vec(i1:i2), coef(:nfc), trend=trend, usefft=use_fft, initfft=false )
!
        end if
!
    end do
!
    if ( use_fft ) then
!
!       DEALLOCATE WORK ARRAYS IF REQUIRED.
!
        call end_fft( )
!
    end if
!
!   CREATE THE NEW NETCDF DATASET FOR THE FREQUENCY ANALYSIS.
!
    status = nf90_create( filt_name, cmode=output_mode, ncid=ncid_filt )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_set_fill( ncid_filt, nf90_nofill, i )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!   DEFINE THE DIMENSIONS FOR THE NEW NETCDF DATASET.
!
    if ( use_tlimited ) then
        dimlens(ndim)   = time3
    else
        dimlens(ndim)   = nf90_unlimited
    end if
!
    status = nf90_def_dim( ncid_filt, name=trim(dim_names(ndim)),                         &
                           len=dimlens(ndim), dimid=dimids(ndim) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim(nf90_strerror(status)) )
!
!   DEFINE THE VARIABLES FOR THE NEW NETCDF DATASET.
!
    status = nf90_def_var( ncid_filt, trim(dim_names(ndim)), coor_type(ndim),         &
                           dimids=dimids(ndim), varid=coor_id2 )
    if ( status/=nf90_noerr )                                                         &
    call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_def_var( ncid_filt, trim(var_name)//"_filt",  output_type,              &
                           dimids=dimids(ndim), varid=filtid )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!   ASSIGN ATTRIBUTES FOR THE NEW NETCDF DATASET.
!
    if ( use_coor_variable(ndim) ) then
!
        do i = 1, coor_natts(ndim)
!
            status = nf90_inq_attname( ncid, coor_id(ndim), i, att_name )
            if ( status/=nf90_noerr )                                                      &
            call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           COPY ATTRIBUTES FOR THE COORDINATE VARIABLE.
!
            status = nf90_copy_att(  ncid, coor_id(ndim), att_name, ncid_filt, coor_id2 )
            if ( status/=nf90_noerr )                                                      &
            call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end do
!
    else
!
        status = nf90_put_att( ncid_filt, coor_id2, "long_name", "Time axis" )
        if ( status/=nf90_noerr )                                                          &
        call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_filt, coor_id2, "short_name", "Time" )
        if ( status/=nf90_noerr )                                                          &
        call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end if
!
!   COPY ATTRIBUTES FOR THE FILTERED VARIABLE.
!
    do i = 1, var_natts
!
        status = nf90_inq_attname( ncid, varid, i, att_name )
        if ( status/=nf90_noerr )  call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        if ( att_name=="add_offset"         .or.  &
             att_name=="scale_factor"       .or.  &
             att_name=="missing_value"      .or.  &
             att_name=="_FillValue"               )   cycle
!
!       COPY USEFUL ATTRIBUTES.
!
        status = nf90_copy_att(  ncid, varid, att_name, ncid_filt, filtid )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    end do
!
    if (  output_type==nf90_real8 ) then
!
        status = nf90_put_att( ncid_filt, filtid, "missing_value", real( missing, dp ) )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_filt, filtid, "_FillValue", real( missing, dp ) )
!
    else
!
        status = nf90_put_att( ncid_filt, filtid, "missing_value", real( missing, sp ) )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_filt, filtid, "_FillValue", real( missing, sp ) )
!
    end if
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    if ( ndim==1 ) then
        name = trim(var_name)
    else
        call val_to_string( iindex, fmt, i )
        name = trim(var_name)//"("//fmt(1:i)//")"
    end if
!
!   SET GLOBAL ATTRIBUTES.
!
    status = nf90_put_att( ncid_filt, nf90_global, "title", " Symmetric linear filtering of "//trim(var_name) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_filt, nf90_global, "minimum_period", pl )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_filt, nf90_global, "maximum_period", ph )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_filt, nf90_global, "trend_removal", trend )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_filt, nf90_global, "number_of_filter_coefficients", nfc )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    if ( use_hamming ) then
!
        name = "Hamming"
!
        status = nf90_put_att( ncid_filt, nf90_global, "window_parameter", win )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    else
!
        name = "Lanczos"
!
    end if
!
    status = nf90_put_att( ncid_filt, nf90_global, "Window", trim( name ) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    if ( use_fft ) then
        name = "time domain"
    else
        name = "frequency domain"
    end if
!
    status = nf90_put_att( ncid_filt, nf90_global, "algorithm", trim(name) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_filt, nf90_global, "number_of_time_segments", nperiod )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_filt, nf90_global, "history", trim(history) )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!   LEAVE DEFINE MODE FOR THE NEW NETCDF DATASET.
!
    status = nf90_enddef( ncid_filt )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!   WRITE VARIABLES FOR THE NEW NETCDF DATASET.
!
    if ( use_coor_variable(ndim) ) then
!
        allocate( record(time3), stat=status )
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
        status = nf90_get_var( ncid, coor_id(ndim), record, start=(/ time1 /), count=(/ time3 /) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_var( ncid_filt, coor_id2, record )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        deallocate( record )
!
    else
!
        status = nf90_put_var( ncid_filt, coor_id2, arth(time1,1_i4b,time3 ) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end if
!
!   WRITE FILTERED VARIABLE.
!
    status = nf90_put_var( ncid_filt, filtid, vec(:time3) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!   DEALLOCATE WORK ARRAYS.
!
    if ( detrend_only ) then
        deallocate( vec )
    else
        deallocate( vec, coef )
    end if
!
!   CLOSE THE NETCDF DATASETS.
!
    status = nf90_close( ncid )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_close( ncid_filt )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
! END OF PROGRAM comp_symlin_filter_1d
! ====================================
!
end program comp_symlin_filter_1d
