program comp_trend_1d
!! #############################################################################
!!
!! Copyright 2018 IRD
!!
!! This file is part of NCSTAT.
!!
!! NCSTAT is free software: you can redistribute it and/or modify
!! it under the terms of the GNU Lesser General Public License as 
!! published by the Free Software Foundation, either version 3 of 
!! the License, or (at your option) any later version.
!!
!! NCSTAT is distributed in the hope that it will be useful,
!! but WITHOUT ANY WARRANTY; without even the implied warranty of
!! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!! GNU Lesser General Public License for more details.
!!
!! You can find a copy of the GNU Lesser General Public License
!! in the NCSTAT/doc directory.
!!
!! #############################################################################
!
!+
!
! .. _comp_trend_1d:
!
! =============
! comp_trend_1d
! =============
!
! Authors
! =======
!
! Pascal Terray (LOCEAN/IPSL)
!
! Latest revision
! ===============
!
! 13/09/2018
!
! Purpose
! =======
!
! Extract a trend component from an unidimensional variable in a NetCDF dataset by using
! a LOESS smoother [Cleveland]_ [Cleveland_Devlin]_. The LOESS procedure is a powerful
! statistical technique for describing a discrete time series (see the references
! in the Remarks Section below). In the LOESS procedure, the analyzed time series is decomposed into two terms:
!
!           **X**\ (*t*\ ) = **T**\ (*t*\ ) + **R**\ (*t*\ )
!
! where *t* refers to a time index, the **T** term is used to quantify the trend and low-frequency variations in the time series,
! and, finally, the **R** term contains the residual component.
!
! The trend is estimated through a sequence of applications of locally weighted regression or low-order polynomial (e.g. Loess)
! to data windows whose length is chosen by the user. Other important features of the LOESS method are the specification of
! the amount of trend smoothing according to the choice of the user, the ability to produce a robust estimate
! of the trend component that is not distorted by aberrant behavior in the data and the stationarity of
! the **R** time series. 
!
! This procedure returns the trend component as estimated by the LOESS procedure and, optionally,
! the residuals in a NetCDF dataset, for the time series associated with a NetCDF variable.
!
! If the NetCDF variable is tridimensional or fourdimensional use :ref:`comp_trend_3d` or :ref:`comp_trend_4d`, respectively 
! instead of comp_trend_1d. It the time series has a seasonal (or diurnal) cycle,
! use :ref:`comp_stl_1d` instead of comp_trend_1d in order to estimate the trend component by the Loess procedure.
!
! The exact meaning and default values for most of the optional parameters of comp_trend_1d are exactly the same as in the original
! (STL) procedure described by [Cleveland_etal]_ and the user is referred to this publication for further
! details on the LOESS procedure as implemented here.
!
! Further Details
! ===============
!
! Usage
! -----
!
! ::
!
!       $ comp_trend_1d \
!         -f=input_netcdf_file \
!         -v=input_netcdf_variable \
!         -nt=trend_smoother_length        (nt) \
!         -t=time1,time2                             (optional) \
!         -a=type_of_analysis                        (optional : trend, residual)  \
!         -o=output_netcdf_file                      (optional) \
!         -itdeg=trend_smoother_degree     (itdeg)   (optional : 0, 1, 2 ) \
!         -ntjump=trend_skipping_value     (ntjump)  (optional) \
!         -maxit=max_robustness_iterations (maxit)   (optional) \
!         -smt=trend_smoothing_factor                (optional) \
!         -robust                                    (optional) \
!         -double                                    (optional) \
!         -hdf5                                      (optional) \
!         -tlimited                                  (optional)
!
! By default
! ----------
!
!        *-t=*
!         the whole time period associated with the *netcdf_variable*
!        *-a=*
!         the *type_of_analysis* is set to ``trend``. This means that the residual times series
!         is not computed and not stored in the *output_netcdf_file*
!        *-o=*
!         the *output_netcdf_file* is named  ``trend_``\ *netcdf_variable*\ ``.nc``
!        *-itdeg=*
!         the *trend_smoother_degree* is set to ``1``
!        *-ntjump=*
!         the *trend_skipping_value* is set to ``nt/10`` where ``nt`` is the value of
!         the *trend_smoother_length* argument
!        *-maxit=*
!         the *max_robustness_iterations* is set to ``15``
!        *-smt=*
!         no smoothing is applied to the trend component
!        *-robust*
!         robustness iterations are not used, by default
!        *-double*
!         the results are stored as single-precision floating
!         point numbers in the output NetCDF file. If **-double** is activated, the
!         results are stored as double-precision floating point numbers 
!        *-hdf5*
!         a NetCDF classical format file is created. If **-hdf5** is activated,
!         the output NetCDF file is a NetCDF-4/HDF5 format file
!        *-tlimited*
!         the time dimension is defined as unlimited in the output NetCDF file.
!         However, if **-tlimited** is activated, the time dimension is defined
!         as limited in the output NetCDF file
!
! Remarks
! -------
!
!   #) The **-v=**\ *netcdf_variable* argument specifies the NetCDF variable from which the time
!      series must be decomposed and the **-f=**\ *input_netcdf_file* argument specifies that this NetCDF variable
!      must be extracted from the NetCDF file *input_netcdf_file*.
!
!   #) If the **-t=**\ *time1,time2* argument is missing, the whole time period
!      associated with the *netcdf_variable* is used to decompose the time series.
!
!      The selected time period is a vector of two integers specifying the first
!      and last time observations. The indices are relative to ``1``\ . Note that the
!      output NetCDF file will have ``ntime`` = *time2* - *time1* + ``1`` time observations.
!
!   #) The **-nt=** argument specifies the length of the trend smoother, ``nt``. The value of ``nt``
!      should be an odd integer greater than or equal to ``3``. As ``nt`` increases the values of
!      the trend component become smoother.
!
!   #) The **-itdeg=** argument specifies the degree of the locally-fitted polynomial in trend
!      smoothing.  The value is ``0``, ``1`` or ``2``.
!
!   #) The **-ntjump=** argument specifies the skipping value for trend smoothing.
!      The trend smoother skips ahead  ``ntjump``  points and then
!      linearly interpolates in between.  The  value  of  ``ntjump``
!      should  be  a  positive  integer; if ``ntjump`` is set to ``1``, a trend
!      smooth is calculated  at  all points in the time series. To make the
!      procedure run faster, a reasonable choice for ``ntjump`` is
!      ``10``\ % or ``20``\ % of ``nt``.
!
!   #) The **-a=** argument specifies if the residuals from the trend component
!      are stored in the output NetCDF file. If:
!
!      * **-a=**\ ``trend``, the residuals are not computed
!
!      * **-a=**\ ``residual``, the residuals are computed and stored.
!
!      The default is **-a=**\ ``trend`` , e.g. the residuals are not stored. Note that in all cases, the
!      trend component is computed and stored in the output NetCDF file.
!
!   #) If **-robust** is specified, robustness iterations are carried out until
!      convergence of the trend component or with a maximum of ``maxit`` iterations
!      as specified by the **-maxit=** argument. Convergence occurs if the maximum
!      changes in individual trend fits are less than ``1``\ % of the component's range
!      after the previous iteration.
!
!   #) **-smt=**\ *trend_smoothing_factor* means that the trend component extracted from
!      the *netcdf_variable* (e.g. the **-v=** argument) must be smoothed with
!      a moving average of approximately ``2``\ *\ *trend_smoothing_factor*\ +\ ``1`` terms.
!      The smoothing is applied before estimating the residuals (e.g. if **-a=** argument
!      is set to ``residual`` ).
!      *trend_smoothing_factor* must be a strictly positive integer (e.g. greater than ``0``).
!      Note that this additional smoothing is not implemented in the original LOESS procedure.
!
!   #) The **-double** argument specifies that, the results are stored as double-precision floating
!      point numbers in the output NetCDF file.
!      By default, the results are stored as single-precision floating point numbers
!      in the output NetCDF file.
!
!   #) The **-hdf5** argument is allowed only if the NCSTAT software has been
!      compiled with the _USE_NETCDF4 CPP macro (e.g. ``-D_USE_NETCDF4``) and linked
!      to the NetCDF 4 library or higher. 
!      If this argument is specified, the *output_netcdf_file* will be a NetCDF-4/HDF5
!      format file instead of a NetCDF classic format file. However, this
!      argument is recognized in the procedure only if the NCSTAT software has
!      been built with the _USE_NETCDF4 CPP macro.
!
!   #) It is assumed that the data has no missing values.
!
!   #) It the time series have a seasonal or diurnal cycle, use :ref:`comp_stl_1d` instead of comp_trend_1d.
!
!   #) Duplicate parameters are allowed, but this is always the last occurrence
!      of a parameter which will be used for the computations. Moreover, the
!      number of specified parameters must not be greater than the total number
!      of allowed parameters.
!
!   #) For more details on the LOESS procedure, see
!
!      * "Robust Locally Weighted Regression and Smoothing Scatterplots",
!        by Cleveland, W.S., Journal of the American Statistical Association, Vol. 74, 829-836, 1979. doi: `10.1080/01621459.1979.10481038`_
!
!      * "Locally Weighted Regression: An Approach to Regression Analysis by Local Fitting",
!        by Cleveland, W.S., and Devlin, S.J., Journal of the American Statistical Association, Vol. 83, 596-610, 1988. doi: `10.1080/01621459.1988.10478639`_
!
!      * "A Seasonal-Trend Decomposition Procedure Based on Loess",
!        by Cleveland, R.B.,  Cleveland, W.S., McRae, J.E., and Terpennings, I.,
!        Journal of Official Statistics, 6, 3-73, 1990. http://www.jos.nu/Articles/abstract.asp?article=613
!
! .. _10.1080/01621459.1979.10481038 : http://dx.doi.org/10.1080/01621459.1979.10481038
! .. _10.1080/01621459.1988.10478639 : http://dx.doi.org/10.1080/01621459.1988.10478639
!
! Outputs
! -------
!
!   comp_trend_1d creates an output NetCDF file that contains the trend and residual components extracted from the
!   time series associated with the input NetCDF variable. The output NetCDF dataset contains the
!   following NetCDF variables (in the description below,  ntime is the selected number of time observations) :
!
!   #) *netcdf_variable*\ _trend\ ``(ntime)`` : the trend component for the time series
!      associated with the input NetCDF variable.
!
!   #) *netcdf_variable*\ _residual\ ``(ntime)`` : the residual component for the time series
!      associated with the input NetCDF variable.
!
!      This variable is stored only if the **-a**\ =\ ``residual`` argument has been specified when calling
!      comp_trend_1d.
!
! Examples
! --------
!
!   #)   For estimating a trend from the unidimensional NetCDF variable called ``ts`` extracted from the
!        file ``ts_gfdl_cm2_0.picntrl_monthly_glob.nc``, which includes a monthly anomalies time series, and store
!        the results in the NetCDF file ``ts_gfdl_cm2_0.picntrl_monthly_glob_trend.nc``, use the following
!        command :
!
!        ::
!
!          $ comp_trend_1d \
!            -f=ts_gfdl_cm2_0.picntrl_monthly_glob.nc \
!            -v=ts \
!            -nt=111 \
!            -a=trend \
!            -robust \
!            -o=ts_gfdl_cm2_0.picntrl_monthly_glob_trend.nc
!
!-
! ================================================================================================
!
!
! USED MODULES 
! ============
!
#ifdef _USE_NAGWARE
    use f90_unix_env,     only : iargc, getarg
#endif
!
#ifdef _USE_NETCDF4
    use netcdf,    only : nf90_hdf5
#endif
!
    use netcdf,   only : nf90_max_var_dims, nf90_max_name, nf90_real4, nf90_real8, nf90_clobber, &
                         nf90_open, nf90_nowrite, nf90_noerr, nf90_strerror, nf90_get_var,       &
                         nf90_get_att, nf90_create, nf90_clobber, nf90_set_fill, nf90_nofill,    &
                         nf90_unlimited, nf90_def_dim, nf90_def_var, nf90_inq_attname,           &
                         nf90_copy_att, nf90_put_att, nf90_inquire_attribute, nf90_global,       &
                         nf90_enddef, nf90_put_var, nf90_close
!
    use Statpack, only : i1b, i4b, stnd, extd, lgl, true, false, ten, c0_9,   & 
                         kint, tolower, case_change, string_to_val,           &
                         allocate_error, merror, arth, comp_trend, comp_smooth
!
    use NCSTAT_Utilities, only : prtunit, nchar, get_listarg, get_variable_dimcoords
!
!
! STRONG TYPING IMPOSED 
! =====================
!
    implicit none
!
!
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='comp_trend_1d : ',                 &
!
                                       cmethod(2)=(/"trend   ","residual"/),     &
!
                                       ctrendtype(2)=(/"not_robust","robust    "/)
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
!
#ifndef _USE_NAGWARE
    integer(i4b)  :: iargc
#endif
    integer(i4b)  :: narg, status, ncid, ncid_trend,                         &
                     varid, residid, trendid, coor_id2,                      &
                     i, ntime, time1, time2, time3,                          &
                     method, trendtype, output_type, output_mode, maxiter,   &
                     nt, itdeg, ntjump, trend_smooth_factor
    integer(i4b), dimension(nf90_max_var_dims) :: dimids, dimlens, coor_natts, coor_id, coor_type
    integer(i1b)                               :: kcode
!
    real(stnd), dimension(:),     allocatable  :: y, trend
    real(stnd)                                 :: tmp
    real(extd), dimension(:),     allocatable  :: record
!
    logical(lgl)                               :: use_tlimited, robust, comp_resid, ok
    logical(lgl), dimension(nf90_max_var_dims) :: use_coor_variable
!
    character(len=nchar)                       :: name, history
    character(len=nf90_max_name)               :: fic_name, var_name, trend_name, att_name
    character(len=nf90_max_name), dimension(nf90_max_var_dims) :: dim_names
    character(len=14)                                          :: fmt
!
!
! EXECUTABLE STATEMENTS
! =====================
!
!   GET THE NUMBER OF COMMAND LINE ARGUMENTS.
!
    narg = iargc()
!
!   CHECK THE NUMBER OF COMMAND LINE ARGUMENTS.
!
    if ( narg==0 ) then
!
        write (prtunit,*)
        write (prtunit,*) ' Purpose : '
        write (prtunit,*)
        write (prtunit,*) ' Extract a trend component from a unidimensional variable in '
        write (prtunit,*) ' a NetCDF dataset by using a LOESS smoother. It returns the ' 
        write (prtunit,*) ' trend component and, optionally, the residuals in a NetCDF dataset.'
        write (prtunit,*)
        write (prtunit,*) ' Usage : '
        write (prtunit,*)
        write (prtunit,*) ' comp_trend_1d  '
        write (prtunit,*) '        -f=input_netcdf_file'
        write (prtunit,*) '        -v=netcdf_variable'
        write (prtunit,*) '        -nt=trend_smoother_length'
        write (prtunit,*) '        -t=time1,time2                   (optional)'
        write (prtunit,*) '        -a=type_of_analysis              (optional : trend, residual)'
        write (prtunit,*) '        -o=output_netcdf_file            (optional)'
        write (prtunit,*) '        -itdeg=trend_smoother_degree     (optional : 0, 1, 2)'
        write (prtunit,*) '        -ntjump=trend_skipping_value     (optional)'
        write (prtunit,*) '        -maxit=max_robustness_iterations (optional)'
        write (prtunit,*) '        -smt=trend_smoothing_factor      (optional)'
        write (prtunit,*) '        -robust                          (optional)'
        write (prtunit,*) '        -double                          (optional)'
#ifdef _USE_NETCDF4
        write (prtunit,*) '        -hdf5                            (optional)'
#endif
        write (prtunit,*) '        -tlimited                        (optional)'
        write (prtunit,*)
        write (prtunit,*) ' By default : '
        write (prtunit,*)
        write (prtunit,*) '-t= the whole time period associated with the netcdf_variable'
        write (prtunit,*) '-a=trend'
        write (prtunit,*) '-o=trend_netcdf_variable.nc'
        write (prtunit,*) '-itdeg=1'
        write (prtunit,*) '-ntjump=nt/10 where nt is the value of trend_smoother_length'
        write (prtunit,*) '-maxit=15'
        write (prtunit,*) '-smt= by default, no smoothing is applied to the trend component'
        write (prtunit,*) '-robust   : by default, no robustness iterations are to be used'
        write (prtunit,*) '-double   : by default, the results are stored as single floating numbers'
#ifdef _USE_NETCDF4
        write (prtunit,*) '-hdf5     : by default, a NetCDF classical format file is created'
#endif
        write (prtunit,*) '-tlimited : by default, the time dimension is defined as unlimited'
        write (prtunit,*)
!
        stop
!
    else if ( narg<2 .or. narg>14 ) then
!
        call merror( name_proc//"Wrong number of line arguments" )
!
    end if
!
    fic_name   = " "
    var_name   = " "
    trend_name = " "
!
    history = "comp_trend_1d"
!
!   DEFAULT VALUES FOR COMMAND LINE ARGUMENTS.
!
    output_mode   = nf90_clobber
    output_type   = nf90_real4
!
    nt            = -1
    itdeg         = 1
    ntjump        = -1
    method        = 1
    trendtype     = 1
    maxiter       = 15
!
    comp_resid    = false
    robust        = false
!
    trend_smooth_factor  = 0
!
    time1     = 1
    time2     = huge( time2 )
!
    use_tlimited = false
!
!   GET THE COMMAND LINE ARGUMENTS.
!
    do i = 1, narg
!
        call getarg( i, name )
!
        history = trim(history)//" "//trim(name)
!
        call case_change( name(:3), tolower )
!
        select case ( name(1:3 ) )
!
            case ('-f=')
!
!               GET THE NAME OF THE NETCDF DATASET.
!
                fic_name = name(4:)
!
            case ('-v=')
!
!               GET THE NAME OF THE NETCDF VARIABLE IN THE DATASET.
!
                var_name = name(4:)
!
            case ('-t=')
!
!               GET THE PERIOD DEFINITION.
!
                call get_listarg( name_proc, name(4:),                    &
                                  "time1 and time2 are integer values",   &
                                  "Wrong period definition",              &
                                  time1, time2 )
!
            case ('-o=')
!
!               GET THE NAME OF THE NEW NETCDF DATASET FOR STORING THE RESULTS.
!
                trend_name = name(4:)
!
            case ('-a=')
!
!               DETERMINE IF THE RESIDUALS ARE REQUIRED.
!
                call case_change( name(4:), tolower )
!
                select case ( name(4:) )
                    case ( cmethod(1) )
                        method = 1
                    case ( cmethod(2) )
                        method     = 2
                        comp_resid = true
                    case default
                        call merror( name_proc//"Unknown analysis option" )
                end select
!
            case default
!
                select case ( name(1:4 ) )
!
                    case ('-nt=')
!
!                       GET THE LENGTH OF THE TREND SMOOTHER.
!
                        call string_to_val( name(5:), kcode, fmt )
                        if ( kcode==kint ) then
                            read( name(5:), fmt ) nt
                        end if
                        if ( kcode/=kint .or. nt<=2  )                           &
                        call merror( name_proc//"The -nt= argument is an integer > 2" )
!
                    case default
!
                        call case_change( name(4:5), tolower )
!
                        select case ( name(1:5) )
!
                            case ('-smt=')
!
!                               DETERMINE THE TREND SMOOTHING FACTOR.
!
                                call string_to_val( name(6:), kcode, fmt )
                                if ( kcode==kint ) then
                                     read( name(6:), fmt ) trend_smooth_factor
                                end if
                                if ( trend_smooth_factor<=0 )        &
         call merror( name_proc//"The -smt= argument is a strictly positive integer value " )
!
                            case default
!
                                call case_change( name(6:7), tolower )
!
                                select case ( name(1:7) )
!
                                    case ('-itdeg=')
!
!                                       GET THE DEGREE OF LOCALL-FITTED POLYNOMIAL IN TREND SMOOTHING.
!
                                        call string_to_val( name(8:), kcode, fmt )
                                        if ( kcode==kint ) then
                                            read( name(8:), fmt ) itdeg
                                        end if
                                        if ( kcode/=kint .or. (itdeg<0 .or. itdeg>2)  )                &
                                        call merror( name_proc//"The -itdeg= argument must be 0, 1 or 2." )
!
                                    case ('-maxit=')
!
!                                       GET THE MAXIMUM NUMBER OF ROBUSTNESS ITERATIONS.
!
                                        call string_to_val( name(8:), kcode, fmt )
                                        if ( kcode==kint ) then
                                            read( name(8:), fmt ) maxiter
                                        end if
                                        if ( kcode/=kint .or. maxiter<=0  )                &
                          call merror( name_proc//"The -maxit= argument is a strictly positive integer" )
!
                                    case default
!
                                        select case ( name(1:8) )
!
                                            case ('-ntjump=')
!
!                                               GET THE TREND SKIPPING VALUE.
!
                                                call string_to_val( name(9:), kcode, fmt )
                                                if ( kcode==kint ) then
                                                    read( name(9:), fmt ) ntjump
                                                end if
                                                if ( kcode/=kint .or. ntjump<=0  )                &
                                 call merror( name_proc//"The -ntjump= argument is a stricly positive integer" )
!
                                            case default
!
                                                call case_change( name(8:), tolower )
!
                                                select case ( name )
!
                                                    case ('-robust')
!
!                                                       ROBUST ITERATIONS ARE REQUIRED.
!
                                                        robust  = true
                                                        trendtype = 2
!
                                                    case ('-double')
!
!                                                       STORE RESULTS AS DOUBLE PRECISION NUMBERS.
!
                                                        output_type = nf90_real8
!
#ifdef _USE_NETCDF4
                                                    case ('-hdf5')
!
!                                                       REDEFINE THE CREATION MODE FLAG FOR THE OUTPUT FILE.
!
                                                        output_mode = nf90_hdf5
#endif
!
                                                    case ('-tlimited')
!
!                                                       DEFINE THE TIME DIMENSION AS LIMITED.
!
                                                        use_tlimited = true
!
                                                    case default
!
!                                                       WRONG ARGUMENT.
!
                                                        call merror( name_proc//"Wrong argument" )
!
                                                end select
!
                                        end select
!
                                end select
!
                        end select
!
                end select
!
        end select
!
    end do
!
!   CHECK THE COMMAND LINE ARGUMENTS.
!
    if ( fic_name==" " )                                                 &
    call merror( name_proc//"Input NETCDF file is missing" )
!
    if ( var_name==" " )                                                 &
    call merror( name_proc//"Input NETCDF variable is missing" )
!
    if ( nt==-1_i4b )                                                    &
    call merror( name_proc//"The -nt= argument is missing" )
    if ( mod(nt,2_i4b)==0_i4b) nt = nt + 1_i4b
!
    if ( ntjump==-1_i4b ) then
        ntjump = max( 1_i4b, int(real(nt,stnd)/ten + c0_9,i4b) )
    end if
!
    if ( trend_name==" " ) trend_name="trend_"//trim(var_name)//".nc"
!
!   OPEN THE NETCDF DATASETS.
!
    status = nf90_open( trim(fic_name), nf90_nowrite, ncid )
    if ( status/=nf90_noerr )                                            &
    call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!   GET THE VARIABLE ID FROM ITS NAME.
!   FIND THE DIMENSION LENGTHS AND NAMES FOR THE NETCDF VARIABLE.
!   GET THE COORDINATE VARIABLES IDS FROM THEIR NAMES, IF ANY.
!   FIND THE EXTERNAL TYPE, THE DIMENSION ID AND THE NUMBER OF ATTRIBUTES 
!   FOR THE COORDINATE VARIABLES.
!
    call get_variable_dimcoords(  name_proc, trim(var_name), ncid,                    &
                                  varid,  dimids_out=dimids, dim_names_out=dim_names, &
                                  dimlens_out=dimlens, coor_id_out=coor_id,           &
                                  coor_natts_out=coor_natts, coor_type_out=coor_type, &
                                  use_coor_variable_out=use_coor_variable             )
!
    if ( count(dimlens/=0)/=1 )           &
    call merror( name_proc//"Input NETCDF variable is not unidimensional" )
!
    ntime = dimlens(1)
!
!   CHECK THE TIME DEFINITION.
!
    time1 = max( time1, 1 )
    time2 = min( time2, ntime )
    time3 = time2 - time1 + 1
    if ( time3<=0  ) call merror( name_proc//"Wrong time definition" )
!
!   ALLOCATE WORK ARRAYS.
!
    allocate( y(time3), trend(time3), stat=status   )
    if ( status/=0 ) call merror( name_proc//allocate_error )
!
!   FIRST READ THE NETCDF VARIABLE.
!
    status = nf90_get_var( ncid, varid, y(:time3), start=(/ time1 /), count=(/ time3 /) )  
!
!   FIND THE MISSING VALUE ATTRIBUTE ASSOCIATED WITH THE NETCDF VARIABLE, IF ANY.
!
    status = nf90_get_att( ncid, varid, "missing_value", tmp )
    if ( status==nf90_noerr ) then
!
        if ( any( y(:time3)==tmp ) )      &
        call merror( name_proc//"Missing values are not allowed" )
!
    end if
!
!   FIND THE SCALE_FACTOR ATTRIBUTE ASSOCIATED WITH THE NETCDF VARIABLE, IF ANY.
!
    status = nf90_get_att( ncid, varid, "scale_factor", tmp )
    if ( status==nf90_noerr ) y(:time3) = y(:time3)*tmp
!
!   FIND THE ADD_OFFSET ATTRIBUTE ASSOCIATED WITH THE NETCDF VARIABLE, IF ANY.
!
    status = nf90_get_att( ncid, varid, "add_offset", tmp )
    if ( status==nf90_noerr )  y(:time3) = y(:time3) + tmp
!
!   EXTRACT THE TREND COMPONENT FROM THE TIME SERIES.
!
    call comp_trend( y(:time3), nt, itdeg, robust, trend(:time3),  &
                     ntjump=ntjump, maxiter=maxiter, ok=ok )
!
!   SMOOTH THE TREND COMPONENT IF REQUIRED.
!
    if ( trend_smooth_factor/=0 ) then
        call comp_smooth( trend(:time3), trend_smooth_factor )
    end if
!
!   COMPUTE RESIDUALS IF REQUIRED.
!
    if ( method==2 ) then
         y(:time3) = y(:time3) - trend(:time3)
    end if
!
!   CREATE THE NEW NETCDF DATASET FOR THE TREND ANALYSIS.
!
    status = nf90_create( trend_name, cmode=output_mode, ncid=ncid_trend )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_set_fill( ncid_trend, nf90_nofill, i )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!   DEFINE THE DIMENSIONS FOR THE NEW NETCDF DATASET.
!
    if ( use_tlimited ) then
        dimlens(1)   = time3
    else
        dimlens(1)   = nf90_unlimited
    end if
!
    status = nf90_def_dim( ncid_trend, name=trim(dim_names(1)), len=dimlens(1), dimid=dimids(1) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim(nf90_strerror(status)) )
!
!   DEFINE THE VARIABLES FOR THE NEW NETCDF DATASET.
!
    status = nf90_def_var( ncid_trend, trim(dim_names(1)), coor_type(1),                &
                           dimids=dimids(1), varid=coor_id2 )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim(nf90_strerror(status)) )
!
    status = nf90_def_var( ncid_trend, trim(var_name)//"_trend", output_type, dimids=dimids(1), varid=trendid )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    if ( comp_resid ) then
!
        status = nf90_def_var( ncid_trend, trim(var_name)//"_"//cmethod(method), output_type,    &
                               dimids=dimids(1), varid=residid )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    end if
!
!   ASSIGN ATTRIBUTES FOR THE NEW NETCDF DATASET.
!
    if ( use_coor_variable(1) ) then
!
        do i = 1, coor_natts(1)
!
            status = nf90_inq_attname( ncid, coor_id(1), i, att_name )
            if ( status/=nf90_noerr )                                                     &
            call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           COPY ATTRIBUTES FOR THE TIME VARIABLE.
!
            status = nf90_copy_att(  ncid, coor_id(1), att_name, ncid_trend, coor_id2 )
            if ( status/=nf90_noerr )                                                     &
            call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end do
!
    else
!
        status = nf90_put_att( ncid_trend, coor_id2, "long_name", "time axis" )
        if ( status/=nf90_noerr )                                                  &
        call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_trend, coor_id2, "short_name", "time" )
        if ( status/=nf90_noerr )                                                  &
        call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end if
!
    status = nf90_put_att( ncid_trend, trendid, "long_name", "trend of "//trim(var_name)  )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_trend, trendid, "short_name", "trend"  )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    if ( trend_smooth_factor/=0_i4b ) then
        status = nf90_put_att( ncid_trend, trendid, "smoothing_factor", trend_smooth_factor )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
    end if
!
    if ( comp_resid ) then
        status = nf90_put_att( ncid_trend, residid, "long_name", cmethod(method)//" of "//trim(var_name)  )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_trend, residid, "short_name", cmethod(method)  )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
    end if
!
    status = nf90_inquire_attribute( ncid, varid, "units" )
    if ( status==nf90_noerr ) then
!
        status = nf90_copy_att(  ncid, varid, "units", ncid_trend, trendid )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        if ( comp_resid ) then
            status = nf90_copy_att(  ncid, varid, "units", ncid_trend, residid )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
        end if
!
    end if
!
!   SET GLOBAL ATTRIBUTES.
!
    status = nf90_put_att( ncid_trend, nf90_global, "title", "TREND analysis of "//trim(var_name) )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_trend, nf90_global, "trend_smoother_length", nt )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_trend, nf90_global, "trend_smoother_degree", itdeg )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_trend, nf90_global, "trend_skipping_value", ntjump )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    if ( robust ) then
!
        if ( ok ) then
            fmt = "1 on 1"
        else
            fmt = "0 on 1"
        end if
!
        status = nf90_put_att( ncid_trend, nf90_global, "convergence_factor", fmt(1:6) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_trend, nf90_global, "max_robustness_iterations", maxiter )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end if
!
    status = nf90_put_att( ncid_trend, nf90_global, "trendtype", ctrendtype(trendtype) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_trend, nf90_global, "method", cmethod(method) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_trend, nf90_global, "history", trim(history) )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!   LEAVE DEFINE MODE FOR THE NEW NETCDF DATASET.
!
    status = nf90_enddef( ncid_trend )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!   WRITE VARIABLES FOR THE NEW  NETCDF DATASET.
!
!   WRITE TIME VARIABLE FOR THE NEW  NETCDF DATASET.
!
    if ( use_coor_variable(1) ) then
!
!       ALLOCATE WORK ARRAY.
!
        allocate( record(time3), stat=status )
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
        status = nf90_get_var( ncid, coor_id(1), record, start=(/ time1 /) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_var( ncid_trend, coor_id2, record )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!       DEALLOCATE WORK ARRAY.
!
        deallocate( record )
!
    else
!
        status = nf90_put_var( ncid_trend, coor_id2, arth(time1,1_i4b,time3 ) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end if
!
!   WRITE THE TREND COMPONENT.
!
    status = nf90_put_var( ncid_trend, trendid, trend(:time3) )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )

    if ( comp_resid ) then
!
!       WRITE THE RESIDUALS IF REQUIRED.
!
        status = nf90_put_var( ncid_trend, residid, y(:time3) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end if
!
!   DEALLOCATE WORK ARRAYS.
!
    deallocate( y, trend )
!
!   CLOSE THE NETCDF DATASETS.
!
    status = nf90_close( ncid )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_close( ncid_trend )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
! END OF PROGRAM comp_trend_1d
! ============================
!
end program comp_trend_1d
