program comp_trend_3d
!! #############################################################################
!!
!! Copyright 2018 IRD
!!
!! This file is part of NCSTAT.
!!
!! NCSTAT is free software: you can redistribute it and/or modify
!! it under the terms of the GNU Lesser General Public License as 
!! published by the Free Software Foundation, either version 3 of 
!! the License, or (at your option) any later version.
!!
!! NCSTAT is distributed in the hope that it will be useful,
!! but WITHOUT ANY WARRANTY; without even the implied warranty of
!! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!! GNU Lesser General Public License for more details.
!!
!! You can find a copy of the GNU Lesser General Public License
!! in the NCSTAT/doc directory.
!!
!! #############################################################################
!
!+
!
! .. _comp_trend_3d:
!
! =============
! comp_trend_3d
! =============
!
! Authors
! =======
!
! Pascal Terray (LOCEAN/IPSL)
!
! Latest revision
! ===============
!
! 13/09/2018
!
! Purpose
! =======
!
! Extract a trend component from a tridimensional variable in a NetCDF dataset by using
! a LOESS smoother [Cleveland]_ [Cleveland_Devlin]_. The LOESS procedure is a powerful
! statistical technique for describing a discrete time series (see the references
! in the Remarks Section below). In the LOESS procedure, the analyzed multi-channel
! time series is decomposed into two terms:
!
!           **X**\ (*t*\ ) = **T**\ (*t*\ ) + **R**\ (*t*\ )
!
! where *t* refers to a time index, the **T** term is used to quantify the trend and low-frequency variations in the time series,
! and, finally, the **R** term contains the residual component.
!
! The trend is estimated through a sequence of applications of locally weighted regression or low-order polynomial (e.g. Loess)
! to data windows whose length is chosen by the user. Other important features of the LOESS method are the specification of
! the amount of trend smoothing according to the choice of the user, the ability to produce robust estimates
! of the trend component that are not distorted by aberrant behavior in the data and the stationarity of
! the **R** time series. 
!
! This procedure returns the trend component as estimated by the LOESS procedure and, optionally,
! the residuals in a NetCDF dataset, for the multi-channel time series associated with a NetCDF variable.
! 
! If the NetCDF variable is unidimensional or fourdimensional use :ref:`comp_trend_1d` or :ref:`comp_trend_4d`, respectively 
! instead of comp_trend_3d. It the time series have a seasonal (or diurnal) cycle,
! use :ref:`comp_stl_3d` instead of comp_trend_3d in order to estimate the trend component by the Loess procedure.
! 
! The exact meaning and default values for most of the optional parameters of comp_trend_3d are exactly the same as in the original
! (STL) procedure described by [Cleveland_etal]_ and the user is referred to this publication for further
! details on the LOESS procedure as implemented here.
! 
! This procedure is parallelized if OpenMP is used.
!
! Further Details
! ===============
!
! Usage
! -----
!
! ::
!
!       $ comp_trend_3d \
!         -f=input_netcdf_file \
!         -v=input_netcdf_variable \
!         -nt=trend_smoother_length        (nt) \
!         -m=input_mesh_mask_netcdf_file             (optional) \
!         -g=grid_type                               (optional : n, t, u, v, w, f) \
!         -x=lon1,lon2                               (optional) \
!         -y=lat1,lat2                               (optional) \
!         -t=time1,time2                             (optional) \
!         -a=type_of_analysis                        (optional : trend, residual)  \
!         -o=output_netcdf_file                      (optional) \
!         -itdeg=trend_smoother_degree     (itdeg)   (optional : 0, 1, 2 ) \
!         -ntjump=trend_skipping_value     (ntjump)  (optional) \
!         -maxit=max_robustness_iterations (maxit)   (optional) \
!         -smt=trend_smoothing_factor                (optional) \
!         -robust                                    (optional) \
!         -mi=missing_value                          (optional) \
!         -double                                    (optional) \
!         -hdf5                                      (optional) \
!         -bigfile                                   (optional) \
!         -tlimited                                  (optional)
!
! By default
! ----------
!
!        *-m=*
!         an *input_mesh_mask_netcdf_file* is not used
!        *-g=*
!         the *grid_type* is set to ``n`` which means that the 2-D grid-mesh associated with
!         the input NetCDF variable is assumed to be regular or Gaussian
!        *-x=*
!         the whole longitude domain associated with the *netcdf_variable*
!        *-y=*
!         the whole latitude domain associated with the *netcdf_variable*
!        *-t=*
!         the whole time period associated with the *netcdf_variable*
!        *-a=*
!         the *type_of_analysis* is set to ``trend``. This means that the residual times series
!         are not computed and not stored in the *output_netcdf_file*
!        *-o=*
!         the *output_netcdf_file* is named  ``trend_``\ *netcdf_variable*\ ``.nc``
!        *-itdeg=*
!         the *trend_smoother_degree* is set to ``1``
!        *-ntjump=*
!         the *trend_skipping_value* is set to ``nt/10`` where ``nt`` is the value of
!         the *trend_smoother_length* argument
!        *-maxit=*
!         the *max_robustness_iterations* is set to ``15``
!        *-smt=*
!         no smoothing is applied to the trend component
!        *-robust*
!         robustness iterations are not used, by default
!        *-mi=*
!         the *missing_value* for the output variables is equal to ``1.e+20``
!        *-double*
!         the results are stored as single-precision floating
!         point numbers in the output NetCDF file. If **-double** is activated, the
!         results are stored as double-precision floating point numbers 
!        *-bigfile*
!         a NetCDF classical format file is created. If **-bigfile** is activated,
!         the output NetCDF file is a 64-bit offset format file
!        *-hdf5*
!         a NetCDF classical format file is created. If **-hdf5** is activated,
!         the output NetCDF file is a NetCDF-4/HDF5 format file
!        *-tlimited*
!         the time dimension is defined as unlimited in the output NetCDF file.
!         However, if **-tlimited** is activated, the time dimension is defined
!         as limited in the output NetCDF file
!
! Remarks
! -------
!
!   #) The **-v=**\ *netcdf_variable* argument specifies the NetCDF variable from which the multi-channel time
!      series must be decomposed and the **-f=**\ *input_netcdf_file* argument specifies that this NetCDF variable
!      must be extracted from the NetCDF file *input_netcdf_file*.
!
!   #) If the **-x=**\ *lon1,lon2* and **-y=**\ *lat1,lat2* arguments are missing, the whole
!      geographical domain associated with the *netcdf_variable* is used to select the multi-channel
!      time series.
!
!      The longitude or latitude range must be a vector of two integers specifying
!      the first and last selected indices along each dimension. The indices are relative to ``1``\ .
!      Negative values are allowed for *lon1*. In this case the longitude domain
!      is from ``nlon``\ +\ *lon1*\ +\ ``1`` to *lon2* where ``nlon`` is the number of longitude
!      points in the grid associated with the NetCDF variable and it is assumed that the grid
!      is periodic.
!
!      Refer to :ref:`comp_mask_3d` for transforming geographical coordinates as indices before
!      using comp_trend_3d.
!
!   #) If the **-t=**\ *time1,time2* argument is missing, the whole time period
!      associated with the *netcdf_variable* is used to decompose the time series.
!
!      The selected time period is a vector of two integers specifying the first
!      and last time observations. The indices are relative to ``1``\ . Note that the
!      output NetCDF file will have ``ntime`` = *time2* - *time1* + ``1`` time observations.
!
!   #) The geographical shapes of the *netcdf_variable* (in the *input_netcdf_file*) and the mask
!      (in the *input_mesh_mask_netcdf_file*) must agree if the **-m=** argument is used.
!
!   #) If **-g=** is set to ``t``, ``u``, ``v``, ``w`` or ``f`` it is assumed that the
!      NetCDF variable is from an experiment with the NEMO or ORCA model.
!
!      If **-g=** is set to ``n``, it is assumed that the 2-D grid-mesh is regular or Gaussian
!      and as such has no duplicate points.
!
!      This argument is also used to determine the name of the NetCDF mesh_mask variable if an 
!      *input_mesh_mask_netcdf_file* is used as specified with the **-m=** argument.
!
!   #) The **-nt=** argument specifies the length of the trend smoother, ``nt``. The value of ``nt``
!      should be an odd integer greater than or equal to ``3``. As ``nt`` increases the values of
!      the trend component become smoother.
!
!   #) The **-itdeg=** argument specifies the degree of the locally-fitted polynomial in trend
!      smoothing.  The value is ``0``, ``1`` or ``2``.
!
!   #) The **-ntjump=** argument specifies the skipping value for trend smoothing.
!      The trend smoother skips ahead  ``ntjump``  points and then
!      linearly interpolates in between.  The  value  of  ``ntjump``
!      should  be  a  positive  integer; if ``ntjump`` is set to ``1``, a trend
!      smooth is calculated  at  all points in the time series. To make the
!      procedure run faster, a reasonable choice for ``ntjump`` is
!      ``10``\ % or ``20``\ % of ``nt``.
!
!   #) The **-a=** argument specifies if the residuals from the trend component
!      are stored in the output NetCDF file. If:
!
!      * **-a=**\ ``trend``, the residuals are not computed
!
!      * **-a=**\ ``residual``, the residuals are computed and stored.
!
!      The default is **-a=**\ ``trend`` , e.g. the residuals are not stored. Note that in all cases, the
!      trend component is computed and stored in the output NetCDF file.
!
!   #) If **-robust** is specified, robustness iterations are carried out until
!      convergence of the trend component or with a maximum of ``maxit`` iterations
!      as specified by the **-maxit=** argument. Convergence occurs if the maximum
!      changes in individual trend fits are less than ``1``\ % of the component's range
!      after the previous iteration.
!
!   #) **-smt=**\ *trend_smoothing_factor* means that the trend component extracted from
!      the *netcdf_variable* (e.g. the **-v=** argument) must be smoothed with
!      a moving average of approximately ``2``\ *\ *trend_smoothing_factor*\ +\ ``1`` terms.
!      The smoothing is applied before estimating the residuals (e.g. if **-a=** argument
!      is set to ``residual`` ).
!      *trend_smoothing_factor* must be a strictly positive integer (e.g. greater than ``0``).
!      Note that this additional smoothing is not implemented in the original LOESS procedure.
!
!   #) The **-mi=**\ *missing_value* argument specifies the missing value attribute for
!      the output variables in the *output_netcdf_file*. 
!      If the **-mi=** argument is not specified, the *missing_value* is set
!      to ``1.e+20``.
!
!   #) The **-double** argument specifies that, the results are stored as double-precision floating
!      point numbers in the output NetCDF file.
!      By default, the results are stored as single-precision floating point numbers
!      in the output NetCDF file.
!
!   #) The **-bigfile** argument is allowed only if the NCSTAT software has been compiled with the
!      _USE_NETCDF36 or _USE_NETCDF4 CPP macros (e.g. ``-D_USE_NETCDF36`` or ``-D_USE_NETCDF4``) and
!      linked to the NetCDF 3.6 library or higher.
!      If this argument is specified, the *output_netcdf_file* will be a 64-bit offset format file
!      instead of a NetCDF classic format file. However, this argument is recognized in the procedure
!      only if the NCSTAT software has been built with the _USE_NETCDF36 or _USE_NETCDF4 CPP macros.
!
!   #) The **-hdf5** argument is allowed only if the NCSTAT software has been
!      compiled with the _USE_NETCDF4 CPP macro (e.g. ``-D_USE_NETCDF4``) and linked
!      to the NetCDF 4 library or higher. 
!      If this argument is specified, the *output_netcdf_file* will be a NetCDF-4/HDF5
!      format file instead of a NetCDF classic format file. However, this
!      argument is recognized in the procedure only if the NCSTAT software has
!      been built with the _USE_NETCDF4 CPP macro.
!
!   #) It is assumed that the data has no missing values excepted those associated with a constant land-sea
!      mask.
!
!   #) It the time series have a seasonal or diurnal cycle, use :ref:`comp_stl_3d` instead of comp_trend_3d.
!
!   #) Duplicate parameters are allowed, but this is always the last occurrence
!      of a parameter which will be used for the computations. Moreover, the
!      number of specified parameters must not be greater than the total number
!      of allowed parameters.
!
!   #) For more details on the LOESS procedure, see
!
!      * "Robust Locally Weighted Regression and Smoothing Scatterplots",
!        by Cleveland, W.S., Journal of the American Statistical Association, Vol. 74, 829-836, 1979. doi: `10.1080/01621459.1979.10481038`_
!
!      * "Locally Weighted Regression: An Approach to Regression Analysis by Local Fitting",
!        by Cleveland, W.S., and Devlin, S.J., Journal of the American Statistical Association, Vol. 83, 596-610, 1988. doi: `10.1080/01621459.1988.10478639`_
!
!      * "A Seasonal-Trend Decomposition Procedure Based on Loess",
!        by Cleveland, R.B.,  Cleveland, W.S., McRae, J.E., and Terpennings, I.,
!        Journal of Official Statistics, 6, 3-73, 1990. http://www.jos.nu/Articles/abstract.asp?article=613
!
! .. _10.1080/01621459.1979.10481038 : http://dx.doi.org/10.1080/01621459.1979.10481038
! .. _10.1080/01621459.1988.10478639 : http://dx.doi.org/10.1080/01621459.1988.10478639
!
! Outputs
! -------
!
!   comp_trend_3d creates an output NetCDF file that contains the trend and residual components extracted from the
!   time series associated with the input NetCDF variable. The output NetCDF dataset contains the
!   following NetCDF variables (in the description below, nlat and nlon are the length of the spatial dimensions
!   of the input NetCDF variable and ntime is the selected number of time observations) :
!
!   #) *netcdf_variable*\ _trend\ ``(ntime,nlat,nlon)`` : the trend component for each of the time series 
!      of the 2-D grid-mesh associated with the input NetCDF variable.
!
!   #) *netcdf_variable*\ _residual\ ``(ntime,nlat,nlon)`` : the residual component for each of
!      the time series of the 2-D grid-mesh associated with the input NetCDF variable.
!
!      This variable is stored only if the **-a**\ =\ ``residual`` argument has been specified when calling
!      comp_trend_3d.
!
!   The trend and residual components are packed in tridimensional variables whose first and second dimensions
!   are exactly the same as those associated with the input NetCDF variable *netcdf_variable*
!   even if you restrict the geographical domain with the **-x=** and **-y=** arguments. However,
!   outside the selected domain, the output NetCDF variables are filled with missing values.
!
! Examples
! --------
!
!   #)   For estimating a trend from the tridimensional NetCDF variable called ``sosstsst`` extracted from the
!        file ``ST7_1m_00101_20012_sosstsst_ano_grid_T.nc``, which includes monthly anomalies time series, and store
!        the results in the NetCDF file ``stl_ST7_1m_00101_20012_sosstsst_ano_grid_T.nc``, use the following
!        command :
!
!        ::
!
!          $ comp_trend_3d \
!            -f=ST7_1m_00101_20012_sosstsst_ano_grid_T.nc \
!            -v=sosstsst \
!            -nt=127 \
!            -a=residual \
!            -robust \
!            -o=stl_ST7_1m_00101_20012_sosstsst_ano_grid_T.nc
!
!-
! ================================================================================================
!
#ifndef _OPENMP
#undef _PARALLEL_READ
#endif
!
#ifdef _USE_NETCDF4
#define _USE_NETCDF36
#endif
!
! USED MODULES 
! ============
!
#ifdef _USE_NAGWARE
    use f90_unix_env,     only : iargc, getarg
#endif
!
#ifdef _PARALLEL_READ
    use netcdf,    only : nf90_share
    use Statpack,  only : omp_limit
    use omp_lib,   only : omp_get_num_procs, omp_get_max_threads
#endif
!
#ifdef _USE_NETCDF36
    use netcdf,    only : nf90_64bit_offset
#endif
!
#ifdef _USE_NETCDF4
    use netcdf,    only : nf90_hdf5
#endif
!
    use netcdf,   only : nf90_max_var_dims, nf90_max_name, nf90_real4, nf90_real8,                       &
                         nf90_sizehint_default, nf90_open, nf90_nowrite, nf90_noerr,                     &
                         nf90_strerror,  nf90_get_att, nf90_get_var, nf90_create, nf90_clobber,          &
                         nf90_set_fill, nf90_nofill, nf90_unlimited, nf90_def_dim, nf90_def_var,         &
                         nf90_inq_attname, nf90_inq_varid, nf90_copy_att, nf90_put_att, nf90_global,     &
                         nf90_enddef, nf90_inquire_attribute, nf90_put_var, nf90_close
!
    use Statpack, only : i1b, i4b, stnd, extd, lgl, true, false, zero, one,  ten, c0_9,               & 
                         kfix, kexp, kint, tolower, case_change, string_to_val, val_to_string,        &
                         allocate_error, merror, arth, comp_trend, comp_smooth
!
    use NCSTAT_Utilities, only : prtunit, nchar, sp, dp, def_missing,                          &
                                 get_listarg, get_variable_dims, get_variable_dimcoords
!
!
! STRONG TYPING IMPOSED 
! =====================
!
    implicit none
!
!
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='comp_trend_3d : ',      &
!
            cmethod(2)=(/"trend   ","residual"/),                     &
!
            ctrendtype(2)=(/"not_robust","robust    "/)
!
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
!
#ifndef _USE_NAGWARE
    integer(i4b)  :: iargc
#endif
    integer(i4b)  :: narg, status, ncid, ncid_mask, ncid_trend,                               &
                     varid, maskid, residid, trendid, convid,                                 &
                     i, j, j1, j2, k, nlon, nlat, ntime, nvar,                                &
                     lon1, lon2, lon3, lat1, lat2, lat3,                                      &
                     time1, time2, time3, rec, offset_rec,                                    &
                     method, trendtype, maxiter,                                              &
                     nt, itdeg, ntjump,                                                       &
                     trend_smooth_factor, output_type, output_mode
    integer(i4b), dimension(3)                 :: coor_id2, startlec, countlec
    integer(i4b), dimension(nf90_max_var_dims) :: dimids, dimlens, coor_natts, coor_id, coor_type
    integer(i1b)                               :: kcode
!
    real(stnd), dimension(:,:),   allocatable  :: y, trend, temp
    real(stnd)                                 :: missing, tmp
    real(extd), dimension(:),     allocatable  :: record
!
    logical(lgl)                               :: use_tlimited, robust, comp_resid,    &
                                                  use_x, use_y, use_mask, create_hdf5
    logical(lgl), dimension(nf90_max_var_dims) :: use_coor_variable
    logical(lgl), dimension(:,:), allocatable  :: mask
    logical(lgl), dimension(:),   allocatable  :: ok
!
    character(len=nchar)                       :: name, history
    character(len=nf90_max_name)               :: fic_name, var_name, mask_name, trend_name, att_name
    character(len=nf90_max_name), dimension(nf90_max_var_dims) :: dim_names
    character(len=14)                                          :: fmt, fmt2
    character                                                  :: chr, grid_type
!
#ifdef _PARALLEL_READ
    integer(i4b)  :: ncid2, varid2
!
    logical  :: test_par
!
#endif
!
!
! EXECUTABLE STATEMENTS
! =====================
!
!   GET THE NUMBER OF COMMAND LINE ARGUMENTS.
!
    narg = iargc()
!
!   CHECK THE NUMBER OF COMMAND LINE ARGUMENTS.
!
    if ( narg==0 ) then
!
        write (prtunit,*)
        write (prtunit,*) ' Purpose : '
        write (prtunit,*)
        write (prtunit,*) ' Extract trend components from a tridimensional variable in '
        write (prtunit,*) ' a NetCDF dataset by using a LOESS smoother. It returns the' 
        write (prtunit,*) ' trend components and, optionally, the residuals in a NetCDF dataset.'
        write (prtunit,*)
        write (prtunit,*) ' Usage : '
        write (prtunit,*)
        write (prtunit,*) ' comp_trend_3d  '
        write (prtunit,*) '        -f=input_netcdf_file'
        write (prtunit,*) '        -v=netcdf_variable'
        write (prtunit,*) '        -nt=trend_smoother_length'
        write (prtunit,*) '        -m=input_mesh_mask_netcdf_file   (optional)'
        write (prtunit,*) '        -g=grid_type                     (optional : n, t, u, v, w, f)'
        write (prtunit,*) '        -x=lon1,lon2                     (optional)'
        write (prtunit,*) '        -y=lat1,lat2                     (optional)'
        write (prtunit,*) '        -t=time1,time2                   (optional)'
        write (prtunit,*) '        -a=type_of_analysis              (optional : trend, residual)'
        write (prtunit,*) '        -o=output_netcdf_file            (optional)'
        write (prtunit,*) '        -itdeg=trend_smoother_degree     (optional : 0, 1, 2)'
        write (prtunit,*) '        -ntjump=trend_skipping_value     (optional)'
        write (prtunit,*) '        -maxit=max_robustness_iterations (optional)'
        write (prtunit,*) '        -smt=trend_smoothing_factor      (optional)'
        write (prtunit,*) '        -robust                          (optional)'
        write (prtunit,*) '        -mi=missing_value                (optional)'
        write (prtunit,*) '        -double                          (optional)'
#ifdef _USE_NETCDF36
        write (prtunit,*) '        -bigfile                         (optional)'
#endif
#ifdef _USE_NETCDF4
        write (prtunit,*) '        -hdf5                            (optional)'
#endif
        write (prtunit,*) '        -tlimited                        (optional)'
        write (prtunit,*)
        write (prtunit,*) ' By default : '
        write (prtunit,*)
        write (prtunit,*) '-m= an input_mesh_mask_netcdf_file is not used'
        write (prtunit,*) '-g= the grid type is set to n'
        write (prtunit,*) '-x= the whole longitude domain associated with the netcdf_variable'
        write (prtunit,*) '-y= the whole latitude domain associated with the netcdf_variable'
        write (prtunit,*) '-t= the whole time period associated with the netcdf_variable'
        write (prtunit,*) '-a=trend'
        write (prtunit,*) '-o=trend_netcdf_variable.nc'
        write (prtunit,*) '-itdeg=1'
        write (prtunit,*) '-ntjump=nt/10 where nt is the value of trend_smoother_length'
        write (prtunit,*) '-maxit=15'
        write (prtunit,*) '-smt= by default, no smoothing is applied to the trend component'
        write (prtunit,*) '-robust : by default, no robustness iterations are to be used'
        write (prtunit,*) '-mi= by default, the missing_value is equal to 1.e+20'
        write (prtunit,*) '-double   : by default, the results are stored as single floating numbers'
#ifdef _USE_NETCDF36
        write (prtunit,*) '-bigfile : by default, a netCDF classical format file is created'
#endif
#ifdef _USE_NETCDF4
        write (prtunit,*) '-hdf5     : by default, a NetCDF classical format file is created'
#endif
        write (prtunit,*) '-tlimited : by default, the time dimension is defined as unlimited'
#ifdef _OPENMP
        write (prtunit,*)
        write (prtunit,*) 'This procedure is parallelized with OpenMP'
#endif
        write (prtunit,*)
!
        stop
!
    else if ( narg<3 .or. narg>20 ) then
!
        call merror( name_proc//"Wrong number of line arguments" )
!
    end if
!
    fic_name   = " "
    var_name   = " "
    mask_name  = " "
    trend_name = " "
!
    history = "comp_trend_3d"
!
!   DEFAULT VALUES FOR COMMAND LINE ARGUMENTS.
!
    output_mode   = nf90_clobber
    output_type   = nf90_real4
!
    missing       = def_missing
!
    grid_type     = "n"
!
    nt            = -1
    itdeg         = 1
    ntjump        = -1
    method        = 1
    trendtype     = 1
    maxiter       = 15
!
    comp_resid    = false
    robust        = false
!
    trend_smooth_factor  = 0
!
    lon1      = 1
    lon2      = huge( lon2 )
    lat1      = 1
    lat2      = huge( lat2 )
    time1     = 1
    time2     = huge( time2 )
!
    use_mask      = false
    use_x         = false
    use_y         = false
    create_hdf5   = false
    use_tlimited  = false
!
!   GET THE COMMAND LINE ARGUMENTS.
!
    do i = 1, narg
!
        call getarg( i, name )
!
        history = trim(history)//" "//trim(name)
!
        call case_change( name(:3), tolower )
!
        select case ( name(1:3 ) )
!
            case ('-f=')
!
!               GET THE NAME OF THE NETCDF DATASET.
!
                fic_name = name(4:)
!
            case ('-v=')
!
!               GET THE NAME OF THE NETCDF VARIABLE IN THE DATASET.
!
                var_name = name(4:)
!
            case ('-g=')
!
!               GET THE GRID-TYPE.
!
                call case_change( name(4:), tolower )
!
                select case ( name(4:) )
                    case ( 'n', 'u', 'v', 'w', 't', 'f' )
                        grid_type = name(4:4)
                    case default
                        call merror( name_proc//"Unknown grid type" )
                end select
!
            case ('-x=')
!
!               GET THE DOMAIN DEFINITION (Longitude).
!
                call get_listarg( name_proc, name(4:),                    &
                                  "lon1 and lon2 are integer values",     &
                                  "Wrong domain definition (longitude)",  &
                                  lon1, lon2 )
!
                use_x = true
!
            case ('-y=')
!
!               GET THE DOMAIN DEFINITION (latitude).
!
                call get_listarg( name_proc, name(4:),                    &
                                  "lat1 and lat2 are integer values",     &
                                  "Wrong domain definition (latitude)",   &
                                  lat1, lat2 )
!
                use_y = true
!
            case ('-t=')
!
!               GET THE PERIOD DEFINITION.
!
                call get_listarg( name_proc, name(4:),                    &
                                  "time1 and time2 are integer values",   &
                                  "Wrong period definition",              &
                                  time1, time2 )
!
            case ('-m=')
!
!               GET THE NAME OF THE NETCDF MESH-MASK DATASET.
!
                mask_name = name(4:)
!
                use_mask = true
!
            case ('-o=')
!
!               GET THE NAME OF THE NEW NETCDF DATASET FOR STORING THE RESULTS.
!
                trend_name = name(4:)
!
            case ('-a=')
!
!               DETERMINE IF THE RESIDUALS ARE REQUIRED.
!
                call case_change( name(4:), tolower )
!
                select case ( name(4:) )
                    case ( cmethod(1) )
                        method = 1
                    case ( cmethod(2) )
                        method     = 2
                        comp_resid = true
                    case default
                        call merror( name_proc//"Unknown analysis option" )
                end select
!
            case default
!
                select case ( name(1:4 ) )
!
                    case ('-nt=')
!
!                       GET THE LENGTH OF THE TREND SMOOTHER.
!
                        call string_to_val( name(5:), kcode, fmt )
                        if ( kcode==kint ) then
                            read( name(5:), fmt ) nt
                        end if
                        if ( kcode/=kint .or. nt<=2  )                           &
                        call merror( name_proc//"The -nt= argument is an integer > 2" )
!
                    case ('-mi=')
!
!                       GET THE MISSING VALUE INDICATOR.
!
                        call string_to_val( name(5:), kcode, fmt )
                        if ( kcode/=kfix .and. kcode/=kexp )                &
                    call merror( name_proc//"The -mi= argument must be a real number " )
                        read( name(5:), fmt ) missing
!
                    case default
!
                        call case_change( name(4:5), tolower )
!
                        select case ( name(1:5 ) )
!
                            case ('-smt=')
!
!                               DETERMINE THE TREND SMOOTHING FACTOR.
!
                                call string_to_val( name(6:), kcode, fmt )
                                if ( kcode==kint ) then
                                     read( name(6:), fmt ) trend_smooth_factor
                                end if
                                if ( trend_smooth_factor<=0 )        &
         call merror( name_proc//"The -smt= argument is a strictly positive integer value " )
!
                            case default
!
                                call case_change( name(6:7), tolower )
!
                                select case ( name(1:7) )
!
                                    case ('-itdeg=')
!
!                                       GET THE DEGREE OF LOCALLY-FITTED POLYNOMIAL IN TREND SMOOTHING.
!
                                        call string_to_val( name(8:), kcode, fmt )
                                        if ( kcode==kint ) then
                                            read( name(8:), fmt ) itdeg
                                        end if
                                        if ( kcode/=kint .or. (itdeg<0 .or. itdeg>2)  )                &
                                        call merror( name_proc//"The -itdeg= argument must be 0, 1 or 2." )
!
                                    case ('-maxit=')
!
!                                       GET THE MAXIMUM NUMBER OF ROBUSTNESS ITERATIONS.
!
                                        call string_to_val( name(8:), kcode, fmt )
                                        if ( kcode==kint ) then
                                            read( name(8:), fmt ) maxiter
                                        end if
                                        if ( kcode/=kint .or. maxiter<=0  )                &
                                 call merror( name_proc//"The -maxit= argument is a strictly positive integer" )
!
                                    case default
!
                                        select case ( name(1:8) )
!
                                            case ('-ntjump=')
!
!                                               GET THE TREND SKIPPING VALUE.
!
                                                call string_to_val( name(9:), kcode, fmt )
                                                if ( kcode==kint ) then
                                                    read( name(9:), fmt ) ntjump
                                                end if
                                                if ( kcode/=kint .or. ntjump<=0  )                &
                                 call merror( name_proc//"The -ntjump= argument is a stricly positive integer" )
!
                                            case default
!
                                                call case_change( name(8:), tolower )
!
                                                select case ( name )
!
                                                    case ('-robust')
!
!                                                       ROBUST ITERATIONS ARE REQUIRED.
!
                                                        robust  = true
                                                        trendtype = 2
!
                                                    case ('-double')
!
!                                                       STORE RESULTS AS DOUBLE PRECISION NUMBERS.
!
                                                        output_type = nf90_real8
!
#ifdef _USE_NETCDF36
                                                    case ('-bigfile')
!
!                                                       REDEFINE THE CREATION MODE FLAG FOR THE OUTPUT FILE.
!
                                                        output_mode = nf90_64bit_offset
#endif
!
#ifdef _USE_NETCDF4
                                                    case ('-hdf5')
!
!                                                       REDEFINE THE CREATION MODE FLAG FOR THE OUTPUT FILE.
!
                                                        output_mode = nf90_hdf5
                                                        create_hdf5 = true
#endif
!
                                                    case ('-tlimited')
!
!                                                       DEFINE THE TIME DIMENSION AS LIMITED.
!
                                                        use_tlimited = true
!
                                                    case default
!
!                                                       WRONG ARGUMENT.
!
                                                        call merror( name_proc//"Wrong argument" )
!
                                                  end select
!
                                          end select
!
                                  end select
!
                          end select
!
                end select
!
        end select
!
    end do
!
!   CHECK THE COMMAND LINE ARGUMENTS.
!
    if ( fic_name==" " )                                                 &
    call merror( name_proc//"Input NETCDF file is missing" )
!
    if ( var_name==" " )                                                 &
    call merror( name_proc//"Input NETCDF variable is missing" )
!
    if ( nt==-1_i4b )                                                    &
    call merror( name_proc//"The -nt= argument is missing" )
    if ( mod(nt,2_i4b)==0_i4b) nt = nt + 1_i4b
!
    if ( ntjump==-1_i4b ) then
        ntjump = max( 1_i4b, int(real(nt,stnd)/ten + c0_9,i4b) )
    end if
!
    if ( trend_name==" " ) trend_name="trend_"//trim(var_name)//".nc"
!
!   OPEN THE NETCDF DATASETS.
!
    k = nf90_sizehint_default
    status = nf90_open( trim(fic_name), nf90_nowrite, ncid, chunksize=k )
    if ( status/=nf90_noerr )                                               &
    call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    if ( use_mask ) then
        status = nf90_open( trim(mask_name), nf90_nowrite, ncid_mask )
        if ( status/=nf90_noerr )                                             &
        call merror( name_proc//trim( nf90_strerror( status ) ) )
    end if
!
!   GET THE VARIABLE ID FROM ITS NAME.
!   FIND THE DIMENSION LENGTHS AND NAMES FOR THE NETCDF VARIABLE.
!   GET THE COORDINATE VARIABLES IDS FROM THEIR NAMES, IF ANY.
!   FIND THE EXTERNAL TYPE, THE DIMENSION ID AND THE NUMBER OF ATTRIBUTES 
!   FOR THE COORDINATE VARIABLES.
!
    call get_variable_dimcoords(  name_proc, trim(var_name), ncid,                    &
                                  varid,  dimids_out=dimids, dim_names_out=dim_names, &
                                  dimlens_out=dimlens, coor_id_out=coor_id,           &
                                  coor_natts_out=coor_natts, coor_type_out=coor_type, &
                                  use_coor_variable_out=use_coor_variable             )
!
    if ( count(dimlens/=0)/=3 )           &
    call merror( name_proc//"Input NETCDF variable is not tridimensional" )
!
    nlon   = dimlens(1)
    nlat   = dimlens(2)
    ntime  = dimlens(3)
!
    if ( use_mask ) then
!
        chr = merge( 't', grid_type, grid_type=='w' )
!
!       GET THE MASK ID FROM ITS NAME.
!       VERIFY THE DIMENSION LENGTHS FOR THE NETCDF MASK VARIABLE.
!
        call get_variable_dims( name_proc, chr//"mask", ncid_mask, maskid, dimids, dimlens )
!
        if ( dimlens(1)/=nlon .or. dimlens(2)/=nlat )                      &
        call merror( name_proc//"Shape disagreement with the mask" )
!
        if ( .not.use_x ) then
!
            status = nf90_get_att( ncid_mask, maskid, "lon1_Eastern_limit", i )
            if ( status==nf90_noerr ) lon1 = i
!
            status = nf90_get_att( ncid_mask, maskid, "lon2_Western_limit", i )
            if ( status==nf90_noerr ) lon2 = i
!
        end if
!
        if ( .not.use_y ) then
!
            status = nf90_get_att( ncid_mask, maskid, "lat1_Southern_limit", i )
            if ( status==nf90_noerr ) lat1 = i
!
            status = nf90_get_att( ncid_mask, maskid, "lat2_Northern_limit", i )
            if ( status==nf90_noerr ) lat2 = i
!
        end if
!
    end if
!
!   CHECK THE BOUNDARIES OF THE GEOGRAPHICAL DOMAIN.
!
    if ( lon1<0 ) then
        lon2 = max( lon2, 1 )
        lon3 = nlon + lon1 - lon2 + 1
    else
        lon1 = max( lon1, 1 )
        lon2 = min( lon2, nlon )
        lon3 = lon2 - lon1 + 1
    end if
!
    if ( lon3<=0  )  call merror( name_proc//"Wrong domain definition (longitude)" )
!
    if ( lon1<0 ) then
       lon3 = nlon
       j1   = 1
       j2   = nlon
    else
       j1 = lon1
       j2 = lon2
    end if
!
    lat1 = max( lat1, 1 )
    lat2 = min( lat2, nlat )
    lat3 = lat2 - lat1 + 1
    if ( lat3<=0  )  call merror( name_proc//"Wrong domain definition (latitude)" )
!
!   CHECK THE TIME DEFINITION.
!
    time1 = max( time1, 1 )
    time2 = min( time2, ntime )
    time3 = time2 - time1 + 1
    if ( time3<=0  ) call merror( name_proc//"Wrong time definition" )
!
!   ALLOCATE WORK ARRAYS.
!
    allocate( temp(nlon,nlat), mask(nlon,nlat), stat=status )
    if ( status/=0 ) call merror( name_proc//allocate_error )
!
    if ( use_mask ) then
!
!       GET THE MASK.
!
        status = nf90_get_var( ncid_mask, maskid, temp )     
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!       DETERMINE THE GRID POINTS USED IN THE STL ANALYSIS FROM THE MASK.
!
        mask(:nlon,:nlat) = temp(:nlon,:nlat)==one
!
    else
!
!       CONSTRUCT A DUMMY MASK.
!
        mask(:nlon,:nlat) = true
!
    end if
!
!   SELECT THE DOMAIN.
!
    if ( lon1>0 ) then
        mask(1:lon1-1,:nlat)    = false
        mask(lon2+1:nlon,:nlat) = false
    else
        mask(lon2+1:nlon+lon1,:nlat) = false
    end if
!
    mask(:nlon,1:lat1-1)    = false
    mask(:nlon,lat2+1:nlat) = false
!   
    nvar = count( mask )
    if ( nvar<=0  )  call merror( name_proc//"Wrong domain definition : domain is empty" )
!
!   ALLOCATE WORK ARRAYS.
!
    allocate( y(time3,nvar), trend(time3,nvar), ok(nvar),  stat=status   )
    if ( status/=0 ) call merror( name_proc//allocate_error )
!
!   COMPUTE THE OFFSET FOR THE RECORD NUMBER.
!
    offset_rec = time1 - 1
!
!   FIRST READ THE NETCDF VARIABLE.
!
    countlec(1:3)  = (/ lon3, lat3, 1_i4b /)
!
#ifdef _PARALLEL_READ
!
    i = omp_get_num_procs()
    j = omp_get_max_threads()
    test_par = i>1 .and. j>1 .and. time3*nvar>omp_limit
!
    if ( test_par ) then
!
!       DEALLOCATE WORK ARRAY.
!
        deallocate( temp )
!
!$OMP PARALLEL PRIVATE(i,k,status,startlec,ncid2,varid2,rec,temp)      &
!$OMP         ,SHARED(y,mask,varid,j1,j2,lon3,lat1,lat2,lat3,time1,time2,countlec,nvar,offset_rec)
!
!       ALLOCATE WORK ARRAY.
!
        allocate( temp(lon3,lat3), stat=status )
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
!$OMP CRITICAL (opennc)
!
        status = nf90_open( trim(fic_name), nf90_share, ncid2  )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!       GET THE VARIABLE ID FROM ITS NAME.
!
        status = nf90_inq_varid( ncid2, trim(var_name), varid2 )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!$OMP END CRITICAL (opennc)
!
        startlec(1)  = j1
        startlec(2)  = lat1
!
!$OMP DO SCHEDULE(STATIC)
        do i = time1, time2
!
            startlec(3) = i
!
!           FILL THE WORK ARRAY WITH A RECORD FROM THE NETCDF VARIABLE IN THE INPUT FILE.
!
            status = nf90_get_var( ncid2, varid2, temp(:lon3,:lat3), start=startlec, count=countlec  )     
            if ( status/=nf90_noerr )   call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           PACK THE DATA.
!
            rec           = i - offset_rec
            y(rec,1:nvar) = pack( temp(:lon3,:lat3), mask(j1:j2,lat1:lat2) )
!
        end do
!$OMP END DO
!
!       DEALLOCATE WORK ARRAY.
!
        deallocate( temp )
!
!$OMP CRITICAL (closenc)
!
        status = nf90_close( ncid2 )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!$OMP END CRITICAL (closenc)
!
!$OMP END PARALLEL
!
!       REALLOCATE WORK ARRAY.
!
        allocate( temp(nlon,nlat), stat=status )
        if ( status/=0 ) call merror( name_proc//allocate_error )
!
    else
!
#endif
!
        startlec(1:2)  = (/ j1, lat1 /)
!
        do i = time1, time2
!
            startlec(3) = i
!
!           FILL THE WORK ARRAY WITH A RECORD FROM THE NETCDF VARIABLE IN THE INPUT FILE.
!
            status = nf90_get_var( ncid, varid, temp(:lon3,:lat3), start=startlec, count=countlec  )     
            if ( status/=nf90_noerr )   call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           PACK THE DATA.
!
            rec           = i - offset_rec
            y(rec,1:nvar) = pack( temp(:lon3,:lat3), mask(j1:j2,lat1:lat2) )
!
        end do
!
#ifdef _PARALLEL_READ
!
    end if
!
#endif
!
!   FIND THE SCALE_FACTOR ATTRIBUTE ASSOCIATED WITH THE NETCDF VARIABLE, IF ANY.
!
    status = nf90_get_att( ncid, varid, "scale_factor", tmp )
    if ( status==nf90_noerr ) then
        y(1:time3,1:nvar) = y(1:time3,1:nvar)*tmp
    end if
!
!   FIND THE ADD_OFFSET ATTRIBUTE ASSOCIATED WITH THE NETCDF VARIABLE, IF ANY.
!
    status = nf90_get_att( ncid, varid, "add_offset", tmp )
    if ( status==nf90_noerr )  then
        y(1:time3,1:nvar) = y(1:time3,1:nvar) + tmp
    end if
!
!   EXTRACT THE TREND COMPONENT FROM THE TIME SERIES.
!
    call comp_trend( y(:time3,:nvar), nt, itdeg, robust, trend(:time3,:nvar),  &
                     ntjump=ntjump, maxiter=maxiter, ok=ok(:nvar) )
!
!   SMOOTH THE TREND COMPONENT IF REQUIRED.
!
    if ( trend_smooth_factor/=0 ) then
        call comp_smooth( trend(:time3,:nvar), trend_smooth_factor, dimvar=2_i4b )
    end if
!
!   COMPUTE RESIDUALS IF REQUIRED.
!
    if ( method==2 ) then
         y(:time3,:nvar) = y(:time3,:nvar) - trend(:time3,:nvar)
    end if
!
!   CREATE THE NEW NETCDF DATASET FOR THE TREND ANALYSIS.
!
    if ( create_hdf5 ) then
        status = nf90_create( trim(trend_name), cmode=output_mode, ncid=ncid_trend )
    else
        k = nf90_sizehint_default
        status = nf90_create( trim(trend_name), cmode=output_mode, ncid=ncid_trend, chunksize=k )
    end if
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_set_fill( ncid_trend, nf90_nofill, i )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!   DEFINE THE DIMENSIONS FOR THE NEW NETCDF DATASET.
!
    dimlens(1)   = nlon
    dimlens(2)   = nlat
    if ( use_tlimited ) then
        dimlens(3)   = time3
    else
        dimlens(3)   = nf90_unlimited
    end if
!
    do i = 1, 3
!
        status = nf90_def_dim( ncid_trend, name=trim(dim_names(i)),                         &
                               len=dimlens(i), dimid=dimids(i) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim(nf90_strerror(status)) )
!
    end do
!
!   DEFINE THE VARIABLES FOR THE NEW NETCDF DATASET.
!
    do i = 1, 3
!
        status = nf90_def_var( ncid_trend, trim(dim_names(i)), coor_type(i),                &
                               dimids=dimids(i), varid=coor_id2(i) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim(nf90_strerror(status)) )
!
    end do
!
    if ( robust ) then
        status = nf90_def_var( ncid_trend, trim(var_name)//"_convergence", nf90_real4,      &
                               dimids=dimids(:2), varid=convid )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
    end if
!
    status = nf90_def_var( ncid_trend, trim(var_name)//"_trend", output_type, dimids=dimids(:3), varid=trendid )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    if ( comp_resid ) then
        status = nf90_def_var( ncid_trend, trim(var_name)//"_"//cmethod(method), output_type,      &
                               dimids=dimids(:3), varid=residid )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
    end if
!
!   ASSIGN ATTRIBUTES FOR THE NEW NETCDF DATASET.
!
    do j = 1, 3
!
        if ( .not.use_coor_variable(j) ) cycle
!
        do i = 1, coor_natts(j)
!
            status = nf90_inq_attname( ncid, coor_id(j), i, att_name )
            if ( status/=nf90_noerr )                                                     &
            call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!           COPY ATTRIBUTES FOR THE COORDINATE VARIABLES.
!
            status = nf90_copy_att(  ncid, coor_id(j), att_name, ncid_trend, coor_id2(j) )
            if ( status/=nf90_noerr )                                                     &
            call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end do
!
    end do
!
    if ( .not.use_coor_variable(3) ) then
!
        status = nf90_put_att( ncid_trend, coor_id2(3), "long_name", "Time axis" )
        if ( status/=nf90_noerr )                                                  &
        call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_trend, coor_id2(3), "short_name", "Time" )
        if ( status/=nf90_noerr )                                                  &
        call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end if
!
    if ( robust ) then
!
        status = nf90_put_att( ncid_trend, convid, "long_name",     &
                               "Convergence or robust iterations for  "//trim(var_name)  )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_trend, convid, "short_name", "Convergence"  )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_trend, convid, "missing_value", real( missing, sp ) )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_trend, convid, "_FillValue", real( missing, sp ) )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    end if
!
    status = nf90_put_att( ncid_trend, trendid, "long_name", "trend of "//trim(var_name)  )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_trend, trendid, "short_name", "trend"  )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    if ( trend_smooth_factor/=0_i4b ) then
!
        status = nf90_put_att( ncid_trend, trendid, "smoothing_factor", trend_smooth_factor )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end if
!
    if ( comp_resid ) then
!
        status = nf90_put_att( ncid_trend, residid, "long_name", cmethod(method)//" of "//trim(var_name)  )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_trend, residid, "short_name", cmethod(method)  )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    end if
!
    if (  output_type==nf90_real8 ) then
!
        status = nf90_put_att( ncid_trend, trendid, "missing_value", real( missing, dp ) )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_trend, trendid, "_FillValue", real( missing, dp ) )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        if ( comp_resid ) then
!
            status = nf90_put_att( ncid_trend, residid, "missing_value", real( missing, dp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_trend, residid, "_FillValue", real( missing, dp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end if
!
    else
!
        status = nf90_put_att( ncid_trend, trendid, "missing_value", real( missing, sp ) )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_trend, trendid, "_FillValue", real( missing, sp ) )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        if ( comp_resid ) then
!
            status = nf90_put_att( ncid_trend, residid, "missing_value", real( missing, sp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_att( ncid_trend, residid, "_FillValue", real( missing, sp ) )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        end if
!
    end if
!
    status = nf90_inquire_attribute( ncid, varid, "units" )
    if ( status==nf90_noerr ) then
!
        status = nf90_copy_att(  ncid, varid, "units", ncid_trend, trendid )
        if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
        if ( comp_resid ) then
            status = nf90_copy_att(  ncid, varid, "units", ncid_trend, residid )
            if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
        end if
!
    end if
!
!   SET GLOBAL ATTRIBUTES.
!
    status = nf90_put_att( ncid_trend, nf90_global, "title", "TREND analysis of "//trim(var_name) )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_trend, nf90_global, "grid_type", grid_type )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_trend, nf90_global, "trend_smoother_length", nt )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_trend, nf90_global, "trend_smoother_degree", itdeg )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_trend, nf90_global, "trend_skipping_value", ntjump )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    if ( robust ) then
!
        k = count( ok(:nvar) )
        call val_to_string( k, fmt, i )
        call val_to_string( nvar, fmt2, j )
        name = fmt(1:i)//" on "//fmt2(1:j)
!
        status = nf90_put_att( ncid_trend, nf90_global, "convergence_factor", trim(name) )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        status = nf90_put_att( ncid_trend, nf90_global, "max_robustness_iterations", maxiter )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end if
!
    status = nf90_put_att( ncid_trend, nf90_global, "trendtype", ctrendtype(trendtype) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_trend, nf90_global, "method", cmethod(method) )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    status = nf90_put_att( ncid_trend, nf90_global, "history", trim(history) )
    if ( status/=nf90_noerr ) call merror( name_proc//trim( nf90_strerror( status ) ) )
!
!   LEAVE DEFINE MODE FOR THE NEW NETCDF DATASET.
!
    status = nf90_enddef( ncid_trend )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
!   WRITE VARIABLES FOR THE NEW  NETCDF DATASET.
!
!
!   ALLOCATE WORK ARRAY.
!
    i = max( nlon, nlat, time3 )
    allocate( record(i), stat=status )
    if ( status/=0 ) call merror( name_proc//allocate_error )
!
    dimlens(3) = time3
!
    do j = 1, 3
!
        j1 = merge( 1_i4b, time1, j/=3_i4b)
        j2 = dimlens(j)
!
        if ( use_coor_variable(j) ) then
            status = nf90_get_var( ncid, coor_id(j), record(:j2), start=(/ j1 /), count=(/ j2 /) )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
            status = nf90_put_var( ncid_trend, coor_id2(j), record(:j2), count=(/ j2 /) )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
        else
            status = nf90_put_var( ncid_trend, coor_id2(j), arth(j1,1_i4b,j2 ) )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
        end if
!
    end do
!
!   DEALLOCATE WORK ARRAY.
!
    deallocate( record )
!
    startlec(1:2)  = 1_i4b
!
!   WRITE THE TREND COMPONENT.
!
    do i = 1, time3
!
!       UNPACK THE TREND COMPONENT.
!
        temp(:nlon,:nlat) = unpack( trend(i,1:nvar), mask(:nlon,:nlat), missing )
!
        startlec(3)  = i
!
        status = nf90_put_var( ncid_trend, trendid, temp, start=startlec )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end do
!
!   WRITE THE CONVERGENCE INDICATOR, IF REQUIRED.
!
    if ( robust ) then
!
        where ( ok )
            trend(1,1:nvar) = one
        elsewhere
            trend(1,1:nvar) = zero
        end where
        temp(:nlon,:nlat) = unpack( trend(1,1:nvar), mask(:nlon,:nlat), missing )
!
        status = nf90_put_var( ncid_trend, convid, temp )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    end if
!
    if ( comp_resid ) then
!
!       WRITE THE RESIDUALS IF REQUIRED.
!
        do i = 1, time3
!
!           UNPACK THE RESIDUALS.
!
            temp(:nlon,:nlat) = unpack( y(i,1:nvar), mask(:nlon,:nlat), missing )
!
            startlec(3)  = i
!
            status = nf90_put_var( ncid_trend, residid, temp, start=startlec )
            if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
        end do
!
    end if
!
!   DEALLOCATE WORK ARRAYS.
!
    deallocate( temp, mask, y, trend, ok )
!
!   CLOSE THE NETCDF DATASETS.
!
    status = nf90_close( ncid )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
    if ( use_mask ) then
        status = nf90_close( ncid_mask )
        if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
    end if
!
    status = nf90_close( ncid_trend )
    if ( status/=nf90_noerr ) call merror(  name_proc//trim( nf90_strerror( status ) ) )
!
! END OF PROGRAM comp_trend_3d
! ============================
!
end program comp_trend_3d
